@@ -1,7 +1,8 @@
 #include <filezilla.h>
 
-CConnectCommand::CConnectCommand(CServer const& server, Credentials const& credentials, bool retry_connecting)
+CConnectCommand::CConnectCommand(CServer const& server, ServerHandle const& handle, Credentials const& credentials, bool retry_connecting)
 	: server_(server)
+	, handle_(handle)
 	, credentials_(credentials)
 	, retry_connecting_(retry_connecting)
 {
@@ -93,8 +94,8 @@ std::wstring CRawCommand::GetCommand() const
 	return m_command;
 }
 
-CDeleteCommand::CDeleteCommand(const CServerPath& path, std::deque<std::wstring>&& files)
-	: m_path(path), m_files(files)
+CDeleteCommand::CDeleteCommand(const CServerPath& path, std::vector<std::wstring>&& files)
+	: m_path(path), files_(files)
 {
 }
 
