@@ -4,6 +4,11 @@
 
 #include <algorithm>
 
+void CDirentry::clear()
+{
+	*this = CDirentry();
+}
+
 std::wstring CDirentry::dump() const
 {
 	std::wstring str = fz::sprintf(L"name=%s\nsize=%d\npermissions=%s\nownerGroup=%s\ndir=%d\nlink=%d\ntarget=%s\nunsure=%d\n",
