@@ -1,11 +1,12 @@
 #include <filezilla.h>
 #include "directorylistingparser.h"
-#include "ControlSocket.h"
+#include "controlsocket.h"
 
 #include <libfilezilla/format.hpp>
 
 #include <algorithm>
 #include <vector>
+#include <limits>
 
 #include <string.h>
 
@@ -226,13 +227,18 @@ public:
 		switch (base) {
 		default:
 		case decimal:
-			if (m_number == -1) {
+			if (m_number == std::numeric_limits<int64_t>::min()) {
+				constexpr int64_t max = (std::numeric_limits<int64_t>::max() - 9) / 10;
 				if (IsNumeric() || IsLeftNumeric()) {
 					m_number = 0;
 					for (unsigned int i = 0; i < m_len; ++i) {
 						if (m_pToken[i] < '0' || m_pToken[i] > '9') {
 							break;
 						}
+						if (m_number > max) {
+							m_number = -1;
+							break;
+						}
 						m_number *= 10;
 						m_number += m_pToken[i] - '0';
 					}
@@ -244,6 +250,10 @@ public:
 						--start;
 					}
 					for (unsigned int i = start; i < m_len; ++i) {
+						if (m_number > max) {
+							m_number = -1;
+							break;
+						}
 						m_number *= 10;
 						m_number += m_pToken[i] - '0';
 					}
@@ -252,8 +262,12 @@ public:
 			return m_number;
 		case hex:
 			{
+				constexpr int64_t max = (std::numeric_limits<int64_t>::max() - 15) / 16;
 				int64_t number = 0;
 				for (unsigned int i = 0; i < m_len; ++i) {
+					if (number > max) {
+						return -1;
+					}
 					const wchar_t& c = m_pToken[i];
 					if (c >= '0' && c <= '9') {
 						number *= 16;
@@ -277,7 +291,7 @@ public:
 	}
 
 protected:
-	int64_t m_number{-1};
+	int64_t m_number{std::numeric_limits<int64_t>::min()};
 
 	wchar_t const* m_pToken{};
 
@@ -1222,7 +1236,9 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine & line, int &index, CDiren
 
 		// Some servers use times only for files newer than 6 months
 		if (year <= 0) {
-			assert(month != -1 && day != -1);
+			if (month == -1 || day == -1) {
+				return false;
+			}
 			tm const t = fz::datetime::now().get_tm(fz::datetime::utc);
 			year = t.tm_year + 1900;
 			int const currentDayOfYear = t.tm_mday + 31 * t.tm_mon;
@@ -1451,9 +1467,9 @@ bool CDirectoryListingParser::ParseShortDate(CToken &token, CDirentry &entry, bo
 		gotYear = true;
 	}
 
-	assert(gotYear);
-	assert(gotMonth);
-	assert(gotDay);
+	if (!gotYear || !gotMonth || !gotDay) {
+		return false;
+	}
 
 	if (!entry.time.set(fz::datetime::utc, year, month, day)) {
 		return false;
@@ -2078,7 +2094,7 @@ bool CDirectoryListingParser::AddData(char *pData, int len)
 bool CDirectoryListingParser::AddLine(std::wstring && line, std::wstring && name, fz::datetime const& time)
 {
 	if (m_pControlSocket) {
-		m_pControlSocket->LogMessageRaw(MessageType::RawList, line);
+		m_pControlSocket->log_raw(logmsg::listing, line);
 	}
 
 	CDirentry override;
@@ -2129,7 +2145,7 @@ CLine *CDirectoryListingParser::GetLine(bool breakAtEnd, bool &error)
 				if (iter == m_DataList.end()) {
 					if (reslen > 10000) {
 						if (m_pControlSocket) {
-							m_pControlSocket->LogMessage(MessageType::Error, _("Received a line exceeding 10000 characters, aborting."));
+							m_pControlSocket->log(logmsg::error, _("Received a line exceeding 10000 characters, aborting."));
 						}
 						error = true;
 						return nullptr;
@@ -2146,7 +2162,7 @@ CLine *CDirectoryListingParser::GetLine(bool breakAtEnd, bool &error)
 
 		if (reslen > 10000) {
 			if (m_pControlSocket) {
-				m_pControlSocket->LogMessage(MessageType::Error, _("Received a line exceeding 10000 characters, aborting."));
+				m_pControlSocket->log(logmsg::error, _("Received a line exceeding 10000 characters, aborting."));
 			}
 			error = true;
 			return nullptr;
@@ -2198,7 +2214,7 @@ CLine *CDirectoryListingParser::GetLine(bool breakAtEnd, bool &error)
 		std::wstring buffer;
 		if (m_pControlSocket) {
 			buffer = m_pControlSocket->ConvToLocal(res, buflen);
-			m_pControlSocket->LogMessageRaw(MessageType::RawList, buffer);
+			m_pControlSocket->log_raw(logmsg::listing, buffer);
 		}
 		else {
 			buffer = fz::to_wstring_from_utf8(res);
@@ -3164,7 +3180,7 @@ void CDirectoryListingParser::DeduceEncoding()
 
 	if ((count[0x1f] || count[0x15] || count[0x25]) && !count[0x0a] && count[static_cast<unsigned char>('@')] && count[static_cast<unsigned char>('@')] > count[static_cast<unsigned char>(' ')] && count_ebcdic > count_normal) {
 		if (m_pControlSocket) {
-			m_pControlSocket->LogMessage(MessageType::Status, _("Received a directory listing which appears to be encoded in EBCDIC."));
+			m_pControlSocket->log(logmsg::status, _("Received a directory listing which appears to be encoded in EBCDIC."));
 		}
 		m_listingEncoding = listingEncoding::ebcdic;
 		for (auto & data : m_DataList) {
