@@ -32,6 +32,12 @@
  * get concatenated to be parsed again (and discarded if not recognized).
  */
 
+#include <directorylisting.h>
+#include <server.h>
+
+#include <deque>
+#include <vector>
+
 class CLine;
 class CToken;
 class CControlSocket;
