@@ -1,12 +1,11 @@
 #include <filezilla.h>
-#include "ControlSocket.h"
+#include "controlsocket.h"
 #include "directorycache.h"
 #include "engineprivate.h"
 #include "ftp/ftpcontrolsocket.h"
 #include "http/httpcontrolsocket.h"
 #include "logging_private.h"
 #include "pathcache.h"
-#include "ratelimiter.h"
 #include "sftp/sftpcontrolsocket.h"
 #if ENABLE_STORJ
 #include "storj/storjcontrolsocket.h"
@@ -16,36 +15,44 @@
 
 #include <algorithm>
 
-fz::mutex CFileZillaEnginePrivate::mutex_;
+fz::mutex CFileZillaEnginePrivate::global_mutex_{false};
 std::vector<CFileZillaEnginePrivate*> CFileZillaEnginePrivate::m_engineList;
 std::atomic_int CFileZillaEnginePrivate::m_activeStatus[2] = {{0}, {0}};
 std::list<CFileZillaEnginePrivate::t_failedLogins> CFileZillaEnginePrivate::m_failedLogins;
 
+struct invalid_current_working_dir_event_type{};
+typedef fz::simple_event<invalid_current_working_dir_event_type, CServer, CServerPath> CInvalidateCurrentWorkingDirEvent;
+
+
 namespace {
-unsigned int get_next_engine_id(fz::mutex& mutex)
+unsigned int get_next_engine_id()
 {
-	fz::scoped_lock lock(mutex);
-	static unsigned int id = 0;
-	return ++id;
+	static std::atomic<int> next_{};
+	return ++next_;
 }
 }
 
 CFileZillaEnginePrivate::CFileZillaEnginePrivate(CFileZillaEngineContext& context, CFileZillaEngine& parent, EngineNotificationHandler& notificationHandler)
 	: event_handler(context.GetEventLoop())
 	, transfer_status_(*this)
+	, opLockManager_(context.GetOpLockManager())
 	, notification_handler_(notificationHandler)
-	, m_engine_id(get_next_engine_id(mutex_))
+	, m_engine_id(get_next_engine_id())
 	, m_options(context.GetOptions())
-	, m_rateLimiter(context.GetRateLimiter())
+	, rate_limiter_(context.GetRateLimiter())
 	, directory_cache_(context.GetDirectoryCache())
 	, path_cache_(context.GetPathCache())
 	, parent_(parent)
 	, thread_pool_(context.GetThreadPool())
 	, encoding_converter_(context.GetCustomEncodingConverter())
+	, context_(context)
 {
-	m_engineList.push_back(this);
+	{
+		fz::scoped_lock lock(global_mutex_);
+		m_engineList.push_back(this);
+	}
 
-	m_pLogging = new CLogging(*this);
+	logger_ = std::make_unique<CLogging>(*this);
 
 	{
 		bool queue_logs = ShouldQueueLogsFromOptions();
@@ -69,9 +76,11 @@ bool CFileZillaEnginePrivate::ShouldQueueLogsFromOptions() const
 CFileZillaEnginePrivate::~CFileZillaEnginePrivate()
 {
 	remove_handler();
+	UnregisterAllOptions();
+
 	m_maySendNotificationEvent = false;
 
-	m_pControlSocket.reset();
+	controlSocket_.reset();
 	m_pCurrentCommand.reset();
 
 	// Delete notification list
@@ -80,15 +89,16 @@ CFileZillaEnginePrivate::~CFileZillaEnginePrivate()
 	}
 
 	// Remove ourself from the engine list
-	m_engineList.erase(std::remove(m_engineList.begin(), m_engineList.end(), this), m_engineList.end());
-	for (auto iter = m_engineList.begin(); iter != m_engineList.end(); ++iter) {
-		if (*iter == this) {
-			m_engineList.erase(iter);
-			break;
+	{
+		fz::scoped_lock lock(global_mutex_);
+		m_engineList.erase(std::remove(m_engineList.begin(), m_engineList.end(), this), m_engineList.end());
+		for (auto iter = m_engineList.begin(); iter != m_engineList.end(); ++iter) {
+			if (*iter == this) {
+				m_engineList.erase(iter);
+				break;
+			}
 		}
 	}
-
-	delete m_pLogging;
 }
 
 void CFileZillaEnginePrivate::OnEngineEvent(EngineNotificationType type)
@@ -112,11 +122,12 @@ bool CFileZillaEnginePrivate::IsBusy() const
 bool CFileZillaEnginePrivate::IsConnected() const
 {
 	fz::scoped_lock lock(mutex_);
-	if (!m_pControlSocket) {
+	if (!controlSocket_) {
 		return false;
 	}
 
-	return m_pControlSocket->Connected();
+	// FIXME: This is unsafe!
+	return controlSocket_->Connected();
 }
 
 const CCommand *CFileZillaEnginePrivate::GetCurrentCommand() const
@@ -132,7 +143,7 @@ Command CFileZillaEnginePrivate::GetCurrentCommandId() const
 		return Command::none;
 	}
 	else {
-		return GetCurrentCommand()->GetId();
+		return m_pCurrentCommand->GetId();
 	}
 }
 
@@ -157,14 +168,14 @@ void CFileZillaEnginePrivate::AddLogNotification(CLogmsgNotification *pNotificat
 {
 	fz::scoped_lock lock(notification_mutex_);
 
-	if (pNotification->msgType == MessageType::Error) {
+	if (pNotification->msgType == logmsg::error) {
 		queue_logs_ = false;
 
 		m_NotificationList.insert(m_NotificationList.end(), queued_logs_.begin(), queued_logs_.end());
 		queued_logs_.clear();
 		AddNotification(lock, pNotification);
 	}
-	else if (pNotification->msgType == MessageType::Status) {
+	else if (pNotification->msgType == logmsg::status) {
 		ClearQueuedLogs(lock, false);
 		AddNotification(lock, pNotification);
 	}
@@ -217,11 +228,11 @@ void CFileZillaEnginePrivate::ClearQueuedLogs(bool reset_flag)
 int CFileZillaEnginePrivate::ResetOperation(int nErrorCode)
 {
 	fz::scoped_lock lock(mutex_);
-	m_pLogging->LogMessage(MessageType::Debug_Debug, L"CFileZillaEnginePrivate::ResetOperation(%d)", nErrorCode);
+	logger_->log(logmsg::debug_debug, L"CFileZillaEnginePrivate::ResetOperation(%d)", nErrorCode);
 
 	if (m_pCurrentCommand) {
 		if ((nErrorCode & FZ_REPLY_NOTSUPPORTED) == FZ_REPLY_NOTSUPPORTED) {
-			m_pLogging->LogMessage(MessageType::Error, _("Command not supported by this protocol"));
+			logger_->log(logmsg::error, _("Command not supported by this protocol"));
 		}
 
 		if (m_pCurrentCommand->GetId() == Command::connect) {
@@ -239,7 +250,7 @@ int CFileZillaEnginePrivate::ResetOperation(int nErrorCode)
 						if (!delay) {
 							delay = fz::duration::from_seconds(1);
 						}
-						m_pLogging->LogMessage(MessageType::Status, _("Waiting to retry..."));
+						logger_->log(logmsg::status, _("Waiting to retry..."));
 						stop_timer(m_retryTimer);
 						m_retryTimer = add_timer(delay, true);
 						return FZ_REPLY_WOULDBLOCK;
@@ -297,14 +308,14 @@ int CFileZillaEnginePrivate::Connect(CConnectCommand const& command)
 	// Need to delete before setting m_pCurrentCommand.
 	// The destructor can call CFileZillaEnginePrivate::ResetOperation
 	// which would delete m_pCurrentCommand
-	m_pControlSocket.reset();
+	controlSocket_.reset();
 	m_nControlSocketError = 0;
 
 	auto const& server = command.GetServer();
 	if (server.GetPort() != CServer::GetDefaultPort(server.GetProtocol())) {
 		ServerProtocol protocol = CServer::GetProtocolFromPort(server.GetPort(), true);
 		if (protocol != UNKNOWN && protocol != server.GetProtocol()) {
-			m_pLogging->LogMessage(MessageType::Status, _("Selected port usually in use by a different protocol."));
+			logger_->log(logmsg::status, _("Selected port usually in use by a different protocol."));
 		}
 	}
 
@@ -314,9 +325,9 @@ int CFileZillaEnginePrivate::Connect(CConnectCommand const& command)
 int CFileZillaEnginePrivate::Disconnect(CDisconnectCommand const&)
 {
 	int res = FZ_REPLY_OK;
-	if (m_pControlSocket) {
-		res = m_pControlSocket->Disconnect();
-		m_pControlSocket.reset();
+	if (controlSocket_) {
+		res = controlSocket_->Disconnect();
+		controlSocket_.reset();
 	}
 
 	return res;
@@ -328,18 +339,29 @@ int CFileZillaEnginePrivate::List(CListCommand const& command)
 	bool const refresh = (command.GetFlags() & LIST_FLAG_REFRESH) != 0;
 	bool const avoid = (command.GetFlags() & LIST_FLAG_AVOID) != 0;
 
+	if (flags & LIST_FLAG_CLEARCACHE) {
+		GetDirectoryCache().InvalidateServer(controlSocket_->GetCurrentServer());
+		GetPathCache().InvalidateServer(controlSocket_->GetCurrentServer());
+	}
+
 	if (!refresh && !command.GetPath().empty()) {
-		CServer const& server = m_pControlSocket->GetCurrentServer();
+		CServer const& server = controlSocket_->GetCurrentServer();
 		if (server) {
 			CServerPath path(path_cache_.Lookup(server, command.GetPath(), command.GetSubDir()));
 			if (path.empty()) {
 				if (command.GetSubDir().empty()) {
 					path = command.GetPath();
 				}
-				else if (server.GetProtocol() == STORJ) {
-                                        path = command.GetPath();
-                                        path.ChangePath(command.GetSubDir());
-                                }
+				else if (server.GetProtocol() == S3 || server.GetProtocol() == STORJ ||
+						server.GetProtocol() == WEBDAV || server.GetProtocol() == INSECURE_WEBDAV ||
+						server.GetProtocol() == AZURE_FILE || server.GetProtocol() == AZURE_BLOB ||
+						server.GetProtocol() == SWIFT || server.GetProtocol() == GOOGLE_CLOUD ||
+						server.GetProtocol() == GOOGLE_DRIVE || server.GetProtocol() == DROPBOX ||
+						server.GetProtocol() == ONEDRIVE || server.GetProtocol() == B2 ||
+						server.GetProtocol() == BOX) {
+					path = command.GetPath();
+					path.ChangePath(command.GetSubDir());
+				}
 			}
 			if (!path.empty()) {
 				CDirectoryListing *pListing = new CDirectoryListing;
@@ -351,7 +373,7 @@ int CFileZillaEnginePrivate::List(CListCommand const& command)
 					}
 					else {
 						if (!avoid) {
-							CDirectoryListingNotification *pNotification = new CDirectoryListingNotification(pListing->path);
+							CDirectoryListingNotification *pNotification = new CDirectoryListingNotification(pListing->path, true);
 							AddNotification(pNotification);
 						}
 						delete pListing;
@@ -366,13 +388,13 @@ int CFileZillaEnginePrivate::List(CListCommand const& command)
 		}
 	}
 
-	m_pControlSocket->List(command.GetPath(), command.GetSubDir(), flags);
+	controlSocket_->List(command.GetPath(), command.GetSubDir(), flags);
 	return FZ_REPLY_CONTINUE;
 }
 
 int CFileZillaEnginePrivate::FileTransfer(CFileTransferCommand const& command)
 {
-	m_pControlSocket->FileTransfer(command.GetLocalFile(), command.GetRemotePath(), command.GetRemoteFile(), command.Download(), command.GetTransferSettings());
+	controlSocket_->FileTransfer(command.GetLocalFile(), command.GetRemotePath(), command.GetRemoteFile(), command.Download(), command.GetTransferSettings());
 	return FZ_REPLY_CONTINUE;
 }
 
@@ -382,54 +404,54 @@ int CFileZillaEnginePrivate::RawCommand(CRawCommand const& command)
 		fz::scoped_lock lock(notification_mutex_);
 		queue_logs_ = false;
 	}
-	m_pControlSocket->RawCommand(command.GetCommand());
+	controlSocket_->RawCommand(command.GetCommand());
 	return FZ_REPLY_CONTINUE;
 }
 
 int CFileZillaEnginePrivate::Delete(CDeleteCommand& command)
 {
 	if (command.GetFiles().size() == 1) {
-		m_pLogging->LogMessage(MessageType::Status, _("Deleting \"%s\""), command.GetPath().FormatFilename(command.GetFiles().front()));
+		logger_->log(logmsg::status, _("Deleting \"%s\""), command.GetPath().FormatFilename(command.GetFiles().front()));
 	}
 	else {
-		m_pLogging->LogMessage(MessageType::Status, _("Deleting %u files from \"%s\""), static_cast<unsigned int>(command.GetFiles().size()), command.GetPath().GetPath());
+		logger_->log(logmsg::status, _("Deleting %u files from \"%s\""), static_cast<unsigned int>(command.GetFiles().size()), command.GetPath().GetPath());
 	}
-	m_pControlSocket->Delete(command.GetPath(), command.ExtractFiles());
+	controlSocket_->Delete(command.GetPath(), command.ExtractFiles());
 	return FZ_REPLY_CONTINUE;
 }
 
 int CFileZillaEnginePrivate::RemoveDir(CRemoveDirCommand const& command)
 {
-	m_pControlSocket->RemoveDir(command.GetPath(), command.GetSubDir());
+	controlSocket_->RemoveDir(command.GetPath(), command.GetSubDir());
 	return FZ_REPLY_CONTINUE;
 }
 
 int CFileZillaEnginePrivate::Mkdir(CMkdirCommand const& command)
 {
-	m_pControlSocket->Mkdir(command.GetPath());
+	controlSocket_->Mkdir(command.GetPath());
 	return FZ_REPLY_CONTINUE;
 }
 
 int CFileZillaEnginePrivate::Rename(CRenameCommand const& command)
 {
-	m_pControlSocket->Rename(command);
+	controlSocket_->Rename(command);
 	return FZ_REPLY_CONTINUE;
 }
 
 int CFileZillaEnginePrivate::Chmod(CChmodCommand const& command)
 {
-	m_pControlSocket->Chmod(command);
+	controlSocket_->Chmod(command);
 	return FZ_REPLY_CONTINUE;
 }
 
 void CFileZillaEnginePrivate::RegisterFailedLoginAttempt(const CServer& server, bool critical)
 {
-	fz::scoped_lock lock(mutex_);
+	fz::scoped_lock lock(global_mutex_);
 	std::list<t_failedLogins>::iterator iter = m_failedLogins.begin();
 	while (iter != m_failedLogins.end()) {
 		fz::duration const span = fz::monotonic_clock::now() - iter->time;
 		if (span.get_seconds() >= m_options.GetOptionVal(OPTION_RECONNECTDELAY) ||
-			iter->server == server || (!critical && (iter->server.GetHost() == server.GetHost() && iter->server.GetPort() == server.GetPort())))
+			iter->server.SameResource(server) || (!critical && (iter->server.GetHost() == server.GetHost() && iter->server.GetPort() == server.GetPort())))
 		{
 			std::list<t_failedLogins>::iterator prev = iter;
 			++iter;
@@ -449,7 +471,7 @@ void CFileZillaEnginePrivate::RegisterFailedLoginAttempt(const CServer& server,
 
 fz::duration CFileZillaEnginePrivate::GetRemainingReconnectDelay(CServer const& server)
 {
-	fz::scoped_lock lock(mutex_);
+	fz::scoped_lock lock(global_mutex_);
 	std::list<t_failedLogins>::iterator iter = m_failedLogins.begin();
 	while (iter != m_failedLogins.end()) {
 		fz::duration const span = fz::monotonic_clock::now() - iter->time;
@@ -481,19 +503,19 @@ void CFileZillaEnginePrivate::OnTimer(fz::timer_id)
 	m_retryTimer = 0;
 
 	if (!m_pCurrentCommand || m_pCurrentCommand->GetId() != Command::connect) {
-		m_pLogging->LogMessage(MessageType::Debug_Warning, L"CFileZillaEnginePrivate::OnTimer called without pending Command::connect");
+		logger_->log(logmsg::debug_warning, L"CFileZillaEnginePrivate::OnTimer called without pending Command::connect");
 		return;
 	}
 
-	m_pControlSocket.reset();
+	controlSocket_.reset();
 
 	int res = ContinueConnect();
 	res |= m_nControlSocketError;
 	m_nControlSocketError = 0;
 
 	if (res == FZ_REPLY_CONTINUE) {
-		assert(m_pControlSocket);
-		m_pControlSocket->SendNextCommand();
+		assert(controlSocket_);
+		controlSocket_->SendNextCommand();
 	}
 	else if (res != FZ_REPLY_WOULDBLOCK) {
 		ResetOperation(res);
@@ -505,7 +527,7 @@ int CFileZillaEnginePrivate::ContinueConnect()
 	fz::scoped_lock lock(mutex_);
 
 	if (!m_pCurrentCommand || m_pCurrentCommand->GetId() != Command::connect) {
-		m_pLogging->LogMessage(MessageType::Debug_Warning, L"CFileZillaEnginePrivate::ContinueConnect called without pending Command::connect");
+		logger_->log(logmsg::debug_warning, L"CFileZillaEnginePrivate::ContinueConnect called without pending Command::connect");
 		return ResetOperation(FZ_REPLY_INTERNALERROR);
 	}
 
@@ -513,7 +535,7 @@ int CFileZillaEnginePrivate::ContinueConnect()
 	const CServer& server = pConnectCommand->GetServer();
 	fz::duration const& delay = GetRemainingReconnectDelay(server);
 	if (delay) {
-		m_pLogging->LogMessage(MessageType::Status, fztranslate("Delaying connection for %d second due to previously failed connection attempt...", "Delaying connection for %d seconds due to previously failed connection attempt...", (delay.get_milliseconds() + 999) / 1000), (delay.get_milliseconds() + 999) / 1000);
+		logger_->log(logmsg::status, fztranslate("Delaying connection for %d second due to previously failed connection attempt...", "Delaying connection for %d seconds due to previously failed connection attempt...", (delay.get_milliseconds() + 999) / 1000), (delay.get_milliseconds() + 999) / 1000);
 		stop_timer(m_retryTimer);
 		m_retryTimer = add_timer(delay, true);
 		return FZ_REPLY_WOULDBLOCK;
@@ -525,52 +547,59 @@ int CFileZillaEnginePrivate::ContinueConnect()
 	case FTPS:
 	case FTPES:
 	case INSECURE_FTP:
-		m_pControlSocket = std::make_unique<CFtpControlSocket>(*this);
+		controlSocket_ = std::make_unique<CFtpControlSocket>(*this);
 		break;
 	case SFTP:
-		m_pControlSocket = std::make_unique<CSftpControlSocket>(*this);
+		controlSocket_ = std::make_unique<CSftpControlSocket>(*this);
 		break;
 	case HTTP:
 	case HTTPS:
-		m_pControlSocket = std::make_unique<CHttpControlSocket>(*this);
+		controlSocket_ = std::make_unique<CHttpControlSocket>(*this);
 		break;
 #if ENABLE_STORJ
 	case STORJ:
-		m_pControlSocket = std::make_unique<CStorjControlSocket>(*this);
+		controlSocket_ = std::make_unique<CStorjControlSocket>(*this);
 		break;
 #endif
 	default:
-		m_pLogging->LogMessage(MessageType::Error, _("'%s' is not a supported protocol."), CServer::GetProtocolName(server.GetProtocol()));
+		logger_->log(logmsg::error, _("'%s' is not a supported protocol."), CServer::GetProtocolName(server.GetProtocol()));
 		return FZ_REPLY_SYNTAXERROR|FZ_REPLY_DISCONNECTED;
 	}
 
-	m_pControlSocket->Connect(server, pConnectCommand->GetCredentials());
+	controlSocket_->SetHandle(pConnectCommand->GetHandle());
+	controlSocket_->Connect(server, pConnectCommand->GetCredentials());
 	return FZ_REPLY_CONTINUE;
 }
 
+void CFileZillaEnginePrivate::OnInvalidateCurrentWorkingDir(CServer const& server, CServerPath const& path)
+{
+	if (!controlSocket_ || controlSocket_->GetCurrentServer() != server) {
+		return;
+	}
+	controlSocket_->InvalidateCurrentWorkingDir(path);
+}
+
 void CFileZillaEnginePrivate::InvalidateCurrentWorkingDirs(const CServerPath& path)
 {
-	fz::scoped_lock lock(mutex_);
-
-	assert(m_pControlSocket);
-	CServer const& ownServer  = m_pControlSocket->GetCurrentServer();
-	assert(ownServer);
-
+	CServer ownServer;
+	{
+		fz::scoped_lock lock(mutex_);
+		if (controlSocket_) {
+			ownServer = controlSocket_->GetCurrentServer();
+		}
+	}
+	if (!ownServer) {
+		// May happen during destruction
+		return;
+	}
+	
+	fz::scoped_lock lock(global_mutex_);
 	for (auto & engine : m_engineList) {
 		if (!engine || engine == this) {
 			continue;
 		}
 
-		if (!engine->m_pControlSocket) {
-			continue;
-		}
-
-		CServer const& server = engine->m_pControlSocket->GetCurrentServer();
-		if (server != ownServer) {
-			continue;
-		}
-
-		engine->m_pControlSocket->InvalidateCurrentWorkingDir(path);
+		engine->send_event(new CInvalidateCurrentWorkingDirEvent(ownServer, path));
 	}
 }
 
@@ -578,17 +607,19 @@ void CFileZillaEnginePrivate::operator()(fz::event_base const& ev)
 {
 	fz::scoped_lock lock(mutex_);
 
-	fz::dispatch<CFileZillaEngineEvent, CCommandEvent, CAsyncRequestReplyEvent, fz::timer_event>(ev, this,
+	fz::dispatch<CFileZillaEngineEvent, CCommandEvent, CAsyncRequestReplyEvent, fz::timer_event, CInvalidateCurrentWorkingDirEvent>(ev, this,
 		&CFileZillaEnginePrivate::OnEngineEvent,
 		&CFileZillaEnginePrivate::OnCommandEvent,
 		&CFileZillaEnginePrivate::OnSetAsyncRequestReplyEvent,
-		&CFileZillaEnginePrivate::OnTimer
+		&CFileZillaEnginePrivate::OnTimer,
+		&CFileZillaEnginePrivate::OnInvalidateCurrentWorkingDir
 		);
 }
 
 int CFileZillaEnginePrivate::CheckCommandPreconditions(CCommand const& command, bool checkBusy)
 {
 	if (!command.valid()) {
+		logger_->log(logmsg::debug_warning, L"Command not valid");
 		return FZ_REPLY_SYNTAXERROR;
 	}
 	else if (checkBusy && IsBusy()) {
@@ -597,7 +628,7 @@ int CFileZillaEnginePrivate::CheckCommandPreconditions(CCommand const& command,
 	else if (command.GetId() != Command::connect && command.GetId() != Command::disconnect && !IsConnected()) {
 		return FZ_REPLY_NOTCONNECTED;
 	}
-	else if (command.GetId() == Command::connect && m_pControlSocket) {
+	else if (command.GetId() == Command::connect && controlSocket_) {
 		return FZ_REPLY_ALREADYCONNECTED;
 	}
 	return FZ_REPLY_OK;
@@ -645,6 +676,19 @@ void CFileZillaEnginePrivate::OnCommandEvent()
 			case Command::chmod:
 				res = Chmod(static_cast<CChmodCommand const&>(command));
 				break;
+			case Command::httprequest:
+				{
+					auto * http_socket = dynamic_cast<CHttpControlSocket*>(controlSocket_.get());
+					if (http_socket) {
+						http_socket->FileTransfer(static_cast<CHttpRequestCommand const&>(command));
+						res = FZ_REPLY_CONTINUE;
+					}
+					else {
+						logger_->log(logmsg::error, _("Command not supported by this protocol"));
+						res = FZ_REPLY_NOTSUPPORTED;
+					}
+				}
+				break;
 			default:
 				res = FZ_REPLY_SYNTAXERROR;
 			}
@@ -659,8 +703,8 @@ void CFileZillaEnginePrivate::OnCommandEvent()
 		m_nControlSocketError = 0;
 
 		if (res == FZ_REPLY_CONTINUE) {
-			assert(m_pControlSocket);
-			m_pControlSocket->SendNextCommand();
+			assert(controlSocket_);
+			controlSocket_->SendNextCommand();
 		}
 		else if (res != FZ_REPLY_WOULDBLOCK) {
 			ResetOperation(res);
@@ -678,14 +722,14 @@ void CFileZillaEnginePrivate::DoCancel()
 	if (m_retryTimer) {
 		assert(m_pCurrentCommand && m_pCurrentCommand->GetId() == Command::connect);
 
-		m_pControlSocket.reset();
+		controlSocket_.reset();
 
 		m_pCurrentCommand.reset();
 
 		stop_timer(m_retryTimer);
 		m_retryTimer = 0;
 
-		m_pLogging->LogMessage(MessageType::Error, _("Connection attempt interrupted by user"));
+		logger_->log(logmsg::error, _("Connection attempt interrupted by user"));
 		COperationNotification *notification = new COperationNotification();
 		notification->nReplyCode = FZ_REPLY_DISCONNECTED | FZ_REPLY_CANCELED;
 		notification->commandId = Command::connect;
@@ -694,8 +738,8 @@ void CFileZillaEnginePrivate::DoCancel()
 		ClearQueuedLogs(true);
 	}
 	else {
-		if (m_pControlSocket) {
-			m_pControlSocket->Cancel();
+		if (controlSocket_) {
+			controlSocket_->Cancel();
 		}
 		else {
 			ResetOperation(FZ_REPLY_CANCELED);
@@ -718,7 +762,7 @@ bool CFileZillaEnginePrivate::CheckAsyncRequestReplyPreconditions(std::unique_pt
 		match = reply->requestNumber == m_asyncRequestCounter;
 	}
 
-	return match && m_pControlSocket;
+	return match && controlSocket_;
 }
 
 void CFileZillaEnginePrivate::OnSetAsyncRequestReplyEvent(std::unique_ptr<CAsyncRequestNotification> const& reply)
@@ -728,8 +772,7 @@ void CFileZillaEnginePrivate::OnSetAsyncRequestReplyEvent(std::unique_ptr<CAsync
 		return;
 	}
 
-	m_pControlSocket->SetAlive();
-	m_pControlSocket->SetAsyncRequestReply(reply.get());
+	controlSocket_->CallSetAsyncRequestReply(reply.get());
 }
 
 int CFileZillaEnginePrivate::Execute(const CCommand &command)
@@ -821,10 +864,10 @@ int CFileZillaEnginePrivate::CacheLookup(const CServerPath& path, CDirectoryList
 		return FZ_REPLY_ERROR;
 	}
 
-	assert(m_pControlSocket->GetCurrentServer());
+	assert(controlSocket_->GetCurrentServer());
 
 	bool is_outdated = false;
-	if (!directory_cache_.Lookup(listing, m_pControlSocket->GetCurrentServer(), path, true, is_outdated)) {
+	if (!directory_cache_.Lookup(listing, controlSocket_->GetCurrentServer(), path, true, is_outdated)) {
 		return FZ_REPLY_ERROR;
 	}
 
@@ -854,6 +897,11 @@ void CFileZillaEnginePrivate::OnOptionsChanged(changed_options_t const&)
 	}
 }
 
+fz::logger_interface& CFileZillaEnginePrivate::GetLogger()
+{
+	return *logger_;
+}
+
 
 CTransferStatusManager::CTransferStatusManager(CFileZillaEnginePrivate& engine)
 	: engine_(engine)
