@@ -11,10 +11,11 @@
 #include "option_change_event_handler.h"
 
 #include <atomic>
+#include <list>
+#include <deque>
 
 class CControlSocket;
 class CLogging;
-class CRateLimiter;
 class OpLockManager;
 
 enum EngineNotificationType
@@ -87,11 +88,12 @@ public:
 	std::unique_ptr<CNotification> GetNextNotification();
 
 	COptionsBase& GetOptions() { return m_options; }
-	CRateLimiter& GetRateLimiter() { return m_rateLimiter; }
+	fz::rate_limiter& GetRateLimiter() { return rate_limiter_; }
 	CDirectoryCache& GetDirectoryCache() { return directory_cache_; }
 	CPathCache& GetPathCache() { return path_cache_; }
 	fz::thread_pool& GetThreadPool() { return thread_pool_; }
 	CFileZillaEngineContext& GetContext() { return context_; }
+	CFileZillaEngine& GetParent() { return parent_; }
 
 	// If deleting or renaming a directory, it could be possible that another
 	// engine's CControlSocket instance still has that directory as
@@ -109,6 +111,9 @@ public:
 	CustomEncodingConverterBase const& GetEncodingConverter() const { return encoding_converter_; }
 
 	OpLockManager & opLockManager_;
+
+	fz::logger_interface& GetLogger();
+
 protected:
 	virtual void OnOptionsChanged(changed_options_t const& options);
 
@@ -177,7 +182,7 @@ protected:
 
 	COptionsBase& m_options;
 
-	CLogging* m_pLogging;
+	std::unique_ptr<CLogging> logger_;
 
 	// Everything related to the retry code
 	// ------------------------------------
@@ -197,7 +202,7 @@ protected:
 	int m_retryCount{};
 	fz::timer_id m_retryTimer{};
 
-	CRateLimiter& m_rateLimiter;
+	fz::rate_limiter& rate_limiter_;
 	CDirectoryCache& directory_cache_;
 	CPathCache& path_cache_;
 
