@@ -18,7 +18,7 @@ int CFtpDeleteOpData::Send()
 		return FZ_REPLY_CONTINUE;
 	}
 	else if (opState == del_del) {
-		std::wstring const& file = files_.front();
+		std::wstring const& file = files_.back();
 		if (file.empty()) {
 			log(logmsg::debug_info, L"Empty filename");
 			return FZ_REPLY_INTERNALERROR;
@@ -46,7 +46,7 @@ int CFtpDeleteOpData::ParseResponse()
 		deleteFailed_ = true;
 	}
 	else {
-		std::wstring const& file = files_.front();
+		std::wstring const& file = files_.back();
 
 		engine_.GetDirectoryCache().RemoveFile(currentServer_, path_, file);
 
@@ -61,7 +61,7 @@ int CFtpDeleteOpData::ParseResponse()
 		}
 	}
 
-	files_.pop_front();
+	files_.pop_back();
 
 	if (!files_.empty()) {
 		return FZ_REPLY_CONTINUE;
