@@ -3,7 +3,7 @@
 #include "cwd.h"
 #include "chmod.h"
 #include "delete.h"
-#include "directorycache.h"
+#include "../directorycache.h"
 #include "directorylistingparser.h"
 #include "engineprivate.h"
 #include "externalipresolver.h"
@@ -20,13 +20,12 @@
 #include "rename.h"
 #include "rmd.h"
 #include "servercapabilities.h"
-#include "socket_errors.h"
-#include "tlssocket.h"
 #include "transfersocket.h"
 
 #include <libfilezilla/file.hpp>
 #include <libfilezilla/iputils.hpp>
 #include <libfilezilla/local_filesys.hpp>
+#include <libfilezilla/tls_layer.hpp>
 #include <libfilezilla/util.hpp>
 
 #include <algorithm>
@@ -34,15 +33,6 @@
 CFtpControlSocket::CFtpControlSocket(CFileZillaEnginePrivate & engine)
 	: CRealControlSocket(engine)
 {
-	// Enable TCP_NODELAY, speeds things up a bit.
-	socket_->set_flags(fz::socket::flag_nodelay | fz::socket::flag_keepalive);
-
-	// Enable SO_KEEPALIVE, lots of clueless users have broken routers and
-	// firewalls which terminate the control connection on long transfers.
-	int v = engine_.GetOptions().GetOptionVal(OPTION_TCP_KEEPALIVE_INTERVAL);
-	if (v >= 1 && v < 10000) {
-		socket_->set_keepalive_interval(fz::duration::from_minutes(v));
-	}
 }
 
 CFtpControlSocket::~CFtpControlSocket()
@@ -54,17 +44,20 @@ CFtpControlSocket::~CFtpControlSocket()
 
 void CFtpControlSocket::OnReceive()
 {
-	LogMessage(MessageType::Debug_Verbose, L"CFtpControlSocket::OnReceive()");
+	log(logmsg::debug_verbose, L"CFtpControlSocket::OnReceive()");
+
+	size_t const max = 65536;
 
 	for (;;) {
 		int error;
-		int read = m_pBackend->Read(m_receiveBuffer + m_bufferLen, RECVBUFFERSIZE - m_bufferLen, error);
 
+		size_t const toRead = max - receiveBuffer_.size();
+		int read = active_layer_->read(receiveBuffer_.get(toRead), toRead, error);
 		if (read < 0) {
 			if (error != EAGAIN) {
-				LogMessage(MessageType::Error, _("Could not read from socket: %s"), fz::socket_error_description(error));
+				log(logmsg::error, _("Could not read from socket: %s"), fz::socket_error_description(error));
 				if (GetCurrentCommandId() != Command::connect) {
-					LogMessage(MessageType::Error, _("Disconnected from server"));
+					log(logmsg::error, _("Disconnected from server"));
 				}
 				DoClose();
 			}
@@ -72,44 +65,47 @@ void CFtpControlSocket::OnReceive()
 		}
 
 		if (!read) {
-			auto messageType = (GetCurrentCommandId() == Command::none) ? MessageType::Status : MessageType::Error;
-			LogMessage(messageType, _("Connection closed by server"));
+			auto messageType = (GetCurrentCommandId() == Command::none) ? logmsg::status : logmsg::error;
+			log(messageType, _("Connection closed by server"));
 			DoClose();
 			return;
 		}
 
+		size_t i = receiveBuffer_.size();
+		receiveBuffer_.add(read);
+
 		SetActive(CFileZillaEngine::recv);
 
-		char* start = m_receiveBuffer;
-		m_bufferLen += read;
-
-		for (int i = start - m_receiveBuffer; i < m_bufferLen; ++i) {
-			char const& p = m_receiveBuffer[i];
+		while (i < receiveBuffer_.size()) {
+			auto const& p = receiveBuffer_[i];
 			if (p == '\r' ||
 				p == '\n' ||
 				p == 0)
 			{
-				int len = i - (start - m_receiveBuffer);
-				if (!len) {
-					++start;
-					continue;
+				if (!i) {
+					receiveBuffer_.consume(1);
 				}
+				else {
+					std::wstring line = ConvToLocal(reinterpret_cast<char const*>(receiveBuffer_.get()), i);
+					receiveBuffer_.consume(i + 1);
+					i = 0;
 
-				std::wstring line = ConvToLocal(start, i - (start - m_receiveBuffer));
-				start = m_receiveBuffer + i + 1;
+					ParseLine(line);
 
-				ParseLine(line);
-
-				// Abort if connection got closed
-				if (!currentServer_) {
-					return;
+					// Abort if connection got closed
+					if (!currentServer_) {
+						return;
+					}
 				}
 			}
+			else {
+				++i;
+			}
 		}
-		memmove(m_receiveBuffer, start, m_bufferLen - (start - m_receiveBuffer));
-		m_bufferLen -= (start -m_receiveBuffer);
-		if (m_bufferLen > MAXLINELEN) {
-			m_bufferLen = MAXLINELEN;
+		if (receiveBuffer_.size() == max) {
+			log(fz::logmsg::error, _("Received too long response line from server, closing connection."));
+			DoClose();
+			return;
 		}
 	}
 }
@@ -117,7 +113,7 @@ void CFtpControlSocket::OnReceive()
 void CFtpControlSocket::ParseLine(std::wstring line)
 {
 	m_rtt.Stop();
-	LogMessageRaw(MessageType::Response, line);
+	log_raw(logmsg::reply, line);
 	SetAlive();
 
 	if (!operations_.empty() && operations_.back()->opId == Command::connect) {
@@ -138,7 +134,7 @@ void CFtpControlSocket::ParseLine(std::wstring line)
 		else if (data.opState == LOGON_WELCOME) {
 			if (!data.gotFirstWelcomeLine) {
 				if (fz::str_tolower_ascii(line).substr(0, 3) == L"ssh") {
-					LogMessage(MessageType::Error, _("Cannot establish FTP connection to an SFTP server. Please select proper protocol."));
+					log(logmsg::error, _("Cannot establish FTP connection to an SFTP server. Please select proper protocol."));
 					DoClose(FZ_REPLY_CRITICALERROR);
 					return;
 				}
@@ -182,38 +178,29 @@ void CFtpControlSocket::OnConnect()
 	SetAlive();
 
 	if (currentServer_.GetProtocol() == FTPS) {
-		if (!m_pTlsSocket) {
-			LogMessage(MessageType::Status, _("Connection established, initializing TLS..."));
+		if (!tls_layer_) {
+			log(logmsg::status, _("Connection established, initializing TLS..."));
 
-			assert(!m_pTlsSocket);
-			delete m_pBackend;
-			m_pTlsSocket = new CTlsSocket(this, *socket_, this);
-			m_pBackend = m_pTlsSocket;
+			tls_layer_ = std::make_unique<fz::tls_layer>(event_loop_, this, *active_layer_, &engine_.GetContext().GetTlsSystemTrustStore(), logger_);
+			active_layer_ = tls_layer_.get();
 
-			if (!m_pTlsSocket->Init()) {
-				LogMessage(MessageType::Error, _("Failed to initialize TLS."));
-				DoClose();
-				return;
-			}
-
-			int res = m_pTlsSocket->Handshake();
-			if (res == FZ_REPLY_ERROR) {
+			if (!tls_layer_->client_handshake(this)) {
 				DoClose();
 			}
 
 			return;
 		}
 		else {
-			LogMessage(MessageType::Status, _("TLS connection established, waiting for welcome message..."));
+			log(logmsg::status, _("TLS connection established, waiting for welcome message..."));
 		}
 	}
-	else if ((currentServer_.GetProtocol() == FTPES || currentServer_.GetProtocol() == FTP) && m_pTlsSocket) {
-		LogMessage(MessageType::Status, _("TLS connection established."));
+	else if ((currentServer_.GetProtocol() == FTPES || currentServer_.GetProtocol() == FTP) && tls_layer_) {
+		log(logmsg::status, _("TLS connection established."));
 		SendNextCommand();
 		return;
 	}
 	else {
-		LogMessage(MessageType::Status, _("Connection established, waiting for welcome message..."));
+		log(logmsg::status, _("Connection established, waiting for welcome message..."));
 	}
 	m_pendingReplies = 1;
 	m_repliesToSkip = 0;
@@ -222,22 +209,22 @@ void CFtpControlSocket::OnConnect()
 void CFtpControlSocket::ParseResponse()
 {
 	if (m_Response.empty()) {
-		LogMessage(MessageType::Debug_Warning, L"No reply in ParseResponse");
+		log(logmsg::debug_warning, L"No reply in ParseResponse");
 		return;
 	}
 
 	if (m_Response[0] != '1') {
 		if (m_pendingReplies > 0) {
-			m_pendingReplies--;
+			--m_pendingReplies;
 		}
 		else {
-			LogMessage(MessageType::Debug_Warning, L"Unexpected reply, no reply was pending.");
+			log(logmsg::debug_warning, L"Unexpected reply, no reply was pending.");
 			return;
 		}
 	}
 
 	if (m_repliesToSkip) {
-		LogMessage(MessageType::Debug_Info, L"Skipping reply after cancelled operation or keepalive command.");
+		log(logmsg::debug_info, L"Skipping reply after cancelled operation or keepalive command.");
 		if (m_Response[0] != '1') {
 			--m_repliesToSkip;
 		}
@@ -256,12 +243,12 @@ void CFtpControlSocket::ParseResponse()
 	}
 
 	if (operations_.empty()) {
-		LogMessage(MessageType::Debug_Info, L"Skipping reply without active operation.");
+		log(logmsg::debug_info, L"Skipping reply without active operation.");
 		return;
 	}
 
 	auto & data = *operations_.back();
-	LogMessage(MessageType::Debug_Verbose, L"%s::ParseResponse() in state %d", data.name_, data.opState);
+	log(logmsg::debug_verbose, L"%s::ParseResponse() in state %d", data.name_, data.opState);
 	int res = data.ParseResponse();
 	if (res == FZ_REPLY_OK) {
 		ResetOperation(FZ_REPLY_OK);
@@ -300,15 +287,15 @@ int CFtpControlSocket::SendCommand(std::wstring const& str, bool maskArgs, bool
 	size_t pos;
 	if (maskArgs && (pos = str.find(' ')) != std::wstring::npos) {
 		std::wstring stars(str.size() - pos - 1, '*');
-		LogMessageRaw(MessageType::Command, str.substr(0, pos + 1) + stars);
+		log_raw(logmsg::command, str.substr(0, pos + 1) + stars);
 	}
 	else {
-		LogMessageRaw(MessageType::Command, str);
+		log_raw(logmsg::command, str);
 	}
 
 	std::string buffer = ConvToServer(str);
 	if (buffer.empty()) {
-		LogMessage(MessageType::Error, _("Failed to convert command to 8 bit charset"));
+		log(logmsg::error, _("Failed to convert command to 8 bit charset"));
 		return FZ_REPLY_ERROR;
 	}
 	buffer += "\r\n";
@@ -335,10 +322,10 @@ void CFtpControlSocket::List(CServerPath const& path, std::wstring const& subDir
 	}
 
 	if (newPath.empty()) {
-		LogMessage(MessageType::Status, _("Retrieving directory listing..."));
+		log(logmsg::status, _("Retrieving directory listing..."));
 	}
 	else {
-		LogMessage(MessageType::Status, _("Retrieving directory listing of \"%s\"..."), newPath.GetPath());
+		log(logmsg::status, _("Retrieving directory listing of \"%s\"..."), newPath.GetPath());
 	}
 
 	Push(std::make_unique<CFtpListOpData>(*this, path, subDir, flags));
@@ -346,7 +333,7 @@ void CFtpControlSocket::List(CServerPath const& path, std::wstring const& subDir
 
 int CFtpControlSocket::ResetOperation(int nErrorCode)
 {
- 	LogMessage(MessageType::Debug_Verbose, L"CFtpControlSocket::ResetOperation(%d)", nErrorCode);
+ 	log(logmsg::debug_verbose, L"CFtpControlSocket::ResetOperation(%d)", nErrorCode);
 
 	m_pTransferSocket.reset();
 	m_pIPResolver.reset();
@@ -376,7 +363,7 @@ int CFtpControlSocket::ResetOperation(int nErrorCode)
 				// Download failed and a new local file was created before, but
 				// nothing has been written to it. Remove it again, so we don't
 				// leave a bunch of empty files all over the place.
-				LogMessage(MessageType::Debug_Verbose, L"Deleting empty file");
+				log(logmsg::debug_verbose, L"Deleting empty file");
 				fz::remove_file(fz::to_native(data.localFile_));
 			}
 		}
@@ -411,10 +398,10 @@ int CFtpControlSocket::ResetOperation(int nErrorCode)
 	return CControlSocket::ResetOperation(nErrorCode);
 }
 
-bool CFtpControlSocket::CanSendNextCommand() const
+bool CFtpControlSocket::CanSendNextCommand()
 {
 	if (m_repliesToSkip) {
-		LogMessage(MessageType::Status, L"Waiting for replies to skip before sending next command...");
+		log(logmsg::status, L"Waiting for replies to skip before sending next command...");
 		return false;
 	}
 
@@ -442,7 +429,7 @@ void CFtpControlSocket::FileTransfer(std::wstring const& localFile, CServerPath
 									std::wstring const& remoteFile, bool download,
 									CFileTransferCommand::t_transferSettings const& transferSettings)
 {
-	LogMessage(MessageType::Debug_Verbose, L"CFtpControlSocket::FileTransfer()");
+	log(logmsg::debug_verbose, L"CFtpControlSocket::FileTransfer()");
 
 	auto pData = std::make_unique<CFtpFileTransferOpData>(*this, download, localFile, remoteFile, remotePath, transferSettings);
 	pData->binary = transferSettings.binary;
@@ -451,20 +438,20 @@ void CFtpControlSocket::FileTransfer(std::wstring const& localFile, CServerPath
 
 void CFtpControlSocket::TransferEnd()
 {
-	LogMessage(MessageType::Debug_Verbose, L"CFtpControlSocket::TransferEnd()");
+	log(logmsg::debug_verbose, L"CFtpControlSocket::TransferEnd()");
 
 	// If m_pTransferSocket is zero, the message was sent by the previous command.
 	// We can safely ignore it.
 	// It does not cause problems, since before creating the next transfer socket, other
 	// messages which were added to the queue later than this one will be processed first.
 	if (operations_.empty() || !m_pTransferSocket || operations_.back()->opId != PrivCommand::rawtransfer) {
-		LogMessage(MessageType::Debug_Verbose, L"Call to TransferEnd at unusual time, ignoring");
+		log(logmsg::debug_verbose, L"Call to TransferEnd at unusual time, ignoring");
 		return;
 	}
 
 	TransferEndReason reason = m_pTransferSocket->GetTransferEndreason();
 	if (reason == TransferEndReason::none) {
-		LogMessage(MessageType::Debug_Info, L"Call to TransferEnd at unusual time");
+		log(logmsg::debug_info, L"Call to TransferEnd at unusual time");
 		return;
 	}
 
@@ -489,18 +476,14 @@ void CFtpControlSocket::TransferEnd()
 		ResetOperation((reason == TransferEndReason::successful) ? FZ_REPLY_OK : FZ_REPLY_ERROR);
 		break;
 	default:
-		LogMessage(MessageType::Debug_Info, L"TransferEnd at unusual op state %d, ignoring", data.opState);
+		log(logmsg::debug_info, L"TransferEnd at unusual op state %d, ignoring", data.opState);
 		break;
 	}
 }
 
 bool CFtpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotification)
 {
-	if (operations_.empty() || !operations_.back()->waitForAsyncRequest) {
-		LogMessage(MessageType::Debug_Info, L"Not waiting for request reply, ignoring request reply %d", pNotification->GetRequestID());
-		return false;
-	}
-	operations_.back()->waitForAsyncRequest = false;
+	log(logmsg::debug_verbose, L"CFtpControlSocket::SetAsyncRequestReply");
 
 	const RequestId requestId = pNotification->GetRequestID();
 	switch (requestId)
@@ -508,7 +491,7 @@ bool CFtpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotific
 	case reqId_fileexists:
 		{
 			if (operations_.empty() || operations_.back()->opId != Command::transfer) {
-				LogMessage(MessageType::Debug_Info, L"No or invalid operation in progress, ignoring request reply %d", pNotification->GetRequestID());
+				log(logmsg::debug_info, L"No or invalid operation in progress, ignoring request reply %d", pNotification->GetRequestID());
 				return false;
 			}
 
@@ -519,7 +502,7 @@ bool CFtpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotific
 	case reqId_interactiveLogin:
 		{
 			if (operations_.empty() || operations_.back()->opId != Command::connect) {
-				LogMessage(MessageType::Debug_Info, L"No or invalid operation in progress, ignoring request reply %d", pNotification->GetRequestID());
+				log(logmsg::debug_info, L"No or invalid operation in progress, ignoring request reply %d", pNotification->GetRequestID());
 				return false;
 			}
 
@@ -536,15 +519,15 @@ bool CFtpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotific
 		break;
 	case reqId_certificate:
 		{
-			if (!m_pTlsSocket || m_pTlsSocket->GetState() != CTlsSocket::TlsState::verifycert) {
-				LogMessage(MessageType::Debug_Info, L"No or invalid operation in progress, ignoring request reply %d", pNotification->GetRequestID());
+			if (!tls_layer_ || tls_layer_->get_state() != fz::socket_state::connecting) {
+				log(logmsg::debug_info, L"No or invalid operation in progress, ignoring request reply %d", pNotification->GetRequestID());
 				return false;
 			}
 
 			CCertificateNotification* pCertificateNotification = static_cast<CCertificateNotification *>(pNotification);
-			m_pTlsSocket->TrustCurrentCert(pCertificateNotification->m_trusted);
+			tls_layer_->set_verification_result(pCertificateNotification->trusted_);
 
-			if (!pCertificateNotification->m_trusted) {
+			if (!pCertificateNotification->trusted_) {
 				DoClose(FZ_REPLY_CRITICALERROR);
 				return false;
 			}
@@ -556,9 +539,9 @@ bool CFtpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotific
 			}
 		}
 		break;
-	case reqId_insecure_ftp:
+	case reqId_insecure_connection:
 		{
-			auto & notification = static_cast<CInsecureFTPNotification&>(*pNotification);
+			auto & notification = static_cast<CInsecureConnectionNotification&>(*pNotification);
 			if (!notification.allow_) {
 				ResetOperation(FZ_REPLY_CANCELED);
 				return false;
@@ -569,7 +552,7 @@ bool CFtpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotific
 			}
 		}
 	default:
-		LogMessage(MessageType::Debug_Warning, L"Unknown request %d", pNotification->GetRequestID());
+		log(logmsg::debug_warning, L"Unknown request %d", pNotification->GetRequestID());
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		return false;
 	}
@@ -583,7 +566,7 @@ void CFtpControlSocket::RawCommand(std::wstring const& command)
 	Push(std::make_unique<CFtpRawCommandOpData>(*this, command));
 }
 
-void CFtpControlSocket::Delete(CServerPath const& path, std::deque<std::wstring>&& files)
+void CFtpControlSocket::Delete(CServerPath const& path, std::vector<std::wstring>&& files)
 {
 	auto pData = std::make_unique<CFtpDeleteOpData>(*this);
 	pData->path_ = path;
@@ -611,7 +594,7 @@ void CFtpControlSocket::Mkdir(CServerPath const& path)
 	 */
 
 	if (operations_.empty() && !path.empty()) {
-		LogMessage(MessageType::Status, _("Creating directory '%s'..."), path.GetPath());
+		log(logmsg::status, _("Creating directory '%s'..."), path.GetPath());
 	}
 
 	auto pData = std::make_unique<CFtpMkdirOpData>(*this);
@@ -622,14 +605,14 @@ void CFtpControlSocket::Mkdir(CServerPath const& path)
 
 void CFtpControlSocket::Rename(CRenameCommand const& command)
 {
-	LogMessage(MessageType::Status, _("Renaming '%s' to '%s'"), command.GetFromPath().FormatFilename(command.GetFromFile()), command.GetToPath().FormatFilename(command.GetToFile()));
+	log(logmsg::status, _("Renaming '%s' to '%s'"), command.GetFromPath().FormatFilename(command.GetFromFile()), command.GetToPath().FormatFilename(command.GetToFile()));
 
 	Push(std::make_unique<CFtpRenameOpData>(*this, command));
 }
 
 void CFtpControlSocket::Chmod(CChmodCommand const& command)
 {
-	LogMessage(MessageType::Status, _("Setting permissions of '%s' to '%s'"), command.GetPath().FormatFilename(command.GetFile()), command.GetPermission());
+	log(logmsg::status, _("Setting permissions of '%s' to '%s'"), command.GetPath().FormatFilename(command.GetFile()), command.GetPermission());
 
 	Push(std::make_unique<CFtpChmodOpData>(*this, command));
 }
@@ -655,14 +638,14 @@ int CFtpControlSocket::GetExternalIPAddress(std::string& address)
 				return FZ_REPLY_OK;
 			}
 
-			LogMessage(MessageType::Debug_Warning, _("No external IP address set, trying default."));
+			log(logmsg::debug_warning, _("No external IP address set, trying default."));
 		}
 		else if (mode == 2) {
 			if (!m_pIPResolver) {
 				std::string localAddress = socket_->local_ip(true);
 
 				if (!localAddress.empty() && localAddress == fz::to_string(engine_.GetOptions().GetOption(OPTION_LASTRESOLVEDIP))) {
-					LogMessage(MessageType::Debug_Verbose, L"Using cached external IP address");
+					log(logmsg::debug_verbose, L"Using cached external IP address");
 
 					address = localAddress;
 					return FZ_REPLY_OK;
@@ -670,22 +653,22 @@ int CFtpControlSocket::GetExternalIPAddress(std::string& address)
 
 				std::wstring resolverAddress = engine_.GetOptions().GetOption(OPTION_EXTERNALIPRESOLVER);
 
-				LogMessage(MessageType::Debug_Info, _("Retrieving external IP address from %s"), resolverAddress);
+				log(logmsg::debug_info, _("Retrieving external IP address from %s"), resolverAddress);
 
 				m_pIPResolver = std::make_unique<CExternalIPResolver>(engine_.GetThreadPool(), *this);
 				m_pIPResolver->GetExternalIP(resolverAddress, fz::address_type::ipv4);
 				if (!m_pIPResolver->Done()) {
-					LogMessage(MessageType::Debug_Verbose, L"Waiting for resolver thread");
+					log(logmsg::debug_verbose, L"Waiting for resolver thread");
 					return FZ_REPLY_WOULDBLOCK;
 				}
 			}
 			if (!m_pIPResolver->Successful()) {
 				m_pIPResolver.reset();
 
-				LogMessage(MessageType::Debug_Warning, _("Failed to retrieve external IP address, using local address"));
+				log(logmsg::debug_warning, _("Failed to retrieve external IP address, using local address"));
 			}
 			else {
-				LogMessage(MessageType::Debug_Info, L"Got external IP address");
+				log(logmsg::debug_info, L"Got external IP address");
 				address = m_pIPResolver->GetIP();
 
 				engine_.GetOptions().SetOption(OPTION_LASTRESOLVEDIP, fz::to_wstring(address));
@@ -700,7 +683,7 @@ int CFtpControlSocket::GetExternalIPAddress(std::string& address)
 getLocalIP:
 	address = socket_->local_ip(true);
 	if (address.empty()) {
-		LogMessage(MessageType::Error, _("Failed to retrieve local IP address."), 1);
+		log(logmsg::error, _("Failed to retrieve local IP address."), 1);
 		return FZ_REPLY_ERROR;
 	}
 
@@ -709,9 +692,9 @@ getLocalIP:
 
 void CFtpControlSocket::OnExternalIPAddress()
 {
-	LogMessage(MessageType::Debug_Verbose, L"CFtpControlSocket::OnExternalIPAddress()");
+	log(logmsg::debug_verbose, L"CFtpControlSocket::OnExternalIPAddress()");
 	if (!m_pIPResolver) {
-		LogMessage(MessageType::Debug_Info, L"Ignoring event");
+		log(logmsg::debug_info, L"Ignoring event");
 		return;
 	}
 
@@ -729,7 +712,7 @@ void CFtpControlSocket::Transfer(std::wstring const& cmd, CFtpTransferOpData* ol
 	pData->pOldData = oldData;
 	pData->pOldData->transferEndReason = TransferEndReason::successful;
 
-	if (m_pProxyBackend) {
+	if (proxy_layer_) {
 		// Only passive suported
 		// Theoretically could use reverse proxy ability in SOCKS5, but
 		// it is too fragile to set up with all those broken routers and
@@ -768,7 +751,7 @@ void CFtpControlSocket::Transfer(std::wstring const& cmd, CFtpTransferOpData* ol
 void CFtpControlSocket::Connect(CServer const& server, Credentials const& credentials)
 {
 	if (!operations_.empty()) {
-		LogMessage(MessageType::Debug_Warning, L"CFtpControlSocket::Connect(): deleting stale operations");
+		log(logmsg::debug_warning, L"CFtpControlSocket::Connect(): deleting stale operations");
 		operations_.clear();
 	}
 
@@ -792,7 +775,7 @@ void CFtpControlSocket::OnTimer(fz::timer_id id)
 		return;
 	}
 
-	LogMessage(MessageType::Status, _("Sending keep-alive command"));
+	log(logmsg::status, _("Sending keep-alive command"));
 
 	std::wstring cmd;
 	auto i = fz::random_number(0, 2);
@@ -857,5 +840,25 @@ void CFtpControlSocket::operator()(fz::event_base const& ev)
 		return;
 	}
 
+	if (fz::dispatch<fz::certificate_verification_event>(ev, this, &CFtpControlSocket::OnVerifyCert)) {
+		return;
+	}
+
 	CRealControlSocket::operator()(ev);
 }
+
+void CFtpControlSocket::ResetSocket()
+{
+	receiveBuffer_.clear();
+	tls_layer_.reset();
+	CRealControlSocket::ResetSocket();
+}
+
+void CFtpControlSocket::OnVerifyCert(fz::tls_layer* source, fz::tls_session_info & info)
+{
+	if (!tls_layer_ || source != tls_layer_.get()) {
+		return;
+	}
+
+	SendAsyncRequest(new CCertificateNotification(std::move(info)));
+}
