@@ -3,10 +3,11 @@
 #include "logon.h"
 #include "../proxy.h"
 #include "../servercapabilities.h"
-#include "../tlssocket.h"
+
+#include <libfilezilla/tls_layer.hpp>
 
 CFtpLogonOpData::CFtpLogonOpData(CFtpControlSocket& controlSocket, Credentials const& credentials)
-    : COpData(Command::connect, L"CFtpLogonOpData")
+	: COpData(Command::connect, L"CFtpLogonOpData")
 	, CFtpOpData(controlSocket)
 	, credentials_(credentials)
 {
@@ -41,17 +42,17 @@ int CFtpLogonOpData::Send()
 	switch (opState)
 	{
 	case LOGON_CONNECT:
-	    {
+		{
 			// Do not use FTP proxy if generic proxy is set
 			int const generic_proxy_type = engine_.GetOptions().GetOptionVal(OPTION_PROXY_TYPE);
-			if ((generic_proxy_type <= CProxySocket::unknown || generic_proxy_type >= CProxySocket::proxytype_count) && !currentServer_.GetBypassProxy()) {
+			if ((generic_proxy_type <= static_cast<int>(ProxyType::NONE) || generic_proxy_type >= static_cast<int>(ProxyType::count)) && !currentServer_.GetBypassProxy()) {
 				ftp_proxy_type_ = engine_.GetOptions().GetOptionVal(OPTION_FTP_PROXY_TYPE);
 			}
-		
+
 			if (!PrepareLoginSequence()) {
 				return FZ_REPLY_INTERNALERROR;
 			}
-			
+
 			if (ftp_proxy_type_) {
 				host_ = engine_.GetOptions().GetOption(OPTION_FTP_PROXY_HOST);
 
@@ -60,12 +61,12 @@ int CFtpLogonOpData::Send()
 					// Probably IPv6 address
 					pos = host_.find(']');
 					if (pos == std::wstring::npos) {
-						LogMessage(MessageType::Error, _("Proxy host starts with '[' but no closing bracket found."));
+						log(logmsg::error, _("Proxy host starts with '[' but no closing bracket found."));
 						return FZ_REPLY_DISCONNECTED | FZ_REPLY_CRITICALERROR;
 					}
 					if (host_.size() > (pos + 1) && host_[pos + 1]) {
 						if (host_[pos + 1] != ':') {
-							LogMessage(MessageType::Error, _("Invalid proxy host, after closing bracket only colon and port may follow."));
+							log(logmsg::error, _("Invalid proxy host, after closing bracket only colon and port may follow."));
 							return FZ_REPLY_DISCONNECTED | FZ_REPLY_CRITICALERROR;
 						}
 						++pos;
@@ -87,11 +88,11 @@ int CFtpLogonOpData::Send()
 				}
 
 				if (host_.empty() || port_ < 1 || port_ > 65535) {
-					LogMessage(MessageType::Error, _("Proxy set but proxy host or port invalid"));
+					log(logmsg::error, _("Proxy set but proxy host or port invalid"));
 					return FZ_REPLY_DISCONNECTED | FZ_REPLY_CRITICALERROR;
 				}
 
-				LogMessage(MessageType::Status, _("Connecting to %s through %s proxy"), currentServer_.Format(ServerFormat::with_optional_port, credentials_), L"FTP"); // @translator: Connecting to ftp.example.com through SOCKS5 proxy
+				log(logmsg::status, _("Connecting to %s through %s proxy"), currentServer_.Format(ServerFormat::with_optional_port, credentials_), L"FTP"); // @translator: Connecting to ftp.example.com through SOCKS5 proxy
 			}
 			else {
 				host_ = currentServer_.GetHost();
@@ -99,20 +100,45 @@ int CFtpLogonOpData::Send()
 			}
 
 			opState = LOGON_WELCOME;
-			return controlSocket_.DoConnect(host_, port_);
-	    }
+			int ret = controlSocket_.DoConnect(host_, port_);
+			if (ret == FZ_REPLY_WOULDBLOCK) {
+				// Enable TCP_NODELAY, speeds things up a bit.
+				controlSocket_.socket_->set_flags(fz::socket::flag_nodelay | fz::socket::flag_keepalive, true);
+
+				// Enable SO_KEEPALIVE, lots of clueless users have broken routers and
+				// firewalls which terminate the control connection on long transfers.
+				int v = engine_.GetOptions().GetOptionVal(OPTION_TCP_KEEPALIVE_INTERVAL);
+				if (v >= 1 && v < 10000) {
+					controlSocket_.socket_->set_keepalive_interval(fz::duration::from_minutes(v));
+				}
+			}
+			return ret;
+		}
 	case LOGON_AUTH_WAIT:
-		LogMessage(MessageType::Debug_Info, L"LogonSend() called during LOGON_AUTH_WAIT, ignoring");
+		log(logmsg::debug_info, L"LogonSend() called during LOGON_AUTH_WAIT, ignoring");
 		return FZ_REPLY_WOULDBLOCK;
 	case LOGON_AUTH_TLS:
 		return controlSocket_.SendCommand(L"AUTH TLS", false, false);
 	case LOGON_AUTH_SSL:
 		return controlSocket_.SendCommand(L"AUTH SSL", false, false);
+	case LOGON_SECURITY:
+		if (currentServer_.GetProtocol() == INSECURE_FTP) {
+			log(logmsg::status, _("Plain FTP is insecure. Please switch to FTP over TLS."));
+		}
+		opState = LOGON_LOGON;
+		if (controlSocket_.tls_layer_) {
+			return FZ_REPLY_CONTINUE;
+		}
+		else {
+			auto notification = new CInsecureConnectionNotification(currentServer_);
+			controlSocket_.SendAsyncRequest(notification);
+			return FZ_REPLY_WOULDBLOCK;
+		}
 	case LOGON_SYST:
 		return controlSocket_.SendCommand(L"SYST");
 	case LOGON_LOGON:
-	    {
-		    t_loginCommand cmd = loginSequence.front();
+		{
+			t_loginCommand cmd = loginSequence.front();
 			switch (cmd.type)
 			{
 			case loginCommandType::user:
@@ -132,6 +158,7 @@ int CFtpLogonOpData::Send()
 				if (!challenge.empty()) {
 					CInteractiveLoginNotification *pNotification = new CInteractiveLoginNotification(CInteractiveLoginNotification::interactive, challenge, false);
 					pNotification->server = currentServer_;
+					pNotification->handle_ = controlSocket_.handle_;
 					pNotification->credentials = credentials_;
 					challenge.clear();
 
@@ -167,7 +194,7 @@ int CFtpLogonOpData::Send()
 			default:
 				return FZ_REPLY_INTERNALERROR;
 			}
-	    }
+		}
 		break;
 	case LOGON_FEAT:
 		return controlSocket_.SendCommand(L"FEAT");
@@ -191,19 +218,19 @@ int CFtpLogonOpData::Send()
 		return controlSocket_.SendCommand(L"PROT P");
 	case LOGON_CUSTOMCOMMANDS:
 		if (customCommandIndex >= currentServer_.GetPostLoginCommands().size()) {
-			LogMessage(MessageType::Debug_Warning, L"pData->customCommandIndex >= m_pCurrentServer->GetPostLoginCommands().size()");
+			log(logmsg::debug_warning, L"pData->customCommandIndex >= m_pCurrentServer->GetPostLoginCommands().size()");
 			return FZ_REPLY_INTERNALERROR | FZ_REPLY_DISCONNECTED;
 		}
 		return controlSocket_.SendCommand(currentServer_.GetPostLoginCommands()[customCommandIndex]);
 	case LOGON_OPTSMLST:
-	    {
-		    std::wstring args;
+		{
+			std::wstring args;
 			CServerCapabilities::GetCapability(currentServer_, opst_mlst_command, &args);
 			return controlSocket_.SendCommand(L"OPTS MLST " + args);
-	    }
+		}
 		break;
 	default:
-		LogMessage(MessageType::Debug_Warning, L"unknown op state: %d", opState);
+		log(logmsg::debug_warning, L"unknown op state: %d", opState);
 		break;
 	}
 
@@ -227,14 +254,11 @@ int CFtpLogonOpData::ParseResponse()
 			CServerCapabilities::SetCapability(currentServer_, (opState == LOGON_AUTH_TLS) ? auth_tls_command : auth_ssl_command, no);
 			if (opState == LOGON_AUTH_SSL) {
 				if (currentServer_.GetProtocol() == FTP) {
-					LogMessage(MessageType::Status, _("Insecure server, it does not support FTP over TLS."));
+					log(logmsg::status, _("Insecure server, it does not support FTP over TLS."));
 					neededCommands[LOGON_PBSZ] = 0;
 					neededCommands[LOGON_PROT] = 0;
-
-					auto notification = new CInsecureFTPNotification(currentServer_);
-					controlSocket_.SendAsyncRequest(notification);
-					opState = LOGON_LOGON;
-					return FZ_REPLY_WOULDBLOCK;
+					opState = LOGON_SECURITY;
+					return FZ_REPLY_CONTINUE;
 				}
 				else {
 					return FZ_REPLY_DISCONNECTED | (code == 5 ? FZ_REPLY_CRITICALERROR : FZ_REPLY_ERROR);
@@ -244,22 +268,13 @@ int CFtpLogonOpData::ParseResponse()
 		else {
 			CServerCapabilities::SetCapability(currentServer_, (opState == LOGON_AUTH_TLS) ? auth_tls_command : auth_ssl_command, yes);
 
-			LogMessage(MessageType::Status, _("Initializing TLS..."));
+			log(logmsg::status, _("Initializing TLS..."));
 
-			assert(!controlSocket_.m_pTlsSocket);
-			delete controlSocket_.m_pBackend;
+			controlSocket_.tls_layer_ = std::make_unique<fz::tls_layer>(controlSocket_.event_loop_, &controlSocket_, *controlSocket_.active_layer_, &engine_.GetContext().GetTlsSystemTrustStore(), controlSocket_.logger_);
+			controlSocket_.active_layer_ = controlSocket_.tls_layer_.get();
 
-			controlSocket_.m_pTlsSocket = new CTlsSocket(&controlSocket_, *controlSocket_.socket_, &controlSocket_);
-			controlSocket_.m_pBackend = controlSocket_.m_pTlsSocket;
-
-			if (!controlSocket_.m_pTlsSocket->Init()) {
-				LogMessage(MessageType::Error, _("Failed to initialize TLS."));
-				return FZ_REPLY_INTERNALERROR | FZ_REPLY_DISCONNECTED;
-			}
-
-			int res = controlSocket_.m_pTlsSocket->Handshake();
-			if (res & FZ_REPLY_ERROR) {
-				return res | FZ_REPLY_DISCONNECTED;
+			if (!controlSocket_.tls_layer_->client_handshake(&controlSocket_)) {
+				return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
 			}
 
 			neededCommands[LOGON_AUTH_SSL] = 0;
@@ -275,11 +290,11 @@ int CFtpLogonOpData::ParseResponse()
 			if (cmd.type == loginCommandType::user || cmd.type == loginCommandType::pass) {
 				auto const user = currentServer_.GetUser();
 				if (!user.empty() && (user.front() == ' ' || user.back() == ' ')) {
-					LogMessage(MessageType::Status, _("Check your login credentials. The entered username starts or ends with a space character."));
+					log(logmsg::status, _("Check your login credentials. The entered username starts or ends with a space character."));
 				}
 				auto const pw = credentials_.GetPass();
 				if (!pw.empty() && (pw.front() == ' ' || pw.back() == ' ')) {
-					LogMessage(MessageType::Status, _("Check your login credentials. The entered password starts or ends with a space character."));
+					log(logmsg::status, _("Check your login credentials. The entered password starts or ends with a space character."));
 				}
 			}
 
@@ -298,14 +313,14 @@ int CFtpLogonOpData::ParseResponse()
 				}
 				if (!asciiOnly) {
 					if (ftp_proxy_type_) {
-						LogMessage(MessageType::Status, _("Login data contains non-ASCII characters and server might not be UTF-8 aware. Cannot fall back to local charset since using proxy."));
+						log(logmsg::status, _("Login data contains non-ASCII characters and server might not be UTF-8 aware. Cannot fall back to local charset since using proxy."));
 						int error = FZ_REPLY_DISCONNECTED | FZ_REPLY_ERROR;
 						if (cmd.type == loginCommandType::pass && code == 5) {
 							error |= FZ_REPLY_PASSWORDFAILED;
 						}
 						return error;
 					}
-					LogMessage(MessageType::Status, _("Login data contains non-ASCII characters and server might not be UTF-8 aware. Trying local charset."));
+					log(logmsg::status, _("Login data contains non-ASCII characters and server might not be UTF-8 aware. Trying local charset."));
 					controlSocket_.m_useUTF8 = false;
 					if (!PrepareLoginSequence()) {
 						int error = FZ_REPLY_DISCONNECTED | FZ_REPLY_ERROR;
@@ -332,9 +347,9 @@ int CFtpLogonOpData::ParseResponse()
 			}
 		}
 		else if (code == 3 && loginSequence.empty()) {
-			LogMessage(MessageType::Error, _("Login sequence fully executed yet not logged in, aborting."));
+			log(logmsg::error, _("Login sequence fully executed yet not logged in, aborting."));
 			if (cmd.type == loginCommandType::pass && credentials_.account_.empty()) {
-				LogMessage(MessageType::Error, _("Server might require an account. Try specifying an account using the Site Manager"));
+				log(logmsg::error, _("Server might require an account. Try specifying an account using the Site Manager"));
 			}
 			return FZ_REPLY_CRITICALERROR | FZ_REPLY_DISCONNECTED;
 		}
@@ -392,7 +407,7 @@ int CFtpLogonOpData::ParseResponse()
 
 		const CharsetEncoding encoding = currentServer_.GetEncodingType();
 		if (encoding == ENCODING_AUTO && CServerCapabilities::GetCapability(currentServer_, utf8_command) != yes) {
-			LogMessage(MessageType::Status, _("Server does not support non-ASCII characters."));
+			log(logmsg::status, _("Server does not support non-ASCII characters."));
 			controlSocket_.m_useUTF8 = false;
 		}
 	}
@@ -412,8 +427,8 @@ int CFtpLogonOpData::ParseResponse()
 		++opState;
 
 		if (opState == LOGON_DONE) {
-			LogMessage(MessageType::Status, _("Logged in"));
-			LogMessage(MessageType::Debug_Info, L"Measured latency of %d ms", controlSocket_.m_rtt.GetLatency());
+			log(logmsg::status, _("Logged in"));
+			log(logmsg::debug_info, L"Measured latency of %d ms", controlSocket_.m_rtt.GetLatency());
 			return FZ_REPLY_OK;
 		}
 
@@ -452,7 +467,7 @@ int CFtpLogonOpData::ParseResponse()
 			}
 			const CharsetEncoding encoding = currentServer_.GetEncodingType();
 			if (encoding == ENCODING_AUTO && CServerCapabilities::GetCapability(currentServer_, utf8_command) != yes) {
-				LogMessage(MessageType::Status, _("Server does not support non-ASCII characters."));
+				log(logmsg::status, _("Server does not support non-ASCII characters."));
 				controlSocket_.m_useUTF8 = false;
 			}
 		}
@@ -745,12 +760,12 @@ bool CFtpLogonOpData::PrepareLoginSequence()
 		}
 
 		if (loginSequence.empty()) {
-			LogMessage(MessageType::Error, _("Could not generate custom login sequence."));
+			log(logmsg::error, _("Could not generate custom login sequence."));
 			return false;
 		}
 	}
 	else {
-		LogMessage(MessageType::Error, _("Unknown FTP proxy type, cannot generate login sequence."));
+		log(logmsg::error, _("Unknown FTP proxy type, cannot generate login sequence."));
 		return false;
 	}
 
