@@ -10,6 +10,7 @@ enum loginStates
 	LOGON_AUTH_TLS,
 	LOGON_AUTH_SSL,
 	LOGON_AUTH_WAIT,
+	LOGON_SECURITY,
 	LOGON_LOGON,
 	LOGON_SYST,
 	LOGON_FEAT,
@@ -33,9 +34,9 @@ enum class loginCommandType
 
 struct t_loginCommand
 {
-	bool optional;
-	bool hide_arguments;
-	loginCommandType type;
+	bool optional{};
+	bool hide_arguments{};
+	loginCommandType type{};
 
 	std::wstring command;
 };
@@ -53,7 +54,6 @@ public:
 
 	std::wstring challenge; // Used for interactive logons
 	bool waitChallenge{};
-	bool waitForAsyncRequest{};
 	bool gotFirstWelcomeLine{};
 	Credentials credentials_;
 
@@ -66,7 +66,7 @@ private:
 
 	unsigned int customCommandIndex{};
 
-	int neededCommands[LOGON_DONE];
+	int neededCommands[LOGON_DONE]{};
 
 	std::deque<t_loginCommand> loginSequence;
 
