@@ -6,10 +6,10 @@
 #include "optionsbase.h"
 #include "proxy.h"
 #include "servercapabilities.h"
-#include "socket_errors.h"
-#include "tlssocket.h"
 #include "transfersocket.h"
 
+#include <libfilezilla/rate_limited_layer.hpp>
+#include <libfilezilla/tls_layer.hpp>
 #include <libfilezilla/util.hpp>
 
 CTransferSocket::CTransferSocket(CFileZillaEnginePrivate & engine, CFtpControlSocket & controlSocket, TransferMode transferMode)
@@ -40,17 +40,14 @@ CTransferSocket::~CTransferSocket()
 
 void CTransferSocket::ResetSocket()
 {
-	delete m_pProxyBackend;
-	if (m_pBackend == m_pTlsSocket) {
-		m_pBackend = nullptr;
-	}
-	delete m_pTlsSocket;
-	delete m_pBackend;
 	socketServer_.reset();
+
+	active_layer_ = nullptr;
+
+	tls_layer_.reset();
+	proxy_layer_.reset();
+	ratelimit_layer_.reset();
 	socket_.reset();
-	m_pProxyBackend = nullptr;
-	m_pTlsSocket = nullptr;
-	m_pBackend = nullptr;
 }
 
 std::wstring CTransferSocket::SetupActiveTransfer(std::string const& ip)
@@ -59,7 +56,7 @@ std::wstring CTransferSocket::SetupActiveTransfer(std::string const& ip)
 	socketServer_ = CreateSocketServer();
 
 	if (!socketServer_) {
-		controlSocket_.LogMessage(MessageType::Debug_Warning, L"CreateSocketServer failed");
+		controlSocket_.log(logmsg::debug_warning, L"CreateSocketServer failed");
 		return std::wstring();
 	}
 
@@ -68,14 +65,14 @@ std::wstring CTransferSocket::SetupActiveTransfer(std::string const& ip)
 	if (port == -1)	{
 		ResetSocket();
 
-		controlSocket_.LogMessage(MessageType::Debug_Warning, L"GetLocalPort failed: %s", fz::socket_error_description(error));
+		controlSocket_.log(logmsg::debug_warning, L"GetLocalPort failed: %s", fz::socket_error_description(error));
 		return std::wstring();
 	}
 
 	if (engine_.GetOptions().GetOptionVal(OPTION_LIMITPORTS)) {
 		port += static_cast<int>(engine_.GetOptions().GetOptionVal(OPTION_LIMITPORTS_OFFSET));
 		if (port <= 0 || port >= 65536) {
-			controlSocket_.LogMessage(MessageType::Debug_Warning, L"Port outside valid range");
+			controlSocket_.log(logmsg::debug_warning, L"Port outside valid range");
 			return std::wstring();
 		}
 	}
@@ -93,41 +90,14 @@ std::wstring CTransferSocket::SetupActiveTransfer(std::string const& ip)
 	return portArguments;
 }
 
-void CTransferSocket::OnSocketEvent(fz::socket_event_source*, fz::socket_event_flag t, int error)
+void CTransferSocket::OnSocketEvent(fz::socket_event_source* source, fz::socket_event_flag t, int error)
 {
-	if (m_transferEndReason != TransferEndReason::none) {
-		return;
-	}
-
-	if (m_pProxyBackend) {
-		switch (t)
-		{
-		case fz::socket_event_flag::connection:
-			{
-				if (error) {
-					controlSocket_.LogMessage(MessageType::Error, _("Proxy handshake failed: %s"), fz::socket_error_description(error));
-					TransferEnd(TransferEndReason::failure);
-				}
-				else {
-					delete m_pProxyBackend;
-					m_pProxyBackend = nullptr;
-					OnConnect();
-				}
-			}
-			return;
-		default:
-			// Uninteresting
-			break;
-		}
-		return;
-	}
-
 	if (socketServer_) {
 		if (t == fz::socket_event_flag::connection) {
 			OnAccept(error);
 		}
 		else {
-			controlSocket_.LogMessage(MessageType::Debug_Info, L"Unhandled socket event %d from listening socket", t);
+			controlSocket_.log(logmsg::debug_info, L"Unhandled socket event %d from listening socket", t);
 		}
 		return;
 	}
@@ -136,7 +106,12 @@ void CTransferSocket::OnSocketEvent(fz::socket_event_source*, fz::socket_event_f
 	{
 	case fz::socket_event_flag::connection:
 		if (error) {
-			controlSocket_.LogMessage(MessageType::Error, _("The data connection could not be established: %s"), fz::socket_error_description(error));
+			if (source == proxy_layer_.get()) {
+				controlSocket_.log(logmsg::error, _("Proxy handshake failed: %s"), fz::socket_error_description(error));
+			}
+			else {
+				controlSocket_.log(logmsg::error, _("The data connection could not be established: %s"), fz::socket_error_description(error));
+			}
 			TransferEnd(TransferEndReason::transfer_failure);
 		}
 		else {
@@ -168,53 +143,58 @@ void CTransferSocket::OnSocketEvent(fz::socket_event_source*, fz::socket_event_f
 void CTransferSocket::OnAccept(int error)
 {
 	controlSocket_.SetAlive();
-	controlSocket_.LogMessage(MessageType::Debug_Verbose, L"CTransferSocket::OnAccept(%d)", error);
+	controlSocket_.log(logmsg::debug_verbose, L"CTransferSocket::OnAccept(%d)", error);
 
 	if (!socketServer_) {
-		controlSocket_.LogMessage(MessageType::Debug_Warning, L"No socket server in OnAccept", error);
+		controlSocket_.log(logmsg::debug_warning, L"No socket server in OnAccept", error);
 		return;
 	}
 
-	socket_.reset(socketServer_->accept(error));
+	socket_ = socketServer_->accept(error);
 	if (!socket_) {
 		if (error == EAGAIN) {
-			controlSocket_.LogMessage(MessageType::Debug_Verbose, L"No pending connection");
+			controlSocket_.log(logmsg::debug_verbose, L"No pending connection");
 		}
 		else {
-			controlSocket_.LogMessage(MessageType::Status, _("Could not accept connection: %s"), fz::socket_error_description(error));
+			controlSocket_.log(logmsg::status, _("Could not accept connection: %s"), fz::socket_error_description(error));
 			TransferEnd(TransferEndReason::transfer_failure);
 		}
 		return;
 	}
 	socketServer_.reset();
 
-	OnConnect();
+	if (!InitLayers(true)) {
+		TransferEnd(TransferEndReason::transfer_failure);
+		return;
+	}
+
+	if (active_layer_->get_state() == fz::socket_state::connected) {
+		OnConnect();
+	}
 }
 
 void CTransferSocket::OnConnect()
 {
 	controlSocket_.SetAlive();
-	controlSocket_.LogMessage(MessageType::Debug_Verbose, L"CTransferSocket::OnConnect");
+	controlSocket_.log(logmsg::debug_verbose, L"CTransferSocket::OnConnect");
 
 	if (!socket_) {
-		controlSocket_.LogMessage(MessageType::Debug_Verbose, L"CTransferSocket::OnConnect called without socket");
+		controlSocket_.log(logmsg::debug_verbose, L"CTransferSocket::OnConnect called without socket");
 		return;
 	}
 
-	if (!m_pBackend) {
-		if (!InitBackend()) {
-			TransferEnd(TransferEndReason::transfer_failure);
-			return;
-		}
-	}
-	else if (m_pTlsSocket) {
+	if (tls_layer_) {
 		// Re-enable Nagle algorithm
-		socket_->set_flags(socket_->flags() & (~fz::socket::flag_nodelay));
-		if (CServerCapabilities::GetCapability(controlSocket_.currentServer_, tls_resume) == unknown)	{
-			CServerCapabilities::SetCapability(controlSocket_.currentServer_, tls_resume, m_pTlsSocket->ResumedSession() ? yes : no);
-		}
+		socket_->set_flags(fz::socket::flag_nodelay, false);
 	}
 
+#ifdef FZ_WINDOWS
+	if (m_transferMode == TransferMode::upload) {
+		// For send buffer tuning
+		add_timer(fz::duration::from_seconds(1), false);
+	}
+#endif
+
 	if (m_bActive) {
 		TriggerPostponedEvents();
 	}
@@ -222,143 +202,164 @@ void CTransferSocket::OnConnect()
 
 void CTransferSocket::OnReceive()
 {
-	controlSocket_.LogMessage(MessageType::Debug_Debug, L"CTransferSocket::OnReceive(), m_transferMode=%d", m_transferMode);
-
-	if (!m_pBackend) {
-		controlSocket_.LogMessage(MessageType::Debug_Verbose, L"Postponing receive, m_pBackend was false.");
-		m_postponedReceive = true;
-		return;
-	}
+	controlSocket_.log(logmsg::debug_debug, L"CTransferSocket::OnReceive(), m_transferMode=%d", m_transferMode);
 
 	if (!m_bActive) {
-		controlSocket_.LogMessage(MessageType::Debug_Verbose, L"Postponing receive, m_bActive was false.");
+		controlSocket_.log(logmsg::debug_verbose, L"Postponing receive, m_bActive was false.");
 		m_postponedReceive = true;
 		return;
 	}
 
-	if (m_transferMode == TransferMode::list) {
-		for (;;) {
-			char *pBuffer = new char[4096];
-			int error;
-			int numread = m_pBackend->Read(pBuffer, 4096, error);
-			if (numread < 0) {
-				delete [] pBuffer;
-				if (error != EAGAIN) {
-					controlSocket_.LogMessage(MessageType::Error, L"Could not read from transfer socket: %s", fz::socket_error_description(error));
-					TransferEnd(TransferEndReason::transfer_failure);
-				}
-				return;
-			}
-
-			if (numread > 0) {
-				if (!m_pDirectoryListingParser->AddData(pBuffer, numread)) {
-					TransferEnd(TransferEndReason::transfer_failure);
+	if (m_transferEndReason == TransferEndReason::none) {
+		if (m_transferMode == TransferMode::list) {
+			for (;;) {
+				char *pBuffer = new char[4096];
+				int error;
+				int numread = active_layer_->read(pBuffer, 4096, error);
+				if (numread < 0) {
+					delete [] pBuffer;
+					if (error != EAGAIN) {
+						controlSocket_.log(logmsg::error, L"Could not read from transfer socket: %s", fz::socket_error_description(error));
+						TransferEnd(TransferEndReason::transfer_failure);
+					}
 					return;
 				}
 
+				if (numread > 0) {
+					if (!m_pDirectoryListingParser->AddData(pBuffer, numread)) {
+						TransferEnd(TransferEndReason::transfer_failure);
+						return;
+					}
+
+					controlSocket_.SetActive(CFileZillaEngine::recv);
+					if (!m_madeProgress) {
+						m_madeProgress = 2;
+						engine_.transfer_status_.SetMadeProgress();
+					}
+					engine_.transfer_status_.Update(numread);
+				}
+				else {
+					delete [] pBuffer;
+					TransferEnd(TransferEndReason::successful);
+					return;
+				}
+			}
+			return;
+		}
+		else if (m_transferMode == TransferMode::download) {
+			int error;
+			int numread;
+
+			// Only do a certain number of iterations in one go to keep the event loop going.
+			// Otherwise this behaves like a livelock on very large files written to a very fast
+			// SSD downloaded from a very fast server.
+			for (int i = 0; i < 100; ++i) {
+				if (!CheckGetNextWriteBuffer()) {
+					return;
+				}
+
+				numread = active_layer_->read(m_pTransferBuffer, m_transferBufferLen, error);
+				if (numread <= 0) {
+					break;
+				}
+
 				controlSocket_.SetActive(CFileZillaEngine::recv);
 				if (!m_madeProgress) {
 					m_madeProgress = 2;
 					engine_.transfer_status_.SetMadeProgress();
 				}
 				engine_.transfer_status_.Update(numread);
-			}
-			else {
-				delete [] pBuffer;
-				TransferEnd(TransferEndReason::successful);
-				return;
-			}
-		}
-	}
-	else if (m_transferMode == TransferMode::download) {
-		int error;
-		int numread;
 
-		// Only do a certain number of iterations in one go to keep the event loop going.
-		// Otherwise this behaves like a livelock on very large files written to a very fast
-		// SSD downloaded from a very fast server.
-		for (int i = 0; i < 100; ++i) {
-			if (!CheckGetNextWriteBuffer()) {
-				return;
+				m_pTransferBuffer += numread;
+				m_transferBufferLen -= numread;
 			}
 
-			numread = m_pBackend->Read(m_pTransferBuffer, m_transferBufferLen, error);
-			if (numread <= 0) {
-				break;
-			}
-
-			controlSocket_.SetActive(CFileZillaEngine::recv);
-			if (!m_madeProgress) {
-				m_madeProgress = 2;
-				engine_.transfer_status_.SetMadeProgress();
-			}
-			engine_.transfer_status_.Update(numread);
-
-			m_pTransferBuffer += numread;
-			m_transferBufferLen -= numread;
-		}
-
-		if (numread < 0) {
-			if (error != EAGAIN) {
-				controlSocket_.LogMessage(MessageType::Error, L"Could not read from transfer socket: %s", fz::socket_error_description(error));
-				TransferEnd(TransferEndReason::transfer_failure);
-			}
-		}
-		else if (!numread) {
-			FinalizeWrite();
-		}
-		else {
-			send_event<fz::socket_event>(m_pBackend, fz::socket_event_flag::read, 0);
-		}
-	}
-	else if (m_transferMode == TransferMode::resumetest) {
-		for (;;) {
-			char buffer[2];
-			int error;
-			int numread = m_pBackend->Read(buffer, 2, error);
 			if (numread < 0) {
 				if (error != EAGAIN) {
-					controlSocket_.LogMessage(MessageType::Error, L"Could not read from transfer socket: %s", fz::socket_error_description(error));
+					controlSocket_.log(logmsg::error, L"Could not read from transfer socket: %s", fz::socket_error_description(error));
 					TransferEnd(TransferEndReason::transfer_failure);
 				}
-				return;
 			}
-
-			if (!numread) {
-				if (m_transferBufferLen == 1) {
-					TransferEnd(TransferEndReason::successful);
+			else if (!numread) {
+				FinalizeWrite();
+			}
+			else {
+				send_event<fz::socket_event>(active_layer_, fz::socket_event_flag::read, 0);
+			}
+			return;
+		}
+		else if (m_transferMode == TransferMode::resumetest) {
+			for (;;) {
+				char buffer[2];
+				int error;
+				int numread = active_layer_->read(buffer, 2, error);
+				if (numread < 0) {
+					if (error != EAGAIN) {
+						controlSocket_.log(logmsg::error, L"Could not read from transfer socket: %s", fz::socket_error_description(error));
+						TransferEnd(TransferEndReason::transfer_failure);
+					}
+					return;
 				}
-				else {
-					controlSocket_.LogMessage(MessageType::Debug_Warning, L"Server incorrectly sent %d bytes", m_transferBufferLen);
+
+				if (!numread) {
+					if (m_transferBufferLen == 1) {
+						TransferEnd(TransferEndReason::successful);
+					}
+					else {
+						controlSocket_.log(logmsg::debug_warning, L"Server incorrectly sent %d bytes", m_transferBufferLen);
+						TransferEnd(TransferEndReason::failed_resumetest);
+					}
+					return;
+				}
+				m_transferBufferLen += numread;
+
+				if (m_transferBufferLen > 1) {
+					controlSocket_.log(logmsg::debug_warning, L"Server incorrectly sent %d bytes", m_transferBufferLen);
 					TransferEnd(TransferEndReason::failed_resumetest);
+					return;
 				}
-				return;
 			}
-			m_transferBufferLen += numread;
+			return;
+		}
+	}
 
-			if (m_transferBufferLen > 1) {
-				controlSocket_.LogMessage(MessageType::Debug_Warning, L"Server incorrectly sent %d bytes", m_transferBufferLen);
-				TransferEnd(TransferEndReason::failed_resumetest);
-				return;
-			}
+	char discard[1024];
+	int error;
+	int numread = active_layer_->read(discard, 1024, error);
+
+	if (m_transferEndReason == TransferEndReason::none) {
+		// If we get here we're uploading
+
+		if (numread > 0) {
+			controlSocket_.log(logmsg::error, L"Received data from the server during an upload");
+			TransferEnd(TransferEndReason::transfer_failure);
+		}
+		else if (numread < 0 && error != EAGAIN) {
+			controlSocket_.log(logmsg::error, L"Could not read from transfer socket: %s", fz::socket_error_description(error));
+			TransferEnd(TransferEndReason::transfer_failure);
+		}
+	}
+	else {
+		if (!numread || (numread < 0 && error != EAGAIN)) {
+			ResetSocket();
 		}
 	}
 }
 
 void CTransferSocket::OnSend()
 {
-	if (!m_pBackend) {
-		controlSocket_.LogMessage(MessageType::Debug_Verbose, L"OnSend called without backend. Ignoring event.");
+	if (!active_layer_) {
+		controlSocket_.log(logmsg::debug_verbose, L"OnSend called without backend. Ignoring event.");
 		return;
 	}
 
 	if (!m_bActive) {
-		controlSocket_.LogMessage(MessageType::Debug_Verbose, L"Postponing send");
+		controlSocket_.log(logmsg::debug_verbose, L"Postponing send");
 		m_postponedSend = true;
 		return;
 	}
 
-	if (m_transferMode != TransferMode::upload) {
+	if (m_transferMode != TransferMode::upload || m_transferEndReason != TransferEndReason::none) {
 		return;
 	}
 
@@ -373,14 +374,14 @@ void CTransferSocket::OnSend()
 			return;
 		}
 
-		written = m_pBackend->Write(m_pTransferBuffer, m_transferBufferLen, error);
+		written = active_layer_->write(m_pTransferBuffer, m_transferBufferLen, error);
 		if (written <= 0) {
 			break;
 		}
 
 		controlSocket_.SetActive(CFileZillaEngine::send);
 		if (m_madeProgress == 1) {
-			controlSocket_.LogMessage(MessageType::Debug_Debug, L"Made progress in CTransferSocket::OnSend()");
+			controlSocket_.log(logmsg::debug_debug, L"Made progress in CTransferSocket::OnSend()");
 			m_madeProgress = 2;
 			engine_.transfer_status_.SetMadeProgress();
 		}
@@ -393,64 +394,40 @@ void CTransferSocket::OnSend()
 	if (written < 0) {
 		if (error == EAGAIN) {
 			if (!m_madeProgress) {
-				controlSocket_.LogMessage(MessageType::Debug_Debug, L"First EAGAIN in CTransferSocket::OnSend()");
+				controlSocket_.log(logmsg::debug_debug, L"First EAGAIN in CTransferSocket::OnSend()");
 				m_madeProgress = 1;
 				engine_.transfer_status_.SetMadeProgress();
 			}
 		}
 		else {
-			controlSocket_.LogMessage(MessageType::Error, L"Could not write to transfer socket: %s", fz::socket_error_description(error));
+			controlSocket_.log(logmsg::error, L"Could not write to transfer socket: %s", fz::socket_error_description(error));
 			TransferEnd(TransferEndReason::transfer_failure);
 		}
 	}
 	else if (written > 0) {
-		send_event<fz::socket_event>(m_pBackend, fz::socket_event_flag::write, 0);
+		send_event<fz::socket_event>(active_layer_, fz::socket_event_flag::write, 0);
 	}
 }
 
 void CTransferSocket::OnSocketError(int error)
 {
-	controlSocket_.LogMessage(MessageType::Debug_Verbose, L"CTransferSocket::OnSocketError(%d)", error);
+	controlSocket_.log(logmsg::debug_verbose, L"CTransferSocket::OnSocketError(%d)", error);
 
 	if (m_transferEndReason != TransferEndReason::none) {
 		return;
 	}
 
-	controlSocket_.LogMessage(MessageType::Error, _("Transfer connection interrupted: %s"), fz::socket_error_description(error));
+	controlSocket_.log(logmsg::error, _("Transfer connection interrupted: %s"), fz::socket_error_description(error));
 	TransferEnd(TransferEndReason::transfer_failure);
 }
 
 bool CTransferSocket::SetupPassiveTransfer(std::wstring const& host, int port)
 {
-	std::string ip;
+	std::string ip = fz::to_utf8(host);
 
 	ResetSocket();
 
-	socket_ = std::make_unique<fz::socket>(engine_.GetThreadPool(), this);
-
-	if (controlSocket_.m_pProxyBackend) {
-		m_pProxyBackend = new CProxySocket(this, socket_.get(), &controlSocket_);
-
-		int res = m_pProxyBackend->Handshake(controlSocket_.m_pProxyBackend->GetProxyType(),
-											 host, port,
-											 controlSocket_.m_pProxyBackend->GetUser(), controlSocket_.m_pProxyBackend->GetPass());
-
-		if (res != EINPROGRESS) {
-			ResetSocket();
-			return false;
-		}
-		int error;
-		ip = controlSocket_.socket_->peer_ip();
-		port = controlSocket_.socket_->remote_port(error);
-		if (ip.empty() || port < 1) {
-			controlSocket_.LogMessage(MessageType::Debug_Warning, L"Could not get peer address of control connection.");
-			ResetSocket();
-			return false;
-		}
-	}
-	else {
-		ip = fz::to_utf8(host);
-	}
+	socket_ = std::make_unique<fz::socket>(engine_.GetThreadPool(), nullptr);
 
 	SetSocketBufferSizes(*socket_);
 
@@ -463,26 +440,69 @@ bool CTransferSocket::SetupPassiveTransfer(std::wstring const& host, int port)
 	// same source.
 
 	std::string bindAddress;
-	if (m_pProxyBackend) {
+	if (controlSocket_.proxy_layer_) {
 		bindAddress = controlSocket_.socket_->local_ip();
-		controlSocket_.LogMessage(MessageType::Debug_Info, L"Binding data connection source IP to control connection source IP %s", bindAddress);
+		controlSocket_.log(logmsg::debug_info, L"Binding data connection source IP to control connection source IP %s", bindAddress);
+		socket_->bind(bindAddress);
 	}
 	else {
 		if (controlSocket_.socket_->peer_ip(true) == ip || controlSocket_.socket_->peer_ip(false) == ip) {
 			bindAddress = controlSocket_.socket_->local_ip();
-			controlSocket_.LogMessage(MessageType::Debug_Info, L"Binding data connection source IP to control connection source IP %s", bindAddress);
+			controlSocket_.log(logmsg::debug_info, L"Binding data connection source IP to control connection source IP %s", bindAddress);
+			socket_->bind(bindAddress);
 		}
 		else {
-			controlSocket_.LogMessage(MessageType::Debug_Warning, L"Destination IP of data connection does not match peer IP of control connection. Not binding source address of data connection.");
+			controlSocket_.log(logmsg::debug_warning, L"Destination IP of data connection does not match peer IP of control connection. Not binding source address of data connection.");
 		}
 	}
 
-	int res = socket_->connect(fz::to_native(ip), port, fz::address_type::unknown, bindAddress);
-	if (res && res != EINPROGRESS) {
+	if (!InitLayers(false)) {
 		ResetSocket();
 		return false;
 	}
 
+	int res = active_layer_->connect(fz::to_native(ip), port, fz::address_type::unknown);
+	if (res) {
+		ResetSocket();
+		return false;
+	}
+
+	return true;
+}
+
+bool CTransferSocket::InitLayers(bool active)
+{
+	ratelimit_layer_ = std::make_unique<fz::rate_limited_layer>(nullptr, *socket_, &engine_.GetRateLimiter());
+	active_layer_ = ratelimit_layer_.get();
+
+	if (controlSocket_.proxy_layer_ && !active) {
+		fz::native_string proxy_host = controlSocket_.proxy_layer_->next().peer_host();
+		int error;
+		int proxy_port = controlSocket_.proxy_layer_->next().peer_port(error);
+
+		if (proxy_host.empty() || proxy_port < 1) {
+			controlSocket_.log(logmsg::debug_warning, L"Could not get peer address of control connection.");
+			return false;
+		}
+
+		proxy_layer_ = std::make_unique<CProxySocket>(nullptr, *active_layer_, &controlSocket_, controlSocket_.proxy_layer_->GetProxyType(), proxy_host, proxy_port, controlSocket_.proxy_layer_->GetUser(), controlSocket_.proxy_layer_->GetPass());
+		active_layer_ = proxy_layer_.get();
+	}
+
+	if (controlSocket_.m_protectDataChannel) {
+		// Disable Nagle's algorithm during TLS handshake
+		socket_->set_flags(fz::socket::flag_nodelay, true);
+
+		tls_layer_ = std::make_unique<fz::tls_layer>(controlSocket_.event_loop_, nullptr, *active_layer_, nullptr, controlSocket_.logger_);
+		active_layer_ = tls_layer_.get();
+
+		if (!tls_layer_->client_handshake(controlSocket_.tls_layer_->get_raw_certificate(), controlSocket_.tls_layer_->get_session_parameters(), controlSocket_.tls_layer_->peer_host())) {
+			return false;
+		}
+	}
+
+	active_layer_->set_event_handler(this);
+
 	return true;
 }
 
@@ -502,21 +522,26 @@ void CTransferSocket::SetActive()
 		return;
 	}
 
-	if (socket_->get_state() == fz::socket::connected) {
+	if (socket_->is_connected()) {
 		TriggerPostponedEvents();
 	}
 }
 
 void CTransferSocket::TransferEnd(TransferEndReason reason)
 {
-	controlSocket_.LogMessage(MessageType::Debug_Verbose, L"CTransferSocket::TransferEnd(%d)", reason);
+	controlSocket_.log(logmsg::debug_verbose, L"CTransferSocket::TransferEnd(%d)", reason);
 
 	if (m_transferEndReason != TransferEndReason::none) {
 		return;
 	}
 	m_transferEndReason = reason;
 
-	ResetSocket();
+	if (reason != TransferEndReason::successful) {
+		ResetSocket();
+	}
+	else {
+		active_layer_->shutdown();
+	}
 
 	controlSocket_.send_event<TransferEndEvent>();
 }
@@ -526,7 +551,7 @@ std::unique_ptr<fz::listen_socket> CTransferSocket::CreateSocketServer(int port)
 	auto socket = std::make_unique<fz::listen_socket>(engine_.GetThreadPool(), this);
 	int res = socket->listen(controlSocket_.socket_->address_family(), port);
 	if (res) {
-		controlSocket_.LogMessage(MessageType::Debug_Verbose, L"Could not listen on port %d: %s", port, fz::socket_error_description(res));
+		controlSocket_.log(logmsg::debug_verbose, L"Could not listen on port %d: %s", port, fz::socket_error_description(res));
 		socket.reset();
 	}
 	else {
@@ -593,10 +618,10 @@ bool CTransferSocket::CheckGetNextWriteBuffer()
 		else if (res == IO_Error) {
 			std::wstring error = ioThread_->GetError();
 			if (error.empty()) {
-				controlSocket_.LogMessage(MessageType::Error, _("Can't write data to file."));
+				controlSocket_.log(logmsg::error, _("Can't write data to file."));
 			}
 			else {
-				controlSocket_.LogMessage(MessageType::Error, _("Can't write data to file: %s"), error);
+				controlSocket_.log(logmsg::error, _("Can't write data to file: %s"), error);
 			}
 			TransferEnd(TransferEndReason::transfer_failure_critical);
 			return false;
@@ -616,19 +641,15 @@ bool CTransferSocket::CheckGetNextReadBuffer()
 			return false;
 		}
 		else if (res == IO_Error) {
-			controlSocket_.LogMessage(MessageType::Error, _("Can't read from file"));
+			controlSocket_.log(logmsg::error, _("Can't read from file"));
 			TransferEnd(TransferEndReason::transfer_failure);
 			return false;
 		}
 		else if (res == IO_Success) {
-			if (m_pTlsSocket) {
-				int error = m_pTlsSocket->Shutdown();
-				if (error != 0) {
-					if (error != EAGAIN) {
-						TransferEnd(TransferEndReason::transfer_failure);
-					}
-					return false;
-				}
+			int res = active_layer_->shutdown();
+			if (res && res != EAGAIN) {
+				TransferEnd(TransferEndReason::transfer_failure);
+				return false;
 			}
 			TransferEnd(TransferEndReason::successful);
 			return false;
@@ -668,49 +689,21 @@ void CTransferSocket::FinalizeWrite()
 	else {
 		std::wstring error = ioThread_->GetError();
 		if (error.empty()) {
-			controlSocket_.LogMessage(MessageType::Error, _("Can't write data to file."));
+			controlSocket_.log(logmsg::error, _("Can't write data to file."));
 		}
 		else {
-			controlSocket_.LogMessage(MessageType::Error, _("Can't write data to file: %s"), error);
+			controlSocket_.log(logmsg::error, _("Can't write data to file: %s"), error);
 		}
 		TransferEnd(TransferEndReason::transfer_failure_critical);
 	}
 }
 
-bool CTransferSocket::InitTls(const CTlsSocket* pPrimaryTlsSocket)
-{
-	// Disable Nagle's algorithm during TLS handshake
-	socket_->set_flags(socket_->flags() | fz::socket::flag_nodelay);
-
-	assert(!m_pBackend);
-	m_pTlsSocket = new CTlsSocket(this, *socket_, &controlSocket_);
-
-	if (!m_pTlsSocket->Init()) {
-		delete m_pTlsSocket;
-		m_pTlsSocket = nullptr;
-		return false;
-	}
-
-	bool try_resume = CServerCapabilities::GetCapability(controlSocket_.currentServer_, tls_resume) != no;
-
-	int res = m_pTlsSocket->Handshake(pPrimaryTlsSocket, try_resume);
-	if (res && res != FZ_REPLY_WOULDBLOCK) {
-		delete m_pTlsSocket;
-		m_pTlsSocket = nullptr;
-		return false;
-	}
-
-	m_pBackend = m_pTlsSocket;
-
-	return true;
-}
-
 void CTransferSocket::TriggerPostponedEvents()
 {
 	assert(m_bActive);
 
 	if (m_postponedReceive) {
-		controlSocket_.LogMessage(MessageType::Debug_Verbose, L"Executing postponed receive");
+		controlSocket_.log(logmsg::debug_verbose, L"Executing postponed receive");
 		m_postponedReceive = false;
 		OnReceive();
 		if (m_transferEndReason != TransferEndReason::none) {
@@ -718,38 +711,12 @@ void CTransferSocket::TriggerPostponedEvents()
 		}
 	}
 	if (m_postponedSend) {
-		controlSocket_.LogMessage(MessageType::Debug_Verbose, L"Executing postponed send");
+		controlSocket_.log(logmsg::debug_verbose, L"Executing postponed send");
 		m_postponedSend = false;
 		OnSend();
-		if (m_transferEndReason != TransferEndReason::none) {
-			return;
-		}
 	}
 }
 
-bool CTransferSocket::InitBackend()
-{
-	if (m_pBackend) {
-		return true;
-	}
-
-#ifdef FZ_WINDOWS
-	// For send buffer tuning
-	add_timer(fz::duration::from_seconds(1), false);
-#endif
-
-	if (controlSocket_.m_protectDataChannel) {
-		if (!InitTls(controlSocket_.m_pTlsSocket)) {
-			return false;
-		}
-	}
-	else {
-		m_pBackend = new CSocketBackend(this, *socket_, engine_.GetRateLimiter());
-	}
-
-	return true;
-}
-
 void CTransferSocket::SetSocketBufferSizes(fz::socket_base& socket)
 {
 	const int size_read = engine_.GetOptions().GetOptionVal(OPTION_SOCKET_BUFFERSIZE_RECV);
@@ -771,7 +738,7 @@ void CTransferSocket::operator()(fz::event_base const& ev)
 
 void CTransferSocket::OnTimer(fz::timer_id)
 {
-	if (socket_ && socket_->get_state() == fz::socket::connected) {
+	if (socket_ && socket_->is_connected()) {
 		int const ideal_send_buffer = socket_->ideal_send_buffer_size();
 		if (ideal_send_buffer != -1) {
 			socket_->set_buffer_sizes(-1, ideal_send_buffer);
