@@ -2,6 +2,7 @@
 #define FILEZILLA_ENGINE_IOTHREAD_HEADER
 
 #include <libfilezilla/event.hpp>
+#include <libfilezilla/event_handler.hpp>
 #include <libfilezilla/thread_pool.hpp>
 
 #define BUFFERCOUNT 8
@@ -65,7 +66,7 @@ private:
 
 	int64_t ReadFromFile(char* pBuffer, int64_t maxLen);
 	bool WriteToFile(char* pBuffer, int64_t len);
-	bool DoWrite(const char* pBuffer, int64_t len);
+	bool DoWrite(char const* pBuffer, int64_t len);
 
 	fz::event_handler* m_evtHandler{};
 
