@@ -1,6 +1,12 @@
 #ifndef FILEZILLA_ENGINE_SERVERCAPABILITIES_HEADER
 #define FILEZILLA_ENGINE_SERVERCAPABILITIES_HEADER
 
+#include <libfilezilla/mutex.hpp>
+
+#include <server.h>
+
+#include <map>
+
 enum capabilities
 {
 	unknown,
@@ -29,9 +35,6 @@ enum capabilityNames
 	rest_stream, // supports REST+STOR in addition to APPE
 	epsv_command,
 
-	// FTPS and HTTPS
-	tls_resume, // Does the server support resuming of TLS sessions?
-
 	// Server timezone offset. If using FTP, LIST details are unspecified and
 	// can return different times than the UTC based times using the MLST or
 	// MDTM commands.
@@ -55,9 +58,9 @@ public:
 protected:
 	struct t_cap
 	{
-		capabilities cap;
+		capabilities cap{unknown};
 		std::wstring option;
-		int number;
+		int number{};
 	};
 	std::map<capabilityNames, t_cap> m_capabilityMap;
 };
@@ -74,6 +77,8 @@ public:
 
 protected:
 	static std::map<CServer, CCapabilities> m_serverMap;
+
+	static fz::mutex m_;
 };
 
 #endif
