@@ -1,7 +1,7 @@
 #include <filezilla.h>
 
 #include "chmod.h"
-#include "directorycache.h"
+#include "../directorycache.h"
 
 enum chmodStates
 {
@@ -22,8 +22,7 @@ int CSftpChmodOpData::Send()
 
 		std::wstring quotedFilename = controlSocket_.QuoteFilename(command_.GetPath().FormatFilename(command_.GetFile(), !useAbsolute_));
 
-		return controlSocket_.SendCommand(L"chmod " + command_.GetPermission() + L" " + controlSocket_.WildcardEscape(quotedFilename),
-				L"chmod " + command_.GetPermission() + L" " + quotedFilename);		
+		return controlSocket_.SendCommand(L"chmod " + command_.GetPermission() + L" " + quotedFilename);
 	}
 
 	return FZ_REPLY_INTERNALERROR;
