@@ -1,19 +1,19 @@
 #include <filezilla.h>
 
 #include "delete.h"
-#include "directorycache.h"
+#include "../directorycache.h"
 
 int CSftpDeleteOpData::Send()
 {
-	std::wstring const& file = files_.front();
+	std::wstring const& file = files_.back();
 	if (file.empty()) {
-		LogMessage(MessageType::Debug_Info, L"Empty filename");
+		log(logmsg::debug_info, L"Empty filename");
 		return FZ_REPLY_INTERNALERROR;
 	}
 
 	std::wstring filename = path_.FormatFilename(file);
 	if (filename.empty()) {
-		LogMessage(MessageType::Error, _("Filename cannot be constructed for directory %s and filename %s"), path_.GetPath(), file);
+		log(logmsg::error, _("Filename cannot be constructed for directory %s and filename %s"), path_.GetPath(), file);
 		return FZ_REPLY_ERROR;
 	}
 
@@ -23,7 +23,7 @@ int CSftpDeleteOpData::Send()
 
 	engine_.GetDirectoryCache().InvalidateFile(currentServer_, path_, file);
 
-	return controlSocket_.SendCommand(L"rm " + controlSocket_.WildcardEscape(controlSocket_.QuoteFilename(filename)), L"rm " + controlSocket_.QuoteFilename(filename));
+	return controlSocket_.SendCommand(L"rm " + controlSocket_.QuoteFilename(filename));
 }
 
 int CSftpDeleteOpData::ParseResponse()
@@ -32,7 +32,7 @@ int CSftpDeleteOpData::ParseResponse()
 		deleteFailed_ = true;
 	}
 	else {
-		std::wstring const& file = files_.front();
+		std::wstring const& file = files_.back();
 
 		engine_.GetDirectoryCache().RemoveFile(currentServer_, path_, file);
 
@@ -47,7 +47,7 @@ int CSftpDeleteOpData::ParseResponse()
 		}
 	}
 
-	files_.pop_front();
+	files_.pop_back();
 
 	if (!files_.empty()) {
 		return FZ_REPLY_CONTINUE;
