@@ -7,20 +7,19 @@
 class CSftpListOpData final : public COpData, public CSftpOpData
 {
 public:
-	CSftpListOpData(CSftpControlSocket & controlSocket, CServerPath const& path, std::wstring const& subDir, int flags, bool topLevel)
-		: COpData(Command::list)
+	CSftpListOpData(CSftpControlSocket & controlSocket, CServerPath const& path, std::wstring const& subDir, int flags)
+		: COpData(Command::list, L"CSftpListOpData")
 		, CSftpOpData(controlSocket)
 		, path_(path)
 		, subDir_(subDir)
 		, flags_(flags)
-		, topLevel_(topLevel)
 	{}
 
 	virtual int Send() override;
 	virtual int ParseResponse() override;
 	virtual int SubcommandResult(int prevResult, COpData const& previousOperation) override;
 
-	int ParseEntry(std::wstring && entry, std::wstring const& stime, std::wstring && name);
+	int ParseEntry(std::wstring && entry, uint64_t mtime, std::wstring && name);
 
 private:
 	std::unique_ptr<CDirectoryListingParser> listing_parser_;
@@ -38,8 +37,6 @@ private:
 	CDirectoryListing directoryListing_;
 
 	fz::monotonic_clock time_before_locking_;
-
-	bool topLevel_{};
 };
 
 #endif
