@@ -4,7 +4,7 @@
 #include "connect.h"
 #include "cwd.h"
 #include "delete.h"
-#include "directorycache.h"
+#include "../directorycache.h"
 #include "directorylistingparser.h"
 #include "engineprivate.h"
 #include "event.h"
@@ -26,6 +26,9 @@
 #include <algorithm>
 #include <cwchar>
 
+struct SftpRateAvailableEventType;
+typedef fz::simple_event<SftpRateAvailableEventType, fz::direction::type> SftpRateAvailableEvent;
+
 CSftpControlSocket::CSftpControlSocket(CFileZillaEnginePrivate & engine)
 	: CControlSocket(engine)
 {
@@ -34,19 +37,20 @@ CSftpControlSocket::CSftpControlSocket(CFileZillaEnginePrivate & engine)
 
 CSftpControlSocket::~CSftpControlSocket()
 {
+	remove_bucket();
 	remove_handler();
 	DoClose();
 }
 
 void CSftpControlSocket::Connect(CServer const& server, Credentials const& credentials)
 {
-	LogMessage(MessageType::Status, _("Connecting to %s..."), server.Format(ServerFormat::with_optional_port, credentials));
+	log(logmsg::status, _("Connecting to %s..."), server.Format(ServerFormat::with_optional_port, credentials));
 	SetWait(true);
 
 	m_sftpEncryptionDetails = CSftpEncryptionNotification();
 
 	if (server.GetEncodingType() == ENCODING_CUSTOM) {
-		LogMessage(MessageType::Debug_Info, L"Using custom encoding: %s", server.GetCustomEncoding());
+		log(logmsg::debug_info, L"Using custom encoding: %s", server.GetCustomEncoding());
 		m_useUTF8 = false;
 	}
 	else {
@@ -67,7 +71,7 @@ void CSftpControlSocket::Connect(CServer const& server, Credentials const& crede
 		std::remove_if(pData->keyfiles_.begin(), pData->keyfiles_.end(),
 			[this](std::wstring const& keyfile) {
 				if (fz::local_filesys::get_file_type(fz::to_native(keyfile), true) != fz::local_filesys::file) {
-					LogMessage(MessageType::Status, _("Skipping non-existing key file \"%s\""), keyfile);
+					log(logmsg::status, _("Skipping non-existing key file \"%s\""), keyfile);
 					return true;
 				}
 				return false;
@@ -77,7 +81,7 @@ void CSftpControlSocket::Connect(CServer const& server, Credentials const& crede
 
 	process_ = std::make_unique<fz::process>();
 
-	engine_.GetRateLimiter().AddObject(this);
+	engine_.GetRateLimiter().add(this);
 	Push(std::move(pData));
 }
 
@@ -94,7 +98,7 @@ void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
 	switch (message.type)
 	{
 	case sftpEvent::Reply:
-		LogMessageRaw(MessageType::Response, message.text[0]);
+		log_raw(logmsg::reply, message.text[0]);
 		ProcessReply(FZ_REPLY_OK, message.text[0]);
 		break;
 	case sftpEvent::Done:
@@ -113,16 +117,16 @@ void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
 		}
 		break;
 	case sftpEvent::Error:
-		LogMessageRaw(MessageType::Error, message.text[0]);
+		log_raw(logmsg::error, message.text[0]);
 		break;
 	case sftpEvent::Verbose:
-		LogMessageRaw(MessageType::Debug_Info, message.text[0]);
+		log_raw(logmsg::debug_info, message.text[0]);
 		break;
 	case sftpEvent::Info:
-		LogMessageRaw(MessageType::Command, message.text[0]); // Not exactly the right message type, but it's a silent one.
+		log_raw(logmsg::command, message.text[0]); // Not exactly the right message type, but it's a silent one.
 		break;
 	case sftpEvent::Status:
-		LogMessageRaw(MessageType::Status, message.text[0]);
+		log_raw(logmsg::status, message.text[0]);
 		break;
 	case sftpEvent::Recv:
 		SetActive(CFileZillaEngine::recv);
@@ -167,12 +171,12 @@ void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
 		}
 		break;
 	case sftpEvent::AskHostkeyBetteralg:
-		LogMessage(MessageType::Error, L"Got sftpReqHostkeyBetteralg when we shouldn't have. Aborting connection.");
+		log(logmsg::error, L"Got sftpReqHostkeyBetteralg when we shouldn't have. Aborting connection.");
 		DoClose(FZ_REPLY_INTERNALERROR);
 		break;
 	case sftpEvent::AskPassword:
 		if (operations_.empty() || operations_.back()->opId != Command::connect) {
-			LogMessage(MessageType::Debug_Warning, L"sftpReqPassword outside connect operation, ignoring.");
+			log(logmsg::debug_warning, L"sftpReqPassword outside connect operation, ignoring.");
 			break;
 		}
 		else {
@@ -198,6 +202,7 @@ void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
 				}
 				CInteractiveLoginNotification *pNotification = new CInteractiveLoginNotification(t, challenge, data.lastChallenge == challengeIdentifier);
 				pNotification->server = currentServer_;
+				pNotification->handle_ = handle_;
 				pNotification->credentials = data.credentials_;
 
 				SendAsyncRequest(pNotification);
@@ -206,10 +211,10 @@ void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
 				if (!data.lastChallenge.empty() && data.lastChallengeType != CInteractiveLoginNotification::keyfile) {
 					// Check for same challenge. Will most likely fail as well, so abort early.
 					if (data.lastChallenge == challengeIdentifier) {
-						LogMessage(MessageType::Error, _("Authentication failed."));
+						log(logmsg::error, _("Authentication failed."));
 					}
 					else {
-						LogMessage(MessageType::Error, _("Server sent an additional login prompt. You need to use the interactive login type."));
+						log(logmsg::error, _("Server sent an additional login prompt. You need to use the interactive login type."));
 					}
 					DoClose(FZ_REPLY_CRITICALERROR | FZ_REPLY_PASSWORDFAILED);
 					return;
@@ -231,10 +236,10 @@ void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
 		m_requestInstruction = message.text[0];
 		break;
 	case sftpEvent::UsedQuotaRecv:
-		OnQuotaRequest(CRateLimiter::inbound);
+		OnQuotaRequest(fz::direction::inbound);
 		break;
 	case sftpEvent::UsedQuotaSend:
-		OnQuotaRequest(CRateLimiter::outbound);
+		OnQuotaRequest(fz::direction::outbound);
 		break;
 	case sftpEvent::KexAlgorithm:
 		m_sftpEncryptionDetails.kexAlgorithm = message.text[0];
@@ -259,7 +264,7 @@ void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
 		break;
 	case sftpEvent::Hostkey:
 		{
-			auto tokens = fz::strtok(message.text[0], ' ');
+			auto tokens = fz::strtok_view(message.text[0], ' ');
 			if (!tokens.empty()) {
 				m_sftpEncryptionDetails.hostKeyFingerprintSHA256 = tokens.back();
 				tokens.pop_back();
@@ -277,7 +282,7 @@ void CSftpControlSocket::OnSftpEvent(sftp_message const& message)
 		}
 		break;
 	default:
-		LogMessage(MessageType::Debug_Warning, L"Message type %d not handled", message.type);
+		log(logmsg::debug_warning, L"Message type %d not handled", message.type);
 		break;
 	}
 }
@@ -293,7 +298,7 @@ void CSftpControlSocket::OnSftpListEvent(sftp_list_message const& message)
 	}
 
 	if (operations_.empty() || operations_.back()->opId != Command::list) {
-		LogMessage(MessageType::Debug_Warning, L"sftpEvent::Listentry outside list operation, ignoring.");
+		log(logmsg::debug_warning, L"sftpEvent::Listentry outside list operation, ignoring.");
 		return;
 	}
 	else {
@@ -307,10 +312,10 @@ void CSftpControlSocket::OnSftpListEvent(sftp_list_message const& message)
 void CSftpControlSocket::OnTerminate(std::wstring const& error)
 {
 	if (!error.empty()) {
-		LogMessageRaw(MessageType::Error, error);
+		log_raw(logmsg::error, error);
 	}
 	else {
-		LogMessageRaw(MessageType::Debug_Info, L"CSftpControlSocket::OnTerminate without error");
+		log_raw(logmsg::debug_info, L"CSftpControlSocket::OnTerminate without error");
 	}
 	if (process_) {
 		DoClose();
@@ -321,14 +326,14 @@ int CSftpControlSocket::SendCommand(std::wstring const& cmd, std::wstring const&
 {
 	SetWait(true);
 
-	LogMessageRaw(MessageType::Command, show.empty() ? cmd : show);
+	log_raw(logmsg::command, show.empty() ? cmd : show);
 
 	// Check for newlines in command
 	// a command like "ls\nrm foo/bar" is dangerous
 	if (cmd.find('\n') != std::wstring::npos ||
 		cmd.find('\r') != std::wstring::npos)
 	{
-		LogMessage(MessageType::Debug_Warning, L"Command containing newline characters, aborting.");
+		log(logmsg::debug_warning, L"Command containing newline characters, aborting.");
 		return FZ_REPLY_INTERNALERROR;
 	}
 
@@ -339,7 +344,7 @@ int CSftpControlSocket::AddToStream(std::wstring const& cmd)
 {
 	std::string const str = ConvToServer(cmd);
 	if (str.empty()) {
-		LogMessage(MessageType::Error, _("Could not convert command to server encoding"));
+		log(logmsg::error, _("Could not convert command to server encoding"));
 		return FZ_REPLY_ERROR;
 	}
 
@@ -361,12 +366,7 @@ int CSftpControlSocket::AddToStream(std::string const& cmd)
 
 bool CSftpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotification)
 {
-	if (operations_.empty() || !operations_.back()->waitForAsyncRequest) {
-		LogMessage(MessageType::Debug_Info, L"Not waiting for request reply, ignoring request reply %d", pNotification->GetRequestID());
-		return false;
-	}
-
-	operations_.back()->waitForAsyncRequest = false;
+	log(logmsg::debug_verbose, L"CSftpControlSocket::SetAsyncRequestReply");
 
 	RequestId const requestId = pNotification->GetRequestID();
 	switch(requestId)
@@ -382,7 +382,7 @@ bool CSftpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotifi
 			if (GetCurrentCommandId() != Command::connect ||
 				!currentServer_)
 			{
-				LogMessage(MessageType::Debug_Info, L"SetAsyncRequestReply called to wrong time");
+				log(logmsg::debug_info, L"SetAsyncRequestReply called to wrong time");
 				return false;
 			}
 
@@ -413,7 +413,7 @@ bool CSftpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotifi
 	case reqId_interactiveLogin:
 		{
 			if (operations_.empty() || operations_.back()->opId != Command::connect) {
-				LogMessage(MessageType::Debug_Info, L"No or invalid operation in progress, ignoring request reply %d", pNotification->GetRequestID());
+				log(logmsg::debug_info, L"No or invalid operation in progress, ignoring request reply %d", pNotification->GetRequestID());
 				return false;
 			}
 
@@ -426,14 +426,16 @@ bool CSftpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotifi
 				return false;
 			}
 			std::wstring const& pass = pInteractiveLoginNotification->credentials.GetPass();
-			data.credentials_.SetPass(pass);
+			if (pInteractiveLoginNotification->GetType() != CInteractiveLoginNotification::keyfile) {
+				data.credentials_.SetPass(pass);
+			}
 			std::wstring show = L"Pass: ";
 			show.append(pass.size(), '*');
 			SendCommand(pass, show);
 		}
 		break;
 	default:
-		LogMessage(MessageType::Debug_Warning, L"Unknown async request reply id: %d", requestId);
+		log(logmsg::debug_warning, L"Unknown async request reply id: %d", requestId);
 		return false;
 	}
 
@@ -451,10 +453,10 @@ void CSftpControlSocket::List(CServerPath const& path, std::wstring const& subDi
 	}
 
 	if (newPath.empty()) {
-		LogMessage(MessageType::Status, _("Retrieving directory listing..."));
+		log(logmsg::status, _("Retrieving directory listing..."));
 	}
 	else {
-		LogMessage(MessageType::Status, _("Retrieving directory listing of \"%s\"..."), newPath.GetPath());
+		log(logmsg::status, _("Retrieving directory listing of \"%s\"..."), newPath.GetPath());
 	}
 
 	Push(std::make_unique<CSftpListOpData>(*this, path, subDir, flags));
@@ -480,15 +482,23 @@ void CSftpControlSocket::ChangeDir(CServerPath const& path, std::wstring const&
 void CSftpControlSocket::ProcessReply(int result, std::wstring const& reply)
 {
 	result_ = result;
-	response_ = reply;
+	response_.clear();
 
 	if (operations_.empty()) {
-		LogMessage(MessageType::Debug_Info, L"Skipping reply without active operation.");
+		log(logmsg::debug_info, L"Skipping reply without active operation.");
 		return;
 	}
 
+	if (reply.size() > 65536) {
+		log(fz::logmsg::error, _("Received too long response line, closing connection."));
+		DoClose(FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED);
+		return;
+	}
+
+	response_ = reply;
+
 	auto & data = *operations_.back();
-	LogMessage(MessageType::Debug_Verbose, L"%s::ParseResponse() in state %d", data.name_, data.opState);
+	log(logmsg::debug_verbose, L"%s::ParseResponse() in state %d", data.name_, data.opState);
 	int res = data.ParseResponse();
 	if (res == FZ_REPLY_OK) {
 		ResetOperation(FZ_REPLY_OK);
@@ -518,7 +528,7 @@ void CSftpControlSocket::FileTransfer(std::wstring const& localFile, CServerPath
 
 int CSftpControlSocket::DoClose(int nErrorCode)
 {
-	engine_.GetRateLimiter().RemoveObject(this);
+	remove_bucket();
 
 	if (process_) {
 		process_->kill();
@@ -558,7 +568,7 @@ void CSftpControlSocket::Mkdir(CServerPath const& path)
 	 */
 
 	if (operations_.empty()) {
-		LogMessage(MessageType::Status, _("Creating directory '%s'..."), path.GetPath());
+		log(logmsg::status, _("Creating directory '%s'..."), path.GetPath());
 	}
 
 	auto pData = std::make_unique<CSftpMkdirOpData>(*this);
@@ -571,12 +581,12 @@ std::wstring CSftpControlSocket::QuoteFilename(std::wstring const& filename)
 	return L"\"" + fz::replaced_substrings(filename, L"\"", L"\"\"") + L"\"";
 }
 
-void CSftpControlSocket::Delete(const CServerPath& path, std::deque<std::wstring>&& files)
+void CSftpControlSocket::Delete(const CServerPath& path, std::vector<std::wstring>&& files)
 {
 	// CFileZillaEnginePrivate should have checked this already
 	assert(!files.empty());
 
-	LogMessage(MessageType::Debug_Verbose, L"CSftpControlSocket::Delete");
+	log(logmsg::debug_verbose, L"CSftpControlSocket::Delete");
 	
 	auto pData = std::make_unique<CSftpDeleteOpData>(*this);
 	pData->path_ = path;
@@ -586,7 +596,7 @@ void CSftpControlSocket::Delete(const CServerPath& path, std::deque<std::wstring
 
 void CSftpControlSocket::RemoveDir(CServerPath const& path, std::wstring const& subDir)
 {
-	LogMessage(MessageType::Debug_Verbose, L"CSftpControlSocket::RemoveDir");
+	log(logmsg::debug_verbose, L"CSftpControlSocket::RemoveDir");
 
 	auto pData = std::make_unique<CSftpRemoveDirOpData>(*this);
 	pData->path_ = path;
@@ -596,50 +606,32 @@ void CSftpControlSocket::RemoveDir(CServerPath const& path, std::wstring const&
 
 void CSftpControlSocket::Chmod(CChmodCommand const& command)
 {
-	LogMessage(MessageType::Status, _("Setting permissions of '%s' to '%s'"), command.GetPath().FormatFilename(command.GetFile()), command.GetPermission());
+	log(logmsg::status, _("Setting permissions of '%s' to '%s'"), command.GetPath().FormatFilename(command.GetFile()), command.GetPermission());
 	Push(std::make_unique<CSftpChmodOpData>(*this, command));
 }
 
 void CSftpControlSocket::Rename(CRenameCommand const& command)
 {
-	LogMessage(MessageType::Status, _("Renaming '%s' to '%s'"), command.GetFromPath().FormatFilename(command.GetFromFile()), command.GetToPath().FormatFilename(command.GetToFile()));
+	log(logmsg::status, _("Renaming '%s' to '%s'"), command.GetFromPath().FormatFilename(command.GetFromFile()), command.GetToPath().FormatFilename(command.GetToFile()));
 	Push(std::make_unique<CSftpRenameOpData>(*this, command));
 }
 
-std::wstring CSftpControlSocket::WildcardEscape(std::wstring const& file)
+void CSftpControlSocket::wakeup(fz::direction::type const d)
 {
-	std::wstring ret;
-	// see src/putty/wildcard.c
+	send_event<SftpRateAvailableEvent>(d);
+}
 
-	ret.reserve(file.size());
-	for (size_t i = 0; i < file.size(); ++i) {
-		auto const& c = file[i];
-		switch (c)
-		{
-		case '[':
-		case ']':
-		case '*':
-		case '?':
-		case '\\':
-			ret.push_back('\\');
-			break;
-		default:
-			break;
-		}
-		ret.push_back(c);
+void CSftpControlSocket::OnQuotaRequest(fz::direction::type const d)
+{
+	if (!process_) {
+		return;
 	}
-	return ret;
-}
 
-void CSftpControlSocket::OnRateAvailable(CRateLimiter::rate_direction direction)
-{
-	OnQuotaRequest(direction);
-}
-
-void CSftpControlSocket::OnQuotaRequest(CRateLimiter::rate_direction direction)
-{
-	int64_t bytes = GetAvailableBytes(direction);
-	if (bytes > 0) {
+	size_t bytes = available(d);
+	if (bytes == fz::rate::unlimited) {
+		AddToStream(fz::sprintf("-%d-\n", d));
+	}
+	else if (bytes > 0) {
 		int b;
 		if (bytes > std::numeric_limits<int>::max()) {
 			b = std::numeric_limits<int>::max();
@@ -647,23 +639,18 @@ void CSftpControlSocket::OnQuotaRequest(CRateLimiter::rate_direction direction)
 		else {
 			b = static_cast<int>(bytes);
 		}
-		AddToStream(fz::sprintf("-%d%d,%d\n", direction, b, engine_.GetOptions().GetOptionVal(OPTION_SPEEDLIMIT_INBOUND + static_cast<int>(direction))));
-		UpdateUsage(direction, b);
-	}
-	else if (bytes == 0) {
-		Wait(direction);
-	}
-	else if (bytes < 0) {
-		AddToStream(fz::sprintf("-%d-\n", direction));
+		AddToStream(fz::sprintf("-%d%d,%d\n", d, b, engine_.GetOptions().GetOptionVal(OPTION_SPEEDLIMIT_INBOUND + static_cast<int>(d))));
+		consume(d, static_cast<size_t>(bytes));
 	}
 }
 
 void CSftpControlSocket::operator()(fz::event_base const& ev)
 {
-	if (fz::dispatch<CSftpEvent, CSftpListEvent, CTerminateEvent>(ev, this,
+	if (fz::dispatch<CSftpEvent, CSftpListEvent, CTerminateEvent, SftpRateAvailableEvent>(ev, this,
 		&CSftpControlSocket::OnSftpEvent,
 		&CSftpControlSocket::OnSftpListEvent,
-		&CSftpControlSocket::OnTerminate)) {
+		&CSftpControlSocket::OnTerminate,
+		&CSftpControlSocket::OnQuotaRequest)) {
 		return;
 	}
 
