@@ -3,6 +3,7 @@
 
 class CStorjControlSocket;
 
+#include <libfilezilla/buffer.hpp>
 #include <libfilezilla/thread_pool.hpp>
 
 namespace fz {
@@ -19,14 +20,19 @@ public:
 
 protected:
 
+	bool readFromProcess(std::wstring & error, bool eof_is_error);
 	std::wstring ReadLine(std::wstring &error);
 
 	void entry();
 
+	void processEvent(storjEvent eventType, std::wstring & error);
+
 	fz::process& process_;
 	CStorjControlSocket& owner_;
 
 	fz::async_task thread_;
+
+	fz::buffer recv_buffer_;
 };
 
 #endif
