@@ -1,10 +1,16 @@
-#ifndef __FILEZILLAENGINE_H__
-#define __FILEZILLAENGINE_H__
+#ifndef FILEZILLA_ENGINE_ENGINE_HEADER
+#define FILEZILLA_ENGINE_ENGINE_HEADER
 
 #include "commands.h"
 
+#include <notification.h>
+
+class CAsyncRequestNotification;
 class CFileZillaEngineContext;
 class CFileZillaEnginePrivate;
+class CNotification;
+class EngineNotificationHandler;
+
 class CFileZillaEngine final
 {
 public:
