@@ -1,6 +1,9 @@
 #ifndef FILEZILLA_ENGINE_COMMANDS_HEADER
 #define FILEZILLA_ENGINE_COMMANDS_HEADER
 
+#include "server.h"
+#include "serverpath.h"
+
 #include <libfilezilla/uri.hpp>
 
 // See below for actual commands and their parameters
@@ -23,7 +26,10 @@ enum class Command
 	httprequest, // Only used by HTTP protocol
 
 	// Only used internally
+	lookup,
 	cwd,
+	common_private1, // Internal commands common to multiple protocols
+	common_private2,
 	private1,
 	private2,
 	private3,
@@ -44,7 +50,7 @@ enum class Command
 #define FZ_REPLY_NOTCONNECTED	(0x0020 | FZ_REPLY_ERROR)
 #define FZ_REPLY_DISCONNECTED	(0x0040)
 #define FZ_REPLY_INTERNALERROR	(0x0080 | FZ_REPLY_ERROR) // If you get this reply, the error description will be
-														  // given by the last Debug_Warning log message. This
+														  // given by the last debug_warning log message. This
 														  // should not happen unless there is a bug in FileZilla 3.
 #define FZ_REPLY_BUSY			(0x0100 | FZ_REPLY_ERROR)
 #define FZ_REPLY_ALREADYCONNECTED	(0x0200 | FZ_REPLY_ERROR) // Will be returned by connect if already connected
@@ -55,6 +61,7 @@ enum class Command
 #define FZ_REPLY_LINKNOTDIR		(0x4000 | FZ_REPLY_ERROR)
 
 #define FZ_REPLY_CONTINUE 0x8000 // Used internally
+#define FZ_REPLY_ERROR_NOTFOUND (0x10000 | FZ_REPLY_ERROR) // Used internally
 
 // --------------- //
 // Actual commands //
@@ -100,15 +107,17 @@ class CBasicCommand final : public CCommandHelper<CBasicCommand<id>, id>
 class CConnectCommand final : public CCommandHelper<CConnectCommand, Command::connect>
 {
 public:
-	explicit CConnectCommand(CServer const& server, Credentials const& credentials, bool retry_conncting = true);
+	explicit CConnectCommand(CServer const& server, ServerHandle const& handle, Credentials const& credentials, bool retry_conncting = true);
 
 	CServer const& GetServer() const { return server_; }
+	ServerHandle const& GetHandle() const { return handle_; }
 	Credentials const& GetCredentials() const { return credentials_; }
 	bool RetryConnecting() const { return retry_connecting_; }
 
 	virtual bool valid() const override;
 protected:
 	CServer const server_;
+	ServerHandle const handle_;
 	Credentials const credentials_;
 	bool const retry_connecting_;
 };
@@ -217,16 +226,16 @@ protected:
 class CDeleteCommand final : public CCommandHelper<CDeleteCommand, Command::del>
 {
 public:
-	CDeleteCommand(CServerPath const& path, std::deque<std::wstring> && files);
+	CDeleteCommand(CServerPath const& path, std::vector<std::wstring> && files);
 
 	CServerPath GetPath() const { return m_path; }
-	const std::deque<std::wstring>& GetFiles() const { return m_files; }
-	std::deque<std::wstring>&& ExtractFiles() { return std::move(m_files); }
+	const std::vector<std::wstring>& GetFiles() const { return files_; }
+	std::vector<std::wstring>&& ExtractFiles() { return std::move(files_); }
 
 	bool valid() const { return !GetPath().empty() && !GetFiles().empty(); }
 protected:
 	CServerPath const m_path;
-	std::deque<std::wstring> m_files;
+	std::vector<std::wstring> files_;
 };
 
 class CRemoveDirCommand final : public CCommandHelper<CRemoveDirCommand, Command::removedir>
