@@ -1,7 +1,8 @@
 #ifndef FILEZILLA_ENGINE_EXTERNALIPRESOLVER_HEADER
 #define FILEZILLA_ENGINE_EXTERNALIPRESOLVER_HEADER
 
-#include "socket.h"
+#include <libfilezilla/buffer.hpp>
+#include <libfilezilla/socket.hpp>
 
 struct external_ip_resolve_event_type;
 typedef fz::simple_event<external_ip_resolve_event_type> CExternalIPResolveEvent;
@@ -35,31 +36,26 @@ protected:
 
 	std::string m_data;
 
-	fz::socket *socket_{};
+	std::unique_ptr<fz::socket> socket_;
 
 	virtual void operator()(fz::event_base const& ev);
 	void OnSocketEvent(fz::socket_event_source* source, fz::socket_event_flag t, int error);
 
 	void OnConnect(int error);
-	void OnClose();
 	void OnReceive();
 	void OnHeader();
-	void OnData(char* buffer, unsigned int len);
+	void OnData(unsigned char* buffer, size_t len);
 	void OnChunkedData();
 	void OnSend();
 
 	std::string m_sendBuffer;
 
-	char* m_pRecvBuffer{};
-	unsigned int m_recvBufferPos{};
-
-	static const unsigned int m_recvBufferLen = 4096;
+	fz::buffer recvBuffer_;
 
 	// HTTP data
-	void ResetHttpData(bool resetRedirectCount);
+	void ResetHttpData();
 	bool m_gotHeader{};
 	int m_responseCode{};
-	std::string m_responseString;
 	std::wstring m_location;
 	int m_redirectCount{};
 
@@ -69,15 +65,13 @@ protected:
 		unknown
 	};
 
-	transferEncodings m_transferEncoding;
+	transferEncodings m_transferEncoding{unknown};
 
 	struct t_chunkData {
 		bool getTrailer{};
 		bool terminateChunk{};
-		int64_t size{};
+		uint64_t size{};
 	} m_chunkData;
-
-	bool m_finished{};
 };
 
 #endif
