@@ -88,6 +88,7 @@ enum engineOptions
 
 namespace pugi {
 class xml_document;
+class xml_node;
 }
 
 class COptionsBase
@@ -100,7 +101,8 @@ public:
 
 	virtual bool SetOption(unsigned int nID, int value) = 0;
 	virtual bool SetOption(unsigned int nID, std::wstring const& value) = 0;
-	virtual bool SetOptionXml(unsigned int nID, std::unique_ptr<pugi::xml_document> const& value) = 0;
+	virtual bool SetOptionXml(unsigned int nID, pugi::xml_node const& value) = 0;
+	virtual bool SetOptionXml(unsigned int nID, pugi::xml_document const& value) = 0;
 };
 
 #endif
