@@ -1,6 +1,10 @@
 #ifndef FILEZILLA_ENGINE_SERVER_HEADER
 #define FILEZILLA_ENGINE_SERVER_HEADER
 
+#include <map>
+#include <memory>
+#include <vector>
+
 enum ServerProtocol
 {
 	// Never change any existing values or user's saved sites will become
@@ -34,7 +38,11 @@ enum ServerProtocol
 
 	B2,
 
-	MAX_VALUE = B2
+	BOX,
+
+	INSECURE_WEBDAV,
+
+	MAX_VALUE = INSECURE_WEBDAV
 };
 
 enum ServerType
@@ -89,9 +97,21 @@ enum class ProtocolFeature
 	PostLoginCommands,
 	S3Lifecycle,
 	RecursiveDelete,
-	ServerAssignedHome
+	ServerAssignedHome,
+	TemporaryUrl,
+	S3Sse,
+	Security // Encryption, integrity protection and authentication
 };
 
+enum class CaseSensitivity
+{
+	unspecified,
+	yes,
+	no
+};
+
+CaseSensitivity GetCaseSensitivity(ServerProtocol protocol);
+
 class Credentials;
 class CServerPath;
 class CServer final
@@ -102,6 +122,12 @@ public:
 	CServer() = default;
 	CServer(ServerProtocol protocol, ServerType type, std::wstring const& host, unsigned int);
 
+	CServer(CServer const&) = default;
+	CServer& operator=(CServer const&) = default;
+
+	CServer(CServer &&) noexcept = default;
+	CServer& operator=(CServer &&) noexcept = default;
+
 	void clear();
 
 	void SetType(ServerType type);
@@ -121,11 +147,18 @@ public:
 
 	void SetUser(std::wstring const& user);
 
-	CServer& operator=(const CServer &op);
 	bool operator==(const CServer &op) const;
 	bool operator<(const CServer &op) const;
 	bool operator!=(const CServer &op) const;
 
+	// Returns whether the argument refers to the same resource.
+	// Compares things like protocol and hostname, but excludes things like the name or the timezone offset.
+	bool SameResource(CServer const& other) const;
+
+	// Stricter than SameResource, also compares things like the encoding.
+	// Can be used as key comparator in content caches
+	bool SameContent(CServer const& other) const;
+
 	bool SetTimezoneOffset(int minutes);
 	void SetPasvMode(PasvMode pasvMode);
 	void MaximumMultipleConnections(int maximum);
@@ -151,17 +184,14 @@ public:
 	bool HasFeature(ProtocolFeature const feature) const;
 	static bool ProtocolHasFeature(ServerProtocol const protocol, ProtocolFeature const feature);
 
+	CaseSensitivity GetCaseSensitivity() const;
+
 	// These commands will be executed after a successful login.
 	std::vector<std::wstring> const& GetPostLoginCommands() const { return m_postLoginCommands; }
 	bool SetPostLoginCommands(std::vector<std::wstring> const& postLoginCommands);
 
 	void SetBypassProxy(bool val);
 
-	// Abstract server name.
-	// Not compared in ==, < and related operators
-	void SetName(std::wstring const& name) { m_name = name; }
-	std::wstring GetName() const { return m_name; }
-
 	static std::wstring GetNameFromServerType(ServerType type);
 	static ServerType GetServerTypeFromName(std::wstring const& name);
 
@@ -169,9 +199,11 @@ public:
 	explicit operator bool() const { return !empty(); }
 
 	void ClearExtraParameters();
-	std::wstring GetExtraParameter(std::string const& name) const;
-	std::map<std::string, std::wstring> const& GetExtraParameters() const;
-	void SetExtraParameter(std::string const& name, std::wstring const& value);
+	std::wstring GetExtraParameter(std::string_view const& name) const;
+	std::map<std::string, std::wstring, std::less<>> const& GetExtraParameters() const;
+	bool HasExtraParameter(std::string_view const& name) const;
+	void SetExtraParameter(std::string_view const& name, std::wstring const& value);
+	void ClearExtraParameter(std::string_view const& name);
 
 protected:
 	ServerProtocol m_protocol{UNKNOWN};
@@ -184,12 +216,11 @@ protected:
 	int m_maximumMultipleConnections{};
 	CharsetEncoding m_encodingType{ENCODING_AUTO};
 	std::wstring m_customEncoding;
-	std::wstring m_name;
 
 	std::vector<std::wstring> m_postLoginCommands;
 	bool m_bypassProxy{};
 
-	std::map<std::string, std::wstring> extraParameters_;
+	std::map<std::string, std::wstring, std::less<>> extraParameters_;
 };
 
 
@@ -216,6 +247,7 @@ namespace ParameterSection {
 		user,
 		credentials,
 		extra,
+		custom,
 
 		section_count
 	};
@@ -263,13 +295,23 @@ public:
 	std::wstring keyFile_;
 
 	void ClearExtraParameters();
-	std::wstring GetExtraParameter(std::string const& name) const;
-	std::map<std::string, std::wstring> const& GetExtraParameters() const;
-	void SetExtraParameter(ServerProtocol protocol, std::string const& name, std::wstring const& value);
+	std::wstring GetExtraParameter(std::string_view const& name) const;
+	std::map<std::string, std::wstring, std::less<>> const& GetExtraParameters() const;
+	bool HasExtraParameter(std::string_view const& name) const;
+	void SetExtraParameter(ServerProtocol protocol, std::string_view const& name, std::wstring const& value);
 
 protected:
 	std::wstring password_;
-	std::map<std::string, std::wstring> extraParameters_;
+	std::map<std::string, std::wstring, std::less<>> extraParameters_;
 };
 
+struct ServerHandleData {
+protected:
+	ServerHandleData() = default;
+	virtual ~ServerHandleData() = default;
+	ServerHandleData(ServerHandleData const&) = default;
+	ServerHandleData& operator=(ServerHandleData const&) = default;
+};
+typedef std::weak_ptr<ServerHandleData> ServerHandle;
+
 #endif
