@@ -12,20 +12,19 @@
 #include "msgbox.h"
 #include "themeprovider.h"
 #include "wxfilesystem_blob_handler.h"
-
 #include <libfilezilla/local_filesys.hpp>
+#include <libfilezilla/translate.hpp>
 
 #include <wx/evtloop.h>
 
-#ifdef __WXMSW__
-#include <wx/dynlib.h>
-#endif
 #ifdef WITH_LIBDBUS
 #include <../dbus/session_manager.h>
 #endif
 
 #if defined(__WXMAC__) || defined(__UNIX__)
 #include <wx/stdpaths.h>
+#elif defined(__WXMSW__)
+#include <shobjidl.h>
 #endif
 
 #include "locale_initializer.h"
@@ -34,11 +33,6 @@
 #include "osx_sandbox_userdirs.h"
 #endif
 
-#ifdef ENABLE_BINRELOC
-	#define BR_PTHREADS 0
-	#include "prefix.h"
-#endif
-
 #ifndef __WXGTK__
 IMPLEMENT_APP(CFileZillaApp)
 #else
@@ -49,6 +43,81 @@ IMPLEMENT_APP_NO_MAIN(CFileZillaApp)
   #error Please build wxWidgets with support for positional arguments.
 #endif
 
+namespace {
+std::wstring GetEnv(std::string const& name)
+{
+	std::wstring ret;
+
+	char const* const p = getenv(name.c_str());
+	if (p && *p) {
+		ret = fz::to_wstring(std::string(p));
+	}
+
+	return ret;
+}
+
+#if FZ_WINDOWS
+std::wstring const PATH_SEP = L";";
+#else
+std::wstring const PATH_SEP = L":";
+#endif
+}
+
+// If non-empty, always terminated by a separator
+std::wstring GetOwnExecutableDir()
+{
+#ifdef FZ_WINDOWS
+	// Add executable path
+	std::wstring path;
+	path.resize(4095);
+	DWORD res;
+	while (true) {
+		res = GetModuleFileNameW(0, &path[0], path.size() - 1);
+		if (!res) {
+			// Failure
+			return std::wstring();
+		}
+
+		if (res < path.size() - 1) {
+			path.resize(res);
+			break;
+		}
+
+		path.resize(path.size() * 2 + 1);
+	}
+	size_t pos = path.find_last_of(L"\\/");
+	if (pos != std::wstring::npos) {
+		return path.substr(0, pos + 1);
+	}
+#elif defined(FZ_MAC)
+	// Fixme: Remove wxDependency and move entire function to libfilezilla
+	std::wstring executable = wxStandardPaths::Get().GetExecutablePath().ToStdWstring();
+	size_t pos = executable.rfind('/');
+	if (pos != std::wstring::npos) {
+		return executable.substr(0, pos + 1);
+	}
+#else
+	std::string path;
+	path.resize(4095);
+	while (true) {
+		int res = readlink("/proc/self/exe", &path[0], path.size());
+		if (res < 0) {
+			return std::wstring();
+		}
+		if (static_cast<size_t>(res) < path.size()) {
+			path.resize(res);
+			break;
+		}
+		path.resize(path.size() * 2 + 1);
+	}
+	size_t pos = path.rfind('/');
+	if (pos != std::wstring::npos) {
+		return fz::to_wstring(path.substr(0, pos + 1));
+	}
+#endif
+	return std::wstring();
+}
+
 CFileZillaApp::CFileZillaApp()
 {
 	m_profile_start = fz::monotonic_clock::now();
@@ -60,56 +129,6 @@ CFileZillaApp::~CFileZillaApp()
 	COptions::Destroy();
 }
 
-#ifdef __WXMSW__
-namespace {
-extern "C"
-{
-	typedef HRESULT(WINAPI *t_SetCurrentProcessExplicitAppUserModelID)(PCWSTR AppID);
-}
-
-static void SetAppId()
-{
-	wxDynamicLibrary dll;
-	if (!dll.Load(_T("shell32.dll"))) {
-		return;
-	}
-
-	if (!dll.HasSymbol(_T("SetCurrentProcessExplicitAppUserModelID"))) {
-		return;
-	}
-
-	t_SetCurrentProcessExplicitAppUserModelID pSetCurrentProcessExplicitAppUserModelID =
-		(t_SetCurrentProcessExplicitAppUserModelID)dll.GetSymbol(_T("SetCurrentProcessExplicitAppUserModelID"));
-
-	if (!pSetCurrentProcessExplicitAppUserModelID) {
-		return;
-	}
-
-	pSetCurrentProcessExplicitAppUserModelID(_T("FileZilla.Client.AppID"));
-}
-
-static bool InitWinsock()
-{
-	WSADATA d{};
-	int res = WSAStartup((2 << 8) | 8, &d);
-	if (res != 0) {
-		int err = WSAGetLastError();
-		wxString msg = wxString::Format(_("Could not initialize Winsock (%d): %s"), err, wxSysErrorMsg(err));
-		wxMessageBoxEx(msg, _("Failed to initialize networking"), wxICON_EXCLAMATION);
-		return false;
-	}
-
-	return true;
-}
-
-static void UninitWinsock()
-{
-	WSACleanup();
-}
-}
-
-#endif //__WXMSW__
-
 void CFileZillaApp::InitLocale()
 {
 	wxString language = COptions::Get()->GetOption(OPTION_LANGUAGE);
@@ -193,14 +212,15 @@ bool CFileZillaApp::OnInit()
 {
 	AddStartupProfileRecord("CFileZillaApp::OnInit()");
 
+	// Turn off idle events, we don't need them
+	wxIdleEvent::SetMode(wxIDLE_PROCESS_SPECIFIED);
+
+	wxUpdateUIEvent::SetMode(wxUPDATE_UI_PROCESS_SPECIFIED);
+
 	fz::set_translators(translator, translator_pf);
 
 #ifdef __WXMSW__
-	if (!InitWinsock()) {
-		return false;
-	}
-
-	SetAppId();
+	SetCurrentProcessExplicitAppUserModelID(L"FileZilla.Client.AppID");
 #endif
 
 	//wxSystemOptions is slow, if a value is not set, it keeps querying the environment
@@ -235,9 +255,9 @@ bool CFileZillaApp::OnInit()
 
 #if USE_MAC_SANDBOX
 	// Set PUTTYDIR so that fzsftp uses the sandboxed home to put settings.
-	wxString home;
-	if (wxGetEnv(L"HOME", &home) && !home.empty()) {
-		if (home[home.size() - 1] != '/') {
+	std::wstring home = GetEnv("Home");
+	if (!home.empty()) {
+		if (home.back() != '/') {
 			home += '/';
 		}
 		wxSetEnv("PUTTYDIR", home + L".config/putty");
@@ -261,8 +281,8 @@ DO NOT complain about it\r\n\
 USE AT OWN RISK"), _T("Important Information"));
 	}
 	else {
-		wxString v;
-		if (!wxGetEnv(_T("FZDEBUG"), &v) || v != _T("1")) {
+		std::wstring v = GetEnv("FZDEBUG");
+		if (v != L"1") {
 			COptions::Get()->SetOption(OPTION_LOGGING_DEBUGLEVEL, 0);
 			COptions::Get()->SetOption(OPTION_LOGGING_RAWLISTING, 0);
 		}
@@ -274,22 +294,16 @@ USE AT OWN RISK"), _T("Important Information"));
 		return false;
 	}
 
+	themeProvider_ = std::make_unique<CThemeProvider>();
 	CheckExistsFzsftp();
 #if ENABLE_STORJ
 	CheckExistsFzstorj();
 #endif
 
-	// Turn off idle events, we don't need them
-	wxIdleEvent::SetMode(wxIDLE_PROCESS_SPECIFIED);
-
-	wxUpdateUIEvent::SetMode(wxUPDATE_UI_PROCESS_SPECIFIED);
-
 #ifdef WITH_LIBDBUS
 	CSessionManager::Init();
 #endif
 
-	themeProvider_ = std::make_unique<CThemeProvider>();
-
 	// Load the text wrapping engine
 	m_pWrapEngine = std::make_unique<CWrapEngine>();
 	m_pWrapEngine->LoadCache();
@@ -298,14 +312,14 @@ USE AT OWN RISK"), _T("Important Information"));
 #ifdef USE_MAC_SANDBOX
 	OSXSandboxUserdirs::Get().Load();
 
-    if (OSXSandboxUserdirs::Get().GetDirs().empty()) {
+	if (OSXSandboxUserdirs::Get().GetDirs().empty()) {
 		CWelcomeDialog welcome;
 		welcome.Run(nullptr, false);
 		welcome_skip = true;
 
-        OSXSandboxUserdirsDialog dlg;
-        dlg.Run(nullptr, true);
-	    if (OSXSandboxUserdirs::Get().GetDirs().empty()) {
+		OSXSandboxUserdirsDialog dlg;
+		dlg.Run(nullptr, true);
+		if (OSXSandboxUserdirs::Get().GetDirs().empty()) {
 			return false;
 		}
     }
@@ -345,9 +359,6 @@ int CFileZillaApp::OnExit()
 		_exit(0);
 	}
 
-#ifdef __WXMSW__
-	UninitWinsock();
-#endif
 	return wxApp::OnExit();
 }
 
@@ -356,7 +367,7 @@ bool CFileZillaApp::FileExists(std::wstring const& file) const
 	return fz::local_filesys::get_file_type(fz::to_native(file), true) == fz::local_filesys::file;
 }
 
-CLocalPath CFileZillaApp::GetDataDir(std::wstring fileToFind) const
+CLocalPath CFileZillaApp::GetDataDir(std::vector<std::wstring> const& fileToFind, std::wstring const& prefixSub, bool searchSelfDir) const
 {
 	/*
 	 * Finding the resources in all cases is a difficult task,
@@ -369,109 +380,99 @@ CLocalPath CFileZillaApp::GetDataDir(std::wstring fileToFind) const
 	 * At least on OS X it's simple: All inside application bundle.
 	 */
 
+	CLocalPath ret;
+
+	auto testPath = [&](std::wstring const& path) {
+		ret = CLocalPath(path);
+		if (ret.empty()) {
+			return false;
+		}
+
+		for (auto const& file : fileToFind) {
+			if (FileExists(ret.GetPath() + file)) {
+				return true;
+			}
+		}
+		return false;
+	};
+
 #ifdef __WXMAC__
-	CLocalPath path(wxStandardPaths::Get().GetDataDir().ToStdWstring());
-	if (FileExists(path.GetPath() + fileToFind)) {
-		return path;
+	(void)prefixSub;
+
+	if (searchSelfDir && testPath(wxStandardPaths::Get().GetDataDir().ToStdWstring())) {
+		return ret;
 	}
 
 	return CLocalPath();
 #else
 
-	wxPathList pathList;
-	// FIXME: --datadir cmdline
-
 	// First try the user specified data dir.
-	pathList.AddEnvList(_T("FZ_DATADIR"));
-
-	// Next try the current path and the current executable path.
-	// Without this, running development versions would be difficult.
-	pathList.Add(wxGetCwd());
-
-#ifdef ENABLE_BINRELOC
-	const char* path = SELFPATH;
-	if (path && *path) {
-		wxString datadir(SELFPATH , *wxConvCurrent);
-		wxFileName fn(datadir);
-		datadir = fn.GetPath();
-		if (!datadir.empty())
-			pathList.Add(datadir);
-
+	if (searchSelfDir) {
+		if (testPath(GetEnv("FZ_DATADIR"))) {
+			return ret;
+		}
 	}
-	path = DATADIR;
-	if (path && *path) {
-		wxString datadir(DATADIR, *wxConvCurrent);
-		if (!datadir.empty())
-			pathList.Add(datadir);
+
+	std::wstring selfDir = GetOwnExecutableDir();
+	if (!selfDir.empty()) {
+		if (searchSelfDir && testPath(selfDir)) {
+			return ret;
+		}
+
+		if (!prefixSub.empty() && selfDir.size() > 5 && fz::ends_with(selfDir, std::wstring(L"/bin/"))) {
+			std::wstring path = selfDir.substr(0, selfDir.size() - 4) + prefixSub + L"/";
+			if (testPath(path)) {
+				return ret;
+			}
+		}
+
+		// Development paths
+		if (searchSelfDir && selfDir.size() > 7 && fz::ends_with(selfDir, std::wstring(L"/.libs/"))) {
+			std::wstring path = selfDir.substr(0, selfDir.size() - 6);
+			if (FileExists(path + L"Makefile")) {
+				if (testPath(path)) {
+					return ret;
+				}
+			}
+		}
 	}
-#elif defined __WXMSW__
-	wxChar path[1024];
-	int res = GetModuleFileName(0, path, 1000);
-	if (res > 0 && res < 1000) {
-		wxFileName fn(path);
-		pathList.Add(fn.GetPath(wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR));
-	}
-#endif //ENABLE_BINRELOC and __WXMSW__ blocks
 
 	// Now scan through the path
-	pathList.AddEnvList(_T("PATH"));
+	if (!prefixSub.empty()) {
+		std::wstring path = GetEnv("PATH");
+		auto const segments = fz::strtok(path, PATH_SEP);
 
-#ifndef __WXMSW__
-	// Try some common paths
-	pathList.Add(_T("/usr/share/filezilla"));
-	pathList.Add(_T("/usr/local/share/filezilla"));
+		for (auto const& segment : segments) {
+			auto const cur = CLocalPath(segment).GetPath();
+			if (cur.size() > 5 && fz::ends_with(cur, std::wstring(L"/bin/"))) {
+				std::wstring path = cur.substr(0, cur.size() - 4) + prefixSub + L"/";
+				if (testPath(path)) {
+					return ret;
+				}
+			}
+		}
+	}
+
+	ret.clear();
+	return ret;
 #endif
-
-	// For each path, check for the resources
-	wxPathList::const_iterator node;
-	for (node = pathList.begin(); node != pathList.end(); ++node) {
-		auto const cur = CLocalPath(node->ToStdWstring()).GetPath();
-		if (FileExists(cur + fileToFind)) {
-			return CLocalPath(cur);
-		}
-		if (FileExists(cur + _T("share/filezilla/") + fileToFind)) {
-			return CLocalPath(cur + _T("/share/filezilla"));
-		}
-		if (FileExists(cur + _T("filezilla/") + fileToFind)) {
-			return CLocalPath(cur + _T("filezilla"));
-		}
-	}
-
-	for (node = pathList.begin(); node != pathList.end(); ++node) {
-		auto const cur = CLocalPath(node->ToStdWstring()).GetPath();
-		if (FileExists(cur + _T("../") + fileToFind)) {
-			return CLocalPath(cur + _T("/.."));
-		}
-		if (FileExists(cur + _T("../share/filezilla/") + fileToFind)) {
-			return CLocalPath(cur + _T("../share/filezilla"));
-		}
-	}
-
-	for (node = pathList.begin(); node != pathList.end(); ++node) {
-		auto const cur = CLocalPath(node->ToStdWstring()).GetPath();
-		if (FileExists(cur + _T("../../") + fileToFind)) {
-			return CLocalPath(cur + _T("../.."));
-		}
-	}
-
-	return CLocalPath();
-#endif //__WXMAC__
 }
 
 bool CFileZillaApp::LoadResourceFiles()
 {
 	AddStartupProfileRecord("CFileZillaApp::LoadResourceFiles");
-	m_resourceDir = GetDataDir(_T("resources/defaultfilters.xml"));
+	m_resourceDir = GetDataDir({L"resources/defaultfilters.xml"}, L"share/filezilla");
 
 	wxImage::AddHandler(new wxPNGHandler());
 
 	if (m_resourceDir.empty()) {
-		wxString msg = _("Could not find the resource files for FileZilla, closing FileZilla.\nYou can set the data directory of FileZilla using the '--datadir <custompath>' commandline option or by setting the FZ_DATADIR environment variable.");
+		wxString msg = _("Could not find the resource files for FileZilla, closing FileZilla.\nYou can specify the data directory of FileZilla by setting the FZ_DATADIR environment variable.");
 		wxMessageBoxEx(msg, _("FileZilla Error"), wxOK | wxICON_ERROR);
 		return false;
 	}
-
-	m_resourceDir.AddSegment(_T("resources"));
+	else {
+		m_resourceDir.AddSegment(_T("resources"));
+	}
 
 	// Useful for XRC files with embedded image data.
 	wxFileSystem::AddHandler(new wxFileSystemBlobHandler);
@@ -484,9 +485,9 @@ bool CFileZillaApp::InitDefaultsDir()
 	AddStartupProfileRecord("InitDefaultsDir");
 #ifdef __WXGTK__
 	m_defaultsDir = COptions::GetUnadjustedSettingsDir();
-	if (m_defaultsDir.empty() || !wxFileName::FileExists(m_defaultsDir.GetPath() + _T("fzdefaults.xml"))) {
-		if (wxFileName::FileExists(_T("/etc/filezilla/fzdefaults.xml"))) {
-			m_defaultsDir.SetPath(_T("/etc/filezilla"));
+	if (m_defaultsDir.empty() || !FileExists(m_defaultsDir.GetPath() + L"fzdefaults.xml")) {
+		if (FileExists(L"/etc/filezilla/fzdefaults.xml")) {
+			m_defaultsDir.SetPath(L"/etc/filezilla");
 		}
 		else {
 			m_defaultsDir.clear();
@@ -495,7 +496,7 @@ bool CFileZillaApp::InitDefaultsDir()
 
 #endif
 	if (m_defaultsDir.empty()) {
-		m_defaultsDir = GetDataDir(_T("fzdefaults.xml"));
+		m_defaultsDir = GetDataDir({L"fzdefaults.xml"}, L"share/filezilla");
 	}
 
 	return !m_defaultsDir.empty();
@@ -504,24 +505,15 @@ bool CFileZillaApp::InitDefaultsDir()
 bool CFileZillaApp::LoadLocales()
 {
 	AddStartupProfileRecord("CFileZillaApp::LoadLocales");
-#ifndef __WXMAC__
-	m_localesDir = GetDataDir(_T("../locale/de/filezilla.mo"));
-	if (m_localesDir.empty()) {
-		m_localesDir = GetDataDir(_T("../locale/de/LC_MESSAGES/filezilla.mo"));
-	}
+	m_localesDir = GetDataDir({L"locales/de/filezilla.mo"}, std::wstring());
 	if (!m_localesDir.empty()) {
-		m_localesDir.ChangePath( _T("../locale") );
+		m_localesDir.AddSegment(_T("locales"));
 	}
+#ifndef __WXMAC__
 	else {
-		m_localesDir = GetDataDir(_T("locales/de/filezilla.mo"));
-		if (!m_localesDir.empty()) {
-			m_localesDir.AddSegment(_T("locales"));
-		}
+		m_localesDir = GetDataDir({L"de/filezilla.mo", L"de/LC_MESSAGES/filezilla.mo"}, L"share/locale", false);
 	}
-#else
-	m_localesDir.SetPath(wxStandardPaths::Get().GetDataDir().ToStdWstring() + _T("/locales"));
 #endif
-
 	if (!m_localesDir.empty()) {
 		wxLocale::AddCatalogLookupPathPrefix(m_localesDir.GetPath());
 	}
@@ -545,6 +537,7 @@ bool CFileZillaApp::SetLocale(int language)
 	if (!pLocale->AddCatalog(_T("filezilla")) && language != wxLANGUAGE_DEFAULT) {
 		return false;
 	}
+	pLocale->AddCatalog(_T("libfilezilla"));
 
 	if (m_pLocale) {
 		// Now unload old locale
@@ -561,6 +554,7 @@ bool CFileZillaApp::SetLocale(int language)
 		else if (!pLocale->AddCatalog(_T("filezilla")) && language != wxLANGUAGE_DEFAULT) {
 			return false;
 		}
+		pLocale->AddCatalog(_T("libfilezilla"));
 	}
 	m_pLocale = std::move(pLocale);
 
@@ -611,126 +605,103 @@ CWrapEngine* CFileZillaApp::GetWrapEngine()
 void CFileZillaApp::CheckExistsFzsftp()
 {
 	AddStartupProfileRecord("FileZillaApp::CheckExistsFzsftp");
-	CheckExistsTool(L"fzsftp", {L"/src/putty", L"/putty"}, L"FZ_FZSFTP", OPTION_FZSFTP_EXECUTABLE, _("SFTP support"));
+	CheckExistsTool(L"fzsftp", L"../putty/", "FZ_FZSFTP", OPTION_FZSFTP_EXECUTABLE, fztranslate("SFTP support"));
 }
 
 #if ENABLE_STORJ
 void CFileZillaApp::CheckExistsFzstorj()
 {
 	AddStartupProfileRecord("FileZillaApp::CheckExistsFzstorj");
-	CheckExistsTool(L"fzstorj", {L"/src/storj", L"/storj"}, L"FZ_FZSTORJ", OPTION_FZSTORJ_EXECUTABLE, _("Storj support"));
+	CheckExistsTool(L"fzstorj", L"../storj/", "FZ_FZSTORJ", OPTION_FZSTORJ_EXECUTABLE, fztranslate("Storj support"));
 }
 #endif
 
-void CFileZillaApp::CheckExistsTool(std::wstring const& tool, std::vector<std::wstring> const& searchPaths, std::wstring const& env, int setting, wxString const& description)
+void CFileZillaApp::CheckExistsTool(std::wstring const& tool, std::wstring const& buildRelPath, std::string const& env, int setting, std::wstring const& description)
 {
 	// Get the correct path to the specified tool
 
-#ifdef __WXMAC__
-	wxString executable = wxStandardPaths::Get().GetExecutablePath();
-	int pos = executable.Find('/', true);
-	if (pos != -1) {
-		executable = executable.Left(pos);
-	}
-	executable += _T("/") + tool;
-	if (!wxFileName::FileExists(executable.ToStdWstring())) {
-		wxMessageBoxEx(wxString::Format(_("%s could not be found. Without this component of FileZilla, %s will not work.\n\nPlease download FileZilla again. If this problem persists, please submit a bug report."), executable, description),
-			_("File not found"), wxICON_ERROR);
-		executable.clear();
-	}
+	bool found = false;
+	std::wstring executable;
 
+	std::wstring program = tool;
+#ifdef __WXMAC__
+	(void)buildRelPath;
+
+	// On Mac we only look inside the bundle
+	std::wstring path = GetOwnExecutableDir();
+	if (!path.empty()) {
+		executable = path + '/' + tool;
+		if (FileExists(executable)) {
+			found = true;
+		}
+	}
 #else
 
-	wxString program = tool;
 #ifdef __WXMSW__
-	program += _T(".exe");
+	program += L".exe";
 #endif
 
-	bool found = false;
-
 	// First check the given environment variable
-	wxString executable;
-	if (wxGetEnv(env, &executable)) {
-		if (wxFileName::FileExists(executable.ToStdWstring())) {
+	executable = GetEnv(env);
+	if (!executable.empty()) {
+		if (FileExists(executable)) {
 			found = true;
 		}
 	}
 
 	if (!found) {
-		wxPathList pathList;
-
-		// Add current working directory
-		const wxString &cwd = wxGetCwd();
-		pathList.Add(cwd);
-#ifdef __WXMSW__
-
-		// Add executable path
-		wxChar modulePath[1000];
-		DWORD len = GetModuleFileName(0, modulePath, 999);
-		if (len) {
-			modulePath[len] = 0;
-			wxString path(modulePath);
-			int pos = path.Find('\\', true);
-			if (pos != -1) {
-				path = path.Left(pos);
-				pathList.Add(path);
-			}
-		}
-#endif
-
-		// Add a few paths relative to the current working directory
-		pathList.Add(cwd + _T("/bin"));
-		for (auto const& path : searchPaths) {
-			pathList.Add(cwd + path);
-		}
-
-		executable = pathList.FindAbsoluteValidPath(program);
-		if (!executable.empty()) {
-			found = true;
-		}
-	}
-
-#ifdef __UNIX__
-	if (!found) {
-		const wxString prefix = ((const wxStandardPaths&)wxStandardPaths::Get()).GetInstallPrefix();
-		if (prefix != _T("/usr/local")) {
-			// /usr/local is the fallback value. /usr/local/bin is most likely in the PATH
-			// environment variable already so we don't have to check it. Furthermore, other
-			// directories might be listed before it (For example a developer's own
-			// application prefix)
-			wxFileName fn(prefix + _T("/bin/"), program);
-			fn.Normalize();
-			if (fn.FileExists()) {
-				executable = fn.GetFullPath();
+		std::wstring path = GetOwnExecutableDir();
+		if (!path.empty()) {
+			executable = path + program;
+			if (FileExists(executable)) {
 				found = true;
 			}
+			else {
+				// Check if running from build dir
+				if (path.size() > 7 && fz::ends_with(path, std::wstring(L"/.libs/"))) {
+					if (FileExists(path.substr(0, path.size() - 6) + L"Makefile")) {
+						executable = path + L"../" + buildRelPath + program;
+						if (FileExists(executable)) {
+							found = true;
+						}
+					}
+				}
+				else if (FileExists(path + L"Makefile")) {
+					executable = path + buildRelPath + program;
+					if (FileExists(executable)) {
+						found = true;
+					}
+				}
+			}
 		}
 	}
-#endif
 
 	if (!found) {
 		// Check PATH
-		wxPathList pathList;
-		pathList.AddEnvList(_T("PATH"));
-		executable = pathList.FindAbsoluteValidPath(program);
-		if (!executable.empty()) {
-			found = true;
+		std::wstring path = GetEnv("PATH");
+		auto const segments = fz::strtok(path, PATH_SEP);
+		for (auto const& segment : segments) {
+			auto const cur = CLocalPath(segment).GetPath();
+			executable = cur + program;
+			if (!cur.empty() && FileExists(executable)) {
+				found = true;
+				break;
+			}
 		}
 	}
-
-	if (!found) {
-		// Quote path if it contains spaces
-		if (executable.Find(_T(" ")) != -1 && executable[0] != '"' && executable[0] != '\'') {
-			executable = _T("\"") + executable + _T("\"");
-		}
-
-		wxMessageBoxEx(wxString::Format(_("%s could not be found. Without this component of FileZilla, %s will not work.\n\nPossible solutions:\n- Make sure %s is in a directory listed in your PATH environment variable.\n- Set the full path to %s in the %s environment variable."), program, description, program, program, env),
-			_("File not found"), wxICON_ERROR | wxOK);
-		executable.clear();
-	}
 #endif
 
-	COptions::Get()->SetOption(setting, executable.ToStdWstring());
+	if (!found) {
+		// Quote path if it contains spaces
+		if (executable.find(' ') != std::wstring::npos && executable.front() != '"' && executable.front() != '\'') {
+			executable = L"\"" + executable + L"\"";
+		}
+
+		wxMessageBoxEx(fz::sprintf(fztranslate("%s could not be found. Without this component of FileZilla, %s will not work.\n\nPossible solutions:\n- Make sure %s is in a directory listed in your PATH environment variable.\n- Set the full path to %s in the %s environment variable."), program, description, program, program, env),
+			_("File not found"), wxICON_ERROR | wxOK);
+		executable.clear();
+	}
+	COptions::Get()->SetOption(setting, executable);
 }
 
 #ifdef __WXMSW__
@@ -786,14 +757,32 @@ void CFileZillaApp::AddStartupProfileRecord(std::string const& msg)
 void CFileZillaApp::ShowStartupProfile()
 {
 	if (m_profile_start && m_pCommandLine && m_pCommandLine->HasSwitch(CCommandLine::debug_startup)) {
+		AddStartupProfileRecord("CFileZillaApp::ShowStartupProfile");
 		wxString msg = _T("Profile:\n");
+
+		size_t const max_digits = fz::to_string((m_startupProfile.back().first - m_profile_start).get_milliseconds()).size();
+		
+		int64_t prev{};
 		for (auto const& p : m_startupProfile) {
 			auto const diff = p.first - m_profile_start;
+			auto absolute = std::to_wstring(diff.get_milliseconds());
+			if (absolute.size() < max_digits) {
+				msg.append(max_digits - absolute.size(), wchar_t(0x2007)); // FIGURE SPACE
+			}
+			msg += absolute;
+			msg += L" ";
+
+			auto relative = std::to_wstring(diff.get_milliseconds() - prev);
+			if (relative.size() < max_digits) {
+				msg.append(max_digits - relative.size(), wchar_t(0x2007)); // FIGURE SPACE
+			}
+			msg += relative;
+			msg += L" ";
 
-			msg += std::to_wstring(diff.get_milliseconds());
-			msg += _T(" ");
 			msg += fz::to_wstring(p.second);
-			msg += _T("\n");
+			msg += L"\n";
+
+			prev = diff.get_milliseconds();
 		}
 		wxMessageBoxEx(msg);
 	}
