@@ -8,6 +8,7 @@
 #include "filezillaapp.h"
 #include "filter.h"
 #include "file_utils.h"
+#include "infotext.h"
 #include "inputdialog.h"
 #include <algorithm>
 #include "dndobjects.h"
@@ -114,7 +115,7 @@ public:
 				return wxDragNone;
 			}
 
-			if (!m_pLocalListView->m_state.GetServer() || m_pRemoteDataObject->GetServer().server != m_pLocalListView->m_state.GetServer().server) {
+			if (!m_pLocalListView->m_state.GetSite() || m_pRemoteDataObject->GetSite().server != m_pLocalListView->m_state.GetSite().server) {
 				wxMessageBoxEx(_("Drag&drop between different servers has not been implemented yet."));
 				return wxDragNone;
 			}
@@ -256,6 +257,7 @@ BEGIN_EVENT_TABLE(CLocalListView, CFileListCtrl<CLocalFileData>)
 	EVT_MENU(XRCID("ID_DELETE"), CLocalListView::OnMenuDelete)
 	EVT_MENU(XRCID("ID_RENAME"), CLocalListView::OnMenuRename)
 	EVT_KEY_DOWN(CLocalListView::OnKeyDown)
+	EVT_SIZE(CLocalListView::OnSize)
 	EVT_LIST_BEGIN_DRAG(wxID_ANY, CLocalListView::OnBeginDrag)
 	EVT_MENU(XRCID("ID_OPEN"), CLocalListView::OnMenuOpen)
 	EVT_MENU(XRCID("ID_EDIT"), CLocalListView::OnMenuEdit)
@@ -278,7 +280,7 @@ CLocalListView::CLocalListView(CView* pParent, CState& state, CQueueView *pQueue
 	m_state.RegisterHandler(this, STATECHANGE_LOCAL_REFRESH_FILE);
 	m_state.RegisterHandler(this, STATECHANGE_SERVER);
 
-	const unsigned long widths[4] = { 120, 80, 100, 120 };
+	const unsigned long widths[4] = { 170, 80, 120, 120 };
 
 	AddColumn(_("Filename"), wxLIST_FORMAT_LEFT, widths[0], true);
 	AddColumn(_("Filesize"), wxLIST_FORMAT_RIGHT, widths[1]);
@@ -297,6 +299,8 @@ CLocalListView::CLocalListView(CView* pParent, CState& state, CQueueView *pQueue
 	EnablePrefixSearch(true);
 
 	m_windowTinter = std::make_unique<CWindowTinter>(*GetMainWindow());
+
+	m_pInfoText = new CInfoText(*this);
 }
 
 CLocalListView::~CLocalListView()
@@ -311,7 +315,6 @@ CLocalListView::~CLocalListView()
 
 bool CLocalListView::DisplayDir(CLocalPath const& dirname)
 {
-
 	CancelLabelEdit();
 
 	std::wstring focused;
@@ -384,11 +387,26 @@ regular_dir:
 		CStateFilterManager const& filter = m_state.GetStateFilterManager();
 		fz::local_filesys local_filesys;
 
-		if (!local_filesys.begin_find_files(fz::to_native(m_dir.GetPath()), false)) {
+		auto result = local_filesys.begin_find_files(fz::to_native(m_dir.GetPath()), false);
+		if (!result) {
+			
+			if (result.error_ == fz::result::noperm) {
+				SetInfoText(_("You do not have permission to list this directory"));
+			}
+			else {
+				SetInfoText(_("Could not list directory contents"));
+			}
+
 			SetItemCount(1);
+			if (m_pFilelistStatusBar) {
+				m_pFilelistStatusBar->SetDirectoryContents(0, 0, 0, 0, 0);
+			}
+
 			return false;
 		}
 
+		SetInfoText(wxString());
+
 		int64_t totalSize{};
 		int unknown_sizes = 0;
 		int totalFileCount = 0;
@@ -409,23 +427,23 @@ regular_dir:
 			m_fileData.push_back(data);
 			if (!filter.FilenameFiltered(data.name, m_dir.GetPath(), data.dir, data.size, true, data.attributes, data.time)) {
 				if (data.dir) {
-					totalDirCount++;
+					++totalDirCount;
 				}
 				else {
 					if (data.size != -1) {
 						totalSize += data.size;
 					}
 					else {
-						unknown_sizes++;
+						++unknown_sizes;
 					}
-					totalFileCount++;
+					++totalFileCount;
 				}
 				m_indexMapping.push_back(num);
 			}
 			else {
-				hidden++;
+				++hidden;
 			}
-			num++;
+			++num;
 		}
 
 		if (m_pFilelistStatusBar) {
@@ -572,8 +590,8 @@ void CLocalListView::OnItemActivated(wxListEvent &event)
 		return;
 	}
 
-	ServerWithCredentials const& server = m_state.GetServer();
-	if (!server) {
+	Site const& site = m_state.GetSite();
+	if (!site) {
 		wxBell();
 		return;
 	}
@@ -586,7 +604,7 @@ void CLocalListView::OnItemActivated(wxListEvent &event)
 
 	const bool queue_only = action == 1;
 
-	m_pQueue->QueueFile(queue_only, false, data->name, wxEmptyString, m_dir, path, server, data->size);
+	m_pQueue->QueueFile(queue_only, false, data->name, wxEmptyString, m_dir, path, site, data->size);
 	m_pQueue->QueueFile_Finish(true);
 }
 
@@ -623,9 +641,10 @@ void CLocalListView::OnMenuEnter(wxCommandEvent &)
 #ifdef __WXMSW__
 void CLocalListView::DisplayDrives()
 {
+	SetInfoText(wxString());
 	int count = m_fileData.size();
 
-	std::list<std::wstring> drives = CVolumeDescriptionEnumeratorThread::GetDrives();
+	std::vector<std::wstring> drives = CVolumeDescriptionEnumeratorThread::GetDrives();
 	for (auto it = drives.begin(); it != drives.end(); ++it) {
 		std::wstring drive = *it;
 		if (!drive.empty() && drive.back() == '\\') {
@@ -649,7 +668,7 @@ void CLocalListView::DisplayDrives()
 	}
 
 	if (!volumeEnumeratorThread_) {
-		volumeEnumeratorThread_ = std::make_unique<CVolumeDescriptionEnumeratorThread>(this);
+		volumeEnumeratorThread_ = std::make_unique<CVolumeDescriptionEnumeratorThread>(this, m_state.pool_);
 		if (volumeEnumeratorThread_->Failed()) {
 			volumeEnumeratorThread_.reset();
 		}
@@ -658,6 +677,8 @@ void CLocalListView::DisplayDrives()
 
 void CLocalListView::DisplayShares(wxString computer)
 {
+	SetInfoText(wxString());
+
 	// Cast through a union to avoid warning about breaking strict aliasing rule
 	union
 	{
@@ -860,8 +881,8 @@ void CLocalListView::OnContextMenu(wxContextMenuEvent& event)
 
 void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 {
-	ServerWithCredentials const& server = m_state.GetServer();
-	if (!server) {
+	Site const& site = m_state.GetSite();
+	if (!site) {
 		wxBell();
 		return;
 	}
@@ -915,7 +936,7 @@ void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 			root.add_dir_to_visit(localPath, remotePath);
 		}
 		else {
-			m_pQueue->QueueFile(queue_only, false, data->name, wxEmptyString, m_dir, remotePath, server, data->size);
+			m_pQueue->QueueFile(queue_only, false, data->name, wxEmptyString, m_dir, remotePath, site, data->size);
 			added = true;
 		}
 	}
@@ -1298,7 +1319,7 @@ void CLocalListView::ReselectItems(const std::vector<std::wstring>& selectedName
 
 		if (focusedItem != -1 && GetItemCount() != 0) {
 			if (focusedItem >= GetItemCount()) {
-				--focusedItem;
+				focusedItem = GetItemCount() - 1;
 			}
 			SetItemState(focusedItem, wxLIST_STATE_FOCUSED, wxLIST_STATE_FOCUSED);
 		}
@@ -1362,7 +1383,15 @@ void CLocalListView::ReselectItems(const std::vector<std::wstring>& selectedName
 	}
 }
 
-void CLocalListView::OnStateChange(t_statechange_notifications notification, const wxString& data, const void*)
+void CLocalListView::OnSize(wxSizeEvent& event)
+{
+	event.Skip();
+	if (m_pInfoText) {
+		m_pInfoText->Reposition();
+	}
+}
+
+void CLocalListView::OnStateChange(t_statechange_notifications notification, std::wstring const& data, const void*)
 {
 	if (notification == STATECHANGE_LOCAL_DIR) {
 		DisplayDir(m_state.GetLocalDir());
@@ -1371,11 +1400,32 @@ void CLocalListView::OnStateChange(t_statechange_notifications notification, con
 		ApplyCurrentFilter();
 	}
 	else if (notification == STATECHANGE_SERVER) {
-		m_windowTinter->SetBackgroundTint(m_state.GetSite().m_colour);
+		if (m_windowTinter) {
+			m_windowTinter->SetBackgroundTint(m_state.GetSite().m_colour);
+		}
+		if (m_pInfoText) {
+			m_pInfoText->SetBackgroundTint(m_state.GetSite().m_colour);
+		}
 	}
 	else {
 		wxASSERT(notification == STATECHANGE_LOCAL_REFRESH_FILE);
-		RefreshFile(data.ToStdWstring());
+		RefreshFile(data);
+	}
+}
+
+void CLocalListView::SetInfoText(wxString const& text)
+{
+	if (!m_pInfoText) {
+		return;
+	}
+
+	if (IsComparing() || text.empty()) {
+		m_pInfoText->Hide();
+	}
+	else {
+		m_pInfoText->SetText(text);
+		m_pInfoText->Reposition();
+		m_pInfoText->Show();
 	}
 }
 
@@ -1650,8 +1700,7 @@ void CLocalListView::StartComparison()
 
 	m_comparisonIndex = -1;
 
-	const CLocalFileData& last = m_fileData[m_fileData.size() - 1];
-	if (last.comparison_flags != fill) {
+	if (m_fileData.empty() || m_fileData.back().comparison_flags != fill) {
 		CLocalFileData data;
 		data.dir = false;
 		data.icon = -1;
@@ -1661,7 +1710,7 @@ void CLocalListView::StartComparison()
 	}
 }
 
-bool CLocalListView::get_next_file(wxString& name, bool& dir, int64_t& size, fz::datetime& date)
+bool CLocalListView::get_next_file(std::wstring & name, bool& dir, int64_t& size, fz::datetime& date)
 {
 	if (++m_comparisonIndex >= (int)m_originalIndexMapping.size()) {
 		return false;
@@ -1672,7 +1721,7 @@ bool CLocalListView::get_next_file(wxString& name, bool& dir, int64_t& size, fz:
 		return false;
 	}
 
-	const CLocalFileData& data = m_fileData[index];
+	CLocalFileData const& data = m_fileData[index];
 
 	name = data.name;
 	dir = data.dir;
@@ -1684,6 +1733,8 @@ bool CLocalListView::get_next_file(wxString& name, bool& dir, int64_t& size, fz:
 
 void CLocalListView::FinishComparison()
 {
+	SetInfoText(wxString());
+
 	SetItemCount(m_indexMapping.size());
 
 	ComparisonRestoreSelections();
@@ -1748,17 +1799,17 @@ wxString CLocalListView::GetItemText(int item, unsigned int column)
 
 void CLocalListView::OnMenuEdit(wxCommandEvent&)
 {
-	ServerWithCredentials server;
+	Site site;
 	CServerPath path;
 
-	if (!m_state.GetServer()) {
+	if (!m_state.GetSite()) {
 		if (COptions::Get()->GetOptionVal(OPTION_EDIT_TRACK_LOCAL)) {
 			wxMessageBoxEx(_("Cannot edit file, not connected to any server."), _("Editing failed"), wxICON_EXCLAMATION);
 			return;
 		}
 	}
 	else {
-		server = m_state.GetServer();
+		site = m_state.GetSite();
 
 		path = m_state.GetRemotePath();
 		if (path.empty()) {
@@ -1794,7 +1845,7 @@ void CLocalListView::OnMenuEdit(wxCommandEvent&)
 	}
 
 	CEditHandler* pEditHandler = CEditHandler::Get();
-	pEditHandler->Edit(CEditHandler::local, selected_item, path, server, this);
+	pEditHandler->Edit(CEditHandler::local, selected_item, path, site, this);
 }
 
 void CLocalListView::OnMenuOpen(wxCommandEvent&)
@@ -1907,8 +1958,7 @@ void CLocalListView::OnVolumesEnumerated(wxCommandEvent& event)
 		return;
 	}
 
-	std::list<CVolumeDescriptionEnumeratorThread::t_VolumeInfo> volumeInfo;
-	volumeInfo = volumeEnumeratorThread_->GetVolumes();
+	std::vector<CVolumeDescriptionEnumeratorThread::t_VolumeInfo> volumeInfo = volumeEnumeratorThread_->GetVolumes();
 
 	if (event.GetEventType() == fzEVT_VOLUMESENUMERATED) {
 		volumeEnumeratorThread_.reset();
@@ -1918,8 +1968,8 @@ void CLocalListView::OnVolumesEnumerated(wxCommandEvent& event)
 		return;
 	}
 
-	for (std::list<CVolumeDescriptionEnumeratorThread::t_VolumeInfo>::const_iterator iter = volumeInfo.begin(); iter != volumeInfo.end(); ++iter) {
-		std::wstring drive = iter->volume;
+	for (auto const& info : volumeInfo) {
+		std::wstring const& drive = info.volume;
 
 		unsigned int item, index;
 		for (item = m_hasParent ? 1 : 0; item < m_indexMapping.size(); ++item) {
@@ -1932,11 +1982,11 @@ void CLocalListView::OnVolumesEnumerated(wxCommandEvent& event)
 			continue;
 		}
 
-		if (!iter->volumeName.empty()) {
-			m_fileData[index].label = fz::sparse_optional<std::wstring>(drive + _T(" (") + iter->volumeName + _T(")"));
+		if (!info.volumeName.empty()) {
+			m_fileData[index].label = fz::sparse_optional<std::wstring>(drive + _T(" (") + info.volumeName + _T(")"));
 		}
-		if (iter->icon != -1) {
-			m_fileData[index].icon = iter->icon;
+		if (info.icon != -1) {
+			m_fileData[index].icon = info.icon;
 		}
 
 		RefreshItem(item);
