@@ -51,9 +51,7 @@ struct t_Option
 #define DEFAULT_FILENAME_SORT   _T("1")
 #endif
 
-// In C++14 we should be able to use this instead:
-//   static_assert(OPTIONS_NUM <= changed_options_t().size());
-static_assert(static_cast<int>(OPTIONS_NUM) <= static_cast<int>(changed_options_size), "OPTIONS_NUM too big for changed_options_t");
+static_assert(OPTIONS_NUM <= changed_options_t().size());
 
 static const t_Option options[OPTIONS_NUM] =
 {
@@ -305,7 +303,7 @@ std::unique_ptr<pugi::xml_document> COptions::GetOptionXml(unsigned int nID)
 	auto value = std::make_unique<pugi::xml_document>();
 	value->append_copy(m_optionsCache[nID].xmlValue->first_child());
 
-	return std::move(value);
+	return value;
 }
 
 bool COptions::SetOption(unsigned int nID, int value)
@@ -831,7 +829,9 @@ void COptions::SaveIfNeeded()
 	}
 
 	m_save_timer.Stop();
-	Save();
+	if (save) {
+		Save();
+	}
 }
 
 namespace {
