@@ -75,7 +75,7 @@ struct t_EngineData final
 	} state;
 
 	CFileItem* pItem;
-	ServerWithCredentials lastServer;
+	Site lastSite;
 	CStatusLineCtrl* pStatusLineCtrl;
 	wxTimer* m_idleDisconnectTimer;
 };
@@ -119,12 +119,12 @@ public:
 	bool QueueFile(const bool queueOnly, const bool download,
 		std::wstring const& localFile, std::wstring const& remoteFile,
 		const CLocalPath& localPath, const CServerPath& remotePath,
-		ServerWithCredentials const& server, int64_t size, CEditHandler::fileType edit = CEditHandler::none,
+		Site const& site, int64_t size, CEditHandler::fileType edit = CEditHandler::none,
 		QueuePriority priority = QueuePriority::normal);
 
 	void QueueFile_Finish(const bool start); // Need to be called after QueueFile
 	bool QueueFiles(const bool queueOnly, CLocalPath const& localPath, const CRemoteDataObject& dataObject);
-	bool QueueFiles(const bool queueOnly, ServerWithCredentials const& server, CLocalRecursiveOperation::listing const& listing);
+	bool QueueFiles(const bool queueOnly, Site const& site, CLocalRecursiveOperation::listing const& listing);
 
 	bool empty() const;
 	int IsActive() const { return m_activeMode; }
@@ -229,7 +229,7 @@ protected:
 
 	bool IsOtherEngineConnected(t_EngineData* pEngineData);
 
-	t_EngineData* GetIdleEngine(ServerWithCredentials const& server = ServerWithCredentials(), bool allowTransient = false);
+	t_EngineData* GetIdleEngine(Site const& site = Site(), bool allowTransient = false);
 	t_EngineData* GetEngineData(const CFileZillaEngine* pEngine);
 
 	std::vector<t_EngineData*> m_engineData;
@@ -298,7 +298,7 @@ protected:
 
 	std::weak_ptr<CActionAfterBlocker> m_actionAfterBlocker;
 
-	virtual void OnStateChange(CState* pState, t_statechange_notifications notification, wxString const& data, const void* data2) override;
+	virtual void OnStateChange(CState* pState, t_statechange_notifications notification, std::wstring const& data, const void* data2) override;
 
 	DECLARE_EVENT_TABLE()
 	void OnChar(wxKeyEvent& event);
