@@ -99,14 +99,13 @@ public:
 				return wxDragNone;
 			}
 
-			if (!m_pRemoteTreeView->m_state.GetServer() || m_pRemoteDataObject->GetServer().server != m_pRemoteTreeView->m_state.GetServer().server) {
+			if (!m_pRemoteTreeView->m_state.GetSite() || m_pRemoteDataObject->GetSite().server != m_pRemoteTreeView->m_state.GetSite().server) {
 				wxMessageBoxEx(_("Drag&drop between different servers has not been implemented yet."));
 				return wxDragNone;
 			}
 
 			// Make sure path path is valid
-			if (path == m_pRemoteDataObject->GetServerPath())
-			{
+			if (path == m_pRemoteDataObject->GetServerPath()) {
 				wxMessageBoxEx(_("Source and path of the drop operation are identical"));
 				return wxDragNone;
 			}
@@ -244,7 +243,7 @@ EVT_MENU(XRCID("ID_GETURL"), CRemoteTreeView::OnMenuGeturl)
 END_EVENT_TABLE()
 
 CRemoteTreeView::CRemoteTreeView(wxWindow* parent, wxWindowID id, CState& state, CQueueView* pQueue)
-	: wxTreeCtrlEx(parent, id, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxTR_EDIT_LABELS | wxTR_LINES_AT_ROOT | wxTR_HAS_BUTTONS | wxNO_BORDER | wxTR_HIDE_ROOT),
+	: wxTreeCtrlEx(parent, id, wxDefaultPosition, wxDefaultSize, DEFAULT_TREE_STYLE | wxTAB_TRAVERSAL | wxTR_EDIT_LABELS | wxNO_BORDER | wxTR_HIDE_ROOT),
 	CSystemImageList(CThemeProvider::GetIconSize(iconSizeSmall).x),
 	CStateEventHandler(state)
 {
@@ -278,7 +277,7 @@ CRemoteTreeView::~CRemoteTreeView()
 	delete m_pImageList;
 }
 
-void CRemoteTreeView::OnStateChange(t_statechange_notifications notification, const wxString&, const void* data2)
+void CRemoteTreeView::OnStateChange(t_statechange_notifications notification, std::wstring const&, const void* data2)
 {
 	if (notification == STATECHANGE_REMOTE_DIR) {
 		SetDirectoryListing(m_state.GetRemoteDir(), data2 ? *reinterpret_cast<bool const*>(data2) : true);
@@ -536,6 +535,8 @@ void CRemoteTreeView::DisplayItem(wxTreeItemId parent, const CDirectoryListing&
 	std::wstring const path = listing.path.GetPath();
 
 	CFilterDialog filter;
+
+	wxTreeItemId last;
 	for (size_t i = 0; i < listing.size(); ++i) {
 		auto const& entry = listing[i];
 		if (!entry.is_dir()) {
@@ -561,7 +562,7 @@ void CRemoteTreeView::DisplayItem(wxTreeItemId parent, const CDirectoryListing&
 			}
 		}
 		else {
-			wxTreeItemId child = AppendItem(parent, name, 1, 3, 0);
+			wxTreeItemId child = InsertItem(parent, last, name, 1, 3, 0);
 			SetItemImages(child, true);
 		}
 	}
@@ -595,14 +596,21 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 		}
 	}
 
-	auto const& sortFunc = CFileListCtrlSortBase::GetCmpFunction(m_nameSortMode);
-	std::sort(dirs.begin(), dirs.end(), [&](auto const& lhs, auto const& rhs) { return sortFunc(lhs, rhs) < 0; });
+	std::sort(dirs.begin(), dirs.end(), [&](auto const& lhs, auto const& rhs) { return sortFunction_(lhs, rhs) < 0; });
+
+	std::vector<wxTreeItemId> toDelete;
 
 	bool inserted = false;
-	child = GetLastChild(parent);
-	auto iter = dirs.rbegin();
-	while (child && iter != dirs.rend()) {
-		int cmp = sortFunc(GetItemText(child).ToStdWstring(), *iter);
+	
+	wxTreeItemIdValue unused;
+	child = GetFirstChild(parent, unused);
+
+	wxTreeItemId last = GetLastChild(parent);
+
+	auto iter = dirs.begin();
+	while (child && iter != dirs.end()) {
+		wxString const& childName = GetItemText(child);
+		int cmp = sortFunction_(std::wstring_view(childName.data(), childName.size()), *iter);
 
 		if (!cmp) {
 			CServerPath childPath = listing.path;
@@ -619,7 +627,7 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 				SetItemImages(child, true);
 			}
 
-			child = GetPrevSibling(child);
+			child = GetNextSibling(child);
 			++iter;
 		}
 		else if (cmp > 0) {
@@ -628,11 +636,10 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 			while (sel && sel != child) {
 				sel = GetItemParent(sel);
 			}
-			wxTreeItemId prev = GetPrevSibling(child);
 			if (!sel || will_select_parent) {
-				Delete(child);
+				toDelete.push_back(child);
 			}
-			child = prev;
+			child = GetNextSibling(child);
 		}
 		else if (cmp < 0) {
 			// New directory
@@ -641,12 +648,12 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 
 			CDirectoryListing subListing;
 			if (m_state.m_pEngine->CacheLookup(childPath, subListing) == FZ_REPLY_OK) {
-				wxTreeItemId childItem = AppendItem(parent, *iter, 0, 2, 0);
-				if (childItem) {
-					SetItemImages(childItem, false);
+				last = InsertItem(parent, last, *iter, 0, 2, 0);
+				if (last) {
+					SetItemImages(last, false);
 
 					if (HasSubdirs(subListing, filter)) {
-						AppendItem(childItem, _T(""), -1, -1);
+						AppendItem(last, _T(""), -1, -1);
 					}
 				}
 			}
@@ -667,24 +674,23 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 		while (sel && sel != child) {
 			sel = GetItemParent(sel);
 		}
-		wxTreeItemId prev = GetPrevSibling(child);
 		if (!sel || will_select_parent) {
-			Delete(child);
+			toDelete.push_back(child);
 		}
-		child = prev;
+		child = GetNextSibling(child);
 	}
-	while (iter != dirs.rend()) {
+	while (iter != dirs.end()) {
 		CServerPath childPath = listing.path;
 		childPath.AddSegment(*iter);
 
 		CDirectoryListing subListing;
 		if (m_state.m_pEngine->CacheLookup(childPath, subListing) == FZ_REPLY_OK) {
-			wxTreeItemId childItem = AppendItem(parent, *iter, 0, 2, 0);
-			if (childItem) {
-				SetItemImages(childItem, false);
+			last = InsertItem(parent, last, *iter, 0, 2, 0);
+			if (last) {
+				SetItemImages(last, false);
 
 				if (HasSubdirs(subListing, filter)) {
-					AppendItem(childItem, _T(""), -1, -1);
+					AppendItem(last, _T(""), -1, -1);
 				}
 			}
 		}
@@ -698,9 +704,13 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 		++iter;
 		inserted = true;
 	}
+	for (auto it = toDelete.rbegin(); it != toDelete.rend(); ++it) {
+		Delete(*it);
+	}
 
-	if (inserted)
+	if (inserted) {
 		SortChildren(parent);
+	}
 }
 
 void CRemoteTreeView::OnItemExpanding(wxTreeEvent& event)
@@ -833,12 +843,12 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 
 	wxDataObjectComposite object;
 
-	ServerWithCredentials server = m_state.GetServer(); // Make a copy as DoDragDrop later runs the event loop
-	if (!server) {
+	Site site = m_state.GetSite(); // Make a copy as DoDragDrop later runs the event loop
+	if (!site) {
 		return;
 	}
 
-	CRemoteDataObject *pRemoteDataObject = new CRemoteDataObject(server, parent);
+	CRemoteDataObject *pRemoteDataObject = new CRemoteDataObject(site, parent);
 	pRemoteDataObject->AddFile(lastSegment, true, -1, false);
 
 	pRemoteDataObject->Finalize();
@@ -861,7 +871,7 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 
 	CDragDropManager* pDragDropManager = CDragDropManager::Init();
 	pDragDropManager->pDragSource = this;
-	pDragDropManager->server = server;
+	pDragDropManager->site = site;
 	pDragDropManager->remoteParent = parent;
 
 	wxDropSource source(this);
@@ -878,8 +888,8 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 #if FZ3_USESHELLEXT
 	if (ext) {
 		if (!pRemoteDataObject->DidSendData()) {
-			server = m_state.GetServer();
-			if (!server || !m_state.IsRemoteIdle() || server != server) {
+			Site newSite = m_state.GetSite();
+			if (!newSite || !m_state.IsRemoteIdle() || site != newSite) {
 				wxBell();
 				return;
 			}
@@ -905,20 +915,26 @@ void CRemoteTreeView::OnContextMenu(wxTreeEvent& event)
 	auto item = new wxMenuItem(&menu, XRCID("ID_DOWNLOAD"), _("&Download"), _("Download selected directory"));
 	item->SetBitmap(wxArtProvider::GetBitmap(_T("ART_DOWNLOAD"), wxART_MENU));
 	menu.Append(item);
-    item = new wxMenuItem(&menu, XRCID("ID_ADDTOQUEUE"), _("&Add to queue"), _("Add selected directory to the transfer queue"));
+	item = new wxMenuItem(&menu, XRCID("ID_ADDTOQUEUE"), _("&Add to queue"), _("Add selected directory to the transfer queue"));
 	item->SetBitmap(wxArtProvider::GetBitmap(_T("ART_DOWNLOADADD"), wxART_MENU));
 	menu.Append(item);
 
-    menu.AppendSeparator();
-    menu.Append(XRCID("ID_MKDIR"), _("&Create directory"), _("Create a new subdirectory in the current directory"));
-    menu.Append(XRCID("ID_MKDIR_CHGDIR"), _("Create director&y and enter it"), _("Create a new subdirectory in the current directory and change into it"));
+	menu.AppendSeparator();
+	menu.Append(XRCID("ID_MKDIR"), _("&Create directory"), _("Create a new subdirectory in the current directory"));
+	menu.Append(XRCID("ID_MKDIR_CHGDIR"), _("Create director&y and enter it"), _("Create a new subdirectory in the current directory and change into it"));
 
-    menu.AppendSeparator();
-    menu.Append(XRCID("ID_DELETE"), _("D&elete"), _("Delete selected directory"));
-    menu.Append(XRCID("ID_RENAME"), _("&Rename"), _("Rename selected directory"));
-    menu.Append(XRCID("ID_GETURL"), _("C&opy URL(s) to clipboard"), _("Copy the URLs of the selected items to clipboard."));
-    menu.Append(XRCID("ID_GETURL_PASSWORD"), _("C&opy URL(s) with password to clipboard"), _("Copy the URLs of the selected items to clipboard, including password."));
-    menu.Append(XRCID("ID_CHMOD"), _("&File Attributes..."), _("Change the file permissions."));
+	menu.AppendSeparator();
+	menu.Append(XRCID("ID_DELETE"), _("D&elete"), _("Delete selected directory"));
+	menu.Append(XRCID("ID_RENAME"), _("&Rename"), _("Rename selected directory"));
+	menu.Append(XRCID("ID_GETURL"), _("C&opy URL(s) to clipboard"), _("Copy the URLs of the selected items to clipboard."));
+	menu.Append(XRCID("ID_GETURL_PASSWORD"), _("C&opy URL(s) with password to clipboard"), _("Copy the URLs of the selected items to clipboard, including password."));
+
+	auto const protocol = m_state.GetSite().server.GetProtocol();
+	bool const hasChmod = protocol == FTP || protocol == FTPS || protocol == FTPES || protocol == INSECURE_FTP || protocol == SFTP;
+
+	if (hasChmod) {
+	    menu.Append(XRCID("ID_CHMOD"), _("&File Attributes..."), _("Change the file permissions."));
+	}
 
 	const CServerPath& path = m_contextMenuItem ? GetPathFromItem(m_contextMenuItem) : CServerPath();
 	if (!m_state.IsRemoteIdle() || path.empty()) {
@@ -927,7 +943,9 @@ void CRemoteTreeView::OnContextMenu(wxTreeEvent& event)
 		menu.Enable(XRCID("ID_MKDIR"), false);
 		menu.Enable(XRCID("ID_MKDIR_CHGDIR"), false);
 		menu.Enable(XRCID("ID_DELETE"), false);
-		menu.Enable(XRCID("ID_CHMOD"), false);
+		if (hasChmod) {
+			menu.Enable(XRCID("ID_CHMOD"), false);
+		}
 		menu.Enable(XRCID("ID_RENAME"), false);
 		menu.Enable(XRCID("ID_GETURL"), false);
 	}
@@ -962,7 +980,8 @@ void CRemoteTreeView::OnMenuChmod(wxCommandEvent&)
 
 	const bool hasParent = path.HasParent();
 
-	CChmodDialog* pChmodDlg = new CChmodDialog;
+	auto chmodData = std::make_unique<ChmodData>();
+	auto chmodDialog = std::make_unique<CChmodDialog>(*chmodData);
 
 	// Get current permissions of directory
 	std::wstring const name = GetItemText(m_contextMenuItem).ToStdWstring();
@@ -983,69 +1002,65 @@ void CRemoteTreeView::OnMenuChmod(wxCommandEvent&)
 					continue;
 				}
 
-				pChmodDlg->ConvertPermissions(*entry.permissions, permissions);
+				chmodData->ConvertPermissions(*entry.permissions, permissions);
 			}
 		}
 	}
 
-	if (!pChmodDlg->Create(this, 0, 1, name, permissions)) {
-		pChmodDlg->Destroy();
-		pChmodDlg = 0;
+	if (!chmodDialog->Create(this, 0, 1, name, permissions)) {
 		return;
 	}
 
-	if (pChmodDlg->ShowModal() != wxID_OK) {
-		pChmodDlg->Destroy();
-		pChmodDlg = 0;
+	if (chmodDialog->ShowModal() != wxID_OK) {
 		return;
 	}
 
 	// State may have changed while chmod dialog was shown
 	if (!m_contextMenuItem || !m_state.IsRemoteConnected() || !m_state.IsRemoteIdle()) {
-		pChmodDlg->Destroy();
-		pChmodDlg = 0;
 		return;
 	}
 
-	const int applyType = pChmodDlg->GetApplyType();
+	int const applyType = chmodData->GetApplyType();
 
 	recursion_root root(hasParent ? path.GetParent() : path, !cached);
 	if (cached) { // Implies hasParent
 		// Change directory permissions
 		if (!applyType || applyType == 2) {
-			std::wstring const newPerms = pChmodDlg->GetPermissions(permissions, true).ToStdWstring();
+			std::wstring const newPerms = chmodData->GetPermissions(permissions, true);
 
 			m_state.m_pCommandQueue->ProcessCommand(new CChmodCommand(path.GetParent(), name, newPerms));
 		}
 
-		if (pChmodDlg->Recursive()) {
+		if (chmodDialog->Recursive()) {
 			// Start recursion
 			root.add_dir_to_visit(path, std::wstring(), CLocalPath());
 		}
 	}
 	else {
 		if (hasParent) {
-			root.add_dir_to_visit_restricted(path.GetParent(), name, pChmodDlg->Recursive());
+			root.add_dir_to_visit_restricted(path.GetParent(), name, chmodDialog->Recursive());
 		}
 		else {
-			root.add_dir_to_visit_restricted(path, std::wstring(), pChmodDlg->Recursive());
+			root.add_dir_to_visit_restricted(path, std::wstring(), chmodDialog->Recursive());
 		}
 	}
 
-	if (!cached || pChmodDlg->Recursive()) {
+	if (!cached || chmodDialog->Recursive()) {
 		CRemoteRecursiveOperation* pRecursiveOperation = m_state.GetRemoteRecursiveOperation();
 		pRecursiveOperation->AddRecursionRoot(std::move(root));
-		pRecursiveOperation->SetChmodDialog(pChmodDlg);
+
+		chmodDialog.reset();
+		pRecursiveOperation->SetChmodData(std::move(chmodData));
 
 		CServerPath currentPath;
 		const wxTreeItemId selected = GetSelection();
-		if (selected)
+		if (selected) {
 			currentPath = GetPathFromItem(selected);
+		}
 		CFilterManager filter;
 		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_chmod, filter.GetActiveFilters(), currentPath);
 	}
 	else {
-		pChmodDlg->Destroy();
 		const wxTreeItemId selected = GetSelection();
 		if (selected) {
 			CServerPath currentPath = GetPathFromItem(selected);
@@ -1117,7 +1132,7 @@ void CRemoteTreeView::OnMenuDelete(wxCommandEvent&)
 	bool const hasParent = pathToDelete.HasParent();
 
 	CFilterManager filter;
-	if (CServer::ProtocolHasFeature(m_state.GetServer().server.GetProtocol(), ProtocolFeature::RecursiveDelete) && !filter.HasActiveRemoteFilters()) {
+	if (CServer::ProtocolHasFeature(m_state.GetSite().server.GetProtocol(), ProtocolFeature::RecursiveDelete) && !filter.HasActiveRemoteFilters()) {
 		if (hasParent) {
 			std::wstring const name = GetItemText(m_contextMenuItem).ToStdWstring();
 			m_state.m_pCommandQueue->ProcessCommand(new CRemoveDirCommand(pathToDelete.GetParent(), name));
@@ -1295,19 +1310,23 @@ void CRemoteTreeView::OnMenuMkdirChgDir(wxCommandEvent&)
 // Returns the name of the new directory
 CServerPath CRemoteTreeView::MenuMkdir()
 {
-	if (!m_state.IsRemoteIdle())
+	if (!m_state.IsRemoteIdle()) {
 		return CServerPath();
+	}
 
-	if (!m_contextMenuItem)
+	if (!m_contextMenuItem) {
 		return CServerPath();
+	}
 
 	const CServerPath& path = GetPathFromItem(m_contextMenuItem);
-	if (path.empty())
+	if (path.empty()) {
 		return CServerPath();
+	}
 
 	CInputDialog dlg;
-	if (!dlg.Create(this, _("Create directory"), _("Please enter the name of the directory which should be created:")))
+	if (!dlg.Create(this, _("Create directory"), _("Please enter the name of the directory which should be created:"))) {
 		return CServerPath();
+	}
 
 	CServerPath newPath = path;
 
@@ -1325,8 +1344,9 @@ CServerPath CRemoteTreeView::MenuMkdir()
 		dlg.SelectText(pos, pos + newName.Length());
 	}
 
-	if (dlg.ShowModal() != wxID_OK)
+	if (dlg.ShowModal() != wxID_OK) {
 		return CServerPath();
+	}
 
 	newPath = path;
 	if (!newPath.ChangePath(dlg.GetValue().ToStdWstring())) {
@@ -1343,19 +1363,21 @@ bool CRemoteTreeView::ListExpand(wxTreeItemId item)
 {
 	const CServerPath path = GetPathFromItem(item);
 	wxASSERT(!path.empty());
-	if (path.empty())
+	if (path.empty()) {
 		return false;
+	}
 
 	CDirectoryListing listing;
-	if (m_state.m_pEngine->CacheLookup(path, listing) == FZ_REPLY_OK)
+	if (m_state.m_pEngine->CacheLookup(path, listing) == FZ_REPLY_OK) {
 		RefreshItem(item, listing, false);
-	else
-	{
+	}
+	else {
 		SetItemImages(item, true);
 
 		wxTreeItemId child = GetLastChild(item);
-		if (!child || GetItemText(child).empty())
+		if (!child || GetItemText(child).empty()) {
 			return false;
+		}
 	}
 
 	return true;
@@ -1388,8 +1410,9 @@ void CRemoteTreeView::ApplyFilters(bool resort)
 	wxTreeItemIdValue cookie;
 	for (wxTreeItemId child = GetFirstChild(root, cookie); child; child = GetNextSibling(child)) {
 		CServerPath path = GetPathFromItem(child);
-		if (path.empty())
+		if (path.empty()) {
 			continue;
+		}
 
 		_parents dir;
 		dir.item = child;
@@ -1459,8 +1482,8 @@ void CRemoteTreeView::OnMenuGeturl(wxCommandEvent& event)
 		return;
 	}
 
-	ServerWithCredentials const& server = m_state.GetServer();
-	if (!server) {
+	Site const& site = m_state.GetSite();
+	if (!site) {
 		wxBell();
 		return;
 	}
@@ -1470,7 +1493,7 @@ void CRemoteTreeView::OnMenuGeturl(wxCommandEvent& event)
 		return;
 	}
 
-	wxString url = server.Format((event.GetId() == XRCID("ID_GETURL_PASSWORD")) ? ServerFormat::url_with_password : ServerFormat::url);
+	std::wstring url = site.Format((event.GetId() == XRCID("ID_GETURL_PASSWORD")) ? ServerFormat::url_with_password : ServerFormat::url);
 
 	std::wstring const pathPart = fz::percent_encode_w(path.GetPath(), true);
 	if (!pathPart.empty() && pathPart[0] != '/') {
@@ -1486,19 +1509,22 @@ void CRemoteTreeView::OnMenuGeturl(wxCommandEvent& event)
 
 void CRemoteTreeView::UpdateSortMode()
 {
+	CFileListCtrlSortBase::NameSortMode sortMode;
 	switch (COptions::Get()->GetOptionVal(OPTION_FILELIST_NAMESORT))
 	{
 	case 0:
 	default:
-		m_nameSortMode = CFileListCtrlSortBase::namesort_caseinsensitive;
+		sortMode = CFileListCtrlSortBase::namesort_caseinsensitive;
 		break;
 	case 1:
-		m_nameSortMode = CFileListCtrlSortBase::namesort_casesensitive;
+		sortMode = CFileListCtrlSortBase::namesort_casesensitive;
 		break;
 	case 2:
-		m_nameSortMode = CFileListCtrlSortBase::namesort_natural;
+		sortMode = CFileListCtrlSortBase::namesort_natural;
 		break;
 	}
+	sortFunction_ = CFileListCtrlSortBase::GetCmpFunction(sortMode);
+	Resort();
 }
 
 void CRemoteTreeView::OnOptionsChanged(changed_options_t const& options)
