@@ -35,22 +35,22 @@ void CAskSavePasswordDialog::OnOk(wxCommandEvent& event)
 {
 	bool useMaster = xrc_call(*this, "ID_PASSWORDS_USEMASTERPASSWORD", &wxRadioButton::GetValue);
 	if (useMaster) {
-		wxString pw = xrc_call(*this, "ID_MASTERPASSWORD", &wxTextCtrl::GetValue);
-		wxString repeat = xrc_call(*this, "ID_MASTERPASSWORD_REPEAT", &wxTextCtrl::GetValue);
+		std::wstring pw = xrc_call(*this, "ID_MASTERPASSWORD", &wxTextCtrl::GetValue).ToStdWstring();
+		std::wstring repeat = xrc_call(*this, "ID_MASTERPASSWORD_REPEAT", &wxTextCtrl::GetValue).ToStdWstring();
 		if (pw != repeat) {
-			wxMessageBox(_("The entered passwords are not the same."), _("Invalid input"));
+			wxMessageBoxEx(_("The entered passwords are not the same."), _("Invalid input"));
 			return;
 		}
 
 		if (pw.size() < 8) {
-			wxMessageBox(_("The master password needs to be at least 8 characters long."), _("Invalid input"));
+			wxMessageBoxEx(_("The master password needs to be at least 8 characters long."), _("Invalid input"));
 			return;
 		}
 
 		auto priv = fz::private_key::from_password(fz::to_utf8(pw), fz::random_bytes(fz::private_key::salt_size));
 		auto pub = priv.pubkey();
 		if (!pub) {
-			wxMessageBox(_("Could not generate key"), _("Error"));
+			wxMessageBoxEx(_("Could not generate key"), _("Error"));
 			return;
 		}
 		else {
