@@ -1,5 +1,6 @@
 #include <filezilla.h>
 #include "chmoddialog.h"
+#include "xrc_helper.h"
 
 BEGIN_EVENT_TABLE(CChmodDialog, wxDialogEx)
 EVT_BUTTON(XRCID("wxID_OK"), CChmodDialog::OnOK)
@@ -8,11 +9,142 @@ EVT_TEXT(XRCID("ID_NUMERIC"), CChmodDialog::OnNumericChanged)
 EVT_CHECKBOX(XRCID("ID_RECURSE"), CChmodDialog::OnRecurseChanged)
 END_EVENT_TABLE()
 
-CChmodDialog::CChmodDialog()
+
+bool ChmodData::ConvertPermissions(std::wstring const& rwx, char* permissions)
+{
+	if (!permissions) {
+		return false;
+	}
+
+	size_t pos = rwx.find('(');
+	if (pos != std::wstring::npos && rwx.back() == ')') {
+		// MLSD permissions:
+		//   foo (0644)
+		return DoConvertPermissions(rwx.substr(pos + 1, rwx.size() - pos - 2), permissions);
+	}
+
+	return DoConvertPermissions(rwx, permissions);
+}
+
+bool ChmodData::DoConvertPermissions(std::wstring const& rwx, char* permissions)
+{
+	if (rwx.size() < 3) {
+		return false;
+	}
+	size_t i;
+	for (i = 0; i < rwx.size(); ++i) {
+		if (rwx[i] < '0' || rwx[i] > '9') {
+			break;
+		}
+	}
+	if (i == rwx.size()) {
+		// Mode, e.g. 0723
+		for (i = 0; i < 3; ++i) {
+			int m = rwx[rwx.size() - 3 + i] - '0';
+
+			for (int j = 0; j < 3; ++j) {
+				if (m & (4 >> j)) {
+					permissions[i * 3 + j] = 2;
+				}
+				else {
+					permissions[i * 3 + j] = 1;
+				}
+			}
+		}
+
+		return true;
+	}
+
+	unsigned char const permchars[3] = { 'r', 'w', 'x' };
+
+	if (rwx.size() != 10) {
+		return false;
+	}
+
+	for (int j = 0; j < 9; ++j) {
+		bool set = rwx[j + 1] == permchars[j % 3];
+		permissions[j] = set ? 2 : 1;
+	}
+	if (rwx[3] == 's') {
+		permissions[2] = 2;
+	}
+	if (rwx[6] == 's') {
+		permissions[5] = 2;
+	}
+	if (rwx[9] == 't') {
+		permissions[8] = 2;
+	}
+
+	return true;
+}
+
+
+std::wstring ChmodData::GetPermissions(const char* previousPermissions, bool dir)
+{
+	// Construct a new permission string
+
+	if (numeric_.size() < 3) {
+		return numeric_;
+	}
+
+	for (size_t i = numeric_.size() - 3; i < numeric_.size(); ++i) {
+		if ((numeric_[i] < '0' || numeric_[i] > '9') && numeric_[i] != 'x') {
+			return numeric_;
+		}
+	}
+
+	if (!previousPermissions) {
+		std::wstring ret = numeric_;
+		size_t const size = ret.size();
+		if (numeric_[size - 1] == 'x') {
+			ret[size - 1] = dir ? '5' : '4';
+		}
+		if (numeric_[size - 2] == 'x') {
+			ret[size - 2] = dir ? '5' : '4';
+		}
+		if (numeric_[size - 3] == 'x') {
+			ret[size - 3] = dir ? '7' : '6';
+		}
+		// Use default of  (0...0)755 for dirs and
+		// 644 for files
+		for (size_t i = 0; i < size - 3; ++i) {
+			if (numeric_[i] == 'x') {
+				ret[i] = '0';
+			}
+		}
+		return ret;
+	}
+
+	// 2 set, 1 unset, 0 keep
+
+	const char defaultPerms[9] = { 2, 2, 2, 2, 1, 2, 2, 1, 2 };
+	char perms[9];
+	memcpy(perms, permissions_, 9);
+
+	std::wstring permission = numeric_.substr(0, numeric_.size() - 3);
+	unsigned int k = 0;
+	for (unsigned int i = numeric_.size() - 3; i < numeric_.size(); ++i, ++k) {
+		for (unsigned int j = k * 3; j < k * 3 + 3; ++j) {
+			if (!perms[j]) {
+				if (previousPermissions[j]) {
+					perms[j] = previousPermissions[j];
+				}
+				else {
+					perms[j] = defaultPerms[j];
+				}
+			}
+		}
+		permission += fz::to_wstring((perms[k * 3] - 1) * 4 + (perms[k * 3 + 1] - 1) * 2 + (perms[k * 3 + 2] - 1) * 1);
+	}
+
+	return permission;
+}
+
+CChmodDialog::CChmodDialog(ChmodData & data)
+	: data_(data)
 {
 	for (int i = 0; i < 9; ++i) {
 		m_checkBoxes[i] = 0;
-		m_permissions[i] = 0;
 	}
 }
 
@@ -22,7 +154,7 @@ bool CChmodDialog::Create(wxWindow* parent, int fileCount, int dirCount,
 	m_noUserTextChange = false;
 	lastChangedNumeric = false;
 
-	memcpy(m_permissions, permissions, 9);
+	memcpy(data_.permissions_, permissions, 9);
 
 	SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
 	SetParent(parent);
@@ -139,14 +271,15 @@ void CChmodDialog::OnOK(wxCommandEvent&)
 	wxRadioButton* pApplyFiles = XRCCTRL(*this, "ID_APPLYFILES", wxRadioButton);
 	wxRadioButton* pApplyDirs = XRCCTRL(*this, "ID_APPLYDIRS", wxRadioButton);
 	if (pApplyFiles->GetValue()) {
-		m_applyType = 1;
+		data_.applyType_ = 1;
 	}
 	else if (pApplyDirs->GetValue()) {
-		m_applyType = 2;
+		data_.applyType_ = 2;
 	}
 	else {
-		m_applyType = 0;
+		data_.applyType_ = 0;
 	}
+	data_.numeric_ = xrc_call(*this, "ID_NUMERIC", &wxTextCtrl::GetValue).ToStdWstring();
 	EndModal(wxID_OK);
 }
 
@@ -164,73 +297,77 @@ void CChmodDialog::OnCheckboxClick(wxCommandEvent&)
 		{
 		default:
 		case wxCHK_UNDETERMINED:
-			m_permissions[i] = 0;
+			data_.permissions_[i] = 0;
 			break;
 		case wxCHK_UNCHECKED:
-			m_permissions[i] = 1;
+			data_.permissions_[i] = 1;
 			break;
 		case wxCHK_CHECKED:
-			m_permissions[i] = 2;
+			data_.permissions_[i] = 2;
 			break;
 		}
 	}
 
 	wxString numericValue;
 	for (int i = 0; i < 3; ++i) {
-		if (!m_permissions[i * 3] || !m_permissions[i * 3 + 1] || !m_permissions[i * 3 + 2]) {
+		if (!data_.permissions_[i * 3] || !data_.permissions_[i * 3 + 1] || !data_.permissions_[i * 3 + 2]) {
 			numericValue += 'x';
 			continue;
 		}
 
-		numericValue += wxString::Format(_T("%d"), (m_permissions[i * 3] - 1) * 4 + (m_permissions[i * 3 + 1] - 1) * 2 + (m_permissions[i * 3 + 2] - 1) * 1);
+		numericValue += wxString::Format(_T("%d"), (data_.permissions_[i * 3] - 1) * 4 + (data_.permissions_[i * 3 + 1] - 1) * 2 + (data_.permissions_[i * 3 + 2] - 1) * 1);
 	}
 
 	wxTextCtrl *pTextCtrl = XRCCTRL(*this, "ID_NUMERIC", wxTextCtrl);
 	wxString oldValue = pTextCtrl->GetValue();
 
 	m_noUserTextChange = true;
-	pTextCtrl->SetValue(oldValue.Left(oldValue.Length() - 3) + numericValue);
+	pTextCtrl->SetValue(oldValue.Left(oldValue.size() - 3) + numericValue);
 	m_noUserTextChange = false;
 	oldNumeric = numericValue;
 }
 
 void CChmodDialog::OnNumericChanged(wxCommandEvent&)
 {
-	if (m_noUserTextChange)
+	if (m_noUserTextChange) {
 		return;
+	}
 
 	lastChangedNumeric = true;
 
 	wxTextCtrl *pTextCtrl = XRCCTRL(*this, "ID_NUMERIC", wxTextCtrl);
 	wxString numeric = pTextCtrl->GetValue();
-	if (numeric.Length() < 3)
+	if (numeric.size() < 3) {
 		return;
+	}
 
 	numeric = numeric.Right(3);
 	for (int i = 0; i < 3; ++i) {
-		if ((numeric[i] < '0' || numeric[i] > '9') && numeric[i] != 'x')
+		if ((numeric[i] < '0' || numeric[i] > '9') && numeric[i] != 'x') {
 			return;
+		}
 	}
 	for (int i = 0; i < 3; ++i) {
-		if (!oldNumeric.empty() && numeric[i] == oldNumeric[i])
+		if (!oldNumeric.empty() && numeric[i] == oldNumeric[i]) {
 			continue;
+		}
 		if (numeric[i] == 'x') {
-			m_permissions[i * 3] = 0;
-			m_permissions[i * 3 + 1] = 0;
-			m_permissions[i * 3 + 2] = 0;
+			data_.permissions_[i * 3] = 0;
+			data_.permissions_[i * 3 + 1] = 0;
+			data_.permissions_[i * 3 + 2] = 0;
 		}
 		else {
 			int value = numeric[i] - '0';
-			m_permissions[i * 3] = (value & 4) ? 2 : 1;
-			m_permissions[i * 3 + 1] = (value & 2) ? 2 : 1;
-			m_permissions[i * 3 + 2] = (value & 1) ? 2 : 1;
+			data_.permissions_[i * 3] = (value & 4) ? 2 : 1;
+			data_.permissions_[i * 3 + 1] = (value & 2) ? 2 : 1;
+			data_.permissions_[i * 3 + 2] = (value & 1) ? 2 : 1;
 		}
 	}
 
 	oldNumeric = numeric;
 
 	for (int i = 0; i < 9; ++i) {
-		switch (m_permissions[i])
+		switch (data_.permissions_[i])
 		{
 		default:
 		case 0:
@@ -246,55 +383,6 @@ void CChmodDialog::OnNumericChanged(wxCommandEvent&)
 	}
 }
 
-wxString CChmodDialog::GetPermissions(const char* previousPermissions, bool dir)
-{
-	// Construct a new permission string
-
-	wxTextCtrl *pTextCtrl = XRCCTRL(*this, "ID_NUMERIC", wxTextCtrl);
-	wxString numeric = pTextCtrl->GetValue();
-	if (numeric.Length() < 3)
-		return numeric;
-
-	for (unsigned int i = numeric.Length() - 3; i < numeric.Length(); ++i) {
-		if ((numeric[i] < '0' || numeric[i] > '9') && numeric[i] != 'x')
-			return numeric;
-	}
-	if (!previousPermissions) {
-		// Use default of  (0...0)755 for dirs and
-		// 644 for files
-		if (numeric[numeric.Length() - 1] == 'x')
-			numeric[numeric.Length() - 1] = dir ? '5' : '4';
-		if (numeric[numeric.Length() - 2] == 'x')
-			numeric[numeric.Length() - 2] = dir ? '5' : '4';
-		if (numeric[numeric.Length() - 3] == 'x')
-			numeric[numeric.Length() - 3] = dir ? '7' : '6';
-		numeric.Replace(_T("x"), _T("0"));
-		return numeric;
-	}
-
-	// 2 set, 1 unset, 0 keep
-
-	const char defaultPerms[9] = { 2, 2, 2, 2, 1, 2, 2, 1, 2 };
-	char perms[9];
-	memcpy(perms, m_permissions, 9);
-
-	wxString permission = numeric.Left(numeric.Length() - 3);
-	unsigned int k = 0;
-	for (unsigned int i = numeric.Length() - 3; i < numeric.Length(); ++i, ++k) {
-		for (unsigned int j = k * 3; j < k * 3 + 3; ++j) {
-			if (!perms[j]) {
-				if (previousPermissions[j])
-					perms[j] = previousPermissions[j];
-				else
-					perms[j] = defaultPerms[j];
-			}
-		}
-		permission += wxString::Format(_T("%d"), (int)(perms[k * 3] - 1) * 4 + (perms[k * 3 + 1] - 1) * 2 + (perms[k * 3 + 2] - 1) * 1);
-	}
-
-	return permission;
-}
-
 bool CChmodDialog::Recursive() const
 {
 	return m_recursive;
@@ -310,57 +398,3 @@ void CChmodDialog::OnRecurseChanged(wxCommandEvent&)
 	pApplyFiles->Enable(pRecurse->GetValue());
 	pApplyDirs->Enable(pRecurse->GetValue());
 }
-
-bool CChmodDialog::ConvertPermissions(wxString rwx, char* permissions)
-{
-	if (!permissions)
-		return false;
-
-	int pos = rwx.Find('(');
-	if (pos != -1 && rwx.Last() == ')') {
-		// MLSD permissions:
-		//   foo (0644)
-		rwx.RemoveLast();
-		rwx = rwx.Mid(pos + 1);
-	}
-
-	if (rwx.Len() < 3)
-		return false;
-	size_t i;
-	for (i = 0; i < rwx.Len(); i++)
-		if (rwx[i] < '0' || rwx[i] > '9')
-			break;
-	if (i == rwx.Len()) {
-		// Mode, e.g. 0723
-		for (i = 0; i < 3; ++i) {
-			int m = rwx[rwx.Len() - 3 + i] - '0';
-
-			for (int j = 0; j < 3; ++j) {
-				if (m & (4 >> j))
-					permissions[i * 3 + j] = 2;
-				else
-					permissions[i * 3 + j] = 1;
-			}
-		}
-
-		return true;
-	}
-
-	const unsigned char permchars[3] = {'r', 'w', 'x'};
-
-	if (rwx.Length() != 10)
-		return false;
-
-	for (int j = 0; j < 9; ++j) {
-		bool set = rwx[j + 1] == permchars[j % 3];
-		permissions[j] = set ? 2 : 1;
-	}
-	if (rwx[3] == 's')
-		permissions[2] = 2;
-	if (rwx[6] == 's')
-		permissions[5] = 2;
-	if (rwx[9] == 't')
-		permissions[8] = 2;
-
-	return true;
-}
