@@ -150,7 +150,7 @@ void CCommandQueue::ProcessReply(int nReplyCode, Command commandId)
 			return;
 		}
 		if (nReplyCode & FZ_REPLY_PASSWORDFAILED) {
-			CLoginManager::Get().CachedPasswordFailed(m_state.GetServer().server);
+			CLoginManager::Get().CachedPasswordFailed(m_state.GetSite().server);
 		}
 	}
 
@@ -176,9 +176,9 @@ void CCommandQueue::ProcessReply(int nReplyCode, Command commandId)
 
 		if (reconnect) {
 			// Try automatic reconnect
-			ServerWithCredentials const& server = m_state.GetServer();
-			if (server) {
-				m_CommandList.emplace_front(normal, std::make_unique<CConnectCommand>(server.server, server.credentials));
+			Site const& site = m_state.GetSite();
+			if (site) {
+				m_CommandList.emplace_front(normal, std::make_unique<CConnectCommand>(site.server, site.Handle(), site.credentials));
 				ProcessNextCommand();
 				return;
 			}
@@ -319,14 +319,14 @@ void CCommandQueue::ProcessDirectoryListing(CDirectoryListingNotification const&
 
 	if (listingIsRecursive) {
 		if (listingNotification.Primary() && m_state.GetRemoteRecursiveOperation()->IsActive()) {
-			m_state.NotifyHandlers(STATECHANGE_REMOTE_DIR_OTHER, wxString(), &pListing);
+			m_state.NotifyHandlers(STATECHANGE_REMOTE_DIR_OTHER, std::wstring(), &pListing);
 		}
 	}
 	else {
 		m_state.SetRemoteDir(pListing, listingNotification.Primary());
 	}
 
-	if (pListing && !listingNotification.Failed() && m_state.GetServer()) {
-		CContextManager::Get()->ProcessDirectoryListing(m_state.GetServer().server, pListing, listingIsRecursive ? 0 : &m_state);
+	if (pListing && !listingNotification.Failed() && m_state.GetSite()) {
+		CContextManager::Get()->ProcessDirectoryListing(m_state.GetSite().server, pListing, listingIsRecursive ? 0 : &m_state);
 	}
 }
