@@ -1,5 +1,6 @@
 #include <filezilla.h>
 #include "conditionaldialog.h"
+#include "dialogex.h"
 #include "filezillaapp.h"
 #include "wrapengine.h"
 #include "Options.h"
@@ -8,9 +9,11 @@ BEGIN_EVENT_TABLE(CConditionalDialog, wxDialog)
 EVT_BUTTON(wxID_ANY, CConditionalDialog::OnButton)
 END_EVENT_TABLE()
 
-CConditionalDialog::CConditionalDialog(wxWindow* parent, DialogType type, Modes mode, bool checked /*=false*/)
-	: wxDialog(parent, wxID_ANY, _T(""), wxDefaultPosition), m_type(type)
+CConditionalDialog::CConditionalDialog(wxWindow* parent, DialogType type, Modes mode, bool checked)
+	: wxDialog(parent, wxID_ANY, wxString(), wxDefaultPosition), m_type(type)
 {
+	DialogLayout layout(this);
+
 	wxSizer* pVertSizer = new wxBoxSizer(wxVERTICAL);
 
 	wxSizer* pMainSizer = new wxBoxSizer(wxHORIZONTAL);
@@ -19,7 +22,7 @@ CConditionalDialog::CConditionalDialog(wxWindow* parent, DialogType type, Modes
 	pMainSizer->AddSpacer(5);
 	wxSizer* pSizer = new wxBoxSizer(wxVERTICAL);
 	pMainSizer->Add(pSizer, 0, wxALL, 5);
-	m_pTextSizer = new wxFlexGridSizer(1, 5, 6);
+	m_pTextSizer = layout.createFlex(1);
 	pSizer->Add(m_pTextSizer, 0, wxTOP, 5);
 
 	wxCheckBox *pCheckBox = new wxCheckBox(this, wxID_HIGHEST + 1, _("&Don't show this dialog again."));
@@ -55,8 +58,9 @@ CConditionalDialog::CConditionalDialog(wxWindow* parent, DialogType type, Modes
 bool CConditionalDialog::Run()
 {
 	wxString dialogs = COptions::Get()->GetOption(OPTION_ONETIME_DIALOGS);
-	if (dialogs.Len() > (unsigned int)m_type && dialogs[m_type] == '1')
+	if (dialogs.size() > static_cast<size_t>(m_type) && dialogs[m_type] == '1') {
 		return true;
+	}
 
 	Fit();
 	wxGetApp().GetWrapEngine()->WrapRecursive(this, 3);
@@ -67,14 +71,16 @@ bool CConditionalDialog::Run()
 
 	auto cb = dynamic_cast<wxCheckBox*>(FindWindow(wxID_HIGHEST + 1));
 	if (cb && cb->GetValue()) {
-		while (dialogs.Len() <= (unsigned int)m_type)
+		while (dialogs.size() <= static_cast<size_t>(m_type)) {
 			dialogs += _T("0");
+		}
 		dialogs[m_type] = '1';
 		COptions::Get()->SetOption(OPTION_ONETIME_DIALOGS, dialogs.ToStdWstring());
 	}
 
-	if (id == wxID_OK || id == wxID_YES)
+	if (id == wxID_OK || id == wxID_YES) {
 		return true;
+	}
 
 	return false;
 }
