@@ -1,5 +1,7 @@
-#ifndef __CONDITIONALDIALOG_H__
-#define __CONDITIONALDIALOG_H__
+#ifndef FILEZILLA_INTERFACE_CONDITIONALDIALOG_HEADER
+#define FILEZILLA_INTERFACE_CONDITIONALDIALOG_HEADER
+
+#include <wx/dialog.h>
 
 class CConditionalDialog final : public wxDialog
 {
@@ -20,7 +22,8 @@ public:
 		confirm_preserve_timestamps,
 		compare_treeviewmismatch,
 		compare_changesorting,
-		many_selected_for_edit
+		many_selected_for_edit,
+		quick_search
 	};
 
 	CConditionalDialog(wxWindow* parent, DialogType type, Modes mode, bool checked = false);
@@ -38,4 +41,4 @@ protected:
 	void OnButton(wxCommandEvent& event);
 };
 
-#endif //__CONDITIONALDIALOG_H__
+#endif
