@@ -56,7 +56,7 @@ void CContextControl::Create(wxWindow *parent)
 	wxSplitterWindow::Create(parent, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxSP_NOBORDER);
 }
 
-void CContextControl::CreateTab()
+bool CContextControl::CreateTab()
 {
 	CLocalPath localPath;
 	Site site;
@@ -68,13 +68,18 @@ void CContextControl::CreateTab()
 		site = controls->pState->GetLastSite();
 		remotePath = controls->pState->GetLastServerPath();
 	}
-	CreateTab(localPath, site, remotePath);
+	return CreateTab(localPath, site, remotePath);
 }
 
-void CContextControl::CreateTab(CLocalPath const& localPath, Site const& site, CServerPath const& remotePath)
+bool CContextControl::CreateTab(CLocalPath const& localPath, Site const& site, CServerPath const& remotePath)
 {
 	wxGetApp().AddStartupProfileRecord("CContextControl::CreateTab");
 
+	if (GetTabCount() >= 200) {
+		wxBell();
+		return false;
+	}
+
 	{
 	#ifdef __WXMSW__
 		// Some reparenting is being done when creating tabs. Reparenting of frozen windows isn't working
@@ -105,15 +110,16 @@ void CContextControl::CreateTab(CLocalPath const& localPath, Site const& site, C
 		}
 		if (!pState) {
 			pState = CContextManager::Get()->CreateState(m_mainFrame);
-			if (!pState->CreateEngine()) {
-				wxMessageBoxEx(_("Failed to initialize FTP engine"));
+			if (!pState) {
+				return false;
 			}
 		}
-
+	
 		pState->SetLastSite(site, remotePath);
 
 		CreateContextControls(*pState);
 
+
 		pState->GetLocalRecursiveOperation()->SetQueue(m_mainFrame.GetQueue());
 		pState->GetRemoteRecursiveOperation()->SetQueue(m_mainFrame.GetQueue());
 
@@ -141,6 +147,8 @@ void CContextControl::CreateTab(CLocalPath const& localPath, Site const& site, C
 	if (m_tabs) {
 		m_tabs->SetSelection(m_tabs->GetPageCount() - 1);
 	}
+
+	return true;
 }
 
 void CContextControl::CreateContextControls(CState& state)
@@ -224,7 +232,9 @@ void CContextControl::CreateContextControls(CState& state)
 		pLocalFilelistStatusBar->Hide();
 	}
 	context_controls.pLocalListViewPanel->SetStatusBar(pLocalFilelistStatusBar);
-	context_controls.pLocalListView->SetFilelistStatusBar(pLocalFilelistStatusBar);
+	if (context_controls.pLocalListView) {
+		context_controls.pLocalListView->SetFilelistStatusBar(pLocalFilelistStatusBar);
+	}
 	pLocalFilelistStatusBar->SetConnected(true);
 
 	CFilelistStatusBar* pRemoteFilelistStatusBar = new CFilelistStatusBar(context_controls.pRemoteListViewPanel);
@@ -232,7 +242,9 @@ void CContextControl::CreateContextControls(CState& state)
 		pRemoteFilelistStatusBar->Hide();
 	}
 	context_controls.pRemoteListViewPanel->SetStatusBar(pRemoteFilelistStatusBar);
-	context_controls.pRemoteListView->SetFilelistStatusBar(pRemoteFilelistStatusBar);
+	if (context_controls.pRemoteListView) {
+		context_controls.pRemoteListView->SetFilelistStatusBar(pRemoteFilelistStatusBar);
+	}
 
 	auto localRecursiveStatus = new CRecursiveOperationStatus(context_controls.pLocalListViewPanel, state, true);
 	context_controls.pLocalListViewPanel->SetFooter(localRecursiveStatus);
@@ -322,7 +334,9 @@ void CContextControl::CreateContextControls(CState& state)
 	m_mainFrame.ConnectNavigationHandler(context_controls.pLocalViewHeader);
 	m_mainFrame.ConnectNavigationHandler(context_controls.pRemoteViewHeader);
 
-	state.GetComparisonManager()->SetListings(context_controls.pLocalListView, context_controls.pRemoteListView);
+	if (context_controls.pLocalListView && context_controls.pRemoteListView) {
+		state.GetComparisonManager()->SetListings(context_controls.pLocalListView, context_controls.pRemoteListView);
+	}
 
 	if (m_tabs) {
 		m_tabs->AddPage(context_controls.pViewSplitter, state.GetTitle());
@@ -592,7 +606,7 @@ void CContextControl::AdvanceTab(bool forward)
 	m_tabs->AdvanceTab(forward);
 }
 
-void CContextControl::OnStateChange(CState* pState, t_statechange_notifications notification, const wxString&, const void*)
+void CContextControl::OnStateChange(CState* pState, t_statechange_notifications notification, std::wstring const&, const void*)
 {
 	if (notification == STATECHANGE_CHANGEDCONTEXT) {
 		if (!pState) {
@@ -650,8 +664,8 @@ void CContextControl::SaveTabs()
 		Site const site = controls->pState->GetLastSite();
 
 		auto tab = tabs.append_child("Tab");
-		SetServer(tab, site.server_);
-		tab.append_child("Site").text().set(fz::to_utf8(site.m_path).c_str());
+		SetServer(tab, site);
+		tab.append_child("Site").text().set(fz::to_utf8(site.SitePath()).c_str());
 		tab.append_child("RemotePath").text().set(fz::to_utf8(controls->pState->GetLastServerPath().GetSafePath()).c_str());
 		tab.append_child("LocalPath").text().set(fz::to_utf8(controls->pState->GetLocalDir().GetPath()).c_str());
 
@@ -697,17 +711,15 @@ void CContextControl::RestoreTabs()
 			CLocalPath localPath(fz::to_wstring_from_utf8(tab.child("LocalPath").child_value()));
 
 			Site site;
-
-			ServerWithCredentials last_server;
 			CServerPath last_path;
 
-			if (GetServer(tab, site.server_) && last_path.SetSafePath(fz::to_wstring_from_utf8(tab.child("RemotePath").child_value()))) {
+			if (GetServer(tab, site) && last_path.SetSafePath(fz::to_wstring_from_utf8(tab.child("RemotePath").child_value()))) {
 				std::wstring last_site_path = fz::to_wstring_from_utf8(tab.child("Site").child_value());
 
 				std::unique_ptr<Site> ssite;
 				if (!last_site_path.empty()) {
 					auto ssite = CSiteManager::GetSiteByPath(last_site_path, false).first;
-					if (ssite && ssite->server_ == site.server_) {
+					if (ssite && ssite->SameResource(site)) {
 						site = *ssite;
 					}
 				}
