@@ -51,8 +51,8 @@ public:
 
 	void Create(wxWindow* parent);
 
-	void CreateTab();
-	void CreateTab(CLocalPath const& localPath, Site const& site, CServerPath const& remotePath);
+	bool CreateTab();
+	bool CreateTab(CLocalPath const& localPath, Site const& site, CServerPath const& remotePath);
 	bool CloseTab(int tab);
 
 	_context_controls* GetCurrentControls();
@@ -91,7 +91,7 @@ protected:
 	void OnTabContextCloseOthers(wxCommandEvent& event);
 	void OnTabContextNew(wxCommandEvent&);
 
-	virtual void OnStateChange(CState* pState, t_statechange_notifications notification, const wxString&, const void*);
+	virtual void OnStateChange(CState* pState, t_statechange_notifications notification, std::wstring const&, const void*) override;
 };
 
 #endif
