@@ -251,9 +251,9 @@ CEditHandler::fileState CEditHandler::GetFileState(wxString const& fileName) con
 	return iter->state;
 }
 
-CEditHandler::fileState CEditHandler::GetFileState(wxString const& fileName, CServerPath const& remotePath, ServerWithCredentials const& server) const
+CEditHandler::fileState CEditHandler::GetFileState(wxString const& fileName, CServerPath const& remotePath, Site const& site) const
 {
-	std::list<t_fileData>::const_iterator iter = GetFile(fileName, remotePath, server);
+	std::list<t_fileData>::const_iterator iter = GetFile(fileName, remotePath, site);
 	if (iter == m_fileDataList[remote].end()) {
 		return unknown;
 	}
@@ -261,11 +261,11 @@ CEditHandler::fileState CEditHandler::GetFileState(wxString const& fileName, CSe
 	return iter->state;
 }
 
-int CEditHandler::GetFileCount(CEditHandler::fileType type, CEditHandler::fileState state, ServerWithCredentials const& server) const
+int CEditHandler::GetFileCount(CEditHandler::fileType type, CEditHandler::fileState state, Site const& site) const
 {
 	int count = 0;
 	if (state == unknown) {
-		wxASSERT(!server);
+		wxASSERT(!site);
 		if (type != remote) {
 			count += m_fileDataList[local].size();
 		}
@@ -274,14 +274,14 @@ int CEditHandler::GetFileCount(CEditHandler::fileType type, CEditHandler::fileSt
 		}
 	}
 	else {
-		auto f = [state, &server](decltype(m_fileDataList[0]) & items) {
+		auto f = [state, &site](decltype(m_fileDataList[0]) & items) {
 			int cnt = 0;
 			for (auto const& data : items) {
 				if (data.state != state) {
 					continue;
 				}
 
-				if (!server || data.server == server) {
+				if (!site || data.site == site) {
 					++cnt;
 				}
 			}
@@ -298,7 +298,7 @@ int CEditHandler::GetFileCount(CEditHandler::fileType type, CEditHandler::fileSt
 	return count;
 }
 
-bool CEditHandler::AddFile(CEditHandler::fileType type, std::wstring& fileName, CServerPath const& remotePath, ServerWithCredentials const& server)
+bool CEditHandler::AddFile(CEditHandler::fileType type, std::wstring& fileName, CServerPath const& remotePath, Site const& site)
 {
 	wxASSERT(type != none);
 
@@ -307,7 +307,7 @@ bool CEditHandler::AddFile(CEditHandler::fileType type, std::wstring& fileName,
 		state = GetFileState(fileName);
 	}
 	else {
-		state = GetFileState(fileName, remotePath, server);
+		state = GetFileState(fileName, remotePath, site);
 	}
 	if (state != unknown) {
 		wxFAIL_MSG(_T("File state not unknown"));
@@ -327,7 +327,7 @@ bool CEditHandler::AddFile(CEditHandler::fileType type, std::wstring& fileName,
 		data.state = edit;
 	}
 	data.remotePath = remotePath;
-	data.server = server;
+	data.site = site;
 
 	if (type == local && !COptions::Get()->GetOptionVal(OPTION_EDIT_TRACK_LOCAL)) {
 		return StartEditing(local, data);
@@ -357,9 +357,9 @@ bool CEditHandler::Remove(const wxString& fileName)
 	return true;
 }
 
-bool CEditHandler::Remove(wxString const& fileName, CServerPath const& remotePath, ServerWithCredentials const& server)
+bool CEditHandler::Remove(wxString const& fileName, CServerPath const& remotePath, Site const& site)
 {
-	std::list<t_fileData>::iterator iter = GetFile(fileName, remotePath, server);
+	std::list<t_fileData>::iterator iter = GetFile(fileName, remotePath, site);
 	if (iter == m_fileDataList[remote].end()) {
 		return true;
 	}
@@ -417,7 +417,7 @@ bool CEditHandler::RemoveAll(bool force)
 	return m_fileDataList[local].empty() && m_fileDataList[remote].empty();
 }
 
-bool CEditHandler::RemoveAll(fileState state, ServerWithCredentials const& server)
+bool CEditHandler::RemoveAll(fileState state, Site const& site)
 {
 	// Others not implemented
 	wxASSERT(state == upload_and_remove_failed);
@@ -433,7 +433,7 @@ bool CEditHandler::RemoveAll(fileState state, ServerWithCredentials const& serve
 			continue;
 		}
 
-		if (server && iter->server != server) {
+		if (site && iter->site != site) {
 			keep.push_back(*iter);
 			continue;
 		}
@@ -475,7 +475,7 @@ std::list<CEditHandler::t_fileData>::const_iterator CEditHandler::GetFile(const
 	return iter;
 }
 
-std::list<CEditHandler::t_fileData>::iterator CEditHandler::GetFile(wxString const& fileName, CServerPath const& remotePath, ServerWithCredentials const& server)
+std::list<CEditHandler::t_fileData>::iterator CEditHandler::GetFile(wxString const& fileName, CServerPath const& remotePath, Site const& site)
 {
 	std::list<t_fileData>::iterator iter;
 	for (iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); ++iter) {
@@ -483,7 +483,7 @@ std::list<CEditHandler::t_fileData>::iterator CEditHandler::GetFile(wxString con
 			continue;
 		}
 
-		if (iter->server != server) {
+		if (iter->site != site) {
 			continue;
 		}
 
@@ -497,7 +497,7 @@ std::list<CEditHandler::t_fileData>::iterator CEditHandler::GetFile(wxString con
 	return iter;
 }
 
-std::list<CEditHandler::t_fileData>::const_iterator CEditHandler::GetFile(wxString const& fileName, CServerPath const& remotePath, ServerWithCredentials const& server) const
+std::list<CEditHandler::t_fileData>::const_iterator CEditHandler::GetFile(wxString const& fileName, CServerPath const& remotePath, Site const& site) const
 {
 	std::list<t_fileData>::const_iterator iter;
 	for (iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); ++iter) {
@@ -505,7 +505,7 @@ std::list<CEditHandler::t_fileData>::const_iterator CEditHandler::GetFile(wxStri
 			continue;
 		}
 
-		if (iter->server != server) {
+		if (iter->site != site) {
 			continue;
 		}
 
@@ -548,9 +548,9 @@ void CEditHandler::FinishTransfer(bool, wxString const& fileName)
 	SetTimerState();
 }
 
-void CEditHandler::FinishTransfer(bool successful, wxString const& fileName, CServerPath const& remotePath, ServerWithCredentials const& server)
+void CEditHandler::FinishTransfer(bool successful, wxString const& fileName, CServerPath const& remotePath, Site const& site)
 {
-	auto iter = GetFile(fileName, remotePath, server);
+	auto iter = GetFile(fileName, remotePath, site);
 	if (iter == m_fileDataList[remote].end()) {
 		return;
 	}
@@ -616,9 +616,9 @@ bool CEditHandler::StartEditing(wxString const& file)
 	return StartEditing(local, *iter);
 }
 
-bool CEditHandler::StartEditing(wxString const& file, CServerPath const& remotePath, ServerWithCredentials const& server)
+bool CEditHandler::StartEditing(wxString const& file, CServerPath const& remotePath, Site const& site)
 {
-	auto iter = GetFile(file, remotePath, server);
+	auto iter = GetFile(file, remotePath, site);
 	if (iter == m_fileDataList[remote].end()) {
 		return false;
 	}
@@ -632,16 +632,19 @@ bool CEditHandler::StartEditing(CEditHandler::fileType type, t_fileData& data)
 	wxASSERT(data.state == edit);
 
 	bool is_link;
-	if (fz::local_filesys::get_file_info(fz::to_native(data.file), is_link, 0, &data.modificationTime, 0) != fz::local_filesys::file)
+	if (fz::local_filesys::get_file_info(fz::to_native(data.file), is_link, 0, &data.modificationTime, 0) != fz::local_filesys::file) {
 		return false;
+	}
 
 	bool program_exists = false;
 	wxString cmd = GetOpenCommand(data.file, program_exists);
-	if (cmd.empty() || !program_exists)
+	if (cmd.empty() || !program_exists) {
 		return false;
+	}
 
-	if (!wxExecute(cmd))
+	if (!wxExecute(cmd)) {
 		return false;
+	}
 
 	return true;
 }
@@ -662,8 +665,9 @@ void CEditHandler::CheckForModifications(bool emitEvent)
 	for (int i = 0; i < 2; ++i) {
 checkmodifications_loopbegin:
 		for (auto iter = m_fileDataList[i].begin(); iter != m_fileDataList[i].end(); ++iter) {
-			if (iter->state != edit)
+			if (iter->state != edit) {
 				continue;
+			}
 
 			fz::datetime mtime;
 			bool is_link;
@@ -674,16 +678,18 @@ checkmodifications_loopbegin:
 				goto checkmodifications_loopbegin;
 			}
 
-			if (mtime.empty())
+			if (mtime.empty()) {
 				continue;
+			}
 
-			if (!iter->modificationTime.empty() && !iter->modificationTime.compare(mtime))
+			if (!iter->modificationTime.empty() && !iter->modificationTime.compare(mtime)) {
 				continue;
+			}
 
 			// File has changed, ask user what to do
 
 			m_busyTimer.Stop();
-			if (!wxDialogEx::CanShowPopupDialog() ) {
+			if (!wxDialogEx::CanShowPopupDialog()) {
 				m_busyTimer.Start(1000, true);
 				insideCheckForModifications = false;
 				return;
@@ -697,8 +703,9 @@ checkmodifications_loopbegin:
 
 			bool remove;
 			int res = DisplayChangeNotification(CEditHandler::fileType(i), iter, remove);
-			if (res == -1)
+			if (res == -1) {
 				continue;
+			}
 
 			if (res == wxID_YES) {
 				UploadFile(CEditHandler::fileType(i), iter, remove);
@@ -721,8 +728,9 @@ checkmodifications_loopbegin:
 				m_fileDataList[i].erase(iter);
 				goto checkmodifications_loopbegin;
 			}
-			else
+			else {
 				iter->modificationTime = mtime;
+			}
 		}
 	}
 
@@ -734,12 +742,15 @@ checkmodifications_loopbegin:
 int CEditHandler::DisplayChangeNotification(CEditHandler::fileType type, std::list<CEditHandler::t_fileData>::const_iterator iter, bool& remove)
 {
 	CChangedFileDialog dlg;
-	if (!dlg.Load(wxTheApp->GetTopWindow(), _T("ID_CHANGEDFILE")))
+	if (!dlg.Load(wxTheApp->GetTopWindow(), _T("ID_CHANGEDFILE"))) {
 		return -1;
-	if (type == remote)
+	}
+	if (type == remote) {
 		XRCCTRL(dlg, "ID_DESC_UPLOAD_LOCAL", wxStaticText)->Hide();
-	else
+	}
+	else {
 		XRCCTRL(dlg, "ID_DESC_UPLOAD_REMOTE", wxStaticText)->Hide();
+	}
 
 	dlg.SetChildLabel(XRCID("ID_FILENAME"), iter->name);
 
@@ -759,11 +770,12 @@ int CEditHandler::DisplayChangeNotification(CEditHandler::fileType type, std::li
 			XRCCTRL(dlg, "ID_DESC_OPENEDAS", wxStaticText)->Hide();
 			XRCCTRL(dlg, "ID_OPENEDAS", wxStaticText)->Hide();
 		}
-		else
+		else {
 			dlg.SetChildLabel(XRCID("ID_OPENEDAS"), file);
+		}
 	}
 
-	dlg.SetChildLabel(XRCID("ID_SERVER"), iter->server.Format(ServerFormat::with_user_and_optional_port));
+	dlg.SetChildLabel(XRCID("ID_SERVER"), iter->site.Format(ServerFormat::with_user_and_optional_port));
 	dlg.SetChildLabel(XRCID("ID_REMOTEPATH"), iter->remotePath.GetPath());
 
 	dlg.GetSizer()->Fit(&dlg);
@@ -775,9 +787,9 @@ int CEditHandler::DisplayChangeNotification(CEditHandler::fileType type, std::li
 	return res;
 }
 
-bool CEditHandler::UploadFile(wxString const& file, CServerPath const& remotePath, ServerWithCredentials const& server, bool unedit)
+bool CEditHandler::UploadFile(wxString const& file, CServerPath const& remotePath, Site const& site, bool unedit)
 {
-	std::list<t_fileData>::iterator iter = GetFile(file, remotePath, server);
+	std::list<t_fileData>::iterator iter = GetFile(file, remotePath, site);
 	return UploadFile(remote, iter, unedit);
 }
 
@@ -826,7 +838,7 @@ bool CEditHandler::UploadFile(fileType type, std::list<t_fileData>::iterator ite
 		return false;
 	}
 
-	m_pQueue->QueueFile(false, false, file, iter->name, localPath, iter->remotePath, iter->server, size, type, QueuePriority::high);
+	m_pQueue->QueueFile(false, false, file, iter->name, localPath, iter->remotePath, iter->site, size, type, QueuePriority::high);
 	m_pQueue->QueueFile_Finish(true);
 
 	return true;
@@ -842,7 +854,7 @@ void CEditHandler::OnTimerEvent(wxTimerEvent&)
 	}
 #endif
 
-	CheckForModifications();
+	CheckForModifications(true);
 }
 
 void CEditHandler::SetTimerState()
@@ -864,76 +876,112 @@ wxString CEditHandler::CanOpen(CEditHandler::fileType type, const wxString& file
 	wxASSERT(type != none);
 
 	wxString command = GetOpenCommand(fileName, program_exists);
-	if (command.empty() || !program_exists)
+	if (command.empty() || !program_exists) {
 		return command;
+	}
 
 	wxFileName fn;
-	if (type == remote)
+	if (type == remote) {
 		fn = wxFileName(m_localDir, fileName);
-	else
+	}
+	else {
 		fn = wxFileName(fileName);
+	}
 
 	wxString name = fn.GetFullPath();
 	wxString tmp = command;
 	wxString args;
-	if (UnquoteCommand(tmp, args) && tmp == name)
+	if (UnquoteCommand(tmp, args) && tmp == name) {
 		dangerous = true;
-	else
+	}
+	else {
 		dangerous = false;
+	}
 
 	return command;
 }
 
 wxString CEditHandler::GetOpenCommand(const wxString& file, bool& program_exists)
 {
+#ifdef __WXMSW__
+	if (file.find('"') != std::wstring::npos) {
+		// Windows doesn't allow double-quotes in filenames.
+		// On top if it, wxExecute does not properly deal with quotes preceeded by backslashes.
+		// This is the safe choice.
+		return wxString();
+	}
+#endif
+
 	if (!COptions::Get()->GetOptionVal(OPTION_EDIT_ALWAYSDEFAULT)) {
 		const wxString command = GetCustomOpenCommand(file, program_exists);
-		if (!command.empty())
+		if (!command.empty()) {
 			return command;
+		}
 
 		if (COptions::Get()->GetOptionVal(OPTION_EDIT_INHERITASSOCIATIONS)) {
 			const wxString sysCommand = GetSystemOpenCommand(file, program_exists);
-			if (!sysCommand.empty())
+			if (!sysCommand.empty()) {
 				return sysCommand;
+			}
 		}
 	}
 
 	wxString command = COptions::Get()->GetOption(OPTION_EDIT_DEFAULTEDITOR);
-	if (command.empty() || command[0] == '0')
+	if (command.empty() || command[0] == '0') {
 		return wxString(); // None set
+	}
 	else if (command[0] == '1') {
 		// Text editor
 		const wxString random = _T("5AC2EE515D18406 space aB77C2C60F1F88952.txt"); // Chosen by fair dice roll. Guaranteed to be random.
 		wxString sysCommand = GetSystemOpenCommand(random, program_exists);
-		if (sysCommand.empty() || !program_exists)
+		if (sysCommand.empty() || !program_exists) {
 			return sysCommand;
+		}
 
 		sysCommand.Replace(random, file);
 		return sysCommand;
 	}
-	else if (command[0] == '2')
+	else if (command[0] == '2') {
 		command = command.Mid(1);
+	}
 
-	if (command.empty())
+	if (command.empty()) {
 		return wxString();
+	}
 
 	wxString args;
 	wxString editor = command;
-	if (!UnquoteCommand(editor, args))
+	if (!UnquoteCommand(editor, args)) {
 		return wxString();
+	}
 
-	if (!ProgramExists(editor))
-	{
+	if (!ProgramExists(editor)) {
 		program_exists = false;
 		return editor;
 	}
 
 	program_exists = true;
+#ifndef __WXMSW__
+	wxString escaped = file;
+	escaped.Replace(L"\\", L"\\\\");
+	escaped.Replace(L"\"", L"\\\"");
+	return command + _T(" \"") + escaped + _T("\"");
+#else
 	return command + _T(" \"") + file + _T("\"");
+#endif
 }
 
 wxString CEditHandler::GetCustomOpenCommand(const wxString& file, bool& program_exists)
 {
+#ifdef __WXMSW__
+	if (file.find('"') != std::wstring::npos) {
+		// Windows doesn't allow double-quotes in filenames.
+		// On top if it, wxExecute does not properly deal with quotes preceeded by backslashes.
+		// This is the safe choice.
+		return wxString();
+	}
+#endif
+
 	wxFileName fn(file);
 
 	wxString ext = fn.GetExt();
@@ -983,7 +1031,13 @@ wxString CEditHandler::GetCustomOpenCommand(const wxString& file, bool& program_
 		}
 
 		program_exists = true;
-		return command + _T(" \"") + fn.GetFullPath() + _T("\"");
+
+		wxString arg = fn.GetFullPath();
+#ifndef __WXMSW__
+		arg.Replace(L"\\", L"\\\\");
+		arg.Replace(L"\"", L"\\\"");
+#endif
+		return command + L" \"" + arg + L"\"";
 	}
 
 	return wxString();
@@ -1087,8 +1141,9 @@ bool CEditHandler::FilenameExists(const wxString& file)
 			wxRemoveFile(file);
 		}
 
-		if (wxFileName::FileExists(file))
+		if (wxFileName::FileExists(file)) {
 			return true;
+		}
 	}
 
 	return false;
@@ -1124,21 +1179,23 @@ CEditHandlerStatusDialog::~CEditHandlerStatusDialog()
 int CEditHandlerStatusDialog::ShowModal()
 {
 	const CEditHandler* const pEditHandler = CEditHandler::Get();
-	if (!pEditHandler)
+	if (!pEditHandler) {
 		return wxID_CANCEL;
+	}
 
-	if (!pEditHandler->GetFileCount(CEditHandler::none, CEditHandler::unknown))
-	{
+	if (!pEditHandler->GetFileCount(CEditHandler::none, CEditHandler::unknown)) {
 		wxMessageBoxEx(_("No files are currently being edited."), _("Cannot show dialog"), wxICON_INFORMATION, m_pParent);
 		return wxID_CANCEL;
 	}
 
-	if (!Load(m_pParent, _T("ID_EDITING")))
+	if (!Load(m_pParent, _T("ID_EDITING"))) {
 		return wxID_CANCEL;
+	}
 
 	wxListCtrl* pListCtrl = XRCCTRL(*this, "ID_FILES", wxListCtrl);
-	if (!pListCtrl)
+	if (!pListCtrl) {
 		return wxID_CANCEL;
+	}
 
 	pListCtrl->InsertColumn(0, _("Filename"));
 	pListCtrl->InsertColumn(1, _("Type"));
@@ -1148,8 +1205,7 @@ int CEditHandlerStatusDialog::ShowModal()
 	{
 		const std::list<CEditHandler::t_fileData>& files = pEditHandler->GetFiles(CEditHandler::remote);
 		unsigned int i = 0;
-		for (std::list<CEditHandler::t_fileData>::const_iterator iter = files.begin(); iter != files.end(); ++iter, ++i)
-		{
+		for (std::list<CEditHandler::t_fileData>::const_iterator iter = files.begin(); iter != files.end(); ++iter, ++i) {
 			pListCtrl->InsertItem(i, iter->name);
 			pListCtrl->SetItem(i, COLUMN_TYPE, _("Remote"));
 			switch (iter->state)
@@ -1176,7 +1232,7 @@ int CEditHandlerStatusDialog::ShowModal()
 				pListCtrl->SetItem(i, COLUMN_STATUS, _("Unknown"));
 				break;
 			}
-			pListCtrl->SetItem(i, COLUMN_REMOTEPATH, iter->server.Format(ServerFormat::with_user_and_optional_port) + iter->remotePath.GetPath());
+			pListCtrl->SetItem(i, COLUMN_REMOTEPATH, iter->site.Format(ServerFormat::with_user_and_optional_port) + iter->remotePath.GetPath());
 			CEditHandler::t_fileData* pData = new CEditHandler::t_fileData(*iter);
 			pListCtrl->SetItemPtrData(i, (wxUIntPtr)pData);
 		}
@@ -1204,14 +1260,15 @@ int CEditHandlerStatusDialog::ShowModal()
 				pListCtrl->SetItem(i, COLUMN_STATUS, _("Unknown"));
 				break;
 			}
-			pListCtrl->SetItem(i, COLUMN_REMOTEPATH, iter->server.Format(ServerFormat::with_user_and_optional_port) + iter->remotePath.GetPath());
+			pListCtrl->SetItem(i, COLUMN_REMOTEPATH, iter->site.Format(ServerFormat::with_user_and_optional_port) + iter->remotePath.GetPath());
 			CEditHandler::t_fileData* pData = new CEditHandler::t_fileData(*iter);
 			pListCtrl->SetItemPtrData(i, (wxUIntPtr)pData);
 		}
 	}
 
-	for (int i = 0; i < 4; i++)
+	for (int i = 0; i < 4; ++i) {
 		pListCtrl->SetColumnWidth(i, wxLIST_AUTOSIZE);
+	}
 	pListCtrl->SetMinSize(wxSize(pListCtrl->GetColumnWidth(0) + pListCtrl->GetColumnWidth(1) + pListCtrl->GetColumnWidth(2) + pListCtrl->GetColumnWidth(3) + 10, pListCtrl->GetMinSize().GetHeight()));
 	GetSizer()->Fit(this);
 
@@ -1222,8 +1279,9 @@ int CEditHandlerStatusDialog::ShowModal()
 
 	int res = wxDialogEx::ShowModal();
 
-	for (int i = 0; i < pListCtrl->GetItemCount(); i++)
+	for (int i = 0; i < pListCtrl->GetItemCount(); ++i) {
 		delete (CEditHandler::t_fileData*)pListCtrl->GetItemData(i);
+	}
 
 	return res;
 }
@@ -1231,8 +1289,9 @@ int CEditHandlerStatusDialog::ShowModal()
 void CEditHandlerStatusDialog::SetCtrlState()
 {
 	const CEditHandler* const pEditHandler = CEditHandler::Get();
-	if (!pEditHandler)
+	if (!pEditHandler) {
 		return;
+	}
 
 	wxListCtrl* pListCtrl = XRCCTRL(*this, "ID_FILES", wxListCtrl);
 
@@ -1241,16 +1300,18 @@ void CEditHandlerStatusDialog::SetCtrlState()
 	bool selectedUploadRemoveFailed = false;
 
 	int item = -1;
-	while ((item = pListCtrl->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
-	{
+	while ((item = pListCtrl->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1) {
 		CEditHandler::fileType type;
 		CEditHandler::t_fileData* pData = GetDataFromItem(item, type);
-		if (pData->state == CEditHandler::edit)
+		if (pData->state == CEditHandler::edit) {
 			selectedEdited = true;
-		else if (pData->state == CEditHandler::upload_and_remove_failed)
+		}
+		else if (pData->state == CEditHandler::upload_and_remove_failed) {
 			selectedUploadRemoveFailed = true;
-		else
+		}
+		else {
 			selectedOther = true;
+		}
 	}
 
 	bool select = selectedEdited && !selectedOther && !selectedUploadRemoveFailed;
@@ -1268,8 +1329,9 @@ void CEditHandlerStatusDialog::OnSelectionChanged(wxListEvent&)
 void CEditHandlerStatusDialog::OnUnedit(wxCommandEvent&)
 {
 	CEditHandler* const pEditHandler = CEditHandler::Get();
-	if (!pEditHandler)
+	if (!pEditHandler) {
 		return;
+	}
 
 	wxListCtrl* pListCtrl = XRCCTRL(*this, "ID_FILES", wxListCtrl);
 
@@ -1299,12 +1361,13 @@ void CEditHandlerStatusDialog::OnUnedit(wxCommandEvent&)
 			pListCtrl->DeleteItem(i);
 		}
 		else {
-			if (pEditHandler->Remove(pData->name, pData->remotePath, pData->server)) {
+			if (pEditHandler->Remove(pData->name, pData->remotePath, pData->site)) {
 				delete pData;
 				pListCtrl->DeleteItem(i);
 			}
-			else
+			else {
 				pListCtrl->SetItem(i, COLUMN_STATUS, _("Pending removal"));
+			}
 		}
 	}
 
@@ -1314,8 +1377,9 @@ void CEditHandlerStatusDialog::OnUnedit(wxCommandEvent&)
 void CEditHandlerStatusDialog::OnUpload(wxCommandEvent& event)
 {
 	CEditHandler* const pEditHandler = CEditHandler::Get();
-	if (!pEditHandler)
+	if (!pEditHandler) {
 		return;
+	}
 
 	wxListCtrl* pListCtrl = XRCCTRL(*this, "ID_FILES", wxListCtrl);
 
@@ -1342,17 +1406,22 @@ void CEditHandlerStatusDialog::OnUpload(wxCommandEvent& event)
 
 		bool unedit = event.GetId() == XRCID("ID_UPLOADANDUNEDIT") || pData->state == CEditHandler::upload_and_remove_failed;
 
-		if (type == CEditHandler::local)
+		if (type == CEditHandler::local) {
 			pEditHandler->UploadFile(pData->file, unedit);
-		else
-			pEditHandler->UploadFile(pData->name, pData->remotePath, pData->server, unedit);
+		}
+		else {
+			pEditHandler->UploadFile(pData->name, pData->remotePath, pData->site, unedit);
+		}
 
-		if (!unedit)
+		if (!unedit) {
 			pListCtrl->SetItem(i, COLUMN_STATUS, _("Uploading"));
-		else if (type == CEditHandler::remote)
+		}
+		else if (type == CEditHandler::remote) {
 			pListCtrl->SetItem(i, COLUMN_STATUS, _("Uploading and pending removal"));
-		else
+		}
+		else {
 			pListCtrl->SetItem(i, COLUMN_STATUS, _("Uploading and unediting"));
+		}
 	}
 
 	SetCtrlState();
@@ -1361,8 +1430,9 @@ void CEditHandlerStatusDialog::OnUpload(wxCommandEvent& event)
 void CEditHandlerStatusDialog::OnEdit(wxCommandEvent&)
 {
 	CEditHandler* const pEditHandler = CEditHandler::Get();
-	if (!pEditHandler)
+	if (!pEditHandler) {
 		return;
+	}
 
 	wxListCtrl* pListCtrl = XRCCTRL(*this, "ID_FILES", wxListCtrl);
 
@@ -1393,18 +1463,20 @@ void CEditHandlerStatusDialog::OnEdit(wxCommandEvent&)
 					delete pData;
 					pListCtrl->DeleteItem(i);
 				}
-				else
+				else {
 					pListCtrl->SetItem(i, COLUMN_STATUS, _("Pending removal"));
+				}
 			}
 		}
 		else {
-			if (!pEditHandler->StartEditing(pData->name, pData->remotePath, pData->server)) {
-				if (pEditHandler->Remove(pData->name, pData->remotePath, pData->server)) {
+			if (!pEditHandler->StartEditing(pData->name, pData->remotePath, pData->site)) {
+				if (pEditHandler->Remove(pData->name, pData->remotePath, pData->site)) {
 					delete pData;
 					pListCtrl->DeleteItem(i);
 				}
-				else
+				else {
 					pListCtrl->SetItem(i, COLUMN_STATUS, _("Pending removal"));
+				}
 			}
 		}
 	}
@@ -1424,10 +1496,12 @@ CEditHandler::t_fileData* CEditHandlerStatusDialog::GetDataFromItem(int item, CE
 	info.SetId(item);
 	info.SetColumn(1);
 	pListCtrl->GetItem(info);
-	if (info.GetText() == _("Local"))
+	if (info.GetText() == _("Local")) {
 		type = CEditHandler::local;
-	else
+	}
+	else {
 		type = CEditHandler::remote;
+	}
 
 	return pData;
 }
@@ -1447,16 +1521,20 @@ CNewAssociationDialog::CNewAssociationDialog(wxWindow *parent)
 
 bool CNewAssociationDialog::Run(const wxString &file)
 {
-	if (!Load(m_pParent, _T("ID_EDIT_NOPROGRAM")))
+	if (!Load(m_pParent, _T("ID_EDIT_NOPROGRAM"))) {
 		return true;
+	}
 
 	int pos = file.Find('.', true);
-	if (!pos)
+	if (!pos) {
 		m_ext = _T(".");
-	else if (pos != -1)
+	}
+	else if (pos != -1) {
 		m_ext = file.Mid(pos + 1);
-	else
+	}
+	else {
 		m_ext.clear();
+	}
 
 	wxStaticText *const pDesc = XRCCTRL(*this, "ID_DESC", wxStaticText);
 	if (pDesc) {
@@ -1465,16 +1543,19 @@ bool CNewAssociationDialog::Run(const wxString &file)
 
 	bool program_exists = false;
 	wxString cmd = GetSystemOpenCommand(_T("foo.txt"), program_exists);
-	if (!program_exists)
+	if (!program_exists) {
 		cmd.clear();
+	}
 	if (!cmd.empty()) {
 		wxString args;
-		if (!UnquoteCommand(cmd, args))
+		if (!UnquoteCommand(cmd, args)) {
 			cmd.clear();
+		}
 	}
 
-	if (!PathExpand(cmd))
+	if (!PathExpand(cmd)) {
 		cmd.clear();
+	}
 
 	if (cmd.empty()) {
 		xrc_call(*this, "ID_USE_CUSTOM", &wxRadioButton::SetValue, true);
@@ -1493,8 +1574,9 @@ bool CNewAssociationDialog::Run(const wxString &file)
 
 	GetSizer()->Fit(this);
 
-	if (ShowModal() != wxID_OK)
+	if (ShowModal() != wxID_OK) {
 		return false;
+	}
 
 	return true;
 }
@@ -1536,30 +1618,36 @@ void CNewAssociationDialog::OnOK(wxCommandEvent&)
 			return;
 		}
 
-		if (always)
+		if (always) {
 			COptions::Get()->SetOption(OPTION_EDIT_DEFAULTEDITOR, _T("2") + cmd.ToStdWstring());
+		}
 		else {
 			wxString associations = COptions::Get()->GetOption(OPTION_EDIT_CUSTOMASSOCIATIONS);
-			if (!associations.empty() && associations.Last() != '\n')
+			if (!associations.empty() && associations.Last() != '\n') {
 				associations += '\n';
-			if (m_ext.empty())
+			}
+			if (m_ext.empty()) {
 				m_ext = _T("/");
+			}
 			associations += m_ext.ToStdWstring() + _T(" ") + cmd.ToStdWstring();
 			COptions::Get()->SetOption(OPTION_EDIT_CUSTOMASSOCIATIONS, associations.ToStdWstring());
 		}
 	}
 	else {
-		if (always)
+		if (always) {
 			COptions::Get()->SetOption(OPTION_EDIT_DEFAULTEDITOR, _T("1"));
+		}
 		else {
 			bool program_exists = false;
 			wxString cmd = GetSystemOpenCommand(_T("foo.txt"), program_exists);
-			if (!program_exists)
+			if (!program_exists) {
 				cmd.clear();
+			}
 			if (!cmd.empty()) {
 				wxString args;
-				if (!UnquoteCommand(cmd, args))
+				if (!UnquoteCommand(cmd, args)) {
 					cmd.clear();
+				}
 			}
 			if (cmd.empty()
 #ifdef __WXGTK__
@@ -1570,13 +1658,16 @@ void CNewAssociationDialog::OnOK(wxCommandEvent&)
 				wxMessageBoxEx(_("The default editor for text files could not be found."), _("Cannot set file association"), wxICON_EXCLAMATION, this);
 				return;
 			}
-			if (cmd.Find(' ') != -1)
+			if (cmd.Find(' ') != -1) {
 				cmd = _T("\"") + cmd + _T("\"");
+			}
 			wxString associations = COptions::Get()->GetOption(OPTION_EDIT_CUSTOMASSOCIATIONS);
-			if (!associations.empty() && associations.Last() != '\n')
+			if (!associations.empty() && associations.Last() != '\n') {
 				associations += '\n';
-			if (m_ext.empty())
+			}
+			if (m_ext.empty()) {
 				m_ext = _T("/");
+			}
 			associations += m_ext + _T(" ") + cmd;
 			COptions::Get()->SetOption(OPTION_EDIT_CUSTOMASSOCIATIONS, associations.ToStdWstring());
 		}
@@ -1619,16 +1710,16 @@ void CNewAssociationDialog::OnBrowseEditor(wxCommandEvent&)
 	XRCCTRL(*this, "ID_CUSTOM", wxTextCtrl)->ChangeValue(editor);
 }
 
-bool CEditHandler::Edit(CEditHandler::fileType type, std::wstring const& fileName, CServerPath const& path, ServerWithCredentials const& server, int64_t size, wxWindow * parent)
+bool CEditHandler::Edit(CEditHandler::fileType type, std::wstring const& fileName, CServerPath const& path, Site const& site, int64_t size, wxWindow * parent)
 {
 	std::vector<FileData> data;
 	FileData d{fileName, size};
 	data.push_back(d);
 
-	return Edit(type, data, path, server, parent);
+	return Edit(type, data, path, site, parent);
 }
 
-bool CEditHandler::Edit(CEditHandler::fileType type, std::vector<FileData> const& data, CServerPath const& path, ServerWithCredentials const& server, wxWindow * parent)
+bool CEditHandler::Edit(CEditHandler::fileType type, std::vector<FileData> const& data, CServerPath const& path, Site const& site, wxWindow * parent)
 {
 	if (type == CEditHandler::remote) {
 		wxString const& localDir = GetLocalDirectory();
@@ -1656,7 +1747,7 @@ bool CEditHandler::Edit(CEditHandler::fileType type, std::vector<FileData> const
 	bool success = true;
 	int already_editing_action{};
 	for (auto const& file : data) {
-		if (!DoEdit(type, file, path, server, parent, data.size(), already_editing_action)) {
+		if (!DoEdit(type, file, path, site, parent, data.size(), already_editing_action)) {
 			success = false;
 		}
 	}
@@ -1664,7 +1755,7 @@ bool CEditHandler::Edit(CEditHandler::fileType type, std::vector<FileData> const
 	return success;
 }
 
-bool CEditHandler::DoEdit(CEditHandler::fileType type, FileData const& file, CServerPath const& path, ServerWithCredentials const& server, wxWindow * parent, size_t fileCount, int & already_editing_action)
+bool CEditHandler::DoEdit(CEditHandler::fileType type, FileData const& file, CServerPath const& path, Site const& site, wxWindow * parent, size_t fileCount, int & already_editing_action)
 {
 	bool dangerous = false;
 	bool program_exists = false;
@@ -1698,7 +1789,7 @@ bool CEditHandler::DoEdit(CEditHandler::fileType type, FileData const& file, CSe
 		state = GetFileState(file.name);
 	}
 	else {
-		state = GetFileState(file.name, path, server);
+		state = GetFileState(file.name, path, site);
 	}
 	switch (state)
 	{
@@ -1709,7 +1800,7 @@ bool CEditHandler::DoEdit(CEditHandler::fileType type, FileData const& file, CSe
 		wxMessageBoxEx(_("A file with that name is already being transferred."), _("Cannot view/edit selected file"), wxICON_EXCLAMATION);
 		return false;
 	case CEditHandler::removing:
-		if (!Remove(file.name, path, server)) {
+		if (!Remove(file.name, path, site)) {
 			wxMessageBoxEx(_("A file with that name is still being edited. Please close it and try again."), _("Selected file is already opened"), wxICON_EXCLAMATION);
 			return false;
 		}
@@ -1776,12 +1867,12 @@ bool CEditHandler::DoEdit(CEditHandler::fileType type, FileData const& file, CSe
 					StartEditing(file.name);
 				}
 				else {
-					StartEditing(file.name, path, server);
+					StartEditing(file.name, path, site);
 				}
 				return true;
 			}
 			else {
-				if (!Remove(file.name, path, server)) {
+				if (!Remove(file.name, path, site)) {
 					wxMessageBoxEx(_("The selected file is still opened in some other program, please close it."), _("Selected file is still being edited"), wxICON_EXCLAMATION);
 					return false;
 				}
@@ -1793,7 +1884,7 @@ bool CEditHandler::DoEdit(CEditHandler::fileType type, FileData const& file, CSe
 	}
 
 	std::wstring localFile = file.name;
-	if (!AddFile(type, localFile, path, server)) {
+	if (!AddFile(type, localFile, path, site)) {
 		if( type == CEditHandler::local) {
 			wxMessageBoxEx(wxString::Format(_("The file '%s' could not be opened:\nThe associated command failed"), file.name), _("Opening failed"), wxICON_EXCLAMATION);
 		}
@@ -1809,7 +1900,7 @@ bool CEditHandler::DoEdit(CEditHandler::fileType type, FileData const& file, CSe
 		CLocalPath localPath(localFile, &localFileName);
 
 		m_pQueue->QueueFile(false, true, file.name, (localFileName != file.name) ? localFileName : std::wstring(),
-			localPath, path, server, file.size, type, QueuePriority::high);
+			localPath, path, site, file.size, type, QueuePriority::high);
 		m_pQueue->QueueFile_Finish(true);
 	}
 
