@@ -4,6 +4,8 @@
 #include "window_state_manager.h"
 #include "Options.h"
 
+#include <libfilezilla/translate.hpp>
+
 BEGIN_EVENT_TABLE(CFilterEditDialog, CFilterConditionsDialog)
 EVT_BUTTON(XRCID("wxID_OK"), CFilterEditDialog::OnOK)
 EVT_BUTTON(XRCID("wxID_CANCEL"), CFilterEditDialog::OnCancel)
@@ -196,6 +198,7 @@ void CFilterEditDialog::OnRename(wxCommandEvent&)
 {
 	const wxString& oldName = XRCCTRL(*this, "ID_NAME", wxTextCtrl)->GetValue();
 	wxTextEntryDialog *pDlg = new wxTextEntryDialog(this, _("Please enter a new name for the filter."), _("Enter filter name"), oldName);
+	pDlg->SetMaxLength(255);
 	if (pDlg->ShowModal() != wxID_OK) {
 		delete pDlg;
 		return;
