@@ -1,12 +1,12 @@
-#ifndef __FILTEREDIT_H__
-#define __FILTEREDIT_H__
+#ifndef FILEZILLA_INTERFACE_FILTEREDIT_HEADER
+#define FILEZILLA_INTERFACE_FILTEREDIT_HEADER
 
 #include "filter.h"
 #include "filter_conditions_dialog.h"
 
 class wxCustomHeightListCtrl;
 class CWindowStateManager;
-class CFilterEditDialog : public CFilterConditionsDialog
+class CFilterEditDialog final : public CFilterConditionsDialog
 {
 public:
 	CFilterEditDialog();
@@ -44,4 +44,4 @@ protected:
 	CWindowStateManager* m_pWindowStateManager;
 };
 
-#endif //__FILTEREDIT_H__
+#endif
