@@ -5,6 +5,7 @@
 #include "inputdialog.h"
 
 #include <libfilezilla/process.hpp>
+#include <libfilezilla/translate.hpp>
 
 CFZPuttyGenInterface::CFZPuttyGenInterface(wxWindow* parent)
 	: m_parent(parent)
@@ -92,12 +93,10 @@ bool CFZPuttyGenInterface::LoadKeyFile(std::wstring& keyFile, bool silent, std::
 
 		msg = wxString::Format(_("Enter the password for the file '%s'.\nThe converted file will be protected with the same password."), keyFile);
 		CInputDialog dlg;
-		if (!dlg.Create(m_parent, _("Password required"), msg)) {
+		if (!dlg.Create(m_parent, _("Password required"), msg, -1, true)) {
 			return false;
 		}
 
-		dlg.SetPasswordMode(true);
-
 		if (dlg.ShowModal() != wxID_OK) {
 			return false;
 		}
