@@ -1,56 +1,46 @@
 #include <filezilla.h>
 #include "inputdialog.h"
+#include "textctrlex.h"
 
 BEGIN_EVENT_TABLE(CInputDialog, wxDialogEx)
 EVT_TEXT(XRCID("ID_STRING"), CInputDialog::OnValueChanged)
-EVT_TEXT(XRCID("ID_STRING_PW"), CInputDialog::OnValueChanged)
-EVT_BUTTON(XRCID("wxID_OK"), CInputDialog::OnOK)
-EVT_BUTTON(XRCID("wxID_CANCEL"), CInputDialog::OnCancel)
 END_EVENT_TABLE()
 
-CInputDialog::CInputDialog()
-	: m_allowEmpty()
-	, m_pTextCtrl()
+bool CInputDialog::Create(wxWindow* parent, wxString const& title, wxString const& text, int max_len, bool password)
 {
-}
-
-bool CInputDialog::Create(wxWindow* parent, const wxString& title, wxString text)
-{
-	m_allowEmpty = false;
 	SetParent(parent);
 
-	if (!Load(parent, _T("ID_INPUTDIALOG"))) {
+	if (!wxDialogEx::Create(parent, -1, title)) {
 		return false;
 	}
 
-	SetTitle(title);
+	auto& lay = layout();
+	auto * main = lay.createMain(this, 1);
 
-	if (!XRCCTRL(*this, "wxID_OK", wxButton))
-		return false;
+	main->Add(new wxStaticText(this, -1, text));
 
-	if (!XRCCTRL(*this, "wxID_CANCEL", wxButton))
-		return false;
+	m_pTextCtrl = new wxTextCtrlEx(this, XRCID("ID_STRING"), wxString(), wxDefaultPosition, wxDefaultSize, password ? wxTE_PASSWORD : 0);
+	main->Add(m_pTextCtrl, lay.grow);
+	if (max_len != -1) {
+		m_pTextCtrl->SetMaxLength(max_len);
+	}
 
-	m_pTextCtrl = XRCCTRL(*this, "ID_STRING", wxTextCtrl);
-	if (!m_pTextCtrl)
-		return false;
+	auto * buttons = lay.createButtonSizer(this, main, true);
 
-	if (!XRCCTRL(*this, "ID_STRING_PW", wxTextCtrl))
-		return false;
+	auto ok = new wxButton(this, wxID_OK, _("&OK"));
+	ok->SetDefault();
+	buttons->AddButton(ok);
 
-	wxStaticText* pText = XRCCTRL(*this, "ID_TEXT", wxStaticText);
-	if (!pText)
-		return false;
+	auto cancel = new wxButton(this, wxID_CANCEL, _("&Cancel"));
+	buttons->AddButton(cancel);
+	buttons->Realize();
+
+	Bind(wxEVT_BUTTON, [this](wxEvent& evt) {EndModal(evt.GetId()); });
 
 	WrapRecursive(this, 2.0);
-	pText->SetLabel(text);
 
-	GetSizer()->Fit(this);
-	GetSizer()->SetSizeHints(this);
-
-	XRCCTRL(*this, "ID_STRING", wxTextCtrl)->SetFocus();
-
-	XRCCTRL(*this, "wxID_OK", wxButton)->Enable(false);
+	m_pTextCtrl->SetFocus();
+	ok->Disable();
 
 	return true;
 }
@@ -58,7 +48,7 @@ bool CInputDialog::Create(wxWindow* parent, const wxString& title, wxString text
 void CInputDialog::AllowEmpty(bool allowEmpty)
 {
 	m_allowEmpty = allowEmpty;
-	XRCCTRL(*this, "wxID_OK", wxButton)->Enable(m_allowEmpty ? true : (m_pTextCtrl->GetValue() != _T("")));
+	XRCCTRL(*this, "wxID_OK", wxButton)->Enable(m_allowEmpty ? true : (!m_pTextCtrl->GetValue().empty()));
 }
 
 void CInputDialog::OnValueChanged(wxCommandEvent&)
@@ -67,7 +57,7 @@ void CInputDialog::OnValueChanged(wxCommandEvent&)
 	XRCCTRL(*this, "wxID_OK", wxButton)->Enable(m_allowEmpty ? true : !value.empty());
 }
 
-void CInputDialog::SetValue(const wxString& value)
+void CInputDialog::SetValue(wxString const& value)
 {
 	m_pTextCtrl->SetValue(value);
 }
@@ -87,26 +77,14 @@ bool CInputDialog::SelectText(int start, int end)
 	return true;
 }
 
-void CInputDialog::OnOK(wxCommandEvent&)
-{
-	EndModal(wxID_OK);
-}
-
-void CInputDialog::OnCancel(wxCommandEvent&)
-{
-	EndModal(wxID_CANCEL);
-}
-
 bool CInputDialog::SetPasswordMode(bool password)
 {
-	if (password)
-	{
+	if (password) {
 		m_pTextCtrl = XRCCTRL(*this, "ID_STRING_PW", wxTextCtrl);
 		m_pTextCtrl->Show();
 		XRCCTRL(*this, "ID_STRING", wxTextCtrl)->Hide();
 	}
-	else
-	{
+	else {
 		m_pTextCtrl = XRCCTRL(*this, "ID_STRING", wxTextCtrl);
 		m_pTextCtrl->Show();
 		XRCCTRL(*this, "ID_STRING_PW", wxTextCtrl)->Hide();
