@@ -6,15 +6,14 @@
 class CInputDialog final : public wxDialogEx
 {
 public:
-	CInputDialog();
-	virtual ~CInputDialog() {}
+	CInputDialog() = default;
 
-	bool Create(wxWindow* parent, const wxString& title, wxString text);
+	bool Create(wxWindow* parent, wxString const& title, wxString const& text, int max_len = -1, bool password = false);
 
 	bool SetPasswordMode(bool password);
 	void AllowEmpty(bool allowEmpty);
 
-	void SetValue(const wxString& value);
+	void SetValue(wxString const& value);
 	wxString GetValue() const;
 
 	bool SelectText(int start, int end);
@@ -22,11 +21,9 @@ public:
 protected:
 	DECLARE_EVENT_TABLE()
 	void OnValueChanged(wxCommandEvent& event);
-	void OnOK(wxCommandEvent& event);
-	void OnCancel(wxCommandEvent& event);
 
-	bool m_allowEmpty;
-	wxTextCtrl* m_pTextCtrl;
+	bool m_allowEmpty{};
+	wxTextCtrl* m_pTextCtrl{};
 };
 
 #endif
