@@ -8,9 +8,9 @@ int CInterProcessMutex::m_fd = -1;
 int CInterProcessMutex::m_instanceCount = 0;
 #endif
 
-std::list<CReentrantInterProcessMutexLocker::t_data> CReentrantInterProcessMutexLocker::m_mutexes;
+std::vector<CReentrantInterProcessMutexLocker::t_data> CReentrantInterProcessMutexLocker::m_mutexes;
 
-CInterProcessMutex::CInterProcessMutex(enum t_ipcMutexType mutexType, bool initialLock /*=true*/)
+CInterProcessMutex::CInterProcessMutex(t_ipcMutexType mutexType, bool initialLock)
 {
 	m_locked = false;
 #ifdef __WXMSW__
@@ -25,23 +25,27 @@ CInterProcessMutex::CInterProcessMutex(enum t_ipcMutexType mutexType, bool initi
 	m_instanceCount++;
 #endif
 	m_type = mutexType;
-	if (initialLock)
+	if (initialLock) {
 		Lock();
+	}
 }
 
 CInterProcessMutex::~CInterProcessMutex()
 {
-	if (m_locked)
+	if (m_locked) {
 		Unlock();
+	}
 #ifdef __WXMSW__
-	if (hMutex)
+	if (hMutex) {
 		::CloseHandle(hMutex);
+	}
 #else
 	m_instanceCount--;
 	// Close file only if this is the last instance. At least under
 	// Linux, closing the lock file has the affect of removing all locks.
-	if (!m_instanceCount && m_fd >= 0)
+	if (!m_instanceCount && m_fd >= 0) {
 		close(m_fd);
+	}
 #endif
 }
 
@@ -49,11 +53,11 @@ bool CInterProcessMutex::Lock()
 {
 	wxASSERT(!m_locked);
 #ifdef __WXMSW__
-	if (hMutex)
+	if (hMutex) {
 		::WaitForSingleObject(hMutex, INFINITE);
+	}
 #else
-	if (m_fd >= 0)
-	{
+	if (m_fd >= 0) {
 		// Lock 1 byte region in the lockfile. m_type specifies the byte to lock.
 		struct flock f = {};
 		f.l_type = F_WRLCK;
@@ -62,10 +66,11 @@ bool CInterProcessMutex::Lock()
 		f.l_len = 1;
 		f.l_pid = getpid();
 
-		while (fcntl(m_fd, F_SETLKW, &f) == -1)
-		{
-			if (errno == EINTR) // Interrupted by signal, retry
+		while (fcntl(m_fd, F_SETLKW, &f) == -1) {
+			if (errno == EINTR) {
+				// Interrupted by signal, retry
 				continue;
+			}
 
 			// Can't do any locking in this case
 			return false;
@@ -83,21 +88,18 @@ int CInterProcessMutex::TryLock()
 	wxASSERT(!m_locked);
 
 #ifdef __WXMSW__
-	if (!hMutex)
-	{
+	if (!hMutex) {
 		m_locked = false;
 		return 0;
 	}
 
 	int res = ::WaitForSingleObject(hMutex, 1);
-	if (res == WAIT_OBJECT_0)
-	{
+	if (res == WAIT_OBJECT_0) {
 		m_locked = true;
 		return 1;
 	}
 #else
-	if (m_fd >= 0)
-	{
+	if (m_fd >= 0) {
 		// Try to lock 1 byte region in the lockfile. m_type specifies the byte to lock.
 		struct flock f = {};
 		f.l_type = F_WRLCK;
@@ -105,13 +107,16 @@ int CInterProcessMutex::TryLock()
 		f.l_start = m_type;
 		f.l_len = 1;
 		f.l_pid = getpid();
-		while (fcntl(m_fd, F_SETLK, &f) == -1)
-		{
-			if (errno == EINTR) // Interrupted by signal, retry
+		while (fcntl(m_fd, F_SETLK, &f) == -1) {
+			if (errno == EINTR) {
+				// Interrupted by signal, retry
 				continue;
+			}
 
-			if (errno == EAGAIN || errno == EACCES) // Lock held by other process
+			if (errno == EAGAIN || errno == EACCES) {
+				// Lock held by other process
 				return 0;
+			}
 
 			// Can't do any locking in this case
 			return -1;
@@ -127,16 +132,17 @@ int CInterProcessMutex::TryLock()
 
 void CInterProcessMutex::Unlock()
 {
-	if (!m_locked)
+	if (!m_locked) {
 		return;
+	}
 	m_locked = false;
 
 #ifdef __WXMSW__
-	if (hMutex)
+	if (hMutex) {
 		::ReleaseMutex(hMutex);
+	}
 #else
-	if (m_fd >= 0)
-	{
+	if (m_fd >= 0) {
 		// Unlock region specified by m_type.
 		struct flock f = {};
 		f.l_type = F_UNLCK;
@@ -144,10 +150,11 @@ void CInterProcessMutex::Unlock()
 		f.l_start = m_type;
 		f.l_len = 1;
 		f.l_pid = getpid();
-		while (fcntl(m_fd, F_SETLKW, &f) == -1)
-		{
-			if (errno == EINTR) // Interrupted by signal, retry
+		while (fcntl(m_fd, F_SETLKW, &f) == -1) {
+			if (errno == EINTR) {
+				// Interrupted by signal, retry
 				continue;
+			}
 
 			// Can't do any locking in this case
 			return;
@@ -156,23 +163,16 @@ void CInterProcessMutex::Unlock()
 #endif
 }
 
-CReentrantInterProcessMutexLocker::CReentrantInterProcessMutexLocker(enum t_ipcMutexType mutexType)
+CReentrantInterProcessMutexLocker::CReentrantInterProcessMutexLocker(t_ipcMutexType mutexType)
 {
 	m_type = mutexType;
 
-	std::list<t_data>::iterator iter;
-	for (iter = m_mutexes.begin(); iter != m_mutexes.end(); ++iter)
-	{
-		if (iter->pMutex->GetType() == mutexType)
-			break;
+	auto it = std::find_if(m_mutexes.begin(), m_mutexes.end(), [&](auto const& v) { return v.pMutex->GetType() == mutexType; });
+	
+	if (it != m_mutexes.end()) {
+		++(it->lockCount);
 	}
-
-	if (iter != m_mutexes.end())
-	{
-		iter->lockCount++;
-	}
-	else
-	{
+	else {
 		t_data data;
 		data.lockCount = 1;
 		data.pMutex = new CInterProcessMutex(mutexType);
@@ -182,22 +182,18 @@ CReentrantInterProcessMutexLocker::CReentrantInterProcessMutexLocker(enum t_ipcM
 
 CReentrantInterProcessMutexLocker::~CReentrantInterProcessMutexLocker()
 {
-	std::list<t_data>::iterator iter;
-	for (iter = m_mutexes.begin(); iter != m_mutexes.end(); ++iter)
-	{
-		if (iter->pMutex->GetType() == m_type)
-			break;
-	}
-
-	wxASSERT(iter != m_mutexes.end());
-	if (iter == m_mutexes.end())
+	auto it = std::find_if(m_mutexes.begin(), m_mutexes.end(), [&](auto const& v) { return v.pMutex->GetType() == m_type; });
+	wxASSERT(it != m_mutexes.cend());
+	if (it == m_mutexes.cend()) {
 		return;
-
-	if (iter->lockCount == 1)
-	{
-		delete iter->pMutex;
-		m_mutexes.erase(iter);
 	}
-	else
-		iter->lockCount--;
+
+	if (it->lockCount == 1) {
+		delete it->pMutex;
+		*it = m_mutexes.back();
+		m_mutexes.pop_back();
+	}
+	else {
+		--(it->lockCount);
+	}
 }
