@@ -11,15 +11,13 @@ CComparableListing::CComparableListing(wxWindow* pParent)
 
 	// Init backgrounds for directory comparison
 	wxColour background = m_pParent->GetBackgroundColour();
-	if (background.Red() + background.Green() + background.Blue() >= 384)
-	{
+	if (background.Red() + background.Green() + background.Blue() >= 384) {
 		// Light background
 		m_comparisonBackgrounds[0].SetBackgroundColour(wxColour(255, 128, 128));
 		m_comparisonBackgrounds[1].SetBackgroundColour(wxColour(255, 255, 128));
 		m_comparisonBackgrounds[2].SetBackgroundColour(wxColour(128, 255, 128));
 	}
-	else
-	{
+	else {
 		// Light background
 		m_comparisonBackgrounds[0].SetBackgroundColour(wxColour(192, 64, 64));
 		m_comparisonBackgrounds[1].SetBackgroundColour(wxColour(192, 192, 64));
@@ -31,27 +29,31 @@ CComparableListing::CComparableListing(wxWindow* pParent)
 
 bool CComparableListing::IsComparing() const
 {
-	if (!m_pComparisonManager)
+	if (!m_pComparisonManager) {
 		return false;
+	}
 
 	return m_pComparisonManager->IsComparing();
 }
 
 void CComparableListing::ExitComparisonMode()
 {
-	if (!m_pComparisonManager)
+	if (!m_pComparisonManager) {
 		return;
+	}
 
 	m_pComparisonManager->ExitComparisonMode();
 }
 
 void CComparableListing::RefreshComparison()
 {
-	if (!m_pComparisonManager)
+	if (!m_pComparisonManager) {
 		return;
+	}
 
-	if (!IsComparing())
+	if (!IsComparing()) {
 		return;
+	}
 
 	if (!CanStartComparison() || !GetOther() || !GetOther()->CanStartComparison()) {
 		return;
@@ -62,8 +64,9 @@ void CComparableListing::RefreshComparison()
 
 bool CComparisonManager::CompareListings()
 {
-	if (!m_pLeft || !m_pRight)
+	if (!m_pLeft || !m_pRight) {
 		return false;
+	}
 
 	CFilterManager filters;
 	if (filters.HasActiveFilters() && !filters.HasSameLocalAndRemoteFilters()) {
@@ -78,7 +81,6 @@ bool CComparisonManager::CompareListings()
 
 	m_state.NotifyHandlers(STATECHANGE_COMPARISON);
 
-	wxString error;
 	if (!m_pLeft->CanStartComparison() || !m_pRight->CanStartComparison()) {
 		return true;
 	}
@@ -89,7 +91,7 @@ bool CComparisonManager::CompareListings()
 	m_pLeft->StartComparison();
 	m_pRight->StartComparison();
 
-	wxString localFile, remoteFile;
+	std::wstring localFile, remoteFile;
 	bool localDir = false;
 	bool remoteDir = false;
 	int64_t localSize, remoteSize;
@@ -108,14 +110,14 @@ bool CComparisonManager::CompareListings()
 			if (!mode) {
 				const CComparableListing::t_fileEntryFlags flag = (localDir || localSize == remoteSize) ? CComparableListing::normal : CComparableListing::different;
 
-				if (!hide_identical || flag != CComparableListing::normal || localFile == _T("..")) {
+				if (!hide_identical || flag != CComparableListing::normal || localFile == L"..") {
 					m_pLeft->CompareAddFile(flag);
 					m_pRight->CompareAddFile(flag);
 				}
 			}
 			else {
 				if (localDate.empty() || remoteDate.empty()) {
-					if (!hide_identical || !localDate.empty() || !remoteDate.empty() || localFile == _T("..")) {
+					if (!hide_identical || !localDate.empty() || !remoteDate.empty() || localFile == L"..") {
 						const CComparableListing::t_fileEntryFlags flag = CComparableListing::normal;
 						m_pLeft->CompareAddFile(flag);
 						m_pRight->CompareAddFile(flag);
@@ -144,7 +146,7 @@ bool CComparisonManager::CompareListings()
 					else if (dateCmp > 0) {
 						localFlag = CComparableListing::newer;
 					}
-					if (!hide_identical || localFlag != CComparableListing::normal || remoteFlag != CComparableListing::normal || localFile == _T("..")) {
+					if (!hide_identical || localFlag != CComparableListing::normal || remoteFlag != CComparableListing::normal || localFile == L"..") {
 						m_pLeft->CompareAddFile(localFlag);
 						m_pRight->CompareAddFile(remoteFlag);
 					}
@@ -171,8 +173,7 @@ bool CComparisonManager::CompareListings()
 		m_pRight->CompareAddFile(CComparableListing::fill);
 		gotLocal = m_pLeft->get_next_file(localFile, localDir, localSize, localDate);
 	}
-	while (gotRemote)
-	{
+	while (gotRemote) {
 		m_pLeft->CompareAddFile(CComparableListing::fill);
 		m_pRight->CompareAddFile(CComparableListing::lonely);
 		gotRemote = m_pRight->get_next_file(remoteFile, remoteDir, remoteSize, remoteDate);
@@ -184,18 +185,19 @@ bool CComparisonManager::CompareListings()
 	return true;
 }
 
-int CComparisonManager::CompareFiles(const int dirSortMode, const wxString& local, const wxString& remote, bool localDir, bool remoteDir)
+int CComparisonManager::CompareFiles(const int dirSortMode, std::wstring const& local, std::wstring const& remote, bool localDir, bool remoteDir)
 {
 	switch (dirSortMode)
 	{
 	default:
-		if (localDir)
-		{
-			if (!remoteDir)
+		if (localDir) {
+			if (!remoteDir) {
 				return -1;
+			}
 		}
-		else if (remoteDir)
+		else if (remoteDir) {
 			return 1;
+		}
 		break;
 	case 2:
 		// Inline
@@ -203,9 +205,9 @@ int CComparisonManager::CompareFiles(const int dirSortMode, const wxString& loca
 	}
 
 #ifdef __WXMSW__
-	return local.CmpNoCase(remote);
+	return fz::stricmp(local, remote);
 #else
-	return local.Cmp(remote);
+	return local.compare(remote);
 #endif
 
 	return 0;
@@ -220,33 +222,41 @@ void CComparisonManager::SetListings(CComparableListing* pLeft, CComparableListi
 {
 	wxASSERT((pLeft && pRight) || (!pLeft && !pRight));
 
-	if (IsComparing())
+	if (IsComparing()) {
 		ExitComparisonMode();
+	}
 
-	if (m_pLeft)
+	if (m_pLeft) {
 		m_pLeft->SetOther(0);
-	if (m_pRight)
+	}
+	if (m_pRight) {
 		m_pRight->SetOther(0);
+	}
 
 	m_pLeft = pLeft;
 	m_pRight = pRight;
 
-	if (m_pLeft)
+	if (m_pLeft) {
 		m_pLeft->SetOther(m_pRight);
-	if (m_pRight)
+	}
+	if (m_pRight) {
 		m_pRight->SetOther(m_pLeft);
+	}
 }
 
 void CComparisonManager::ExitComparisonMode()
 {
-	if (!IsComparing())
+	if (!IsComparing()) {
 		return;
+	}
 
 	m_isComparing = false;
-	if (m_pLeft)
+	if (m_pLeft) {
 		m_pLeft->OnExitComparisonMode();
-	if (m_pRight)
+	}
+	if (m_pRight) {
 		m_pRight->OnExitComparisonMode();
+	}
 
 	m_state.NotifyHandlers(STATECHANGE_COMPARISON);
 }
