@@ -5,7 +5,9 @@
 #include "wrapengine.h"
 #include "externalipresolver.h"
 #include <wx/timer.h>
-#include "socket.h"
+
+#include <libfilezilla/buffer.hpp>
+#include <libfilezilla/socket.hpp>
 
 #define NETCONFBUFFERSIZE 200
 
@@ -22,7 +24,7 @@ public:
 
 protected:
 
-	void PrintMessage(const wxString& msg, int type);
+	void PrintMessage(std::wstring const& msg, int type);
 
 	void ResetTest();
 
@@ -53,7 +55,7 @@ protected:
 	void OnConnect();
 	void OnSend();
 	void CloseSocket();
-	bool Send(wxString cmd);
+	bool Send(std::wstring const& cmd);
 	void OnDataReceive();
 	void OnDataClose();
 
@@ -61,7 +63,7 @@ protected:
 
 	void SendNextCommand();
 
-	wxString GetExternalIPAddress();
+	std::wstring GetExternalIPAddress();
 
 	int CreateListenSocket();
 	int CreateListenSocket(unsigned int port);
@@ -69,7 +71,7 @@ protected:
 	wxString m_nextLabelText;
 
 	// Test data
-	fz::socket* m_socket{};
+	std::unique_ptr<fz::socket> socket_;
 	int m_state;
 
 	char m_recvBuffer[NETCONFBUFFERSIZE];
@@ -90,15 +92,15 @@ protected:
 	} m_testResult;
 
 	CExternalIPResolver* m_pIPResolver{};
-	wxString m_externalIP;
+	std::wstring m_externalIP;
 
-	fz::listen_socket* m_pSocketServer{};
-	fz::socket* m_pDataSocket{};
+	std::unique_ptr<fz::listen_socket> listen_socket_;
+	std::unique_ptr<fz::socket> data_socket_;
 	int m_listenPort{};
 	bool gotListReply{};
 	int m_data{};
 
-	char* m_pSendBuffer{};
+	fz::buffer sendBuffer_;
 
 	wxTimer m_timer;
 };
