@@ -10,10 +10,8 @@
 CPowerManagement* CPowerManagement::m_pPowerManagement = 0;
 
 #ifdef __WXMAC__
-extern "C" {
-	void const* PowerManagmentImpl_SetBusy();
-	void PowerManagmentImpl_SetIdle(void const* activity);
-}
+void const* PowerManagmentImpl_SetBusy();
+void PowerManagmentImpl_SetIdle(void const* activity);
 #endif
 
 void CPowerManagement::Create(CMainFrame* pMainFrame)
@@ -51,7 +49,7 @@ CPowerManagement::~CPowerManagement()
 #endif
 }
 
-void CPowerManagement::OnStateChange(CState*, t_statechange_notifications, wxString const&, void const*)
+void CPowerManagement::OnStateChange(CState*, t_statechange_notifications, std::wstring const&, void const*)
 {
 	auto const queue = m_pMainFrame->GetQueue();
 	if (queue && queue->IsActive()) {
