@@ -19,8 +19,9 @@ enum class Column_type
 
 enum _column_flags
 {
-	not_null = 1,
-	autoincrement
+	not_null = 0x1,
+	default_null = 0x2,
+	autoincrement = 0x4
 };
 
 struct _column
@@ -51,7 +52,8 @@ namespace server_table_column_names
 		bypass_proxy,
 		post_login_commands,
 		name,
-		parameters
+		parameters,
+		site_path
 	};
 }
 
@@ -73,7 +75,8 @@ _column server_table_columns[] = {
 	{ "bypass_proxy", Column_type::integer, 0 },
 	{ "post_login_commands", Column_type::text, 0 },
 	{ "name", Column_type::text, 0 },
-	{ "parameters", Column_type::text, 0 }
+	{ "parameters", Column_type::text, 0 },
+	{ "site_path", Column_type::text, default_null }
 };
 
 namespace file_table_column_names
@@ -160,7 +163,7 @@ public:
 	int64_t GetColumnInt64(sqlite3_stmt* statement, int index, int64_t def = 0);
 	int GetColumnInt(sqlite3_stmt* statement, int index, int def = 0);
 
-	int64_t ParseServerFromRow(ServerWithCredentials& server);
+	int64_t ParseServerFromRow(Site & site);
 	int64_t ParseFileFromRow(CFileItem** pItem);
 
 	bool MigrateSchema();
@@ -292,7 +295,7 @@ bool CQueueStorage::Impl::MigrateSchema()
 	bool ret = sqlite3_exec(db_, "PRAGMA user_version", int_callback, &version, 0) == SQLITE_OK;
 
 	if (ret) {
-		if (version > 4) {
+		if (version > 5) {
 			ret = false;
 		}
 		else if (version > 0) {
@@ -303,9 +306,12 @@ bool CQueueStorage::Impl::MigrateSchema()
 			if (ret && version < 4) {
 				ret = sqlite3_exec(db_, "ALTER TABLE servers ADD COLUMN parameters TEXT", 0, 0, 0) == SQLITE_OK;
 			}
+			if (ret && version < 5) {
+				ret = sqlite3_exec(db_, "ALTER TABLE servers ADD COLUMN site_path TEXT DEFAULT NULL", 0, 0, 0) == SQLITE_OK;
+			}
 		}
-		if (ret && version != 4) {
-			ret = sqlite3_exec(db_, "PRAGMA user_version = 4", 0, 0, 0) == SQLITE_OK;
+		if (ret && version != 5) {
+			ret = sqlite3_exec(db_, "PRAGMA user_version = 5", 0, 0, 0) == SQLITE_OK;
 		}
 	}
 
@@ -401,6 +407,9 @@ std::string CQueueStorage::Impl::CreateColumnDefs(_column const* columns, size_t
 		if (columns[i].flags & not_null) {
 			query += " NOT NULL";
 		}
+		if (columns[i].flags & default_null) {
+			query += " DEFAULT NULL";
+		}
 	}
 	query += ")";
 
@@ -607,19 +616,19 @@ bool CQueueStorage::Impl::SaveServer(CServerItem const& item)
 {
 	bool kiosk_mode = COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) != 0;
 
-	ServerWithCredentials const& server = item.GetServer();
+	Site const& site = item.GetSite();
 
-	Bind(insertServerQuery_, server_table_column_names::host, server.server.GetHost());
-	Bind(insertServerQuery_, server_table_column_names::port, static_cast<int>(server.server.GetPort()));
-	Bind(insertServerQuery_, server_table_column_names::protocol, static_cast<int>(server.server.GetProtocol()));
-	Bind(insertServerQuery_, server_table_column_names::type, static_cast<int>(server.server.GetType()));
+	Bind(insertServerQuery_, server_table_column_names::host, site.server.GetHost());
+	Bind(insertServerQuery_, server_table_column_names::port, static_cast<int>(site.server.GetPort()));
+	Bind(insertServerQuery_, server_table_column_names::protocol, static_cast<int>(site.server.GetProtocol()));
+	Bind(insertServerQuery_, server_table_column_names::type, static_cast<int>(site.server.GetType()));
 
-	ProtectedCredentials credentials = server.credentials;
+	ProtectedCredentials credentials = site.credentials;
 	credentials.Protect();
 
 	LogonType logonType = credentials.logonType_;
 	if (logonType != LogonType::anonymous) {
-		Bind(insertServerQuery_, server_table_column_names::user, server.server.GetUser());
+		Bind(insertServerQuery_, server_table_column_names::user, site.server.GetUser());
 
 		if (logonType == LogonType::normal || logonType == LogonType::account) {
 			if (kiosk_mode) {
@@ -672,9 +681,9 @@ bool CQueueStorage::Impl::SaveServer(CServerItem const& item)
 		Bind(insertServerQuery_, server_table_column_names::logontype, lt);
 	}
 
-	Bind(insertServerQuery_, server_table_column_names::timezone_offset, server.server.GetTimezoneOffset());
+	Bind(insertServerQuery_, server_table_column_names::timezone_offset, site.server.GetTimezoneOffset());
 
-	switch (server.server.GetPasvMode())
+	switch (site.server.GetPasvMode())
 	{
 	case MODE_PASSIVE:
 		Bind(insertServerQuery_, server_table_column_names::transfer_mode, _T("passive"));
@@ -686,9 +695,9 @@ bool CQueueStorage::Impl::SaveServer(CServerItem const& item)
 		Bind(insertServerQuery_, server_table_column_names::transfer_mode, _T("default"));
 		break;
 	}
-	Bind(insertServerQuery_, server_table_column_names::max_connections, server.server.MaximumMultipleConnections());
+	Bind(insertServerQuery_, server_table_column_names::max_connections, site.server.MaximumMultipleConnections());
 
-	switch (server.server.GetEncodingType())
+	switch (site.server.GetEncodingType())
 	{
 	default:
 	case ENCODING_AUTO:
@@ -698,12 +707,12 @@ bool CQueueStorage::Impl::SaveServer(CServerItem const& item)
 		Bind(insertServerQuery_, server_table_column_names::encoding, _T("UTF-8"));
 		break;
 	case ENCODING_CUSTOM:
-		Bind(insertServerQuery_, server_table_column_names::encoding, server.server.GetCustomEncoding());
+		Bind(insertServerQuery_, server_table_column_names::encoding, site.server.GetCustomEncoding());
 		break;
 	}
 
-	if (CServer::ProtocolHasFeature(server.server.GetProtocol(), ProtocolFeature::PostLoginCommands)) {
-		std::vector<std::wstring> const& postLoginCommands = server.server.GetPostLoginCommands();
+	if (CServer::ProtocolHasFeature(site.server.GetProtocol(), ProtocolFeature::PostLoginCommands)) {
+		std::vector<std::wstring> const& postLoginCommands = site.server.GetPostLoginCommands();
 		if (!postLoginCommands.empty()) {
 			std::wstring commands;
 			for (auto const& command : commands) {
@@ -722,15 +731,15 @@ bool CQueueStorage::Impl::SaveServer(CServerItem const& item)
 		BindNull(insertServerQuery_, server_table_column_names::post_login_commands);
 	}
 
-	Bind(insertServerQuery_, server_table_column_names::bypass_proxy, server.server.GetBypassProxy() ? 1 : 0);
-	if (!server.server.GetName().empty()) {
-		Bind(insertServerQuery_, server_table_column_names::name, server.server.GetName());
+	Bind(insertServerQuery_, server_table_column_names::bypass_proxy, site.server.GetBypassProxy() ? 1 : 0);
+	if (!site.GetName().empty()) {
+		Bind(insertServerQuery_, server_table_column_names::name, site.GetName());
 	}
 	else {
 		BindNull(insertServerQuery_, server_table_column_names::name);
 	}
 
-	auto const& parameters = server.server.GetExtraParameters();
+	auto const& parameters = site.server.GetExtraParameters();
 	if (!parameters.empty()) {
 		fz::query_string qs;
 		for (auto const& parameter : parameters) {
@@ -739,6 +748,14 @@ bool CQueueStorage::Impl::SaveServer(CServerItem const& item)
 		Bind(insertServerQuery_, server_table_column_names::parameters, qs.to_string(false));
 	}
 
+	auto const& site_path = site.SitePath();
+	if (site_path.empty()) {
+		BindNull(insertServerQuery_, server_table_column_names::site_path);
+	}
+	else {
+		Bind(insertServerQuery_, server_table_column_names::site_path, site_path);
+	}
+
 	int res;
 	do {
 		res = sqlite3_step(insertServerQuery_);
@@ -879,8 +896,8 @@ std::wstring CQueueStorage::Impl::GetColumnText(sqlite3_stmt* statement, int ind
 	}
 #else
 	char const* text = reinterpret_cast<char const*>(sqlite3_column_text(statement, index));
-	int len = sqlite3_column_bytes(statement, index);
 	if (text) {
+		int len = sqlite3_column_bytes(statement, index);
 		std::string utf8(text, len);
 		ret = fz::to_wstring_from_utf8(utf8);
 	}
@@ -892,8 +909,8 @@ std::wstring CQueueStorage::Impl::GetColumnText(sqlite3_stmt* statement, int ind
 std::string CQueueStorage::Impl::GetColumnTextUtf8(sqlite3_stmt* statement, int index)
 {
 	char const* text = reinterpret_cast<char const*>(sqlite3_column_text(statement, index));
-	int len = sqlite3_column_bytes(statement, index);
 	if (text) {
+		int len = sqlite3_column_bytes(statement, index);
 		return std::string(text, len);
 	}
 
@@ -920,9 +937,9 @@ int CQueueStorage::Impl::GetColumnInt(sqlite3_stmt* statement, int index, int de
 	}
 }
 
-int64_t CQueueStorage::Impl::ParseServerFromRow(ServerWithCredentials& server)
+int64_t CQueueStorage::Impl::ParseServerFromRow(Site & site)
 {
-	server = ServerWithCredentials();
+	site = Site();
 
 	std::wstring host = GetColumnText(selectServersQuery_, server_table_column_names::host);
 	if (host.empty()) {
@@ -934,7 +951,7 @@ int64_t CQueueStorage::Impl::ParseServerFromRow(ServerWithCredentials& server)
 		return INVALID_DATA;
 	}
 
-	if (!server.server.SetHost(host, port)) {
+	if (!site.server.SetHost(host, port)) {
 		return INVALID_DATA;
 	}
 
@@ -942,14 +959,14 @@ int64_t CQueueStorage::Impl::ParseServerFromRow(ServerWithCredentials& server)
 	if (protocol < 0 || protocol > MAX_VALUE) {
 		return INVALID_DATA;
 	}
-	server.server.SetProtocol(static_cast<ServerProtocol>(protocol));
+	site.server.SetProtocol(static_cast<ServerProtocol>(protocol));
 
 	int type = GetColumnInt(selectServersQuery_, server_table_column_names::type);
 	if (type < 0 || type >= SERVERTYPE_MAX) {
 		return INVALID_DATA;
 	}
 
-	server.server.SetType(static_cast<ServerType>(type));
+	site.server.SetType(static_cast<ServerType>(type));
 
 	int64_t logonType = GetColumnInt64(selectServersQuery_, server_table_column_names::logontype);
 	bool const encrypted = logonType & (1ll << 62);
@@ -958,86 +975,91 @@ int64_t CQueueStorage::Impl::ParseServerFromRow(ServerWithCredentials& server)
 		return INVALID_DATA;
 	}
 
-	server.SetLogonType(static_cast<LogonType>(logonType));
+	site.SetLogonType(static_cast<LogonType>(logonType));
 
-	if (server.credentials.logonType_ != LogonType::anonymous) {
+	if (site.credentials.logonType_ != LogonType::anonymous) {
 		std::wstring user = GetColumnText(selectServersQuery_, server_table_column_names::user);
 		std::wstring pass = GetColumnText(selectServersQuery_, server_table_column_names::password);
 
-		server.SetUser(user);
+		site.SetUser(user);
 		if (encrypted) {
 			size_t pos = pass.find(' ');
 			if (pos == std::string::npos) {
 				return INVALID_DATA;
 			}
 			else {
-				server.credentials.encrypted_ = fz::public_key::from_base64(fz::to_utf8(pass.substr(0, pos)));
+				site.credentials.encrypted_ = fz::public_key::from_base64(fz::to_utf8(pass.substr(0, pos)));
 				pass = pass.substr(pos + 1);
 			}
 		}
-		server.credentials.SetPass(pass);
+		site.credentials.SetPass(pass);
 
-		server.credentials.account_ = GetColumnText(selectServersQuery_, server_table_column_names::account);
-		if (server.credentials.account_.empty() && server.credentials.logonType_ == LogonType::account) {
+		site.credentials.account_ = GetColumnText(selectServersQuery_, server_table_column_names::account);
+		if (site.credentials.account_.empty() && site.credentials.logonType_ == LogonType::account) {
 			return INVALID_DATA;
 		}
 
-		server.credentials.keyFile_ = GetColumnText(selectServersQuery_, server_table_column_names::keyfile);
-		if (server.credentials.keyFile_.empty() && server.credentials.logonType_ == LogonType::key) {
+		site.credentials.keyFile_ = GetColumnText(selectServersQuery_, server_table_column_names::keyfile);
+		if (site.credentials.keyFile_.empty() && site.credentials.logonType_ == LogonType::key) {
 			return INVALID_DATA;
 		}
 	}
 
 	int timezoneOffset = GetColumnInt(selectServersQuery_, server_table_column_names::timezone_offset);
-	if (!server.server.SetTimezoneOffset(timezoneOffset)) {
+	if (!site.server.SetTimezoneOffset(timezoneOffset)) {
 		return INVALID_DATA;
 	}
 
 	std::wstring pasvMode = GetColumnText(selectServersQuery_, server_table_column_names::transfer_mode);
 	if (pasvMode == _T("passive")) {
-		server.server.SetPasvMode(MODE_PASSIVE);
+		site.server.SetPasvMode(MODE_PASSIVE);
 	}
 	else if (pasvMode == _T("active")) {
-		server.server.SetPasvMode(MODE_ACTIVE);
+		site.server.SetPasvMode(MODE_ACTIVE);
 	}
 	else {
-		server.server.SetPasvMode(MODE_DEFAULT);
+		site.server.SetPasvMode(MODE_DEFAULT);
 	}
 
 	int maximumMultipleConnections = GetColumnInt(selectServersQuery_, server_table_column_names::max_connections);
 	if (maximumMultipleConnections < 0) {
 		return INVALID_DATA;
 	}
-	server.server.MaximumMultipleConnections(maximumMultipleConnections);
+	site.server.MaximumMultipleConnections(maximumMultipleConnections);
 
 	std::wstring encodingType = GetColumnText(selectServersQuery_, server_table_column_names::encoding);
 	if (encodingType.empty() || encodingType == _T("Auto")) {
-		server.server.SetEncodingType(ENCODING_AUTO);
+		site.server.SetEncodingType(ENCODING_AUTO);
 	}
 	else if (encodingType == _T("UTF-8")) {
-		server.server.SetEncodingType(ENCODING_UTF8);
+		site.server.SetEncodingType(ENCODING_UTF8);
 	}
 	else {
-		if (!server.server.SetEncodingType(ENCODING_CUSTOM, encodingType)) {
+		if (!site.server.SetEncodingType(ENCODING_CUSTOM, encodingType)) {
 			return INVALID_DATA;
 		}
 	}
 
-	if (CServer::ProtocolHasFeature(server.server.GetProtocol(), ProtocolFeature::PostLoginCommands)) {
+	if (CServer::ProtocolHasFeature(site.server.GetProtocol(), ProtocolFeature::PostLoginCommands)) {
 		std::wstring const commands = GetColumnText(selectServersQuery_, server_table_column_names::post_login_commands);
 		std::vector<std::wstring> postLoginCommands = fz::strtok(commands, '\n');
-		if (!server.server.SetPostLoginCommands(postLoginCommands)) {
+		if (!site.server.SetPostLoginCommands(postLoginCommands)) {
 			return INVALID_DATA;
 		}
 	}
 
 
-	server.server.SetBypassProxy(GetColumnInt(selectServersQuery_, server_table_column_names::bypass_proxy) == 1 );
-	server.server.SetName(GetColumnText(selectServersQuery_, server_table_column_names::name));
+	site.server.SetBypassProxy(GetColumnInt(selectServersQuery_, server_table_column_names::bypass_proxy) == 1 );
+	site.SetName(GetColumnText(selectServersQuery_, server_table_column_names::name).substr(0, 255));
 
 	fz::query_string qs = fz::query_string(GetColumnTextUtf8(selectServersQuery_, server_table_column_names::parameters));
 	for (auto const& pair : qs.pairs()) {
-		server.server.SetExtraParameter(pair.first, fz::to_wstring_from_utf8(pair.second));
+		site.server.SetExtraParameter(pair.first, fz::to_wstring_from_utf8(pair.second));
+	}
+
+	std::wstring site_path = GetColumnText(selectServersQuery_, server_table_column_names::site_path);
+	if (!site_path.empty()) {
+		site.SetSitePath(site_path);
 	}
 
 	return GetColumnInt64(selectServersQuery_, server_table_column_names::id);
@@ -1183,7 +1205,7 @@ bool CQueueStorage::SaveQueue(std::vector<CServerItem*> const& queue)
 	return ret;
 }
 
-int64_t CQueueStorage::GetServer(ServerWithCredentials& server, bool fromBeginning)
+int64_t CQueueStorage::GetServer(Site& site, bool fromBeginning)
 {
 	int64_t ret = -1;
 
@@ -1202,7 +1224,7 @@ int64_t CQueueStorage::GetServer(ServerWithCredentials& server, bool fromBeginni
 			while (res == SQLITE_BUSY);
 
 			if (res == SQLITE_ROW) {
-				ret = d_->ParseServerFromRow(server);
+				ret = d_->ParseServerFromRow(site);
 				if (ret > 0) {
 					break;
 				}
