@@ -8,6 +8,7 @@
 #include "Mainfrm.h"
 #include "asksavepassworddialog.h"
 #include "filezillaapp.h"
+#include "textctrlex.h"
 #include "themeprovider.h"
 
 #include <wx/bmpbuttn.h>
@@ -20,15 +21,6 @@ EVT_MENU(wxID_ANY, CQuickconnectBar::OnMenu)
 EVT_TEXT_ENTER(wxID_ANY, CQuickconnectBar::OnQuickconnect)
 END_EVENT_TABLE()
 
-CQuickconnectBar::CQuickconnectBar()
-	: m_pHost()
-	, m_pUser()
-	, m_pPass()
-	, m_pPort()
-	, m_pMainFrame()
-{
-}
-
 bool CQuickconnectBar::Create(CMainFrame* pParent)
 {
 	m_pMainFrame = pParent;
@@ -48,20 +40,20 @@ bool CQuickconnectBar::Create(CMainFrame* pParent)
 	sizer->Add(mainSizer, wxSizerFlags().Border(wxALL, ConvertDialogToPixels(wxPoint(2, 0)).x));
 
 	mainSizer->Add(new wxStaticText(this, -1, _("&Host:")), wxSizerFlags().Align(wxALIGN_CENTER_VERTICAL));
-	m_pHost = new wxTextCtrl(this, -1, wxString(), wxDefaultPosition, ConvertDialogToPixels(wxSize(63, -1)), wxTE_PROCESS_ENTER);
+	m_pHost = new wxTextCtrlEx(this, -1, wxString(), wxDefaultPosition, ConvertDialogToPixels(wxSize(63, -1)), wxTE_PROCESS_ENTER);
 	m_pHost->SetToolTip(_("Enter the address of the server. To specify the server protocol, prepend the host with the protocol identifier. If no protocol is specified, the default protocol (ftp://) will be used. You can also enter complete URLs in the form protocol://user:pass@host:port here, the values in the other fields will be overwritten then.\n\nSupported protocols are:\n- ftp:// for normal FTP with optional encryption\n- sftp:// for SSH file transfer protocol\n- ftps:// for FTP over TLS (implicit)\n- ftpes:// for FTP over TLS (explicit)"));
 	mainSizer->Add(m_pHost, wxSizerFlags().Align(wxALIGN_CENTER_VERTICAL).Border(wxRIGHT, 5));
 
 	mainSizer->Add(new wxStaticText(this, -1, _("&Username:")), wxSizerFlags().Align(wxALIGN_CENTER_VERTICAL));
-	m_pUser = new wxTextCtrl(this, -1, wxString(), wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER);
+	m_pUser = new wxTextCtrlEx(this, -1, wxString(), wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER);
 	mainSizer->Add(m_pUser, wxSizerFlags().Align(wxALIGN_CENTER_VERTICAL).Border(wxRIGHT, 5));
 
 	mainSizer->Add(new wxStaticText(this, -1, _("Pass&word:")), wxSizerFlags().Align(wxALIGN_CENTER_VERTICAL));
-	m_pPass = new wxTextCtrl(this, -1, wxString(), wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER|wxTE_PASSWORD);
+	m_pPass = new wxTextCtrlEx(this, -1, wxString(), wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER|wxTE_PASSWORD);
 	mainSizer->Add(m_pPass, wxSizerFlags().Align(wxALIGN_CENTER_VERTICAL).Border(wxRIGHT, 5));
 
 	mainSizer->Add(new wxStaticText(this, -1, _("&Port:")), wxSizerFlags().Align(wxALIGN_CENTER_VERTICAL));
-	m_pPort = new wxTextCtrl(this, -1, wxString(), wxDefaultPosition, ConvertDialogToPixels(wxSize(27, -1)), wxTE_PROCESS_ENTER);
+	m_pPort = new wxTextCtrlEx(this, -1, wxString(), wxDefaultPosition, ConvertDialogToPixels(wxSize(27, -1)), wxTE_PROCESS_ENTER);
 	m_pPort->SetToolTip(_("Enter the port on which the server listens. The default for FTP is 21, the default for SFTP is 22."));
 	m_pPort->SetMaxLength(5);
 	mainSizer->Add(m_pPort, wxSizerFlags().Align(wxALIGN_CENTER_VERTICAL).Border(wxRIGHT, 5));
@@ -117,12 +109,12 @@ void CQuickconnectBar::OnQuickconnect(wxCommandEvent& event)
 	std::wstring pass = m_pPass->GetValue().ToStdWstring();
 	std::wstring port = m_pPort->GetValue().ToStdWstring();
 
-	ServerWithCredentials server;
+	Site site;
 
 	std::wstring error;
 
 	CServerPath path;
-	if (!server.ParseUrl(host, port, user, pass, error, path)) {
+	if (!site.ParseUrl(host, port, user, pass, error, path)) {
 		wxString msg = _("Could not parse server address:");
 		msg += _T("\n");
 		msg += error;
@@ -130,21 +122,21 @@ void CQuickconnectBar::OnQuickconnect(wxCommandEvent& event)
 		return;
 	}
 
-	host = server.Format(ServerFormat::host_only);
-	ServerProtocol protocol = server.server.GetProtocol();
+	host = site.Format(ServerFormat::host_only);
+	ServerProtocol protocol = site.server.GetProtocol();
 	switch (protocol)
 	{
 	case FTP:
 	case UNKNOWN:
-		if (CServer::GetProtocolFromPort(server.server.GetPort()) != FTP &&
-			CServer::GetProtocolFromPort(server.server.GetPort()) != UNKNOWN)
+		if (CServer::GetProtocolFromPort(site.server.GetPort()) != FTP &&
+			CServer::GetProtocolFromPort(site.server.GetPort()) != UNKNOWN)
 		{
 			host = _T("ftp://") + host;
 		}
 		break;
 	default:
 		{
-			std::wstring const prefix = server.server.GetPrefixFromProtocol(protocol);
+			std::wstring const prefix = site.server.GetPrefixFromProtocol(protocol);
 			if (!prefix.empty()) {
 				host = prefix + _T("://") + host;
 			}
@@ -153,16 +145,16 @@ void CQuickconnectBar::OnQuickconnect(wxCommandEvent& event)
 	}
 
 	m_pHost->SetValue(host);
-	if (server.server.GetPort() != server.server.GetDefaultPort(server.server.GetProtocol())) {
-		m_pPort->SetValue(wxString::Format(_T("%d"), server.server.GetPort()));
+	if (site.server.GetPort() != site.server.GetDefaultPort(site.server.GetProtocol())) {
+		m_pPort->SetValue(wxString::Format(_T("%d"), site.server.GetPort()));
 	}
 	else {
 		m_pPort->ChangeValue(wxString());
 	}
 
-	m_pUser->SetValue(server.server.GetUser());
-	if (server.credentials.logonType_ != LogonType::anonymous) {
-		m_pPass->SetValue(server.credentials.GetPass());
+	m_pUser->SetValue(site.server.GetUser());
+	if (site.credentials.logonType_ != LogonType::anonymous) {
+		m_pPass->SetValue(site.credentials.GetPass());
 	}
 	else {
 		m_pPass->ChangeValue(wxString());
@@ -175,26 +167,24 @@ void CQuickconnectBar::OnQuickconnect(wxCommandEvent& event)
 	}
 
 	if (event.GetId() == 1) {
-		server.server.SetBypassProxy(true);
+		site.server.SetBypassProxy(true);
 	}
 
-	if (server.credentials.logonType_ != LogonType::anonymous && !CAskSavePasswordDialog::Run(this)) {
+	if (site.credentials.logonType_ != LogonType::anonymous && !CAskSavePasswordDialog::Run(this)) {
 		return;
 	}
 
-	if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) && server.credentials.logonType_ == LogonType::normal) {
-		server.SetLogonType(LogonType::ask);
-		CLoginManager::Get().RememberPassword(server);
+	if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) && site.credentials.logonType_ == LogonType::normal) {
+		site.SetLogonType(LogonType::ask);
+		CLoginManager::Get().RememberPassword(site);
 	}
-	Site site;
-	site.server_ = server;
 	Bookmark bm;
 	bm.m_remoteDir = path;
 	if (!m_pMainFrame->ConnectToSite(site, bm)) {
 		return;
 	}
 
-	CRecentServerList::SetMostRecentServer(server);
+	CRecentServerList::SetMostRecentServer(site);
 }
 
 void CQuickconnectBar::OnQuickconnectDropdown(wxCommandEvent& event)
@@ -217,8 +207,7 @@ void CQuickconnectBar::OnQuickconnectDropdown(wxCommandEvent& event)
 			iter != m_recentServers.end();
 			++iter, ++i)
 		{
-			wxString name(iter->Format(ServerFormat::with_user_and_optional_port));
-			name.Replace(_T("&"), _T("&&"));
+			std::wstring name = LabelEscape(iter->Format(ServerFormat::with_user_and_optional_port)).substr(0, 255);
 			pMenu->Append(10 + i, name);
 		}
 	}
@@ -258,8 +247,7 @@ void CQuickconnectBar::OnMenu(wxCommandEvent& event)
 	auto iter = m_recentServers.cbegin();
 	std::advance(iter, index);
 
-	Site site;
-	site.server_ = *iter;
+	Site site = *iter;
 	m_pMainFrame->ConnectToSite(site, Bookmark());
 }
 
