@@ -47,13 +47,9 @@ CRemoteRecursiveOperation::CRemoteRecursiveOperation(CState &state)
 
 CRemoteRecursiveOperation::~CRemoteRecursiveOperation()
 {
-	if (m_pChmodDlg) {
-		m_pChmodDlg->Destroy();
-		m_pChmodDlg = 0;
-	}
 }
 
-void CRemoteRecursiveOperation::OnStateChange(t_statechange_notifications notification, const wxString&, const void* data2)
+void CRemoteRecursiveOperation::OnStateChange(t_statechange_notifications notification, std::wstring const&, const void* data2)
 {
 	if (notification == STATECHANGE_REMOTE_DIR_OTHER && data2) {
 		std::shared_ptr<CDirectoryListing> const& listing = *reinterpret_cast<std::shared_ptr<CDirectoryListing> const*>(data2);
@@ -78,7 +74,7 @@ void CRemoteRecursiveOperation::StartRecursiveOperation(OperationMode mode, Acti
 	wxCHECK_RET(m_state.IsRemoteConnected(), _T("StartRecursiveOperation while disconnected"));
 	wxCHECK_RET(!finalDir.empty(), _T("Empty final dir in recursive operation"));
 
-	if (mode == recursive_chmod && !m_pChmodDlg) {
+	if (mode == recursive_chmod && !chmodData_) {
 		return;
 	}
 
@@ -235,8 +231,11 @@ void CRemoteRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing*
 
 	++m_processedDirectories;
 
-	ServerWithCredentials const& server = m_state.GetServer();
-	wxASSERT(server);
+	Site const& site = m_state.GetSite();
+	if (!site) {
+		StopRecursiveOperation();
+		return;
+	}
 
 	if (!pDirectoryListing->size() && m_operationMode == recursive_transfer) {
 		if (m_immediate) {
@@ -244,7 +243,7 @@ void CRemoteRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing*
 			m_state.RefreshLocalFile(dir.localDir.GetPath());
 		}
 		else {
-			m_pQueue->QueueFile(true, true, _T(""), _T(""), dir.localDir, CServerPath(), server, -1);
+			m_pQueue->QueueFile(true, true, _T(""), _T(""), dir.localDir, CServerPath(), site, -1);
 			m_pQueue->QueueFile_Finish(false);
 		}
 	}
@@ -254,7 +253,7 @@ void CRemoteRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing*
 	// Is operation restricted to a single child?
 	bool const restrict = static_cast<bool>(dir.restrict);
 
-	std::deque<std::wstring> filesToDelete;
+	std::vector<std::wstring> filesToDelete;
 
 	std::wstring const remotePath = pDirectoryListing->path.GetPath();
 
@@ -353,7 +352,7 @@ void CRemoteRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing*
 					}
 					m_pQueue->QueueFile(!m_immediate, true,
 						entry.name, (entry.name == localFile) ? std::wstring() : localFile,
-						dir.localDir, pDirectoryListing->path, server, entry.size);
+						dir.localDir, pDirectoryListing->path, site, entry.size);
 					added = true;
 				}
 				break;
@@ -365,15 +364,15 @@ void CRemoteRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing*
 			}
 		}
 
-		if (m_operationMode == recursive_chmod && m_pChmodDlg) {
-			const int applyType = m_pChmodDlg->GetApplyType();
+		if (m_operationMode == recursive_chmod && chmodData_) {
+			const int applyType = chmodData_->GetApplyType();
 			if (!applyType ||
 				(!entry.is_dir() && applyType == 1) ||
 				(entry.is_dir() && applyType == 2))
 			{
 				char permissions[9];
-				bool res = m_pChmodDlg->ConvertPermissions(*entry.permissions, permissions);
-				std::wstring newPerms = m_pChmodDlg->GetPermissions(res ? permissions : 0, entry.is_dir()).ToStdWstring();
+				bool res = chmodData_->ConvertPermissions(*entry.permissions, permissions);
+				std::wstring newPerms = chmodData_->GetPermissions(res ? permissions : 0, entry.is_dir());
 				m_state.m_pCommandQueue->ProcessCommand(new CChmodCommand(pDirectoryListing->path, entry.name, newPerms), CCommandQueue::recursiveOperation);
 			}
 		}
@@ -391,14 +390,9 @@ void CRemoteRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing*
 	NextOperation();
 }
 
-void CRemoteRecursiveOperation::SetChmodDialog(CChmodDialog* pChmodDialog)
+void CRemoteRecursiveOperation::SetChmodData(std::unique_ptr<ChmodData> && chmodData)
 {
-	wxASSERT(pChmodDialog);
-
-	if (m_pChmodDlg)
-		m_pChmodDlg->Destroy();
-
-	m_pChmodDlg = pChmodDialog;
+	chmodData_ = std::move(chmodData);
 }
 
 void CRemoteRecursiveOperation::StopRecursiveOperation()
@@ -410,10 +404,7 @@ void CRemoteRecursiveOperation::StopRecursiveOperation()
 	}
 	recursion_roots_.clear();
 
-	if (m_pChmodDlg) {
-		m_pChmodDlg->Destroy();
-		m_pChmodDlg = 0;
-	}
+	chmodData_.reset();
 
 	m_actionAfterBlocker.reset();
 }
@@ -466,15 +457,15 @@ void CRemoteRecursiveOperation::LinkIsNotDir()
 	recursion_root::new_dir dir = root.m_dirsToVisit.front();
 	root.m_dirsToVisit.pop_front();
 
-	ServerWithCredentials const& server = m_state.GetServer();
-	if (!server) {
+	Site const& site = m_state.GetSite();
+	if (!site) {
 		NextOperation();
 		return;
 	}
 
 	if (m_operationMode == recursive_delete) {
 		if (!dir.subdir.empty()) {
-			std::deque<std::wstring> files;
+			std::vector<std::wstring> files;
 			files.push_back(dir.subdir);
 			m_state.m_pCommandQueue->ProcessCommand(new CDeleteCommand(dir.parent, std::move(files)), CCommandQueue::recursiveOperation);
 		}
@@ -487,7 +478,7 @@ void CRemoteRecursiveOperation::LinkIsNotDir()
 		if (m_operationMode != recursive_transfer_flatten) {
 			localPath.MakeParent();
 		}
-		m_pQueue->QueueFile(!m_immediate, true, dir.subdir, (dir.subdir == localFile) ? std::wstring() : localFile, localPath, dir.parent, server, -1);
+		m_pQueue->QueueFile(!m_immediate, true, dir.subdir, (dir.subdir == localFile) ? std::wstring() : localFile, localPath, dir.parent, site, -1);
 		m_pQueue->QueueFile_Finish(m_immediate);
 	}
 
