@@ -16,6 +16,7 @@
 #include "xrc_helper.h"
 
 #include <libfilezilla/uri.hpp>
+#include <libfilezilla/translate.hpp>
 
 #include <wx/clipbrd.h>
 
@@ -122,7 +123,7 @@ protected:
 private:
 	virtual bool CanStartComparison() { return false; }
 	virtual void StartComparison() {}
-	virtual bool get_next_file(wxString&, bool &, int64_t&, fz::datetime&) { return false; }
+	virtual bool get_next_file(std::wstring &, bool &, int64_t&, fz::datetime&) override { return false; }
 	virtual void CompareAddFile(CComparableListing::t_fileEntryFlags) {}
 	virtual void FinishComparison() {}
 	virtual void ScrollTopItem(int) {}
@@ -518,7 +519,7 @@ void CSearchDialog::Run()
 	}
 }
 
-void CSearchDialog::OnStateChange(t_statechange_notifications notification, const wxString&, const void* data2)
+void CSearchDialog::OnStateChange(t_statechange_notifications notification, std::wstring const&, const void* data2)
 {
 	if (notification == STATECHANGE_REMOTE_DIR_OTHER && data2) {
 		if (m_searching == search_mode::remote) {
@@ -717,14 +718,15 @@ void CSearchDialog::OnSearch(wxCommandEvent&)
 			return;
 		}
 
-		CServerPath path;
 
-		ServerWithCredentials const& server = m_state.GetServer();
-		if (!server) {
+		Site const& site = m_state.GetSite();
+		if (!site) {
 			wxMessageBoxEx(_("Connection to server lost."), _("Remote file search"), wxICON_EXCLAMATION);
 			return;
 		}
-		path.SetType(server.server.GetType());
+
+		CServerPath path;
+		path.SetType(site.server.GetType());
 		if (!path.SetPath(xrc_call(*this, "ID_PATH", &wxTextCtrl::GetValue).ToStdWstring()) || path.empty()) {
 			wxMessageBoxEx(_("Need to enter valid remote path"), _("Remote file search"), wxICON_EXCLAMATION);
 			return;
@@ -1041,8 +1043,8 @@ void CSearchDialog::OnDownload(wxCommandEvent&)
 		return;
 	}
 
-	ServerWithCredentials const& server = m_state.GetServer();
-	if (!server) {
+	Site const& site = m_state.GetSite();
+	if (!site) {
 		wxBell();
 		return;
 	}
@@ -1074,7 +1076,7 @@ void CSearchDialog::OnDownload(wxCommandEvent&)
 
 		m_pQueue->QueueFile(!start, true,
 			entry.name, (localName != entry.name) ? localName : std::wstring(),
-			target_path, remote_path, server, entry.size);
+			target_path, remote_path, site, entry.size);
 	}
 	m_pQueue->QueueFile_Finish(start);
 
@@ -1122,8 +1124,8 @@ void CSearchDialog::OnUpload(wxCommandEvent&)
 		return;
 	}
 
-	ServerWithCredentials const& server = m_state.GetServer();
-	if (!server) {
+	Site const& site = m_state.GetSite();
+	if (!site) {
 		wxBell();
 		return;
 	}
@@ -1153,7 +1155,7 @@ void CSearchDialog::OnUpload(wxCommandEvent&)
 
 		m_pQueue->QueueFile(!start, false,
 			entry.name, (localName != entry.name) ? localName : std::wstring(),
-			local_path, target_path, server, entry.size);
+			local_path, target_path, site, entry.size);
 	}
 	m_pQueue->QueueFile_Finish(start);
 
@@ -1209,8 +1211,8 @@ void CSearchDialog::OnEdit(wxCommandEvent&)
 		return;
 	}
 
-	ServerWithCredentials const& server = m_state.GetServer();
-	if (!server) {
+	Site const& site = m_state.GetSite();
+	if (!site) {
 		wxBell();
 		return;
 	}
@@ -1229,7 +1231,7 @@ void CSearchDialog::OnEdit(wxCommandEvent&)
 		const CDirentry& entry = m_results->remoteFileData_[item];
 		const CServerPath path = m_results->remoteFileData_[item].path;
 
-		pEditHandler->Edit(CEditHandler::remote, entry.name, path, server, entry.size, this);
+		pEditHandler->Edit(CEditHandler::remote, entry.name, path, site, entry.size, this);
 	}
 }
 
@@ -1269,7 +1271,8 @@ void CSearchDialog::OnDelete(wxCommandEvent&)
 
 		for (auto const& file : selected_files) {
 			CRemoteSearchFileData const& entry = m_results->remoteFileData_[file];
-			std::deque<std::wstring> files_to_delete;
+			// Possible improvement: Group by path
+			std::vector<std::wstring> files_to_delete;
 			files_to_delete.push_back(entry.name);
 			m_state.m_pCommandQueue->ProcessCommand(new CDeleteCommand(entry.path, std::move(files_to_delete)));
 		}
@@ -1408,8 +1411,8 @@ void CSearchDialog::OnGetUrl(wxCommandEvent& event)
 		return;
 	}
 
-	ServerWithCredentials const& server = m_state.GetServer();
-	if (!server) {
+	Site const& site = m_state.GetSite();
+	if (!site) {
 		wxBell();
 		return;
 	}
@@ -1419,10 +1422,10 @@ void CSearchDialog::OnGetUrl(wxCommandEvent& event)
 		return;
 	}
 
-	wxString const url = server.Format((event.GetId() == XRCID("ID_MENU_SEARCH_GETURL_PASSWORD")) ? ServerFormat::url_with_password : ServerFormat::url);
+	std::wstring const url = site.Format((event.GetId() == XRCID("ID_MENU_SEARCH_GETURL_PASSWORD")) ? ServerFormat::url_with_password : ServerFormat::url);
 
-	auto getUrl = [](wxString const& serverPart, CServerPath const& path, std::wstring const& name) {
-		wxString url = serverPart;
+	auto getUrl = [](std::wstring const& serverPart, CServerPath const& path, std::wstring const& name) {
+		std::wstring url = serverPart;
 
 		auto const pathPart = fz::percent_encode_w(path.FormatFilename(name, false), true);
 		if (!pathPart.empty() && pathPart[0] != '/') {
@@ -1433,7 +1436,7 @@ void CSearchDialog::OnGetUrl(wxCommandEvent& event)
 		return url;
 	};
 
-	wxString urls;
+	std::wstring urls;
 
 	int sel = -1;
 	while ((sel = m_results->GetNextItem(sel, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) >= 0) {
@@ -1445,9 +1448,9 @@ void CSearchDialog::OnGetUrl(wxCommandEvent& event)
 		auto const& entry = m_results->remoteFileData_[index];
 		urls += getUrl(url, entry.path, entry.name);
 #ifdef __WXMSW__
-		urls += _T("\r\n");
+		urls += L"\r\n";
 #else
-		urls += _T("\n");
+		urls += L"\n";
 #endif
 	}
 
