@@ -4,7 +4,170 @@
 #include "serverdata.h"
 #include "Options.h"
 
-bool ServerWithCredentials::ParseUrl(std::wstring const& host, std::wstring const& port, std::wstring const& user, std::wstring const& pass, std::wstring &error, CServerPath &path, ServerProtocol const hint)
+#include <libfilezilla/translate.hpp>
+
+bool Bookmark::operator==(Bookmark const& b) const
+{
+	if (m_localDir != b.m_localDir) {
+		return false;
+	}
+
+	if (m_remoteDir != b.m_remoteDir) {
+		return false;
+	}
+
+	if (m_sync != b.m_sync) {
+		return false;
+	}
+
+	if (m_comparison != b.m_comparison) {
+		return false;
+	}
+
+	if (m_name != b.m_name) {
+		return false;
+	}
+
+	return true;
+}
+
+Site::Site(Site const& s)
+	: server(s.server)
+	, originalServer(s.originalServer)
+	, credentials(s.credentials)
+	, comments_(s.comments_)
+	, m_default_bookmark(s.m_default_bookmark)
+	, m_bookmarks(s.m_bookmarks)
+	, m_colour(s.m_colour)
+{
+	if (s.data_) {
+		data_ = std::make_shared<SiteHandleData>(*s.data_);
+	}
+}
+
+Site& Site::operator=(Site const& s)
+{
+	if (this != &s) {
+		server = s.server;
+		originalServer = s.originalServer;
+		credentials = s.credentials;
+		comments_ = s.comments_;
+		m_default_bookmark = s.m_default_bookmark;
+		m_bookmarks = s.m_bookmarks;
+		m_colour = s.m_colour;
+		data_.reset();
+
+		if (s.data_) {
+			data_ = std::make_shared<SiteHandleData>(*s.data_);
+		}
+	}
+
+	return *this;
+}
+
+bool Site::operator==(Site const& s) const
+{
+	if (server != s.server) {
+		return false;
+	}
+
+	if (comments_ != s.comments_) {
+		return false;
+	}
+
+	if (m_default_bookmark != s.m_default_bookmark) {
+		return false;
+	}
+
+	if (m_bookmarks != s.m_bookmarks) {
+		return false;
+	}
+
+	if (static_cast<bool>(data_) != static_cast<bool>(s.data_)) {
+		return false;
+	}
+
+	if (data_ && *data_ != *s.data_) {
+		return false;
+	}
+
+	if (m_colour != s.m_colour) {
+		return false;
+	}
+
+	return true;
+}
+
+void Site::SetName(std::wstring const& name) {
+	if (!data_) {
+		data_ = std::make_shared<SiteHandleData>();
+	}
+	data_->name_ = name;
+}
+
+std::wstring const& Site::GetName() const
+{
+	if (data_) {
+		return data_->name_;
+	}
+	else {
+		static std::wstring empty;
+		return empty;
+	}
+}
+
+void Site::SetSitePath(std::wstring const& sitePath) {
+	if (!data_) {
+		data_ = std::make_shared<SiteHandleData>();
+	}
+	data_->sitePath_ = sitePath;
+}
+
+std::wstring const& Site::SitePath() const
+{
+	if (data_) {
+		return data_->sitePath_;
+	}
+	else {
+		static std::wstring empty;
+		return empty;
+	}
+}
+
+ServerHandle Site::Handle() const
+{
+	return data_;
+}
+
+void Site::Update(Site const& rhs)
+{
+	CServer newServer;
+	std::optional<CServer> newOriginalServer;
+	if (originalServer && originalServer->SameResource(rhs.GetOriginalServer())) {
+		newOriginalServer = rhs.GetOriginalServer();
+	}
+	else {
+		newOriginalServer = originalServer;
+	}
+
+	if (server.SameResource(rhs.server)) {
+		newServer = rhs.server;
+	}
+	else {
+		newServer = server;
+	}
+
+	std::shared_ptr<SiteHandleData> data = data_;
+	*this = rhs;
+	server = newServer;
+	originalServer = newOriginalServer;
+	if (data && rhs.data_) {
+		*data = *rhs.data_;
+		data_ = data;
+	}
+}
+
+bool Site::ParseUrl(std::wstring const& host, std::wstring const& port, std::wstring const& user, std::wstring const& pass, std::wstring &error, CServerPath &path, ServerProtocol const hint)
 {
 	unsigned int nPort = 0;
 	if (!port.empty()) {
@@ -19,7 +182,7 @@ bool ServerWithCredentials::ParseUrl(std::wstring const& host, std::wstring cons
 	return ParseUrl(host, nPort, user, pass, error, path, hint);
 }
 
-bool ServerWithCredentials::ParseUrl(std::wstring host, unsigned int port, std::wstring user, std::wstring pass, std::wstring &error, CServerPath &path, ServerProtocol const hint)
+bool Site::ParseUrl(std::wstring host, unsigned int port, std::wstring user, std::wstring pass, std::wstring &error, CServerPath &path, ServerProtocol const hint)
 {
 	server.SetType(DEFAULT);
 
@@ -54,7 +217,7 @@ bool ServerWithCredentials::ParseUrl(std::wstring host, unsigned int port, std::
 
 		size_t next_at = host.find('@', pos + 1);
 		while (next_at != std::wstring::npos) {
-			if (slash != std::wstring::npos  && next_at > slash) {
+			if (slash != std::wstring::npos && next_at > slash) {
 				break;
 			}
 
@@ -177,7 +340,7 @@ bool ServerWithCredentials::ParseUrl(std::wstring host, unsigned int port, std::
 	return true;
 }
 
-void ServerWithCredentials::SetLogonType(LogonType logonType)
+void Site::SetLogonType(LogonType logonType)
 {
 	credentials.logonType_ = logonType;
 	if (logonType == LogonType::anonymous) {
@@ -185,7 +348,7 @@ void ServerWithCredentials::SetLogonType(LogonType logonType)
 	}
 }
 
-void ServerWithCredentials::SetUser(std::wstring const& user)
+void Site::SetUser(std::wstring const& user)
 {
 	if (credentials.logonType_ == LogonType::anonymous) {
 		server.SetUser(L"");
@@ -219,6 +382,7 @@ void ProtectedCredentials::Protect(fz::public_key const& key)
 {
 	if (logonType_ != LogonType::normal && logonType_ != LogonType::account) {
 		password_.clear();
+		encrypted_ = fz::public_key();
 		return;
 	}
 
@@ -309,3 +473,16 @@ bool ProtectedCredentials::Unprotect(fz::private_key const& key, bool on_failure
 
 	return ret;
 }
+
+SiteHandleData toSiteHandle(ServerHandle const& handle)
+{
+	auto l = handle.lock();
+	if (l) {
+		auto d = dynamic_cast<SiteHandleData const*>(l.get());
+		if (d) {
+			return *d;
+		}
+	}
+
+	return SiteHandleData();
+}
