@@ -1,10 +1,7 @@
-#ifndef __OPTIONSPAGE_H__
-#define __OPTIONSPAGE_H__
+#ifndef FILEZILLA_INTERFACE_OPTIONSPAGE_HEADER
+#define FILEZILLA_INTERFACE_OPTIONSPAGE_HEADER
 
-#define SAFE_XRCCTRL(id, type) \
-	if (!XRCCTRL(*this, id, type)) \
-		return false; \
-	XRCCTRL(*this, id, type)
+#include <wx/panel.h>
 
 class COptions;
 class CSettingsDialog;
@@ -15,7 +12,6 @@ public:
 
 	void UpdateMaxPageSize(wxSize& maxSize);
 
-	virtual wxString GetResourceName() = 0;
 	virtual bool LoadPage() = 0;
 	virtual bool SavePage() = 0;
 	virtual bool Validate() { return true; }
@@ -51,10 +47,16 @@ public:
 	virtual bool OnDisplayedFirstTime();
 
 protected:
-	COptions* m_pOptions;
-	CSettingsDialog* m_pOwner;
+	virtual wxString GetResourceName() const {
+		return wxString();
+	}
 
-	bool m_was_selected;
+	virtual bool CreateControls(wxWindow* parent);
+
+	COptions* m_pOptions{};
+	CSettingsDialog* m_pOwner{};
+
+	bool m_was_selected{};
 };
 
-#endif //__OPTIONSPAGE_H__
+#endif
