@@ -4,10 +4,10 @@
 class COptionsPageConnectionPassive final : public COptionsPage
 {
 public:
-	virtual wxString GetResourceName() { return _T("ID_SETTINGS_CONNECTION_PASSIVE"); }
-	virtual bool LoadPage();
-	virtual bool SavePage();
-	virtual bool Validate();
+	virtual wxString GetResourceName() const override { return _T("ID_SETTINGS_CONNECTION_PASSIVE"); }
+	virtual bool LoadPage() override;
+	virtual bool SavePage() override;
+	virtual bool Validate() override;
 };
 
 #endif
