@@ -1,15 +1,15 @@
-#ifndef __OPTIONSPAGE_SIZEFORMATTING_H__
-#define __OPTIONSPAGE_SIZEFORMATTING_H__
+#ifndef FILEZILLA_INTERFACE_OPTIONSPAGE_SIZEFORMATTING_HEADER
+#define FILEZILLA_INTERFACE_OPTIONSPAGE_SIZEFORMATTING_HEADER
 
 #include "../sizeformatting.h"
 
 class COptionsPageSizeFormatting final : public COptionsPage
 {
 public:
-	virtual wxString GetResourceName() { return _T("ID_SETTINGS_SIZEFORMATTING"); }
-	virtual bool LoadPage();
-	virtual bool SavePage();
-	virtual bool Validate();
+	virtual wxString GetResourceName() const override { return _T("ID_SETTINGS_SIZEFORMATTING"); }
+	virtual bool LoadPage() override;
+	virtual bool SavePage() override;
+	virtual bool Validate() override;
 
 	void UpdateControls();
 	void UpdateExamples();
@@ -24,4 +24,4 @@ public:
 	wxString FormatSize(int64_t size);
 };
 
-#endif //__OPTIONSPAGE_SIZEFORMATTING_H__
+#endif
