@@ -1,12 +1,15 @@
 #include <filezilla.h>
 #include "sitemanager.h"
 
+#include "dialogex.h"
 #include "filezillaapp.h"
 #include "ipcmutex.h"
 #include "loginmanager.h"
 #include "Options.h"
 #include "xmlfunctions.h"
 
+#include <libfilezilla/translate.hpp>
+
 namespace {
 struct background_color {
 	wxColour const color;
@@ -26,61 +29,6 @@ background_color const background_colors[] = {
 };
 }
 
-bool Bookmark::operator==(Bookmark const& b) const
-{
-	if (m_localDir != b.m_localDir) {
-		return false;
-	}
-
-	if (m_remoteDir != b.m_remoteDir) {
-		return false;
-	}
-
-	if (m_sync != b.m_sync) {
-		return false;
-	}
-
-	if (m_comparison != b.m_comparison) {
-		return false;
-	}
-
-	if (m_name != b.m_name) {
-		return false;
-	}
-
-	return true;
-}
-
-bool Site::operator==(Site const& s) const
-{
-	if (server_ != s.server_) {
-		return false;
-	}
-
-	if (m_comments != s.m_comments) {
-		return false;
-	}
-
-	if (m_default_bookmark != s.m_default_bookmark) {
-		return false;
-	}
-
-	if (m_bookmarks != s.m_bookmarks) {
-		return false;
-	}
-
-	if (m_path != s.m_path) {
-		return false;
-	}
-
-	if (m_colour != s.m_colour) {
-		return false;
-	}
-
-	return true;
-}
-
-
 std::map<int, std::unique_ptr<Site>> CSiteManager::m_idMap;
 
 bool CSiteManager::Load(CSiteManagerXmlHandler& handler)
@@ -112,7 +60,7 @@ bool CSiteManager::Load(pugi::xml_node element, CSiteManagerXmlHandler& handler)
 			}
 
 			const bool expand = GetTextAttribute(child, "expanded") != _T("0");
-			if (!handler.AddFolder(name, expand)) {
+			if (!handler.AddFolder(name.substr(0, 255), expand)) {
 				return false;
 			}
 			Load(child, handler);
@@ -152,18 +100,15 @@ bool CSiteManager::ReadBookmarkElement(Bookmark & bookmark, pugi::xml_node eleme
 
 std::unique_ptr<Site> CSiteManager::ReadServerElement(pugi::xml_node element)
 {
-	ServerWithCredentials server;
-	if (!::GetServer(element, server)) {
-		return 0;
-	}
-	if (server.server.GetName().empty()) {
-		return 0;
-	}
-
 	auto data = std::make_unique<Site>();
-	data->server_ = server;
+	if (!::GetServer(element, *data)) {
+		return nullptr;
+	}
+	if (data->GetName().empty()) {
+		return nullptr;
+	}
 
-	data->m_comments = GetTextElement(element, "Comments");
+	data->comments_ = GetTextElement(element, "Comments");
 	data->m_colour = GetColourFromIndex(GetTextElementInt(element, "Colour"));
 
 	ReadBookmarkElement(data->m_default_bookmark, element);
@@ -177,7 +122,7 @@ std::unique_ptr<Site> CSiteManager::ReadServerElement(pugi::xml_node element)
 
 		Bookmark bookmarkData;
 		if (ReadBookmarkElement(bookmarkData, bookmark)) {
-			bookmarkData.m_name = name;
+			bookmarkData.m_name = name.substr(0, 255);
 			data->m_bookmarks.push_back(bookmarkData);
 		}
 	}
@@ -237,12 +182,12 @@ public:
 
 	virtual bool AddSite(std::unique_ptr<Site> data)
 	{
-		wxString newName(data->server_.server.GetName());
+		std::wstring newName(data->GetName());
 		int i = GetInsertIndex(m_pMenu, newName);
-		newName.Replace(_T("&"), _T("&&"));
+		newName = LabelEscape(newName);
 		wxMenuItem* pItem = m_pMenu->Insert(i, wxID_ANY, newName);
 
-		data->m_path = path + _T("/") + CSiteManager::EscapeSegment(data->server_.server.GetName());
+		data->SetSitePath(path + _T("/") + CSiteManager::EscapeSegment(data->GetName()));
 
 		(*m_idMap)[pItem->GetId()] = std::move(data);
 
@@ -259,9 +204,9 @@ public:
 		wxMenu* pChild = m_pMenu;
 		m_pMenu = m_parents.back();
 		if (pChild->GetMenuItemCount()) {
-			wxString name = m_childNames.back();
+			std::wstring name = m_childNames.back();
 			int i = GetInsertIndex(m_pMenu, name);
-			name.Replace(_T("&"), _T("&&"));
+			name = LabelEscape(name);
 
 			wxMenuItem* pItem = new wxMenuItem(m_pMenu, wxID_ANY, name, _T(""), wxITEM_NORMAL, pChild);
 			m_pMenu->Insert(i, pItem);
@@ -566,13 +511,13 @@ std::pair<std::unique_ptr<Site>, Bookmark> CSiteManager::DoGetSiteByPath(std::ws
 			ret.second = ret.first->m_default_bookmark;
 		}
 
-		ret.first->m_path = BuildPath(c, segments);
+		ret.first->SetSitePath(BuildPath(c, segments));
 	}
 
 	return ret;
 }
 
-std::wstring CSiteManager::AddServer(ServerWithCredentials server)
+std::wstring CSiteManager::AddServer(Site site)
 {
 	// We have to synchronize access to sitemanager.xml so that multiple processed don't write
 	// to the same file or one is reading while the other one writes.
@@ -619,10 +564,10 @@ std::wstring CSiteManager::AddServer(ServerWithCredentials server)
 		name = _("New site").ToStdWstring() + fz::sprintf(L" %d", ++i);
 	}
 
-	server.server.SetName(name);
+	site.SetName(name);
 
 	auto xServer = element.append_child("Server");
-	SetServer(xServer, server);
+	SetServer(xServer, site);
 	AddTextElement(xServer, name);
 
 	if (!file.Save(false)) {
@@ -647,7 +592,7 @@ pugi::xml_node CSiteManager::GetElementByPath(pugi::xml_node node, std::vector<s
 				continue;
 			}
 
-			wxString name = GetTextElement_Trimmed(child, "Name");
+			std::wstring name = GetTextElement_Trimmed(child, "Name");
 			if (name.empty()) {
 				name = GetTextElement_Trimmed(child);
 			}
@@ -670,7 +615,7 @@ pugi::xml_node CSiteManager::GetElementByPath(pugi::xml_node node, std::vector<s
 	return node;
 }
 
-bool CSiteManager::AddBookmark(std::wstring sitePath, const wxString& name, const wxString &local_dir, const CServerPath &remote_dir, bool sync, bool comparison)
+bool CSiteManager::AddBookmark(std::wstring sitePath, wxString const& name, wxString const& local_dir, CServerPath const& remote_dir, bool sync, bool comparison)
 {
 	if (local_dir.empty() && remote_dir.empty()) {
 		return false;
@@ -856,6 +801,7 @@ wxString CSiteManager::GetColourName(int i)
 
 void CSiteManager::Rewrite(CLoginManager & loginManager, pugi::xml_node element, bool on_failure_set_to_ask)
 {
+	bool const forget = COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) != 0;
 	for (auto child = element.first_child(); child; child = child.next_sibling()) {
 		if (!strcmp(child.name(), "Folder")) {
 			Rewrite(loginManager, child, on_failure_set_to_ask);
@@ -863,8 +809,11 @@ void CSiteManager::Rewrite(CLoginManager & loginManager, pugi::xml_node element,
 		else if (!strcmp(child.name(), "Server")) {
 			auto site = ReadServerElement(child);
 			if (site) {
-				loginManager.AskDecryptor(site->server_.credentials.encrypted_, true, false);
-				site->server_.credentials.Unprotect(loginManager.GetDecryptor(site->server_.credentials.encrypted_), on_failure_set_to_ask);
+				if (!forget) {
+					loginManager.AskDecryptor(site->credentials.encrypted_, true, false);
+					site->credentials.Unprotect(loginManager.GetDecryptor(site->credentials.encrypted_), on_failure_set_to_ask);
+				}
+				site->credentials.Protect();
 				Save(child, *site);
 			}
 		}
@@ -897,10 +846,10 @@ void CSiteManager::Rewrite(CLoginManager & loginManager, bool on_failure_set_to_
 
 void CSiteManager::Save(pugi::xml_node element, Site const& site)
 {
-	SetServer(element, site.server_);
+	SetServer(element, site);
 
 	// Save comments
-	AddTextElement(element, "Comments", site.m_comments.ToStdWstring());
+	AddTextElement(element, "Comments", site.comments_);
 
 	// Save colour
 	AddTextElement(element, "Colour", CSiteManager::GetColourIndex(site.m_colour));
@@ -929,3 +878,110 @@ void CSiteManager::Save(pugi::xml_node element, Site const& site)
 		AddTextElementUtf8(node, "DirectoryComparison", bookmark.m_comparison ? "1" : "0");
 	}
 }
+
+namespace {
+pugi::xml_node GetChildWithName(pugi::xml_node element, std::wstring const& name)
+{
+	pugi::xml_node child;
+	for (child = element.first_child(); child; child = child.next_sibling()) {
+		std::wstring childName = GetTextElement_Trimmed(child, "Name");
+		if (childName.empty()) {
+			childName = GetTextElement_Trimmed(child);
+		}
+		if (!fz::stricmp(name, childName)) {
+			return child;
+		}
+	}
+	
+	return pugi::xml_node();
+}
+
+pugi::xml_node GetOrCreateFolderWithName(pugi::xml_node element, std::wstring const& name)
+{
+	pugi::xml_node child = GetChildWithName(element, name);
+	if (child) {
+		if (strcmp(child.name(), "Folder")) {
+			// We do not allow servers and directories to share the same name
+			child = pugi::xml_node();
+		}
+	}
+	else {
+		child = element.append_child("Folder");
+		AddTextElement(child, name);
+	}
+	return child;
+}
+}
+
+bool CSiteManager::ImportSites(pugi::xml_node sites)
+{
+	CInterProcessMutex mutex(MUTEX_SITEMANAGER);
+
+	CXmlFile file(wxGetApp().GetSettingsFile(_T("sitemanager")));
+	auto element = file.Load();
+	if (!element) {
+		wxString msg = wxString::Format(_("Could not load \"%s\", please make sure the file is valid and can be accessed.\nAny changes made in the Site Manager will not be saved."), file.GetFileName());
+		wxMessageBoxEx(msg, _("Error loading xml file"), wxICON_ERROR);
+
+		return false;
+	}
+
+	auto currentSites = element.child("Servers");
+	if (!currentSites) {
+		currentSites = element.append_child("Servers");
+	}
+
+	if (!ImportSites(sites, currentSites)) {
+		return false;
+	}
+
+	return file.Save(true);
+}
+
+bool CSiteManager::ImportSites(pugi::xml_node sitesToImport, pugi::xml_node existingSites)
+{
+	for (auto importFolder = sitesToImport.child("Folder"); importFolder; importFolder = importFolder.next_sibling("Folder")) {
+		std::wstring name = GetTextElement_Trimmed(importFolder, "Name").substr(0, 255);
+		if (name.empty()) {
+			name = GetTextElement_Trimmed(importFolder);
+		}
+		if (name.empty()) {
+			continue;
+		}
+
+		std::wstring newName = name.substr(0, 240);
+		int i = 2;
+		pugi::xml_node folder;
+		while (!(folder = GetOrCreateFolderWithName(existingSites, newName))) {
+			newName = fz::sprintf(L"%s %d", name.substr(0, 240), i++);
+		}
+
+		ImportSites(importFolder, folder);
+	}
+
+	auto & loginManager = CLoginManager::Get();
+
+	for (auto importSite = sitesToImport.child("Server"); importSite; importSite = importSite.next_sibling("Server")) {
+		auto site = ReadServerElement(importSite);
+		if (!site) {
+			continue;
+		}
+			
+		// Find free name
+		auto const name = site->GetName();
+		std::wstring newName = name;
+		int i = 2;
+		while (GetChildWithName(existingSites, newName)) {
+			newName = fz::sprintf(L"%s %d", name.substr(0, 240), i++);
+		}
+		site->SetName(newName);
+
+		site->credentials.Unprotect(loginManager.GetDecryptor(site->credentials.encrypted_), false);
+		site->credentials.Protect();
+
+		auto xsite = existingSites.append_child("Server");
+		Save(xsite, *site);
+	}
+
+	return true;
+}
