@@ -52,6 +52,11 @@ CState* CContextManager::CreateState(CMainFrame &mainFrame)
 
 	NotifyHandlers(pState, STATECHANGE_NEWCONTEXT, _T(""), 0);
 
+	if (!pState->CreateEngine()) {
+		DestroyState(pState);
+		return nullptr;
+	}
+
 	return pState;
 }
 
@@ -161,7 +166,7 @@ size_t CContextManager::HandlerCount(t_statechange_notifications notification) c
 	return m_handlers[notification].size();
 }
 
-void CContextManager::NotifyHandlers(CState* pState, t_statechange_notifications notification, wxString const& data, void const* data2)
+void CContextManager::NotifyHandlers(CState* pState, t_statechange_notifications notification, std::wstring const& data, void const* data2)
 {
 	wxASSERT(notification != STATECHANGE_NONE && notification != STATECHANGE_MAX);
 
@@ -184,14 +189,14 @@ CState* CContextManager::GetCurrentContext()
 	return m_contexts[m_current_context];
 }
 
-void CContextManager::NotifyAllHandlers(t_statechange_notifications notification, wxString const& data, void const* data2)
+void CContextManager::NotifyAllHandlers(t_statechange_notifications notification, std::wstring const& data, void const* data2)
 {
 	for (auto const& context : m_contexts) {
 		context->NotifyHandlers(notification, data, data2);
 	}
 }
 
-void CContextManager::NotifyGlobalHandlers(t_statechange_notifications notification, wxString const& data, void const* data2)
+void CContextManager::NotifyGlobalHandlers(t_statechange_notifications notification, std::wstring const& data, void const* data2)
 {
 	auto const& handlers = m_handlers[notification];
 	for (auto const& handler : handlers) {
@@ -205,14 +210,15 @@ void CContextManager::ProcessDirectoryListing(CServer const& server, std::shared
 		if (state == exempt) {
 			continue;
 		}
-		if (state->GetServer() && state->GetServer().server == server) {
+		if (state->GetSite() && state->GetSite().server == server) {
 			state->SetRemoteDir(listing, false);
 		}
 	}
 }
 
 CState::CState(CMainFrame &mainFrame)
-	: m_mainFrame(mainFrame)
+	: pool_(mainFrame.GetEngineContext().GetThreadPool())
+	, m_mainFrame(mainFrame)
 {
 	m_title = _("Not connected");
 
@@ -275,7 +281,7 @@ bool CState::SetLocalDir(CLocalPath const& dir, std::wstring *error, bool rememb
 	}
 
 	if (!m_sync_browse.local_root.empty()) {
-		wxASSERT(m_site.server_);
+		wxASSERT(m_site);
 
 		if (dir != m_sync_browse.local_root && !dir.IsSubdirOf(m_sync_browse.local_root)) {
 			wxString msg = wxString::Format(_("The local directory '%s' is not below the synchronization root (%s).\nDisable synchronized browsing and continue changing the local directory?"),
@@ -348,7 +354,7 @@ bool CState::SetRemoteDir(std::shared_ptr<CDirectoryListing> const& pDirectoryLi
 
 		if (m_pDirectoryListing) {
 			m_pDirectoryListing = 0;
-			NotifyHandlers(STATECHANGE_REMOTE_DIR, wxString(), &primary);
+			NotifyHandlers(STATECHANGE_REMOTE_DIR, std::wstring(), &primary);
 		}
 		m_previouslyVisitedRemoteSubdir.clear();
 		return true;
@@ -384,7 +390,7 @@ bool CState::SetRemoteDir(std::shared_ptr<CDirectoryListing> const& pDirectoryLi
 
 	m_pDirectoryListing = pDirectoryListing;
 
-	NotifyHandlers(STATECHANGE_REMOTE_DIR, wxString(), &primary);
+	NotifyHandlers(STATECHANGE_REMOTE_DIR, std::wstring(), &primary);
 
 	bool compare = m_changeDirFlags.compare;
 	if (primary) {
@@ -481,9 +487,9 @@ void CState::LocalDirCreated(const CLocalPath& path)
 		return;
 	}
 
-	wxString next_segment = path.GetPath().substr(m_localDir.GetPath().size());
-	int pos = next_segment.Find(CLocalPath::path_separator);
-	if (pos <= 0) {
+	std::wstring next_segment = path.GetPath().substr(m_localDir.GetPath().size());
+	size_t pos = next_segment.find(CLocalPath::path_separator);
+	if (pos == std::wstring::npos || !pos) {
 		// Shouldn't ever come true
 		return;
 	}
@@ -491,18 +497,16 @@ void CState::LocalDirCreated(const CLocalPath& path)
 	// Current local path is /foo/
 	// Called with /foo/bar/baz/
 	// -> Refresh /foo/bar/
-	next_segment = next_segment.Left(pos);
+	next_segment = next_segment.substr(0, pos);
 	NotifyHandlers(STATECHANGE_LOCAL_REFRESH_FILE, next_segment);
 }
 
 void CState::SetSite(Site const& site, CServerPath const& path)
 {
-	if (m_site.server_) {
-		if (site.server_ && site.server_ == m_site.server_ &&
-			site.server_.server.GetName() == m_site.server_.server.GetName() &&
-			site.server_.server.MaximumMultipleConnections() == m_site.server_.server.MaximumMultipleConnections())
-		{
-			// Nothing changes
+	if (m_site) {
+		if (site == m_site) {
+			// Nothing changes, other than possibly credentials
+			m_site = site;
 			return;
 		}
 
@@ -510,11 +514,11 @@ void CState::SetSite(Site const& site, CServerPath const& path)
 		m_pCertificate.reset();
 		m_pSftpEncryptionInfo.reset();
 	}
-	if (site.server_) {
+	if (site) {
 		if (!path.empty()) {
 			m_last_path = path;
 		}
-		else if (m_last_site.server_ != site.server_) {
+		else if (m_last_site.server != site.server) {
 			m_last_path.clear();
 		}
 		m_last_site = site;
@@ -534,11 +538,6 @@ Site const& CState::GetSite() const
 	return m_site;
 }
 
-ServerWithCredentials const& CState::GetServer() const
-{
-	return m_site.server_;
-}
-
 wxString CState::GetTitle() const
 {
 	return m_title;
@@ -546,7 +545,7 @@ wxString CState::GetTitle() const
 
 bool CState::Connect(Site const& site, CServerPath const& path, bool compare)
 {
-	if (!site.server_) {
+	if (!site) {
 		return false;
 	}
 	if (!m_pEngine) {
@@ -557,13 +556,13 @@ bool CState::Connect(Site const& site, CServerPath const& path, bool compare)
 	}
 	m_pRemoteRecursiveOperation->StopRecursiveOperation();
 	SetSyncBrowse(false);
-
-	m_pCommandQueue->ProcessCommand(new CConnectCommand(site.server_.server, site.server_.credentials));
-	m_pCommandQueue->ProcessCommand(new CListCommand(path, std::wstring(), LIST_FLAG_FALLBACK_CURRENT));
+	m_changeDirFlags.compare = compare;
 
 	SetSite(site, path);
 
-	m_changeDirFlags.compare = compare;
+	// Use m_site from here on
+	m_pCommandQueue->ProcessCommand(new CConnectCommand(m_site.server, m_site.Handle(), m_site.credentials));
+	m_pCommandQueue->ProcessCommand(new CListCommand(path, std::wstring(), LIST_FLAG_FALLBACK_CURRENT));
 
 	return true;
 }
@@ -679,7 +678,7 @@ void CState::UnregisterHandler(CStateEventHandler* pHandler, t_statechange_notif
 	}
 }
 
-void CState::NotifyHandlers(t_statechange_notifications notification, const wxString& data, const void* data2)
+void CState::NotifyHandlers(t_statechange_notifications notification, std::wstring const& data, const void* data2)
 {
 	wxASSERT(notification != STATECHANGE_NONE && notification != STATECHANGE_MAX);
 
@@ -722,7 +721,7 @@ CGlobalStateEventHandler::~CGlobalStateEventHandler()
 
 void CState::UploadDroppedFiles(CLocalDataObject const* pLocalDataObject, std::wstring const& subdir, bool queueOnly)
 {
-	if (!m_site.server_ || !m_pDirectoryListing) {
+	if (!m_site || !m_pDirectoryListing) {
 		return;
 	}
 
@@ -739,7 +738,7 @@ void CState::UploadDroppedFiles(CLocalDataObject const* pLocalDataObject, std::w
 
 void CState::UploadDroppedFiles(const wxFileDataObject* pFileDataObject, std::wstring const& subdir, bool queueOnly)
 {
-	if (!m_site.server_ || !m_pDirectoryListing) {
+	if (!m_site || !m_pDirectoryListing) {
 		return;
 	}
 
@@ -762,7 +761,7 @@ void DoUploadDroppedFiles(CState& state, CMainFrame & mainFrame, T const& files,
 		return;
 	}
 
-	if (!state.GetServer()) {
+	if (!state.GetSite()) {
 		return;
 	}
 
@@ -783,7 +782,7 @@ void DoUploadDroppedFiles(CState& state, CMainFrame & mainFrame, T const& files,
 		if (type == fz::local_filesys::file) {
 			std::wstring localFile;
 			CLocalPath const localPath(fz::to_wstring(file), &localFile);
-			mainFrame.GetQueue()->QueueFile(queueOnly, false, localFile, wxEmptyString, localPath, path, state.GetServer(), size);
+			mainFrame.GetQueue()->QueueFile(queueOnly, false, localFile, wxEmptyString, localPath, path, state.GetSite(), size);
 			mainFrame.GetQueue()->QueueFile_Finish(!queueOnly);
 		}
 		else if (type == fz::local_filesys::dir) {
@@ -1044,7 +1043,7 @@ bool CState::IsRemoteConnected() const
 		return false;
 	}
 
-	return static_cast<bool>(m_site.server_);
+	return static_cast<bool>(m_site);
 }
 
 bool CState::IsRemoteIdle(bool ignore_recursive) const
@@ -1105,7 +1104,7 @@ void CState::ListingFailed(int)
 	}
 }
 
-void CState::LinkIsNotDir(const CServerPath& path, const wxString& subdir)
+void CState::LinkIsNotDir(const CServerPath& path, std::wstring const& subdir)
 {
 	m_changeDirFlags.compare = false;
 	m_changeDirFlags.syncbrowse = false;
@@ -1115,7 +1114,7 @@ void CState::LinkIsNotDir(const CServerPath& path, const wxString& subdir)
 
 bool CState::ChangeRemoteDir(CServerPath const& path, std::wstring const& subdir, int flags, bool ignore_busy, bool compare)
 {
-	if (!m_site.server_ || !m_pCommandQueue) {
+	if (!m_site || !m_pCommandQueue) {
 		return false;
 	}
 
@@ -1174,7 +1173,7 @@ bool CState::ChangeRemoteDir(CServerPath const& path, std::wstring const& subdir
 					}
 
 					if (!local_path.Exists(&error)) {
-						wxMessageBox(wxString::Format(_("The local directory '%s' could not be created."), local_path.GetPath()), _("Synchronized browsing"), wxICON_EXCLAMATION);
+						wxMessageBoxEx(wxString::Format(_("The local directory '%s' could not be created."), local_path.GetPath()), _("Synchronized browsing"), wxICON_EXCLAMATION);
 						return false;
 					}
 					m_changeDirFlags.syncbrowse = true;
@@ -1330,23 +1329,24 @@ void CState::SetSecurityInfo(CSftpEncryptionNotification const& info)
 	NotifyHandlers(STATECHANGE_ENCRYPTION);
 }
 
-void CState::UpdateSite(wxString const& oldPath, Site const& newSite)
+void CState::UpdateSite(std::wstring const& oldPath, Site const& newSite)
 {
-	if (newSite.m_path.empty() || !newSite.server_) {
+	if (newSite.SitePath().empty() || !newSite) {
 		return;
 	}
 
 	bool changed = false;
-	if (m_site.server_ && m_site != newSite) {
-		if (m_site.m_path == oldPath && m_site.server_ == newSite.server_) {
-			m_site = newSite;
+	if (m_site && m_site != newSite) {
+		if (m_site.SitePath() == oldPath && m_site.GetOriginalServer().SameResource(newSite.GetOriginalServer())) {
+			// Update handles
+			m_site.Update(newSite);
 			changed = true;
 		}
 	}
-	if (m_last_site.server_ && m_last_site != newSite) {
-		if (m_last_site.m_path == oldPath && m_last_site.server_ == newSite.server_) {
-			m_last_site = newSite;
-			if (!m_site.server_) {
+	if (m_last_site && m_last_site != newSite) {
+		if (m_last_site.SitePath() == oldPath && m_last_site.GetOriginalServer().SameResource(newSite.GetOriginalServer())) {
+			m_last_site.Update(newSite);
+			if (!m_site) {
 				// Active site has precedence over historic data
 				changed = true;
 			}
@@ -1358,17 +1358,72 @@ void CState::UpdateSite(wxString const& oldPath, Site const& newSite)
 	}
 }
 
+void CState::UpdateKnownSites(std::vector<CSiteManagerDialog::_connected_site> const& active_sites)
+{
+	bool changed{};
+	if (m_site) {
+		for (auto const& active_site : active_sites) {
+			if (active_site.old_path == m_site.SitePath()) {
+				if (active_site.old_path == m_site.SitePath() && active_site.site && m_site.GetOriginalServer().SameResource(active_site.site.GetOriginalServer())) {
+					if (m_site != active_site.site) {
+						changed = true;
+						m_site.Update(active_site.site);
+					}
+				}
+				else {
+					changed = true;
+					m_site.SetSitePath(std::wstring());
+				}
+				m_last_site = m_site;
+				break;
+			}
+		}
+	}
+	else if (m_last_site) {
+		for (auto const& active_site : active_sites) {
+			if (active_site.old_path == m_last_site.SitePath()) {
+				if (active_site.old_path == m_last_site.SitePath() && active_site.site && m_last_site.GetOriginalServer().SameResource(active_site.site.GetOriginalServer())) {
+					if (m_last_site != active_site.site) {
+						changed = true;
+						m_last_site.Update(active_site.site);
+					}
+				}
+				else {
+					changed = true;
+					m_last_site.SetSitePath(std::wstring());
+				}
+			}
+			break;
+		}
+	}
+
+	if (changed) {
+		UpdateTitle();
+		NotifyHandlers(STATECHANGE_SERVER);
+	}
+}
+
 void CState::UpdateTitle()
 {
-	if (m_site.server_) {
-		wxString const& name = m_site.server_.server.GetName();
+	if (m_site) {
+		std::wstring const& name = m_site.GetName();
 		m_title.clear();
 		if (!name.empty()) {
 			m_title = name + _T(" - ");
 		}
-		m_title += m_site.server_.Format(ServerFormat::with_user_and_optional_port);
+		m_title += m_site.Format(ServerFormat::with_user_and_optional_port);
 	}
 	else {
 		m_title = _("Not connected");
 	}
 }
+
+void CState::ChangeServer(CServer const& newServer)
+{
+	if (m_site) {
+		if (!m_site.originalServer) {
+			m_site.originalServer = m_site.server;
+		}
+		m_site.server = newServer;
+	}
+}
