@@ -3,6 +3,7 @@
 
 #include "local_path.h"
 #include "sitemanager.h"
+#include "sitemanager_dialog.h"
 #include "filter.h"
 
 #include <memory>
@@ -51,7 +52,7 @@ enum t_statechange_notifications
 class CDirectoryListing;
 class CFileZillaEngine;
 class CCommandQueue;
-class CLocalDataObject; 
+class CLocalDataObject;
 class CLocalRecursiveOperation;
 class CMainFrame;
 class CGlobalStateEventHandler;
@@ -97,13 +98,13 @@ public:
 
 	static CContextManager* Get();
 
-	void NotifyAllHandlers(t_statechange_notifications notification, wxString const& data = wxString(), void const* data2 = 0);
-	void NotifyGlobalHandlers(t_statechange_notifications notification, wxString const& data = wxString(), void const* data2 = 0);
+	void NotifyAllHandlers(t_statechange_notifications notification, std::wstring const& data = std::wstring(), void const* data2 = 0);
+	void NotifyGlobalHandlers(t_statechange_notifications notification, std::wstring const& data = std::wstring(), void const* data2 = 0);
 
 	void SetCurrentContext(CState* pState);
 
 	void ProcessDirectoryListing(CServer const& server, std::shared_ptr<CDirectoryListing> const& listing, CState const* exempt);
-
+	
 protected:
 	CContextManager();
 
@@ -117,7 +118,7 @@ protected:
 	};
 	std::vector<t_handler> m_handlers[STATECHANGE_MAX];
 
-	void NotifyHandlers(CState* pState, t_statechange_notifications notification, wxString const& data, void const* data2);
+	void NotifyHandlers(CState* pState, t_statechange_notifications notification, std::wstring const& data, void const* data2);
 
 	static CContextManager m_the_context_manager;
 };
@@ -148,7 +149,6 @@ public:
 	const CServerPath GetRemotePath() const;
 
 	Site const& GetSite() const;
-	ServerWithCredentials const& GetServer() const;
 	wxString GetTitle() const;
 
 	void RefreshLocal();
@@ -181,18 +181,18 @@ public:
 	CLocalRecursiveOperation* GetLocalRecursiveOperation() { return m_pLocalRecursiveOperation; }
 	CRemoteRecursiveOperation* GetRemoteRecursiveOperation() { return m_pRemoteRecursiveOperation; }
 
-	void NotifyHandlers(t_statechange_notifications notification, wxString const& data = wxString(), void const* data2 = 0);
+	void NotifyHandlers(t_statechange_notifications notification, std::wstring const& data = std::wstring(), void const* data2 = 0);
 
 	bool SuccessfulConnect() const { return m_successful_connect; }
 	void SetSuccessfulConnect() { m_successful_connect = true; }
 
 	void ListingFailed(int error);
-	void LinkIsNotDir(CServerPath const& path, wxString const& subdir);
+	void LinkIsNotDir(CServerPath const& path, std::wstring const& subdir);
 
 	bool SetSyncBrowse(bool enable, CServerPath const& assumed_remote_root = CServerPath());
 	bool GetSyncBrowse() const { return !m_sync_browse.local_root.empty(); }
 
-	Site GetLastSite() const { return m_last_site; }
+	Site const& GetLastSite() const { return m_last_site; }
 	CServerPath GetLastServerPath() const { return m_last_path; }
 	void SetLastSite(Site const& server, CServerPath const& path)
 		{ m_last_site = server; m_last_path = path; }
@@ -209,10 +209,15 @@ public:
 	void ClearPreviouslyVisitedLocalSubdir() { m_previouslyVisitedLocalSubdir.clear(); }
 	void ClearPreviouslyVisitedRemoteSubdir() { m_previouslyVisitedRemoteSubdir.clear(); }
 
-	void UpdateSite(wxString const& oldPath, Site const& newSite);
+	void UpdateKnownSites(std::vector<CSiteManagerDialog::_connected_site> const& active_sites);
+	void UpdateSite(std::wstring const& oldPath, Site const& newSite);
 
 	CStateFilterManager& GetStateFilterManager() { return m_stateFilterManager; }
 
+	void ChangeServer(CServer const& newServer);
+
+	fz::thread_pool & pool_;
+
 protected:
 	void SetSite(Site const& site, CServerPath const& path = CServerPath());
 
@@ -278,7 +283,7 @@ public:
 	CGlobalStateEventHandler() = default;
 	virtual ~CGlobalStateEventHandler();
 
-	virtual void OnStateChange(CState* pState, t_statechange_notifications notification, const wxString& data, const void* data2) = 0;
+	virtual void OnStateChange(CState* pState, t_statechange_notifications notification, std::wstring const& data, const void* data2) = 0;
 };
 
 class CStateEventHandler
@@ -289,7 +294,7 @@ public:
 
 	CState& m_state;
 
-	virtual void OnStateChange(t_statechange_notifications notification, const wxString& data, const void* data2) = 0;
+	virtual void OnStateChange(t_statechange_notifications notification, std::wstring const& data, const void* data2) = 0;
 };
 
 #endif
