@@ -108,8 +108,8 @@ protected:
 
 	void ShowDataTypeMenu();
 
-	virtual void OnOptionsChanged(changed_options_t const& options);
-	virtual void OnStateChange(CState* pState, t_statechange_notifications notification, const wxString& data, const void* data2);
+	virtual void OnOptionsChanged(changed_options_t const& options) override;
+	virtual void OnStateChange(CState* pState, t_statechange_notifications notification, std::wstring const& data, const void* data2) override;
 
 	void DoDisplayQueueSize();
 
