@@ -71,7 +71,9 @@ void CStatusLineCtrl::InitFieldOffsets()
 CStatusLineCtrl::~CStatusLineCtrl()
 {
 	if (!status_.empty() && status_.totalSize >= 0) {
-		m_pEngineData->pItem->SetSize(status_.totalSize);
+		if (m_pEngineData && m_pEngineData->pItem) {
+			m_pEngineData->pItem->SetSize(status_.totalSize);
+		}
 	}
 
 	if (m_transferStatusTimer.IsRunning()) {
@@ -138,19 +140,23 @@ void CStatusLineCtrl::OnPaint(wxPaintEvent&)
 			m_last_elapsed_seconds = elapsed_milli_seconds / 1000;
 		}
 
-		if (COptions::Get()->GetOptionVal(OPTION_SPEED_DISPLAY))
+		if (COptions::Get()->GetOptionVal(OPTION_SPEED_DISPLAY)) {
 			rate = GetMomentarySpeed();
-		else
+		}
+		else {
 			rate = GetAverageSpeed(elapsed_milli_seconds);
+		}
 
 		if (status_.totalSize > 0 && elapsed_milli_seconds >= 1000 && rate > 0) {
 			wxFileOffset r = status_.totalSize - status_.currentOffset;
 			left = r / rate + 1;
-			if (r)
+			if (r) {
 				++left;
+			}
 
-			if (left < 0)
+			if (left < 0) {
 				left = 0;
+			}
 		}
 
 		if (m_last_left != left) {
@@ -161,16 +167,18 @@ void CStatusLineCtrl::OnPaint(wxPaintEvent&)
 		const wxString bytestr = CSizeFormat::Format(status_.currentOffset, true, CSizeFormat::bytes, COptions::Get()->GetOptionVal(OPTION_SIZE_USETHOUSANDSEP) != 0, 0);
 		if (elapsed_milli_seconds >= 1000 && rate > -1) {
 			CSizeFormat::_format format = static_cast<CSizeFormat::_format>(COptions::Get()->GetOptionVal(OPTION_SIZE_FORMAT));
-			if (format == CSizeFormat::bytes)
+			if (format == CSizeFormat::bytes) {
 				format = CSizeFormat::iec;
+			}
 			const wxString ratestr = CSizeFormat::Format(rate, true,
 														 format,
 														 COptions::Get()->GetOptionVal(OPTION_SIZE_USETHOUSANDSEP) != 0,
 														 COptions::Get()->GetOptionVal(OPTION_SIZE_DECIMALPLACES));
 			bytes_and_rate.Printf(_("%s (%s/s)"), bytestr, ratestr );
 		}
-		else
+		else {
 			bytes_and_rate.Printf(_("%s (? B/s)"), bytestr);
+		}
 
 		if (m_last_bytes_and_rate != bytes_and_rate) {
 			refresh |= 8;
@@ -230,8 +238,9 @@ void CStatusLineCtrl::OnPaint(wxPaintEvent&)
 		}
 		if (refresh & 4) {
 			m_mdc->DrawRectangle(m_fieldOffsets[2], 0, m_fieldOffsets[3] - m_fieldOffsets[2], rect.GetHeight() + 1);
-			if (bar_split != -1)
+			if (bar_split != -1) {
 				DrawProgressBar(*m_mdc, m_fieldOffsets[2], 1, rect.GetHeight() - 2, bar_split, permill);
+			}
 		}
 	}
 	dc.Blit(0, 0, rect.GetWidth(), rect.GetHeight(), m_mdc.get(), 0, 0);
@@ -240,11 +249,14 @@ void CStatusLineCtrl::OnPaint(wxPaintEvent&)
 void CStatusLineCtrl::ClearTransferStatus()
 {
 	if (!status_.empty() && status_.totalSize >= 0) {
-		m_pParent->UpdateItemSize(m_pEngineData->pItem, status_.totalSize);
+		if (m_pEngineData && m_pEngineData->pItem) {
+			m_pParent->UpdateItemSize(m_pEngineData->pItem, status_.totalSize);
+		}
 	}
 	status_.clear();
 
-	switch (m_pEngineData->state)
+	auto const state = m_pEngineData ? m_pEngineData->state : t_EngineData::none;
+	switch (state)
 	{
 	case t_EngineData::disconnect:
 		m_statusText = _("Disconnecting from previous server");
@@ -253,15 +265,16 @@ void CStatusLineCtrl::ClearTransferStatus()
 		m_statusText = _("Waiting for transfer to be cancelled");
 		break;
 	case t_EngineData::connect:
-		m_statusText = wxString::Format(_("Connecting to %s"), m_pEngineData->lastServer.Format(ServerFormat::with_user_and_optional_port));
+		m_statusText = wxString::Format(_("Connecting to %s"), m_pEngineData->lastSite.Format(ServerFormat::with_user_and_optional_port));
 		break;
 	default:
 		m_statusText = _("Transferring");
 		break;
 	}
 
-	if (m_transferStatusTimer.IsRunning())
+	if (m_transferStatusTimer.IsRunning()) {
 		m_transferStatusTimer.Stop();
+	}
 
 	m_past_data_count = 0;
 
@@ -279,8 +292,9 @@ void CStatusLineCtrl::SetTransferStatus(CTransferStatus const& status)
 
 		m_lastOffset = status.currentOffset;
 
-		if (!m_transferStatusTimer.IsRunning())
+		if (!m_transferStatusTimer.IsRunning()) {
 			m_transferStatusTimer.Start(100);
+		}
 		Refresh(false);
 	}
 }
@@ -295,8 +309,9 @@ void CStatusLineCtrl::OnTimer(wxTimerEvent&)
 	bool changed;
 	CTransferStatus status = m_pEngineData->pEngine->GetTransferStatus(changed);
 
-	if (status.empty())
+	if (status.empty()) {
 		ClearTransferStatus();
+	}
 	else if (changed) {
 		if (status.madeProgress && !status.list &&
 			m_pEngineData->pItem->GetType() == QueueItemType::File)
@@ -306,8 +321,9 @@ void CStatusLineCtrl::OnTimer(wxTimerEvent&)
 		}
 		SetTransferStatus(status);
 	}
-	else
+	else {
 		m_transferStatusTimer.Stop();
+	}
 }
 
 void CStatusLineCtrl::DrawRightAlignedText(wxDC& dc, wxString const& text, int x, int y)
@@ -404,23 +420,23 @@ wxFileOffset CStatusLineCtrl::GetMomentarySpeed()
 		return -1;
 	}
 
+	if (m_monentary_speed_data.last_offset < 0) {
+		m_monentary_speed_data.last_offset = status_.currentOffset;
+	}
+	
 	if (!m_monentary_speed_data.last_update) {
 		m_monentary_speed_data.last_update = fz::monotonic_clock::now();
-		m_monentary_speed_data.last_offset = status_.startOffset;
 		return -1;
 	}
 
 	fz::duration const time_diff = fz::monotonic_clock::now() - m_monentary_speed_data.last_update;
-	if (time_diff.get_seconds() < 2) {
+	if (time_diff.get_seconds() >= 2) {
+		m_monentary_speed_data.last_update = fz::monotonic_clock::now();
+	}
+	else if (m_monentary_speed_data.last_speed >= 0 || !time_diff) {
 		return m_monentary_speed_data.last_speed;
 	}
-
-	m_monentary_speed_data.last_update = fz::monotonic_clock::now();
-
-	if (m_monentary_speed_data.last_offset < 0) {
-		m_monentary_speed_data.last_offset = status_.startOffset;
-	}
-
+	
 	wxFileOffset const fileOffsetDiff = status_.currentOffset - m_monentary_speed_data.last_offset;
 	m_monentary_speed_data.last_offset = status_.currentOffset;
 	if (fileOffsetDiff >= 0) {
@@ -446,6 +462,5 @@ bool CStatusLineCtrl::Show(bool show)
 
 void CStatusLineCtrl::SetEngineData(const t_EngineData* const pEngineData)
 {
-	wxASSERT(pEngineData);
 	m_pEngineData = pEngineData;
 }
