@@ -1,5 +1,5 @@
-#ifndef __STATUSLINECTRL_H__
-#define __STATUSLINECTRL_H__
+#ifndef FILEZILLA_INTERFACE_STATUSLINECTRL_HEADER
+#define FILEZILLA_INTERFACE_STATUSLINECTRL_HEADER
 
 class CQueueView;
 class CStatusLineCtrl final : public wxWindow
@@ -17,15 +17,15 @@ public:
 
 	int64_t GetLastOffset() const { return status_.empty() ? m_lastOffset : status_.currentOffset; }
 	int64_t GetTotalSize() const { return status_.empty() ? -1 : status_.totalSize; }
-	wxFileOffset GetSpeed(int elapsed_milli_seconds);
-	wxFileOffset GetCurrentSpeed();
+	wxFileOffset GetAverageSpeed(int elapsed_milli_seconds);
+	wxFileOffset GetMomentarySpeed();
 
 	virtual bool Show(bool show = true);
 
 protected:
 	void InitFieldOffsets();
 
-	void DrawRightAlignedText(wxDC& dc, wxString text, int x, int y);
+	void DrawRightAlignedText(wxDC& dc, wxString const& text, int x, int y);
 	void DrawProgressBar(wxDC& dc, int x, int y, int height, int bar_split, int permill);
 
 	CQueueView* m_pParent;
@@ -36,6 +36,7 @@ protected:
 	wxTimer m_transferStatusTimer;
 
 	static int m_fieldOffsets[4];
+	static int m_barWidth;
 	static wxCoord m_textHeight;
 	static bool m_initialized;
 
@@ -52,10 +53,12 @@ protected:
 	} m_past_data[10];
 	int m_past_data_count{};
 
-	//Used by getCurrentSpeed
-	wxDateTime m_gcLastTimeStamp;
-	wxFileOffset m_gcLastOffset{-1};
-	wxFileOffset m_gcLastSpeed{-1};
+	//Used by GetMomentarySpeed
+	struct monentary_speed_data {
+		fz::monotonic_clock last_update;
+		wxFileOffset last_offset{-1};
+		wxFileOffset last_speed{-1};
+	} m_monentary_speed_data;
 
 	//Used to avoid excessive redraws
 	wxBitmap m_data;
@@ -73,4 +76,4 @@ protected:
 	void OnEraseBackground(wxEraseEvent& event);
 };
 
-#endif // __STATUSLINECTRL_H__
+#endif
