@@ -302,6 +302,14 @@ wxBitmap CThemeProvider::CreateBitmap(wxArtID const& id, wxArtClient const& clie
 		if (newSize.x <= 0 || newSize.y <= 0) {
 			newSize = GetIconSize(iconSizeSmall);
 		}
+#ifdef __WXMSW__
+		else {
+			// wxMSW doesn't conside the UI scale factor for menu items
+			if (client == wxART_MENU) {
+				newSize *= GetUIScaleFactor();
+			}
+		}
+#endif
 	}
 	else {
 		newSize = size;
@@ -425,7 +433,7 @@ bool CThemeProvider::GetThemeData(std::wstring const& theme, std::wstring & name
 	return true;
 }
 
-wxIconBundle CThemeProvider::GetIconBundle(const wxArtID& id, const wxArtClient&)
+wxIconBundle CThemeProvider::GetIconBundle(const wxArtID& id, const wxArtClient& client)
 {
 	wxIconBundle iconBundle;
 
@@ -433,19 +441,14 @@ wxIconBundle CThemeProvider::GetIconBundle(const wxArtID& id, const wxArtClient&
 		return iconBundle;
 	}
 
-	wxString name = fz::str_tolower_ascii(id.Mid(4));
-
-	const wxChar* dirs[] = { _T("16x16/"), _T("32x32/"), _T("48x48/") };
-
-	CLocalPath const resourcePath = wxGetApp().GetResourceDir();
-
-	for (auto const& dir : dirs) {
-		wxString file = resourcePath.GetPath() + dir + name + _T(".png");
-		if (!wxFileName::FileExists(file)) {
-			continue;
+	wxSize const sizes[] = { {16,16}, {32,32}, {48,48}, {256,256} };
+	for (auto const& size : sizes) {
+		auto bmp = CThemeProvider::Get()->CreateBitmap(id, client, size);
+		if (bmp.IsOk()) {
+			wxIcon icon;
+			icon.CopyFromBitmap(bmp);
+			iconBundle.AddIcon(icon);
 		}
-
-		iconBundle.AddIcon(wxIcon(file, wxBITMAP_TYPE_PNG));
 	}
 
 	return iconBundle;
@@ -468,7 +471,13 @@ void CThemeProvider::OnOptionsChanged(changed_options_t const&)
 wxSize CThemeProvider::GetIconSize(iconSize size, bool userScaled)
 {
 	int s;
-	if (size == iconSizeSmall) {
+	if (size == iconSizeTiny) {
+		s = wxSystemSettings::GetMetric(wxSYS_SMALLICON_X) * 3 / 4;
+		if (s <= 0) {
+			s = 12;
+		}
+	}
+	else if (size == iconSizeSmall) {
 		s = wxSystemSettings::GetMetric(wxSYS_SMALLICON_X);
 		if (s <= 0) {
 			s = 16;
@@ -492,6 +501,15 @@ wxSize CThemeProvider::GetIconSize(iconSize size, bool userScaled)
 			s += s / 2;
 		}
 	}
+	else if (size == iconSizeHuge) {
+		s = wxSystemSettings::GetMetric(wxSYS_ICON_X);
+		if (s <= 0) {
+			s = 64;
+		}
+		else {
+			s *= 2;
+		}
+	}
 	else {
 		s = wxSystemSettings::GetMetric(wxSYS_ICON_X);
 		if (s <= 0) {
@@ -518,3 +536,14 @@ wxSize CThemeProvider::GetIconSize(iconSize size, bool userScaled)
 
 	return ret;
 }
+
+double CThemeProvider::GetUIScaleFactor()
+{
+	int x = GetIconSize(iconSizeSmall).x;
+	if (!x) {
+		return 1.;
+	}
+	else {
+		return x / 16.;
+	}
+}
