@@ -8,10 +8,12 @@
 
 enum iconSize
 {
+	iconSizeTiny,
 	iconSizeSmall,
 	iconSize24,
 	iconSizeNormal,
-	iconSizeLarge
+	iconSizeLarge,
+	iconSizeHuge
 };
 
 class CTheme final
