@@ -1,5 +1,5 @@
-#ifndef __TOOLBAR_H__
-#define __TOOLBAR_H__
+#ifndef FILEZILLA_INTERFACE_TOOLBAR_HEADER
+#define FILEZILLA_INTERFACE_TOOLBAR_HEADER
 
 #include <option_change_event_handler.h>
 #include "state.h"
@@ -27,7 +27,7 @@ protected:
 	void MakeTool(char const* id, std::wstring const& art, wxString const& tooltip, wxString const& help = wxString(), wxItemKind type = wxITEM_NORMAL);
 	void MakeTools();
 
-	virtual void OnStateChange(CState* pState, t_statechange_notifications notification, const wxString& data, const void* data2);
+	virtual void OnStateChange(CState* pState, t_statechange_notifications notification, std::wstring const& data, const void* data2) override;
 	virtual void OnOptionsChanged(changed_options_t const& options);
 
 	CMainFrame* m_pMainFrame{};
@@ -42,4 +42,4 @@ protected:
 	wxSize iconSize_;
 };
 
-#endif //__TOOLBAR_H__
+#endif
