@@ -4,8 +4,15 @@
 #include <wx/dnd.h>
 #include "filelistctrl.h"
 
+#ifndef __WXMAC__
+	#define DEFAULT_TREE_STYLE wxTR_HAS_BUTTONS | wxTR_LINES_AT_ROOT
+#else
+	#define DEFAULT_TREE_STYLE wxTR_HAS_BUTTONS | wxTR_NO_LINES
+#endif
+
 class wxTreeCtrlEx : public wxNavigationEnabled<wxTreeCtrl>
 {
+	wxDECLARE_CLASS(wxTreeCtrlEx); // Needed for OnCompareItems to work on Windows. Bad library design, why not use normal RTTI?
 public:
 	typedef wxTreeItemId Item;
 
@@ -13,7 +20,7 @@ public:
 	wxTreeCtrlEx(wxWindow *parent, wxWindowID id = wxID_ANY,
 			   const wxPoint& pos = wxDefaultPosition,
 			   const wxSize& size = wxDefaultSize,
-			   long style = wxTR_HAS_BUTTONS | wxTR_LINES_AT_ROOT);
+			   long style = DEFAULT_TREE_STYLE);
 	void SafeSelectItem(wxTreeItemId const& item, bool clearSelection = true);
 
 	// Small wrappers to make wxTreeCtrl(Ex) API more similar to wxListCtrl(ex).
@@ -40,18 +47,24 @@ public:
 	wxTreeItemId GetNextItemSimple(wxTreeItemId const& item, bool includeCollapsed = false) const;
 	wxTreeItemId GetPrevItemSimple(wxTreeItemId const& item) const;
 
+	bool InPrefixSearch() const { return inPrefixSearch_; }
+
+	void Resort();
 protected:
 
+	bool inPrefixSearch_{};
+
 	int m_setSelection{};
 
 #ifdef __WXMAC__
-	DECLARE_EVENT_TABLE()
+	wxDECLARE_EVENT_TABLE();
 	void OnChar(wxKeyEvent& event);
 #endif
 
-	virtual int OnCompareItems(wxTreeItemId const& item1, wxTreeItemId const& item2);
+	virtual int OnCompareItems(wxTreeItemId const& item1, wxTreeItemId const& item2) override;
 
-	CFileListCtrlSortBase::NameSortMode m_nameSortMode;
+	typedef int (*CompareFunction)(std::wstring_view const&, std::wstring_view const&);
+	CompareFunction sortFunction_{};
 };
 
 #endif
