@@ -1,6 +1,7 @@
 #include <filezilla.h>
 #include "viewheader.h"
 #include "commandqueue.h"
+#include "graphics.h"
 
 #ifdef __WXMSW__
 #include <wx/msw/uxtheme.h>
@@ -12,6 +13,8 @@
 #include <wx/combobox.h>
 #include <wx/dcclient.h>
 
+#include <algorithm>
+
 #ifdef __WXMSW__
 const int border_offset = 0;
 #elif defined(__WXMAC__)
@@ -83,6 +86,8 @@ CViewHeader::CViewHeader(wxWindow* pParent, const wxString& label)
 	SetBackgroundStyle(wxBG_STYLE_SYSTEM);
 	m_pComboBox->SetBackgroundStyle(wxBG_STYLE_SYSTEM);
 	m_pLabel->SetBackgroundStyle(wxBG_STYLE_SYSTEM);
+
+	m_windowTinter = std::make_unique<CWindowTinter>(*m_pComboBox);
 }
 
 void CViewHeader::OnSize(wxSizeEvent&)
@@ -93,7 +98,7 @@ void CViewHeader::OnSize(wxSizeEvent&)
 	rect.SetWidth(rect.GetWidth() - m_cbOffset + 2);
 	rect.SetX(m_cbOffset);
 	rect.Deflate(0, border_offset / 2);
-	rect.SetWidth(rect.GetWidth() - border_offset / 2);
+	rect.SetWidth(std::max(0, rect.GetWidth() - border_offset / 2));
 	if (m_pComboBox) {
 		m_pComboBox->SetSize(rect);
 	}
@@ -114,8 +119,7 @@ void CViewHeader::OnSize(wxSizeEvent&)
 void CViewHeader::OnComboPaint(wxPaintEvent& event)
 {
 	// We do a small trick to let the control handle the event before we can paint
-	if (m_alreadyInPaint)
-	{
+	if (m_alreadyInPaint) {
 		event.Skip();
 		return;
 	}
@@ -132,15 +136,18 @@ void CViewHeader::OnComboPaint(wxPaintEvent& event)
 
 	int thumbWidth = ::GetSystemMetrics(SM_CXHTHUMB);
 
-	if (m_bLeftMousePressed)
-	{
+	if (m_bLeftMousePressed) {
 		if (!SendMessage((HWND)box->GetHandle(), CB_GETDROPPEDSTATE, 0, 0))
 			m_bLeftMousePressed = false;
 	}
 
 #if wxUSE_UXTHEME
+#if wxCHECK_VERSION(3, 1, 0)
+	if (wxUxThemeIsActive())
+#else
 	wxUxThemeEngine *p = wxUxThemeEngine::Get();
 	if (p && p->IsThemeActive())
+#endif
 	{
 	}
 	else
@@ -330,10 +337,11 @@ EVT_COMMAND(wxID_ANY, fzEVT_LOCALVIEWHEADERSETTEXTSEL, CLocalViewHeader::OnSelec
 #endif
 END_EVENT_TABLE()
 
-CLocalViewHeader::CLocalViewHeader(wxWindow* pParent, CState* pState)
-	: CViewHeader(pParent, _("Local site:")), CStateEventHandler(pState)
+CLocalViewHeader::CLocalViewHeader(wxWindow* pParent, CState& state)
+	: CViewHeader(pParent, _("Local site:")), CStateEventHandler(state)
 {
-	pState->RegisterHandler(this, STATECHANGE_LOCAL_DIR);
+	state.RegisterHandler(this, STATECHANGE_LOCAL_DIR);
+	state.RegisterHandler(this, STATECHANGE_SERVER);
 }
 
 void CLocalViewHeader::OnTextChanged(wxCommandEvent&)
@@ -341,18 +349,16 @@ void CLocalViewHeader::OnTextChanged(wxCommandEvent&)
 	// This function handles auto-completion
 
 #ifdef __WXGTK__
-	m_autoCompletionText = _T("");
+	m_autoCompletionText.clear();
 #endif
 
 	wxString str = m_pComboBox->GetValue();
-	if (str.empty() || str.Right(1) == _T("/"))
-	{
+	if (str.empty() || str.Right(1) == _T("/")) {
 		m_oldValue = str;
 		return;
 	}
 #ifdef __WXMSW__
-	if (str.Right(1) == _T("\\"))
-	{
+	if (str.Right(1) == _T("\\")) {
 		m_oldValue = str;
 		return;
 	}
@@ -361,25 +367,21 @@ void CLocalViewHeader::OnTextChanged(wxCommandEvent&)
 	if (str == m_oldValue)
 		return;
 
-	if (str.Left(m_oldValue.Length()) != m_oldValue)
-	{
+	if (str.Left(m_oldValue.Length()) != m_oldValue) {
 		m_oldValue = str;
 		return;
 	}
 
 #ifdef __WXMSW__
-	if (str.Left(2) == _T("\\\\"))
-	{
+	if (str.Left(2) == _T("\\\\")) {
 		int pos = str.Mid(2).Find('\\');
-		if (pos == -1)
-		{
+		if (pos == -1) {
 			// Partial UNC path, no full server yet, skip further processing
 			return;
 		}
 
 		pos = str.Mid(pos + 3).Find('\\');
-		if (pos == -1)
-		{
+		if (pos == -1) {
 			// Partial UNC path, no full share yet, skip further processing
 			return;
 		}
@@ -387,8 +389,7 @@ void CLocalViewHeader::OnTextChanged(wxCommandEvent&)
 #endif
 
 	wxFileName fn(str);
-	if (!fn.IsOk())
-	{
+	if (!fn.IsOk()) {
 		m_oldValue = str;
 		return;
 	}
@@ -403,8 +404,7 @@ void CLocalViewHeader::OnTextChanged(wxCommandEvent&)
 	}
 	else {
 		wxLogNull log;
-		if (!dir.Open(path) || !dir.IsOpened())
-		{
+		if (!dir.Open(path) || !dir.IsOpened()) {
 			m_oldValue = str;
 			return;
 		}
@@ -420,8 +420,7 @@ void CLocalViewHeader::OnTextChanged(wxCommandEvent&)
 	}
 
 	wxString tmp;
-	if (dir.GetNext(&tmp))
-	{
+	if (dir.GetNext(&tmp)) {
 		m_oldValue = str;
 		return;
 	}
@@ -455,7 +454,7 @@ void CLocalViewHeader::OnSelectTextEvent(wxCommandEvent&)
 
 	const wxString& oldValue = m_pComboBox->GetValue();
 	const wxString completionText = m_autoCompletionText;
-	m_autoCompletionText = _T("");
+	m_autoCompletionText.clear();
 
 	if (m_pComboBox->GetInsertionPoint() != (int)oldValue.Len())
 		return;
@@ -468,57 +467,61 @@ void CLocalViewHeader::OnSelectTextEvent(wxCommandEvent&)
 void CLocalViewHeader::OnSelectionChanged(wxCommandEvent& event)
 {
 #ifdef __WXGTK__
-	m_autoCompletionText = _T("");
+	m_autoCompletionText.clear();
 #endif
 
-	wxString dir = event.GetString();
-	if (dir.empty())
+	std::wstring dir = event.GetString().ToStdWstring();
+	if (dir.empty()) {
 		return;
+	}
 
-	if (!wxDir::Exists(dir))
-	{
-		const wxString& current = m_pState->GetLocalDir().GetPath();
+	if (!wxDir::Exists(dir)) {
+		const wxString& current = m_state.GetLocalDir().GetPath();
 		int item = m_pComboBox->FindString(current, true);
-		if (item != wxNOT_FOUND)
+		if (item != wxNOT_FOUND) {
 			m_pComboBox->SetSelection(item);
+		}
 
 		wxBell();
 		return;
 	}
 
-	m_pState->SetLocalDir(dir);
+	m_state.SetLocalDir(dir);
 }
 
 void CLocalViewHeader::OnTextEnter(wxCommandEvent&)
 {
 #ifdef __WXGTK__
-	m_autoCompletionText = _T("");
+	m_autoCompletionText.clear();
 #endif
 
-	wxString dir = m_pComboBox->GetValue();
+	std::wstring dir = m_pComboBox->GetValue().ToStdWstring();
 
-	wxString error;
-	if (!m_pState->SetLocalDir(dir, &error))
-	{
-		if (!error.empty())
+	std::wstring error;
+	if (!m_state.SetLocalDir(dir, &error)) {
+		if (!error.empty()) {
 			wxMessageBoxEx(error, _("Failed to change directory"), wxICON_INFORMATION);
-		else
+		}
+		else {
 			wxBell();
-		m_pComboBox->SetValue(m_pState->GetLocalDir().GetPath());
+		}
+		m_pComboBox->SetValue(m_state.GetLocalDir().GetPath());
 	}
 }
 
-void CLocalViewHeader::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString&, const void*)
+void CLocalViewHeader::OnStateChange(t_statechange_notifications notification, std::wstring const&, const void*)
 {
-	wxASSERT(notification == STATECHANGE_LOCAL_DIR);
-	(void)notification;
-
+	if (notification == STATECHANGE_SERVER) {
+		m_windowTinter->SetBackgroundTint(m_state.GetSite().m_colour);
+	}
+	else if (notification == STATECHANGE_LOCAL_DIR) {
 #ifdef __WXGTK__
-	m_autoCompletionText = _T("");
+		m_autoCompletionText.clear();
 #endif
 
-	wxString dir = pState->GetLocalDir().GetPath();
-	AddRecentDirectory(dir);
+		wxString dir = m_state.GetLocalDir().GetPath();
+		AddRecentDirectory(dir);
+	}
 }
 
 BEGIN_EVENT_TABLE(CRemoteViewHeader, CViewHeader)
@@ -526,36 +529,39 @@ EVT_TEXT_ENTER(wxID_ANY, CRemoteViewHeader::OnTextEnter)
 EVT_COMBOBOX(wxID_ANY, CRemoteViewHeader::OnSelectionChanged)
 END_EVENT_TABLE()
 
-CRemoteViewHeader::CRemoteViewHeader(wxWindow* pParent, CState* pState)
-	: CViewHeader(pParent, _("Remote site:")), CStateEventHandler(pState)
+CRemoteViewHeader::CRemoteViewHeader(wxWindow* pParent, CState& state)
+	: CViewHeader(pParent, _("Remote site:")), CStateEventHandler(state)
 {
-	pState->RegisterHandler(this, STATECHANGE_REMOTE_DIR);
+	state.RegisterHandler(this, STATECHANGE_REMOTE_DIR);
+	state.RegisterHandler(this, STATECHANGE_SERVER);
 	Disable();
 }
 
-void CRemoteViewHeader::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString&, const void*)
+void CRemoteViewHeader::OnStateChange(t_statechange_notifications notification, std::wstring const&, const void*)
 {
-	wxASSERT(notification == STATECHANGE_REMOTE_DIR);
-	(void)notification;
-
-	m_path = pState->GetRemotePath();
-	if (m_path.empty()) {
-		m_pComboBox->SetValue(_T(""));
-		Disable();
+	if (notification == STATECHANGE_SERVER) {
+		m_windowTinter->SetBackgroundTint(m_state.GetSite().m_colour);
 	}
-	else {
-		const CServer* const pServer = pState->GetServer();
-		if (pServer && *pServer != m_lastServer) {
-			m_pComboBox->Clear();
-			m_recentDirectories.clear();
-			m_sortedRecentDirectories.clear();
-			m_lastServer = *pServer;
+	else if (notification == STATECHANGE_REMOTE_DIR) {
+		m_path = m_state.GetRemotePath();
+		if (m_path.empty()) {
+			m_pComboBox->SetValue(_T(""));
+			Disable();
 		}
-		Enable();
+		else {
+			Site const& site = m_state.GetSite();
+			if (site && site.server != m_lastServer) {
+				m_pComboBox->Clear();
+				m_recentDirectories.clear();
+				m_sortedRecentDirectories.clear();
+				m_lastServer = site.server;
+			}
+			Enable();
 #ifdef __WXGTK__
-		GetParent()->m_dirtyTabOrder = true;
+			GetParent()->m_dirtyTabOrder = true;
 #endif
-		AddRecentDirectory(m_path.GetPath());
+			AddRecentDirectory(m_path.GetPath());
+		}
 	}
 }
 
@@ -563,17 +569,17 @@ void CRemoteViewHeader::OnTextEnter(wxCommandEvent&)
 {
 	CServerPath path = m_path;
 	wxString value = m_pComboBox->GetValue();
-	if (value.empty() || !path.ChangePath(value)) {
+	if (value.empty() || !path.ChangePath(value.ToStdWstring())) {
 		wxBell();
 		return;
 	}
 
-	if (!m_pState->IsRemoteIdle(true)) {
+	if (!m_state.IsRemoteIdle(true)) {
 		wxBell();
 		return;
 	}
 
-	m_pState->ChangeRemoteDir(path);
+	m_state.ChangeRemoteDir(path);
 }
 
 void CRemoteViewHeader::OnSelectionChanged(wxCommandEvent& event)
@@ -583,15 +589,15 @@ void CRemoteViewHeader::OnSelectionChanged(wxCommandEvent& event)
 		return;
 
 	CServerPath path = m_path;
-	if (!path.SetPath(dir)) {
+	if (!path.SetPath(dir.ToStdWstring())) {
 		wxBell();
 		return;
 	}
 
-	if (!m_pState->IsRemoteIdle(true)) {
+	if (!m_state.IsRemoteIdle(true)) {
 		wxBell();
 		return;
 	}
 
-	m_pState->ChangeRemoteDir(path);
+	m_state.ChangeRemoteDir(path);
 }
