@@ -1,9 +1,7 @@
 #ifndef FILEZILLA_VOLUME_ENUMERATOR_HEADER
 #define FILEZILLA_VOLUME_ENUMERATOR_HEADER
 
-#include <libfilezilla/thread.hpp>
-
-#include <list>
+#include <libfilezilla/thread_pool.hpp>
 
 // Class to enumerate volume labels of volumes assigned
 // a drive letter under MSW.
@@ -25,11 +23,11 @@
 DECLARE_EVENT_TYPE(fzEVT_VOLUMEENUMERATED, -1)
 DECLARE_EVENT_TYPE(fzEVT_VOLUMESENUMERATED, -1)
 
-class CVolumeDescriptionEnumeratorThread final : protected fz::thread
+class CVolumeDescriptionEnumeratorThread final
 {
 public:
-	CVolumeDescriptionEnumeratorThread(wxEvtHandler* pEvtHandler);
-	virtual ~CVolumeDescriptionEnumeratorThread();
+	CVolumeDescriptionEnumeratorThread(wxEvtHandler* pEvtHandler, fz::thread_pool & pool);
+	~CVolumeDescriptionEnumeratorThread();
 
 	bool Failed() const { return m_failure; }
 
@@ -40,28 +38,33 @@ public:
 		int icon{-1};
 	};
 
-	std::list<t_VolumeInfo> GetVolumes();
+	std::vector<t_VolumeInfo> GetVolumes();
 
-	static std::list<std::wstring> GetDrives();
+	static std::vector<std::wstring> GetDrives();
 
 	static long GetDrivesToHide();
 	static bool IsHidden(wchar_t const* drive, long noDrives);
 
 protected:
+	static std::vector<std::wstring> GetDrives(long drivesToHide);
+
 	bool GetDriveLabels();
 	void ProcessDrive(std::wstring const& drive);
 	bool GetDriveLabel(std::wstring const& drive);
 	bool GetDriveIcon(std::wstring const& drive);
-	virtual void entry();
+	void entry();
 
 	wxEvtHandler* m_pEvtHandler;
 
 	bool m_failure{};
 	bool m_stop{};
 
-	std::list<t_VolumeInfo> m_volumeInfo;
+	std::vector<t_VolumeInfo> m_volumeInfo;
 
+	fz::async_task thread_;
 	fz::mutex sync_{false};
+
+	long drivesToHide_{};
 };
 
 #endif
