@@ -68,13 +68,13 @@ bool CWelcomeDialog::Run(wxWindow* parent, bool force)
 	headerLeft->Add(heading);
 	headerLeft->Add(new wxStaticText(this, -1, _("The free open source FTP solution")));
 
-	header->AddSpacer(wxDLG_UNIT(this, wxSize(10, -1)).x);
+	header->AddSpacer(lay.dlgUnits(5));
 
 	header->Add(new wxStaticBitmap(this, -1, CThemeProvider::Get()->CreateBitmap("ART_FILEZILLA", wxString(), CThemeProvider::GetIconSize(iconSizeLarge))), lay.valign);
 
 	main->Add(new wxStaticLine(this), lay.grow);
 
-	int const leftIndent = wxDLG_UNIT(this, wxPoint(10, 0)).x;
+	int const leftIndent = lay.dlgUnits(10);
 	wxString const url = _T("https://welcome.filezilla-project.org/welcome?type=client&category=%s&version=") + ownVersion;
 
 	main->Add(new wxPanel(this, XRCID("ID_HEADERMESSAGE_PANEL")), lay.halign)->Show(false);
@@ -131,7 +131,9 @@ bool CWelcomeDialog::Run(wxWindow* parent, bool force)
 
 void CWelcomeDialog::OnTimer(wxTimerEvent&)
 {
-	Run(parent_, false);
+	if (CanShowPopupDialog()) {
+		Run(parent_, false);
+	}
 	Destroy();
 }
 
@@ -167,7 +169,7 @@ void CWelcomeDialog::InitFooter(wxString const& resources)
 
 			wxXmlResource res(wxXRC_NO_RELOADING);
 			InitHandlers(res);
-			if (res.Load(_T("blob:") + resources)) {
+			if (res.Load(_T("blob64:") + resources)) {
 				CreateMessagePanel(*this, "ID_HEADERMESSAGE_PANEL", res, _T("ID_WELCOME_HEADER"));
 				CreateMessagePanel(*this, "ID_FOOTERMESSAGE_PANEL", res, _T("ID_WELCOME_FOOTER"));
 			}
