@@ -1,11 +1,14 @@
 #include <filezilla.h>
 #include "buildinfo.h"
 #include "xmlfunctions.h"
+#include "xmlutils.h"
 #include "Options.h"
 #include <wx/ffile.h>
 #include <wx/log.h>
 
+#include <libfilezilla/file.hpp>
 #include <libfilezilla/local_filesys.hpp>
+#include <libfilezilla/translate.hpp>
 
 CXmlFile::CXmlFile(std::wstring const& fileName, std::string const& root)
 {
@@ -33,12 +36,12 @@ pugi::xml_node CXmlFile::Load(bool overwriteInvalid)
 
 	GetXmlFile(redirectedName);
 	if (!m_element) {
-		wxString err = wxString::Format(_("The file '%s' could not be loaded."), m_fileName);
+		std::wstring err = fz::sprintf(fztranslate("The file '%s' could not be loaded."), m_fileName);
 		if (m_error.empty()) {
-			err += wxString(_T("\n")) + _("Make sure the file can be accessed and is a well-formed XML document.");
+			err += L"\n" + fztranslate("Make sure the file can be accessed and is a well-formed XML document.");
 		}
 		else {
-			err += _T("\n") + m_error;
+			err += L"\n" + m_error;
 		}
 
 		// Try the backup file
@@ -72,19 +75,19 @@ pugi::xml_node CXmlFile::Load(bool overwriteInvalid)
 		bool res;
 		{
 			wxLogNull null;
-			res = wxCopyFile(redirectedName + _T("~"), redirectedName);
+			res = wxCopyFile(redirectedName + L"~", redirectedName);
 		}
 		if (!res) {
 			// Could not restore backup, give up.
 			Close();
-			m_error = err.ToStdWstring();
-			m_error += _T("\n") + wxString::Format(_("The valid backup file %s could not be restored"), redirectedName + _T("~")).ToStdWstring();
+			m_error = err;
+			m_error += L"\n" + fz::sprintf(fztranslate("The valid backup file %s could not be restored"), redirectedName + L"~");
 			m_modificationTime.clear();
 			return m_element;
 		}
 
 		// We no longer need the backup
-		wxRemoveFile(redirectedName + _T("~"));
+		fz::remove_file(fz::to_native(redirectedName + L"~"));
 		m_error.clear();
 	}
 
@@ -164,99 +167,6 @@ pugi::xml_node CXmlFile::CreateEmpty()
 	return m_element;
 }
 
-pugi::xml_node AddTextElement(pugi::xml_node node, const char* name, std::string const& value, bool overwrite)
-{
-	return AddTextElementUtf8(node, name, fz::to_utf8(value), overwrite);
-}
-
-pugi::xml_node AddTextElement(pugi::xml_node node, const char* name, std::wstring const& value, bool overwrite)
-{
-	return AddTextElementUtf8(node, name, fz::to_utf8(value), overwrite);
-}
-
-void AddTextElement(pugi::xml_node node, const char* name, int64_t value, bool overwrite)
-{
-	if (overwrite) {
-		node.remove_child(name);
-	}
-	auto child = node.append_child(name);
-	child.text().set(static_cast<long long>(value));
-}
-
-pugi::xml_node AddTextElementUtf8(pugi::xml_node node, const char* name, std::string const& value, bool overwrite)
-{
-	assert(node);
-
-	if (overwrite) {
-		node.remove_child(name);
-	}
-
-	auto element = node.append_child(name);
-	if (!value.empty()) {
-		element.text().set(value.c_str());
-	}
-
-	return element;
-}
-
-void AddTextElement(pugi::xml_node node, std::string const& value)
-{
-	AddTextElementUtf8(node, fz::to_utf8(value));
-}
-
-void AddTextElement(pugi::xml_node node, std::wstring const& value)
-{
-	AddTextElementUtf8(node, fz::to_utf8(value));
-}
-
-void AddTextElement(pugi::xml_node node, int64_t value)
-{
-	assert(node);
-	node.text().set(static_cast<long long>(value));
-}
-
-void AddTextElementUtf8(pugi::xml_node node, std::string const& value)
-{
-	assert(node);
-	node.text().set(value.c_str());
-}
-
-std::wstring GetTextElement_Trimmed(pugi::xml_node node, const char* name)
-{
-	return fz::trimmed(GetTextElement(node, name));
-}
-
-std::wstring GetTextElement(pugi::xml_node node, const char* name)
-{
-	wxASSERT(node);
-
-	return fz::to_wstring_from_utf8(node.child_value(name));
-}
-
-std::wstring GetTextElement_Trimmed(pugi::xml_node node)
-{
-	return fz::trimmed(GetTextElement(node));
-}
-
-std::wstring GetTextElement(pugi::xml_node node)
-{
-	wxASSERT(node);
-
-	return fz::to_wstring_from_utf8(node.child_value());
-}
-
-int64_t GetTextElementInt(pugi::xml_node node, const char* name, int defValue /*=0*/)
-{
-	wxASSERT(node);
-	return static_cast<int64_t>(node.child(name).text().as_llong(defValue));
-}
-
-bool GetTextElementBool(pugi::xml_node node, const char* name, bool defValue /*=false*/)
-{
-	wxASSERT(node);
-	return node.child(name).text().as_bool(defValue);
-}
-
 // Opens the specified XML file if it exists or creates a new one otherwise.
 // Returns false on error.
 bool CXmlFile::GetXmlFile(std::wstring const& file)
@@ -314,8 +224,9 @@ bool CXmlFile::SaveXmlFile()
 	wxString redirectedName = GetRedirectedName();
 	if (fz::local_filesys::get_file_info(fz::to_native(redirectedName), isLink, 0, 0, &flags) == fz::local_filesys::file) {
 #ifdef __WXMSW__
-		if (flags & FILE_ATTRIBUTE_HIDDEN)
+		if (flags & FILE_ATTRIBUTE_HIDDEN) {
 			SetFileAttributes(redirectedName.c_str(), flags & ~FILE_ATTRIBUTE_HIDDEN);
+		}
 #endif
 
 		exists = true;
@@ -330,10 +241,40 @@ bool CXmlFile::SaveXmlFile()
 		}
 	}
 
-	bool success = m_document.save_file(static_cast<wchar_t const*>(redirectedName.c_str()));
+	struct flushing_xml_writer final : public pugi::xml_writer
+	{
+	public:
+		static bool save(pugi::xml_document const& document, std::wstring const& filename)
+		{
+			flushing_xml_writer writer(filename);
+			if (!writer.file_.opened()) {
+				return false;
+			}
+			document.save(writer);
 
+			return writer.file_.opened() && writer.file_.fsync();
+		}
+
+	private:
+		flushing_xml_writer(std::wstring const& filename)
+			: file_(fz::to_native(filename), fz::file::writing, fz::file::empty)
+		{
+		}
+
+		virtual void write(void const* data, size_t size) override {
+			if (file_.opened()) {
+				if (file_.write(data, static_cast<int64_t>(size)) != static_cast<int64_t>(size)) {
+					file_.close();
+				}
+			}
+		}
+
+		fz::file file_;
+	};
+
+	bool success = flushing_xml_writer::save(m_document, redirectedName.ToStdWstring());
 	if (!success) {
-		wxRemoveFile(redirectedName);
+		fz::remove_file(fz::to_native(redirectedName));
 		if (exists) {
 			wxLogNull null;
 			wxRenameFile(redirectedName + _T("~"), redirectedName);
@@ -342,13 +283,14 @@ bool CXmlFile::SaveXmlFile()
 		return false;
 	}
 
-	if (exists)
-		wxRemoveFile(redirectedName + _T("~"));
+	if (exists) {
+		fz::remove_file(fz::to_native(redirectedName + L"~"));
+	}
 
 	return true;
 }
 
-bool GetServer(pugi::xml_node node, ServerWithCredentials & server)
+bool GetServer(pugi::xml_node node, Site & site)
 {
 	wxASSERT(node);
 
@@ -362,7 +304,7 @@ bool GetServer(pugi::xml_node node, ServerWithCredentials & server)
 		return false;
 	}
 
-	if (!server.server.SetHost(host, port)) {
+	if (!site.server.SetHost(host, port)) {
 		return false;
 	}
 
@@ -370,30 +312,33 @@ bool GetServer(pugi::xml_node node, ServerWithCredentials & server)
 	if (protocol < 0 || protocol > ServerProtocol::MAX_VALUE) {
 		return false;
 	}
-	server.server.SetProtocol(static_cast<ServerProtocol>(protocol));
+	site.server.SetProtocol(static_cast<ServerProtocol>(protocol));
 
 	int type = GetTextElementInt(node, "Type");
 	if (type < 0 || type >= SERVERTYPE_MAX) {
 		return false;
 	}
 
-	server.server.SetType(static_cast<ServerType>(type));
+	site.server.SetType(static_cast<ServerType>(type));
 
 	int logonType = GetTextElementInt(node, "Logontype");
 	if (logonType < 0 || logonType >= static_cast<int>(LogonType::count)) {
 		return false;
 	}
 
-	server.SetLogonType(static_cast<LogonType>(logonType));
+	site.SetLogonType(static_cast<LogonType>(logonType));
 	
-	if (server.credentials.logonType_ != LogonType::anonymous) {
-		std::wstring user = GetTextElement(node, "User");
-		if (user.empty() && server.credentials.logonType_ != LogonType::interactive && server.credentials.logonType_ != LogonType::ask) {
+	if (site.credentials.logonType_ != LogonType::anonymous) {
+		std::wstring user;
+		if (ProtocolHasUser(site.server.GetProtocol())) {
+			user = GetTextElement(node, "User");
+		}
+		if (user.empty() && site.credentials.logonType_ != LogonType::interactive && site.credentials.logonType_ != LogonType::ask) {
 			return false;
 		}
 
 		std::wstring pass, key;
-		if (server.credentials.logonType_ == LogonType::normal || server.credentials.logonType_ == LogonType::account) {
+		if (site.credentials.logonType_ == LogonType::normal || site.credentials.logonType_ == LogonType::account) {
 			auto passElement = node.child("Pass");
 			if (passElement) {
 				std::wstring encoding = GetTextAttribute(passElement, "encoding");
@@ -404,75 +349,75 @@ bool GetServer(pugi::xml_node node, ServerWithCredentials & server)
 				}
 				else if (encoding == _T("crypt")) {
 					pass = fz::to_wstring_from_utf8(passElement.child_value());
-					server.credentials.encrypted_ = public_key::from_base64(passElement.attribute("pubkey").value());
-					if (!server.credentials.encrypted_) {
+					site.credentials.encrypted_ = fz::public_key::from_base64(passElement.attribute("pubkey").value());
+					if (!site.credentials.encrypted_) {
 						pass.clear();
-						server.SetLogonType(LogonType::ask);
+						site.SetLogonType(LogonType::ask);
 					}
 				}
 				else if (!encoding.empty()) {
-					server.SetLogonType(LogonType::ask);
+					site.SetLogonType(LogonType::ask);
 				}
 				else {
 					pass = GetTextElement(passElement);
 				}
 			}
 		}
-		else if (server.credentials.logonType_ == LogonType::key) {
+		else if (site.credentials.logonType_ == LogonType::key) {
 			key = GetTextElement(node, "Keyfile");
 
 			// password should be empty if we're using a key file
 			pass.clear();
 
-			server.credentials.keyFile_ = key;
+			site.credentials.keyFile_ = key;
 		}
 
-		server.SetUser(user);
-		server.credentials.SetPass(pass);
+		site.SetUser(user);
+		site.credentials.SetPass(pass);
 
-		server.credentials.account_ = GetTextElement(node, "Account");
+		site.credentials.account_ = GetTextElement(node, "Account");
 	}
 
 	int timezoneOffset = GetTextElementInt(node, "TimezoneOffset");
-	if (!server.server.SetTimezoneOffset(timezoneOffset)) {
+	if (!site.server.SetTimezoneOffset(timezoneOffset)) {
 		return false;
 	}
 
 	wxString pasvMode = GetTextElement(node, "PasvMode");
 	if (pasvMode == _T("MODE_PASSIVE")) {
-		server.server.SetPasvMode(MODE_PASSIVE);
+		site.server.SetPasvMode(MODE_PASSIVE);
 	}
 	else if (pasvMode == _T("MODE_ACTIVE")) {
-		server.server.SetPasvMode(MODE_ACTIVE);
+		site.server.SetPasvMode(MODE_ACTIVE);
 	}
 	else {
-		server.server.SetPasvMode(MODE_DEFAULT);
+		site.server.SetPasvMode(MODE_DEFAULT);
 	}
 
 	int maximumMultipleConnections = GetTextElementInt(node, "MaximumMultipleConnections");
-	server.server.MaximumMultipleConnections(maximumMultipleConnections);
+	site.server.MaximumMultipleConnections(maximumMultipleConnections);
 
 	wxString encodingType = GetTextElement(node, "EncodingType");
 	if (encodingType == _T("Auto")) {
-		server.server.SetEncodingType(ENCODING_AUTO);
+		site.server.SetEncodingType(ENCODING_AUTO);
 	}
 	else if (encodingType == _T("UTF-8")) {
-		server.server.SetEncodingType(ENCODING_UTF8);
+		site.server.SetEncodingType(ENCODING_UTF8);
 	}
 	else if (encodingType == _T("Custom")) {
 		std::wstring customEncoding = GetTextElement(node, "CustomEncoding");
 		if (customEncoding.empty()) {
 			return false;
 		}
-		if (!server.server.SetEncodingType(ENCODING_CUSTOM, customEncoding)) {
+		if (!site.server.SetEncodingType(ENCODING_CUSTOM, customEncoding)) {
 			return false;
 		}
 	}
 	else {
-		server.server.SetEncodingType(ENCODING_AUTO);
+		site.server.SetEncodingType(ENCODING_AUTO);
 	}
 
-	if (CServer::SupportsPostLoginCommands(server.server.GetProtocol())) {
+	if (CServer::ProtocolHasFeature(site.server.GetProtocol(), ProtocolFeature::PostLoginCommands)) {
 		std::vector<std::wstring> postLoginCommands;
 		auto element = node.child("PostLoginCommands");
 		if (element) {
@@ -483,22 +428,26 @@ bool GetServer(pugi::xml_node node, ServerWithCredentials & server)
 				}
 			}
 		}
-		if (!server.server.SetPostLoginCommands(postLoginCommands)) {
+		if (!site.server.SetPostLoginCommands(postLoginCommands)) {
 			return false;
 		}
 	}
 
-	server.server.SetBypassProxy(GetTextElementInt(node, "BypassProxy", false) == 1);
-	server.server.SetName(GetTextElement_Trimmed(node, "Name"));
+	site.server.SetBypassProxy(GetTextElementInt(node, "BypassProxy", false) == 1);
+	site.SetName(GetTextElement_Trimmed(node, "Name"));
 
-	if (server.server.GetName().empty()) {
-		server.server.SetName(GetTextElement_Trimmed(node));
+	if (site.GetName().empty()) {
+		site.SetName(GetTextElement_Trimmed(node));
+	}
+
+	for (auto parameter = node.child("Parameter"); parameter; parameter = parameter.next_sibling("Parameter")) {
+		site.server.SetExtraParameter(parameter.attribute("Name").value(), GetTextElement(parameter));
 	}
 
 	return true;
 }
 
-void SetServer(pugi::xml_node node, ServerWithCredentials const& server)
+void SetServer(pugi::xml_node node, Site const& site)
 {
 	if (!node) {
 		return;
@@ -508,15 +457,17 @@ void SetServer(pugi::xml_node node, ServerWithCredentials const& server)
 		node.remove_child(child);
 	}
 
-	AddTextElement(node, "Host", server.server.GetHost());
-	AddTextElement(node, "Port", server.server.GetPort());
-	AddTextElement(node, "Protocol", server.server.GetProtocol());
-	AddTextElement(node, "Type", server.server.GetType());
+	ServerProtocol const protocol = site.server.GetProtocol();
 
-	ProtectedCredentials credentials = server.credentials;
+	AddTextElement(node, "Host", site.server.GetHost());
+	AddTextElement(node, "Port", site.server.GetPort());
+	AddTextElement(node, "Protocol", protocol);
+	AddTextElement(node, "Type", site.server.GetType());
+
+	ProtectedCredentials credentials = site.credentials;
 
 	if (credentials.logonType_ != LogonType::anonymous) {
-		AddTextElement(node, "User", server.server.GetUser());
+		AddTextElement(node, "User", site.server.GetUser());
 
 		credentials.Protect();
 
@@ -547,8 +498,8 @@ void SetServer(pugi::xml_node node, ServerWithCredentials const& server)
 	}
 	AddTextElement(node, "Logontype", static_cast<int>(credentials.logonType_));
 
-	AddTextElement(node, "TimezoneOffset", server.server.GetTimezoneOffset());
-	switch (server.server.GetPasvMode())
+	AddTextElement(node, "TimezoneOffset", site.server.GetTimezoneOffset());
+	switch (site.server.GetPasvMode())
 	{
 	case MODE_PASSIVE:
 		AddTextElementUtf8(node, "PasvMode", "MODE_PASSIVE");
@@ -560,9 +511,9 @@ void SetServer(pugi::xml_node node, ServerWithCredentials const& server)
 		AddTextElementUtf8(node, "PasvMode", "MODE_DEFAULT");
 		break;
 	}
-	AddTextElement(node, "MaximumMultipleConnections", server.server.MaximumMultipleConnections());
+	AddTextElement(node, "MaximumMultipleConnections", site.server.MaximumMultipleConnections());
 
-	switch (server.server.GetEncodingType())
+	switch (site.server.GetEncodingType())
 	{
 	case ENCODING_AUTO:
 		AddTextElementUtf8(node, "EncodingType", "Auto");
@@ -572,12 +523,12 @@ void SetServer(pugi::xml_node node, ServerWithCredentials const& server)
 		break;
 	case ENCODING_CUSTOM:
 		AddTextElementUtf8(node, "EncodingType", "Custom");
-		AddTextElement(node, "CustomEncoding", server.server.GetCustomEncoding());
+		AddTextElement(node, "CustomEncoding", site.server.GetCustomEncoding());
 		break;
 	}
 
-	if (CServer::SupportsPostLoginCommands(server.server.GetProtocol())) {
-		std::vector<std::wstring> const& postLoginCommands = server.server.GetPostLoginCommands();
+	if (CServer::ProtocolHasFeature(site.server.GetProtocol(), ProtocolFeature::PostLoginCommands)) {
+		std::vector<std::wstring> const& postLoginCommands = site.server.GetPostLoginCommands();
 		if (!postLoginCommands.empty()) {
 			auto element = node.append_child("PostLoginCommands");
 			for (auto const& command : postLoginCommands) {
@@ -586,68 +537,16 @@ void SetServer(pugi::xml_node node, ServerWithCredentials const& server)
 		}
 	}
 
-	AddTextElementUtf8(node, "BypassProxy", server.server.GetBypassProxy() ? "1" : "0");
-	std::wstring const& name = server.server.GetName();
+	AddTextElementUtf8(node, "BypassProxy", site.server.GetBypassProxy() ? "1" : "0");
+	std::wstring const& name = site.GetName();
 	if (!name.empty()) {
 		AddTextElement(node, "Name", name);
 	}
-}
 
-void SetTextAttribute(pugi::xml_node node, char const* name, std::string const& value)
-{
-	SetTextAttributeUtf8(node, name, fz::to_utf8(value));
-}
-
-void SetTextAttribute(pugi::xml_node node, char const* name, std::wstring const& value)
-{
-	SetTextAttributeUtf8(node, name, fz::to_utf8(value));
-}
-
-void SetTextAttributeUtf8(pugi::xml_node node, char const* name, std::string const& utf8)
-{
-	assert(node);
-	auto attribute = node.attribute(name);
-	if (!attribute) {
-		attribute = node.append_attribute(name);
+	for (auto const& parameter : site.server.GetExtraParameters()) {
+		auto element = AddTextElement(node, "Parameter", parameter.second);
+		SetTextAttribute(element, "Name", parameter.first);
 	}
-	attribute.set_value(utf8.c_str());
-}
-
-std::wstring GetTextAttribute(pugi::xml_node node, char const* name)
-{
-	assert(node);
-
-	const char* value = node.attribute(name).value();
-	return fz::to_wstring_from_utf8(value);
-}
-
-pugi::xml_node FindElementWithAttribute(pugi::xml_node node, const char* element, const char* attribute, const char* value)
-{
-	pugi::xml_node child = element ? node.child(element) : node.first_child();
-	while (child) {
-		const char* nodeVal = child.attribute(attribute).value();
-		if (nodeVal && !strcmp(value, nodeVal)) {
-			return child;
-		}
-
-		child = element ? child.next_sibling(element) : child.next_sibling();
-	}
-
-	return child;
-}
-
-int GetAttributeInt(pugi::xml_node node, const char* name)
-{
-	return node.attribute(name).as_int();
-}
-
-void SetAttributeInt(pugi::xml_node node, const char* name, int value)
-{
-	auto attribute = node.attribute(name);
-	if (!attribute) {
-		attribute = node.append_attribute(name);
-	}
-	attribute.set_value(value);
 }
 
 namespace {
@@ -657,7 +556,7 @@ struct xml_memory_writer : pugi::xml_writer
 	char* buffer{};
 	size_t remaining{};
 
-	virtual void write(const void* data, size_t size)
+	virtual void write(void const* data, size_t size)
 	{
 		if (buffer && size <= remaining) {
 			memcpy(buffer, data, size);
