@@ -33,15 +33,11 @@ enum {
 
 static const char hex[16] = "0123456789ABCDEF";
 
-static char *mungestr(const char *in)
+static void make_session_filename(const char *in, strbuf *out)
 {
-    char *out, *ret;
-
     if (!in || !*in)
         in = "Default Settings";
 
-    ret = out = snewn(3*strlen(in)+1, char);
-
     while (*in) {
         /*
          * There are remarkably few punctuation characters that
@@ -50,42 +46,36 @@ static char *mungestr(const char *in)
          * opt-in for safe characters rather than opt-out for
          * specific unsafe ones...
          */
-	if (*in!='+' && *in!='-' && *in!='.' && *in!='@' && *in!='_' &&
+        if (*in!='+' && *in!='-' && *in!='.' && *in!='@' && *in!='_' &&
             !(*in >= '0' && *in <= '9') &&
             !(*in >= 'A' && *in <= 'Z') &&
             !(*in >= 'a' && *in <= 'z')) {
-	    *out++ = '%';
-	    *out++ = hex[((unsigned char) *in) >> 4];
-	    *out++ = hex[((unsigned char) *in) & 15];
-	} else
-	    *out++ = *in;
-	in++;
+            put_byte(out, '%');
+            put_byte(out, hex[((unsigned char) *in) >> 4]);
+            put_byte(out, hex[((unsigned char) *in) & 15]);
+        } else
+            put_byte(out, *in);
+        in++;
     }
-    *out = '\0';
-    return ret;
 }
 
-static char *unmungestr(const char *in)
+static void decode_session_filename(const char *in, strbuf *out)
 {
-    char *out, *ret;
-    out = ret = snewn(strlen(in)+1, char);
     while (*in) {
-	if (*in == '%' && in[1] && in[2]) {
-	    int i, j;
+        if (*in == '%' && in[1] && in[2]) {
+            int i, j;
 
-	    i = in[1] - '0';
-	    i -= (i > 9 ? 7 : 0);
-	    j = in[2] - '0';
-	    j -= (j > 9 ? 7 : 0);
+            i = in[1] - '0';
+            i -= (i > 9 ? 7 : 0);
+            j = in[2] - '0';
+            j -= (j > 9 ? 7 : 0);
 
-	    *out++ = (i << 4) + j;
-	    in += 3;
-	} else {
-	    *out++ = *in++;
-	}
+            put_byte(out, (i << 4) + j);
+            in += 3;
+        } else {
+            put_byte(out, *in++);
+        }
     }
-    *out = '\0';
-    return ret;
 }
 
 static char *make_filename(int index, const char *subname)
@@ -97,15 +87,15 @@ static char *make_filename(int index, const char *subname)
      * specific subparts of it, by means of environment variables.
      */
     if (index == INDEX_DIR) {
-	struct passwd *pwd;
+        struct passwd *pwd;
         char *xdg_dir, *old_dir, *old_dir2, *old_dir3, *home, *pwd_home;
 
-	env = getenv("PUTTYDIR");
-	if (env)
-	    return dupstr(env);
+        env = getenv("PUTTYDIR");
+        if (env)
+            return dupstr(env);
 
         home = getenv("HOME");
-	pwd = getpwuid(getuid());
+        pwd = getpwuid(getuid());
         if (pwd && pwd->pw_dir) {
             pwd_home = pwd->pw_dir;
         } else {
@@ -140,11 +130,11 @@ static char *make_filename(int index, const char *subname)
         }
         old_dir3 = dupstr("/.putty");
 
-        if (access(old_dir, F_OK) == 0) {
+        if (old_dir && access(old_dir, F_OK) == 0) {
             ret = old_dir;
             goto out;
         }
-        if (access(old_dir2, F_OK) == 0) {
+        if (old_dir2 && access(old_dir2, F_OK) == 0) {
             ret = old_dir2;
             goto out;
         }
@@ -172,45 +162,45 @@ static char *make_filename(int index, const char *subname)
         return ret;
     }
     if (index == INDEX_SESSIONDIR) {
-	env = getenv("PUTTYSESSIONS");
-	if (env)
-	    return dupstr(env);
-	tmp = make_filename(INDEX_DIR, NULL);
-	ret = dupprintf("%s/sessions", tmp);
-	sfree(tmp);
-	return ret;
+        env = getenv("PUTTYSESSIONS");
+        if (env)
+            return dupstr(env);
+        tmp = make_filename(INDEX_DIR, NULL);
+        ret = dupprintf("%s/sessions", tmp);
+        sfree(tmp);
+        return ret;
     }
     if (index == INDEX_SESSION) {
-        char *munged = mungestr(subname);
-	tmp = make_filename(INDEX_SESSIONDIR, NULL);
-	ret = dupprintf("%s/%s", tmp, munged);
-	sfree(tmp);
-	sfree(munged);
-	return ret;
+        strbuf *sb = strbuf_new();
+        tmp = make_filename(INDEX_SESSIONDIR, NULL);
+        strbuf_catf(sb, "%s/", tmp);
+        sfree(tmp);
+        make_session_filename(subname, sb);
+        return strbuf_to_str(sb);
     }
     if (index == INDEX_HOSTKEYS) {
-	env = getenv("PUTTYSSHHOSTKEYS");
-	if (env)
-	    return dupstr(env);
-	tmp = make_filename(INDEX_DIR, NULL);
-	ret = dupprintf("%s/sshhostkeys", tmp);
-	sfree(tmp);
-	return ret;
+        env = getenv("PUTTYSSHHOSTKEYS");
+        if (env)
+            return dupstr(env);
+        tmp = make_filename(INDEX_DIR, NULL);
+        ret = dupprintf("%s/sshhostkeys", tmp);
+        sfree(tmp);
+        return ret;
     }
     if (index == INDEX_HOSTKEYS_TMP) {
-	tmp = make_filename(INDEX_HOSTKEYS, NULL);
-	ret = dupprintf("%s.tmp", tmp);
-	sfree(tmp);
-	return ret;
+        tmp = make_filename(INDEX_HOSTKEYS, NULL);
+        ret = dupprintf("%s.tmp", tmp);
+        sfree(tmp);
+        return ret;
     }
     if (index == INDEX_RANDSEED) {
-	env = getenv("PUTTYRANDOMSEED");
-	if (env)
-	    return dupstr(env);
-	tmp = make_filename(INDEX_DIR, NULL);
-	ret = dupprintf("%s/randomseed", tmp);
-	sfree(tmp);
-	return ret;
+        env = getenv("PUTTYRANDOMSEED");
+        if (env)
+            return dupstr(env);
+        tmp = make_filename(INDEX_DIR, NULL);
+        ret = dupprintf("%s/randomseed", tmp);
+        sfree(tmp);
+        return ret;
     }
     tmp = make_filename(INDEX_DIR, NULL);
     ret = dupprintf("%s/ERROR", tmp);
@@ -218,7 +208,11 @@ static char *make_filename(int index, const char *subname)
     return ret;
 }
 
-void *open_settings_w(const char *sessionname, char **errmsg)
+struct settings_w {
+    FILE *fp;
+};
+
+settings_w *open_settings_w(const char *sessionname, char **errmsg)
 {
     char *filename, *err;
     FILE *fp;
@@ -252,39 +246,45 @@ void *open_settings_w(const char *sessionname, char **errmsg)
     if (!fp) {
         *errmsg = dupprintf("Unable to save session: open(\"%s\") "
                             "returned '%s'", filename, strerror(errno));
-	sfree(filename);
-	return NULL;                   /* can't open */
+        sfree(filename);
+        return NULL;                   /* can't open */
     }
     sfree(filename);
-    return fp;
+
+    settings_w *toret = snew(settings_w);
+    toret->fp = fp;
+    return toret;
 }
 
-void write_setting_s(void *handle, const char *key, const char *value)
+void write_setting_s(settings_w *handle, const char *key, const char *value)
 {
-    FILE *fp = (FILE *)handle;
-    fprintf(fp, "%s=%s\n", key, value);
+    fprintf(handle->fp, "%s=%s\n", key, value);
 }
 
-void write_setting_i(void *handle, const char *key, int value)
+void write_setting_i(settings_w *handle, const char *key, int value)
 {
-    FILE *fp = (FILE *)handle;
-    fprintf(fp, "%s=%d\n", key, value);
+    fprintf(handle->fp, "%s=%d\n", key, value);
 }
 
-void close_settings_w(void *handle)
+void close_settings_w(settings_w *handle)
 {
-    FILE *fp = (FILE *)handle;
-    fclose(fp);
+    fclose(handle->fp);
+    sfree(handle);
 }
 
-/*
- * Reading settings, for the moment, is done by retrieving X
- * resources from the X display. When we introduce disk files, I
- * think what will happen is that the X resources will override
- * PuTTY's inbuilt defaults, but that the disk files will then
- * override those. This isn't optimal, but it's the best I can
- * immediately work out.
- * FIXME: the above comment is a bit out of date. Did it happen?
+/* ----------------------------------------------------------------------
+ * System for treating X resources as a fallback source of defaults,
+ * after data read from a saved-session disk file.
+ *
+ * The read_setting_* functions will call get_setting(key) as a
+ * fallback if the setting isn't in the file they loaded. That in turn
+ * will hand on to x_get_default, which the front end application
+ * provides, and which actually reads resources from the X server (if
+ * appropriate). In between, there's a tree234 of X-resource shaped
+ * settings living locally in this file: the front end can call
+ * provide_xrm_string() to insert a setting into this tree (typically
+ * in response to an -xrm command line option or similar), and those
+ * will override the actual X resources.
  */
 
 struct skeyval {
@@ -294,73 +294,84 @@ struct skeyval {
 
 static tree234 *xrmtree = NULL;
 
-int keycmp(void *av, void *bv)
+static int keycmp(void *av, void *bv)
 {
     struct skeyval *a = (struct skeyval *)av;
     struct skeyval *b = (struct skeyval *)bv;
     return strcmp(a->key, b->key);
 }
 
-void provide_xrm_string(char *string)
+void provide_xrm_string(const char *string, const char *progname)
 {
-    char *p, *q, *key;
+    const char *p, *q;
+    char *key;
     struct skeyval *xrms, *ret;
 
     p = q = strchr(string, ':');
     if (!q) {
-	fprintf(stderr, "pterm: expected a colon in resource string"
-		" \"%s\"\n", string);
-	return;
+        fprintf(stderr, "%s: expected a colon in resource string"
+                " \"%s\"\n", progname, string);
+        return;
     }
     q++;
     while (p > string && p[-1] != '.' && p[-1] != '*')
-	p--;
+        p--;
     xrms = snew(struct skeyval);
     key = snewn(q-p, char);
     memcpy(key, p, q-p);
     key[q-p-1] = '\0';
     xrms->key = key;
     while (*q && isspace((unsigned char)*q))
-	q++;
+        q++;
     xrms->value = dupstr(q);
 
     if (!xrmtree)
-	xrmtree = newtree234(keycmp);
+        xrmtree = newtree234(keycmp);
 
     ret = add234(xrmtree, xrms);
     if (ret) {
-	/* Override an existing string. */
-	del234(xrmtree, ret);
-	add234(xrmtree, xrms);
+        /* Override an existing string. */
+        del234(xrmtree, ret);
+        add234(xrmtree, xrms);
     }
 }
 
-const char *get_setting(const char *key)
+static const char *get_setting(const char *key)
 {
     struct skeyval tmp, *ret;
     tmp.key = key;
     if (xrmtree) {
-	ret = find234(xrmtree, &tmp, NULL);
-	if (ret)
-	    return ret->value;
+        ret = find234(xrmtree, &tmp, NULL);
+        if (ret)
+            return ret->value;
     }
     return x_get_default(key);
 }
 
-void *open_settings_r(const char *sessionname)
+/* ----------------------------------------------------------------------
+ * Main code for reading settings from a disk file, calling the above
+ * get_setting() as a fallback if necessary.
+ */
+
+struct settings_r {
+    tree234 *t;
+};
+
+settings_r *open_settings_r(const char *sessionname)
 {
     char *filename;
     FILE *fp;
     char *line;
-    tree234 *ret;
+    settings_r *toret;
 
     filename = make_filename(INDEX_SESSION, sessionname);
     fp = fopen(filename, "r");
     sfree(filename);
     if (!fp)
-	return NULL;		       /* can't open */
+        return NULL;                   /* can't open */
 
-    ret = newtree234(keycmp);
+    toret = snew(settings_r);
+    toret->t = newtree234(keycmp);
 
     while ( (line = fgetline(fp)) ) {
         char *value = strchr(line, '=');
@@ -376,63 +387,61 @@ void *open_settings_r(const char *sessionname)
         kv = snew(struct skeyval);
         kv->key = dupstr(line);
         kv->value = dupstr(value);
-        add234(ret, kv);
+        add234(toret->t, kv);
 
         sfree(line);
     }
 
     fclose(fp);
 
-    return ret;
+    return toret;
 }
 
-char *read_setting_s(void *handle, const char *key)
+char *read_setting_s(settings_r *handle, const char *key)
 {
-    tree234 *tree = (tree234 *)handle;
     const char *val;
     struct skeyval tmp, *kv;
 
     tmp.key = key;
-    if (tree != NULL &&
-        (kv = find234(tree, &tmp, NULL)) != NULL) {
+    if (handle != NULL &&
+        (kv = find234(handle->t, &tmp, NULL)) != NULL) {
         val = kv->value;
         assert(val != NULL);
     } else
         val = get_setting(key);
 
     if (!val)
-	return NULL;
+        return NULL;
     else
-	return dupstr(val);
+        return dupstr(val);
 }
 
-int read_setting_i(void *handle, const char *key, int defvalue)
+int read_setting_i(settings_r *handle, const char *key, int defvalue)
 {
-    tree234 *tree = (tree234 *)handle;
     const char *val;
     struct skeyval tmp, *kv;
 
     tmp.key = key;
-    if (tree != NULL &&
-        (kv = find234(tree, &tmp, NULL)) != NULL) {
+    if (handle != NULL &&
+        (kv = find234(handle->t, &tmp, NULL)) != NULL) {
         val = kv->value;
         assert(val != NULL);
     } else
         val = get_setting(key);
 
     if (!val)
-	return defvalue;
+        return defvalue;
     else
-	return atoi(val);
+        return atoi(val);
 }
 
-FontSpec *read_setting_fontspec(void *handle, const char *name)
+FontSpec *read_setting_fontspec(settings_r *handle, const char *name)
 {
     /*
      * In GTK1-only PuTTY, we used to store font names simply as a
      * valid X font description string (logical or alias), under a
      * bare key such as "Font".
-     * 
+     *
      * In GTK2 PuTTY, we have a prefix system where "client:"
      * indicates a Pango font and "server:" an X one; existing
      * configuration needs to be reinterpreted as having the
@@ -440,73 +449,74 @@ FontSpec *read_setting_fontspec(void *handle, const char *name)
      * provided name string (e.g. "Font") to a suffixed one
      * ("FontName").
      */
-    char *suffname = dupcat(name, "Name", NULL);
+    char *suffname = dupcat(name, "Name");
     char *tmp;
 
     if ((tmp = read_setting_s(handle, suffname)) != NULL) {
         FontSpec *fs = fontspec_new(tmp);
-	sfree(suffname);
-	sfree(tmp);
-	return fs;		       /* got new-style name */
+        sfree(suffname);
+        sfree(tmp);
+        return fs;                     /* got new-style name */
     }
     sfree(suffname);
 
     /* Fall back to old-style name. */
     tmp = read_setting_s(handle, name);
     if (tmp && *tmp) {
-        char *tmp2 = dupcat("server:", tmp, NULL);
+        char *tmp2 = dupcat("server:", tmp);
         FontSpec *fs = fontspec_new(tmp2);
-	sfree(tmp2);
-	sfree(tmp);
-	return fs;
+        sfree(tmp2);
+        sfree(tmp);
+        return fs;
     } else {
-	sfree(tmp);
-	return NULL;
+        sfree(tmp);
+        return NULL;
     }
 }
-Filename *read_setting_filename(void *handle, const char *name)
+Filename *read_setting_filename(settings_r *handle, const char *name)
 {
     char *tmp = read_setting_s(handle, name);
     if (tmp) {
         Filename *ret = filename_from_str(tmp);
-	sfree(tmp);
-	return ret;
+        sfree(tmp);
+        return ret;
     } else
-	return NULL;
+        return NULL;
 }
 
-void write_setting_fontspec(void *handle, const char *name, FontSpec *fs)
+void write_setting_fontspec(settings_w *handle, const char *name, FontSpec *fs)
 {
     /*
      * read_setting_fontspec had to handle two cases, but when
      * writing our settings back out we simply always generate the
      * new-style name.
      */
-    char *suffname = dupcat(name, "Name", NULL);
+    char *suffname = dupcat(name, "Name");
     write_setting_s(handle, suffname, fs->name);
     sfree(suffname);
 }
-void write_setting_filename(void *handle, const char *name, Filename *result)
+void write_setting_filename(settings_w *handle,
+                            const char *name, Filename *result)
 {
     write_setting_s(handle, name, result->path);
 }
 
-void close_settings_r(void *handle)
+void close_settings_r(settings_r *handle)
 {
-    tree234 *tree = (tree234 *)handle;
     struct skeyval *kv;
 
-    if (!tree)
+    if (!handle)
         return;
 
-    while ( (kv = index234(tree, 0)) != NULL) {
-        del234(tree, kv);
+    while ( (kv = index234(handle->t, 0)) != NULL) {
+        del234(handle->t, kv);
         sfree((char *)kv->key);
         sfree((char *)kv->value);
         sfree(kv);
     }
 
-    freetree234(tree);
+    freetree234(handle->t);
+    sfree(handle);
 }
 
 void del_settings(const char *sessionname)
@@ -517,7 +527,11 @@ void del_settings(const char *sessionname)
     sfree(filename);
 }
 
-void *enum_settings_start(void)
+struct settings_e {
+    DIR *dp;
+};
+
+settings_e *enum_settings_start(void)
 {
     DIR *dp;
     char *filename;
@@ -526,63 +540,63 @@ void *enum_settings_start(void)
     dp = opendir(filename);
     sfree(filename);
 
-    return dp;
+    settings_e *toret = snew(settings_e);
+    toret->dp = dp;
+    return toret;
 }
 
-char *enum_settings_next(void *handle, char *buffer, int buflen)
+bool enum_settings_next(settings_e *handle, strbuf *out)
 {
-    DIR *dp = (DIR *)handle;
     struct dirent *de;
     struct stat st;
-    char *fullpath;
-    int maxlen, thislen, len;
-    char *unmunged;
+    strbuf *fullpath;
 
-    fullpath = make_filename(INDEX_SESSIONDIR, NULL);
-    maxlen = len = strlen(fullpath);
+    if (!handle->dp)
+        return NULL;
 
-    while ( (de = readdir(dp)) != NULL ) {
-        thislen = len + 1 + strlen(de->d_name);
-	if (maxlen < thislen) {
-	    maxlen = thislen;
-	    fullpath = sresize(fullpath, maxlen+1, char);
-	}
-	fullpath[len] = '/';
-	strncpy(fullpath+len+1, de->d_name, thislen - (len+1));
-	fullpath[thislen] = '\0';
+    fullpath = strbuf_new();
 
-        if (stat(fullpath, &st) < 0 || !S_ISREG(st.st_mode))
+    char *sessiondir = make_filename(INDEX_SESSIONDIR, NULL);
+    put_datapl(fullpath, ptrlen_from_asciz(sessiondir));
+    sfree(sessiondir);
+    put_byte(fullpath, '/');
+
+    size_t baselen = fullpath->len;
+
+    while ( (de = readdir(handle->dp)) != NULL ) {
+        fullpath->len = baselen;
+        put_datapl(fullpath, ptrlen_from_asciz(de->d_name));
+
+        if (stat(fullpath->s, &st) < 0 || !S_ISREG(st.st_mode))
             continue;                  /* try another one */
 
-        unmunged = unmungestr(de->d_name);
-        strncpy(buffer, unmunged, buflen);
-        buffer[buflen-1] = '\0';
-        sfree(unmunged);
-	sfree(fullpath);
-        return buffer;
+        decode_session_filename(de->d_name, out);
+        strbuf_free(fullpath);
+        return true;
     }
 
-    sfree(fullpath);
-    return NULL;
+    strbuf_free(fullpath);
+    return false;
 }
 
-void enum_settings_finish(void *handle)
+void enum_settings_finish(settings_e *handle)
 {
-    DIR *dp = (DIR *)handle;
-    closedir(dp);
+    if (handle->dp)
+        closedir(handle->dp);
+    sfree(handle);
 }
 
 /*
  * Lines in the host keys file are of the form
- * 
+ *
  *   type@port:hostname keydata
- * 
+ *
  * e.g.
- * 
+ *
  *   rsa@22:foovax.example.org 0x23,0x293487364395345345....2343
  */
 int verify_host_key(const char *hostname, int port,
-		    const char *keytype, const char *key)
+                    const char *keytype, const char *key)
 {
     FILE *fp;
     char *filename;
@@ -593,65 +607,65 @@ int verify_host_key(const char *hostname, int port,
     fp = fopen(filename, "r");
     sfree(filename);
     if (!fp)
-	return 1;		       /* key does not exist */
+        return 1;                      /* key does not exist */
 
     ret = 1;
     while ( (line = fgetline(fp)) ) {
-	int i;
-	char *p = line;
-	char porttext[20];
+        int i;
+        char *p = line;
+        char porttext[20];
 
-	line[strcspn(line, "\n")] = '\0';   /* strip trailing newline */
+        line[strcspn(line, "\n")] = '\0';   /* strip trailing newline */
 
-	i = strlen(keytype);
-	if (strncmp(p, keytype, i))
-	    goto done;
-	p += i;
+        i = strlen(keytype);
+        if (strncmp(p, keytype, i))
+            goto done;
+        p += i;
 
-	if (*p != '@')
-	    goto done;
-	p++;
+        if (*p != '@')
+            goto done;
+        p++;
 
-	sprintf(porttext, "%d", port);
-	i = strlen(porttext);
-	if (strncmp(p, porttext, i))
-	    goto done;
-	p += i;
+        sprintf(porttext, "%d", port);
+        i = strlen(porttext);
+        if (strncmp(p, porttext, i))
+            goto done;
+        p += i;
 
-	if (*p != ':')
-	    goto done;
-	p++;
+        if (*p != ':')
+            goto done;
+        p++;
 
-	i = strlen(hostname);
-	if (strncmp(p, hostname, i))
-	    goto done;
-	p += i;
+        i = strlen(hostname);
+        if (strncmp(p, hostname, i))
+            goto done;
+        p += i;
 
-	if (*p != ' ')
-	    goto done;
-	p++;
+        if (*p != ' ')
+            goto done;
+        p++;
 
-	/*
-	 * Found the key. Now just work out whether it's the right
-	 * one or not.
-	 */
-	if (!strcmp(p, key))
-	    ret = 0;		       /* key matched OK */
-	else
-	    ret = 2;		       /* key mismatch */
+        /*
+         * Found the key. Now just work out whether it's the right
+         * one or not.
+         */
+        if (!strcmp(p, key))
+            ret = 0;                   /* key matched OK */
+        else
+            ret = 2;                   /* key mismatch */
 
-	done:
-	sfree(line);
-	if (ret != 1)
-	    break;
+        done:
+        sfree(line);
+        if (ret != 1)
+            break;
     }
 
     fclose(fp);
     return ret;
 }
 
-int have_ssh_host_key(const char *hostname, int port,
-		      const char *keytype)
+bool have_ssh_host_key(const char *hostname, int port,
+                       const char *keytype)
 {
     /*
      * If we have a host key, verify_host_key will return 0 or 2.
@@ -661,7 +675,7 @@ int have_ssh_host_key(const char *hostname, int port,
 }
 
 void store_host_key(const char *hostname, int port,
-		    const char *keytype, const char *key)
+                    const char *keytype, const char *key)
 {
     FILE *rfp, *wfp;
     char *newtext, *line;
@@ -684,7 +698,7 @@ void store_host_key(const char *hostname, int port,
             sfree(tmpfilename);
             return;
         }
-	sfree(dir);
+        sfree(dir);
 
         wfp = fopen(tmpfilename, "w");
     }
@@ -740,11 +754,11 @@ void read_random_seed(noise_consumer_t consumer)
     fd = open(fname, O_RDONLY);
     sfree(fname);
     if (fd >= 0) {
-	char buf[512];
-	int ret;
-	while ( (ret = read(fd, buf, sizeof(buf))) > 0)
-	    consumer(buf, ret);
-	close(fd);
+        char buf[512];
+        int ret;
+        while ( (ret = read(fd, buf, sizeof(buf))) > 0)
+            consumer(buf, ret);
+        close(fd);
     }
 }
 
@@ -769,7 +783,7 @@ void write_random_seed(void *data, int len)
         }
         char *dir, *errmsg;
 
-	dir = make_filename(INDEX_DIR, NULL);
+        dir = make_filename(INDEX_DIR, NULL);
         if ((errmsg = make_dir_path(dir, 0700)) != NULL) {
             nonfatal("Unable to write random seed: %s", errmsg);
             sfree(errmsg);
@@ -777,9 +791,9 @@ void write_random_seed(void *data, int len)
             sfree(dir);
             return;
         }
-	sfree(dir);
+        sfree(dir);
 
-	fd = open(fname, O_CREAT | O_WRONLY, 0600);
+        fd = open(fname, O_CREAT | O_WRONLY, 0600);
         if (fd < 0) {
             nonfatal("Unable to write random seed: open(\"%s\") "
                      "returned '%s'", fname, strerror(errno));
@@ -789,14 +803,14 @@ void write_random_seed(void *data, int len)
     }
 
     while (len > 0) {
-	int ret = write(fd, data, len);
-	if (ret < 0) {
+        int ret = write(fd, data, len);
+        if (ret < 0) {
             nonfatal("Unable to write random seed: write "
                      "returned '%s'", strerror(errno));
             break;
         }
-	len -= ret;
-	data = (char *)data + len;
+        len -= ret;
+        data = (char *)data + len;
     }
 
     close(fd);
