@@ -2,6 +2,7 @@
 #include <directorylistingparser.h>
 
 #include <libfilezilla/format.hpp>
+#include <libfilezilla/util.hpp>
 
 #include <cppunit/extensions/HelperMacros.h>
 #include <list>
@@ -29,6 +30,7 @@ class CDirectoryListingParserTest final : public CppUnit::TestFixture
 		CPPUNIT_TEST(testIndividual);
 	}
 	CPPUNIT_TEST(testAll);
+	CPPUNIT_TEST(testSpecial);
 	CPPUNIT_TEST_SUITE_END();
 
 public:
@@ -1476,11 +1478,11 @@ void CDirectoryListingParserTest::testIndividual()
 
 	CDirectoryListing listing = parser.Parse(CServerPath());
 
-	std::string msg = fz::sprintf("Data: %s, count: %d", entry.data, listing.GetCount());
+	std::string msg = fz::sprintf("Data: %s, count: %u", entry.data, listing.size());
 	fz::replace_substrings(msg, "\r", std::string());
 	fz::replace_substrings(msg, "\n", std::string());
 
-	CPPUNIT_ASSERT_MESSAGE(msg, listing.GetCount() == 1);
+	CPPUNIT_ASSERT_MESSAGE(msg, listing.size() == 1);
 
 	msg = fz::sprintf("Data: %s  Expected:\n%s\n  Got:\n%s", entry.data, entry.reference.dump(), listing[0].dump());
 	CPPUNIT_ASSERT_MESSAGE(msg, listing[0] == entry.reference);
@@ -1500,7 +1502,7 @@ void CDirectoryListingParserTest::testAll()
 	}
 	CDirectoryListing listing = parser.Parse(CServerPath());
 
-	CPPUNIT_ASSERT(listing.GetCount() == m_entries.size());
+	CPPUNIT_ASSERT(listing.size() == m_entries.size());
 
 	unsigned int i = 0;
 	for (auto iter = m_entries.begin(); iter != m_entries.end(); iter++, i++) {
@@ -1510,6 +1512,40 @@ void CDirectoryListingParserTest::testAll()
 	}
 }
 
+void CDirectoryListingParserTest::testSpecial()
+{
+	m_sync.lock();
+
+	static int index = 0;
+	t_entry const& entry = m_entries[index++];
+
+	m_sync.unlock();
+
+	CServer server;
+	server.SetType(entry.serverType);
+
+	// Insert random whitespace, test must not crash
+	static char const chars[] = {'\r', '\n', ' ', '\t', 0};
+	for (size_t pos = 0; pos <= entry.data.size(); ++pos) {
+		std::string const prefix = entry.data.substr(0, pos);
+		std::string const suffix = entry.data.substr(pos);
+		for (size_t j = 1; j <= 5; ++j) {
+			std::string line = prefix;
+			for (size_t k = 0; k < j; ++k) {
+				line += chars[fz::random_number(0, 4)];
+			}
+			line += suffix;
+
+			CDirectoryListingParser parser(0, server);
+
+			char* data = new char[line.size()];
+			memcpy(data, line.c_str(), line.size());
+			parser.AddData(data, line.size());
+			parser.Parse(CServerPath());
+		}
+	}
+}
+
 void CDirectoryListingParserTest::setUp()
 {
 }
